Package: quickcode
Type: Package
Title: Quick and Essential 'R' Tricks for Better Scripts
Version: 1.0.9
Authors@R: c(
    person("Obinna", "Obianom", email = "idonshayo@gmail.com", role = c("aut", "cre")),
    person("Brice", "Richard", email = "wem3mew@msn.com", role = "aut")
    )
Maintainer: Obinna Obianom <idonshayo@gmail.com>
Description: The NOT functions, 'R' tricks and a compilation of some simple quick plus often used 'R' codes to improve your scripts. Improve the quality and reproducibility of 'R' scripts.
License: MIT + file LICENSE
URL: https://quickcode.rpkg.net
BugReports: https://github.com/oobianom/quickcode/issues
Depends: R (> 3.6)
Imports: utils, grDevices, stats, rstudioapi, tools, fitdistrplus
Suggests: rmarkdown, knitr, qpdf, testthat
Encoding: UTF-8
VignetteBuilder: knitr
Language: en-US
RoxygenNote: 7.3.2
Config/testthat/edition: 3
NeedsCompilation: no
Packaged: 2026-04-04 07:53:17 UTC; askph
Author: Obinna Obianom [aut, cre],
  Brice Richard [aut]
Repository: CRAN
Date/Publication: 2026-04-04 08:20:02 UTC
Built: R 4.5.2; ; 2026-04-04 10:13:29 UTC; unix

Package: quickcode
Type: Package
Title: Quick and Essential 'R' Tricks for Better Scripts
Version: 1.2.0
Authors@R: c(
    person("Obinna", "Obianom", email = "idonshayo@gmail.com", role = c("aut", "cre")),
    person("Brice", "Richard", email = "wem3mew@msn.com", role = "aut")
    )
Maintainer: Obinna Obianom <idonshayo@gmail.com>
Description: The NOT functions, 'R' tricks and a compilation of some simple quick plus often used 'R' codes to improve your scripts. Improve the quality and reproducibility of 'R' scripts.
License: MIT + file LICENSE
URL: https://quickcode.rpkg.net
BugReports: https://github.com/oobianom/quickcode/issues
Depends: R (> 3.6)
Imports: utils, grDevices, stats, rstudioapi, tools, fitdistrplus
Suggests: rmarkdown, knitr, qpdf, testthat
Encoding: UTF-8
VignetteBuilder: knitr
Language: en-US
RoxygenNote: 7.3.2
Config/testthat/edition: 3
NeedsCompilation: no
Packaged: 2026-08-26 16:26:24 UTC; askph
Author: Obinna Obianom [aut, cre],
  Brice Richard [aut]
Repository: CRAN
Date/Publication: 2026-08-26 17:00:02 UTC

# Generated by roxygen2: do not edit by hand

export("%.%")
export("%eo%")
export("%match%")
export("%nin%")
export("%or%")
export(add.header)
export(add.sect.comment)
export(add.snippet.clear)
export(add_key)
export(ai.duplicate)
export(archivedPkg)
export(as.boolean)
export(bionic_txt)
export(cat_to_num)
export(cat_to_num2)
export(chain_func)
export(chain_sep)
export(clean)
export(clean_file)
export(clear)
export(cols.rep)
export(compHist)
export(const)
export(create_shape)
export(data_pop)
export(data_pop_filter)
export(data_push)
export(data_rep)
export(data_shuffle)
export(date1to3)
export(date3to1)
export(detect_outlier)
export(detect_outlier2)
export(duplicate)
export(error.out)
export(extract_IP)
export(extract_comment)
export(fAddDate)
export(find_packages)
export(from_tensor_slices)
export(geo.cv)
export(geo.mean)
export(geo.sd)
export(getDate)
export(getDistribution)
export(getGitRepoChange)
export(getGitRepoStart)
export(getWeekSeq)
export(get_func_def)
export(has.error)
export(header.rmd)
export(in.range)
export(inc)
export(indexed)
export(init)
export(insertInText)
export(iqr_outlier)
export(is.Date)
export(is.cauchy)
export(is.decreasing)
export(is.empty)
export(is.gamma)
export(is.image)
export(is.increasing)
export(is.leap)
export(is.logistic)
export(is.lognormal)
export(is.normal)
export(is.poisson)
export(is.uniform)
export(is.weibull)
export(lastwd)
export(learn_rate_scheduler)
export(libraryAll)
export(list_push)
export(list_shuffle)
export(make_dosing_df)
export(math.mm)
export(math.qt)
export(minus)
export(mix.color)
export(mix.cols.btw)
export(mode.calc)
export(multihead_att)
export(mutate_filter)
export(na.cumsum)
export(ndecimal)
export(newSuperVar)
export(nonmem_scaling)
export(normalize.vector)
export(not.Date)
export(not.data)
export(not.duplicated)
export(not.empty)
export(not.environment)
export(not.exists)
export(not.image)
export(not.inherits)
export(not.integer)
export(not.logical)
export(not.na)
export(not.null)
export(not.numeric)
export(not.vector)
export(number)
export(or)
export(pairDist)
export(percent_match)
export(plus)
export(rDecomPkg)
export(randString)
export(rcolorconst)
export(read.csv.print)
export(read.table.print)
export(refresh)
export(remove_comment)
export(remove_content_in_quotes)
export(rows.rep)
export(sample_by_column)
export(setDisAlpha)
export(setOnce)
export(sort_file_type)
export(sort_length)
export(sound_match)
export(strsplit.bool)
export(strsplit.num)
export(sub.range)
export(summarize.envobj)
export(switch_cols)
export(switch_rows)
export(track_func)
export(trim.file)
export(unique_len)
export(unsetDisAlpha)
export(vector_pop)
export(vector_push)
export(vector_shuffle)
export(yesNoBool)
export(zscore)
export(zscoreGrowthCurve)
export(zscore_outlier)
export(zscore_outlier2)
importFrom(grDevices,col2rgb)
importFrom(grDevices,graphics.off)
importFrom(grDevices,rainbow)
importFrom(grDevices,rgb)
importFrom(graphics,abline)
importFrom(graphics,axis)
importFrom(graphics,barplot)
importFrom(graphics,boxplot)
importFrom(graphics,legend)
importFrom(graphics,lines)
importFrom(graphics,par)
importFrom(graphics,points)
importFrom(graphics,polygon)
importFrom(graphics,rug)
importFrom(graphics,segments)
importFrom(graphics,text)
importFrom(stats,approx)
importFrom(stats,density)
importFrom(stats,mad)
importFrom(stats,median)
importFrom(stats,quantile)
importFrom(stats,sd)
importFrom(stats,setNames)
importFrom(utils,adist)
importFrom(utils,read.csv)
importFrom(utils,read.table)

<p align="center">
  <img src="inst/logo.png" alt="quickcode logo" width="160"/>
</p>

<h1 align="center">quickcode</h1>

<p align="center">
  Quick and essential 'R' tricks for better, cleaner, more reproducible scripts.
</p>

<p align="center">
  <a href="https://cran.r-project.org/package=quickcode"><img src="https://www.r-pkg.org/badges/version/quickcode" alt="CRAN status"/></a>
  <a href="https://cran.r-project.org/package=quickcode"><img src="https://cranlogs.r-pkg.org/badges/grand-total/quickcode" alt="CRAN downloads"/></a>
  <img src="https://img.shields.io/badge/license-MIT-blue.svg" alt="License: MIT"/>
</p>

quickcode is a collection of small, dependable helpers for everyday R scripting: a full family of `not.*` validation functions, PHP-style array/data-frame operations (`push`, `pop`, `shuffle`, `rep`), nullish-coalescing and "not in" operators, outlier and distribution detection, super variables, function-usage tracking, RStudio snippet addins, and more. None of it is flashy — it just removes boilerplate you'd otherwise rewrite in every project.

## Installation

Install the released version from CRAN:

```r
install.packages("quickcode")
```

Or install the development version from GitHub:

```r
# install.packages("devtools")
devtools::install_github("oobianom/quickcode")
```

Load it like any other package:

```r
library(quickcode)
```

## Why quickcode

- **Consistent negations.** `is.null()` has no built-in opposite. quickcode adds `not.null()`, `not.na()`, `not.numeric()`, `not.integer()`, `not.logical()`, `not.vector()`, `not.data()`, `not.environment()`, `not.duplicated()`, `not.image()`, `not.inherits()`, `not.exists()` and friends, so conditionals read the way you'd say them out loud.
- **PHP-style array ergonomics.** `vector_push()`, `vector_pop()`, `data_push()`, `data_pop()`, `list_push()`, `data_shuffle()`, `data_rep()` bring the array functions many scripters miss from other languages, working across vectors, lists, and data frames.
- **Operators that save keystrokes.** `%nin%` (not in), `%or%` (nullish coalescing), `%eo%` (error coalescing), and `%match%` (string similarity) shorten common conditional patterns.
- **One-call environment resets.** `refresh()` / `clean()` clear the console, clear the environment, reset the working directory, and reload files/libraries in a single call.
- **Statistics and outlier helpers.** Z-scores, IQR-based and grouped outlier detection, distribution fit-checking (`is.normal`, `is.poisson`, `is.weibull`, ...), geometric mean/SD/CV, and histogram comparisons.
- **Super variables.** `newSuperVar()` creates a variable that is accessible and mutable from any scope, with optional locking, class enforcement, and a limited number of permitted edits.
- **RStudio addins.** Insert header comments, section dividers, or environment-reset snippets straight from the RStudio Addins menu — no need to remember or retype boilerplate.
- **Function usage tracking.** `track_func()` wraps functions to record call frequency, timing, and argument patterns, for lightweight profiling and usage analytics.

## Quick examples

![](https://quickcode.obi.obianom.com/trimfile.gif)


**The `not.*` family**

```r
not.null(NULL)      # TRUE
not.na(NA)           # FALSE
not.integer(45L)     # FALSE
not.integer(45)      # TRUE
```

**Nullish coalescing and "not in"**

```r
NULL %or% "default"        # "default"
NA %or% "default"          # "default"
5 %or% "default"           # 5

5 %nin% c(1:10)            # FALSE
5 %nin% c(11:20)           # TRUE
```

**PHP-style vector and data frame helpers**

```r
p1 <- c(6, 7, 8)
p2 <- c(1, 2, 3)
vector_push(p1, p2)
p1
#> [1] 6 7 8 1 2 3

df1 <- data.frame(ID = 1:10, ID2 = 1:10)
df2 <- data.frame(ID = 11:20, ID2 = 21:30)
data_push(df1, df2, "rows")
```

**One-line variable initialization**

```r
init(a, b, c)               # a, b, c set to NULL
init(x, y, z, value = 5)    # x, y, z set to 5
```

**Environment reset**

```r
quickcode::refresh()

quickcode::clean(
  setwd  = "/path/to/project",
  source = c("file1.R", "file2.R"),
  load   = c("data.RData")
)
```

**Super variables**

```r
newSuperVar(config, value = list(threshold = 10), lock = TRUE)
config                 # view current value
config.set(list(threshold = 20))  # blocked while locked
```

**Outlier and distribution checks**

```r
x <- c(rnorm(100), 50)
detect_outlier2(x)
zscore(x)
is.normal(rnorm(100))
```

See `vignette("quickcode_r_introduction")` for a longer walkthrough.

## Function reference

### NOT / validation functions

| Function | Description |
|---|---|
| `not.null` | Not NULL |
| `not.na` | Not NA |
| `not.empty` / `is.empty` | Not empty / is empty |
| `not.numeric` | Not numeric |
| `not.integer` | Not an integer |
| `not.logical` | Not logical |
| `not.vector` | Not a vector |
| `not.data` | Not a data object |
| `not.duplicated` | Not duplicated elements |
| `not.environment` | Not an environment |
| `not.image` / `is.image` | File extension is/isn't an image |
| `not.inherits` | Does not inherit from specified classes |
| `not.exists` | Object does not exist |
| `has.error` | Check if a call or expression produces an error |

### Operators

| Operator | Description |
|---|---|
| `%nin%` (alias `%!in%`) | Not in vector or array |
| `%or%` | Nullish coalescing operator |
| `%eo%` | Error coalescing operator |
| `%match%` | Percentage string similarity match |
| `%.%` | Simple function chaining |

### PHP-style array / data operations

| Function | Description |
|---|---|
| `vector_push` / `vector_pop` | Add / remove elements from a vector |
| `list_push` | Add elements to a list |
| `data_push` / `data_pop` | Add / remove rows or columns from a data frame |
| `data_pop_filter` | Remove elements from data matching a filter |
| `data_rep` / `rows.rep` / `cols.rep` | Duplicate rows or columns X times |
| `data_shuffle` / `list_shuffle` / `vector_shuffle` | Shuffle a data frame, list, or vector |
| `switch_rows` / `switch_cols` | Swap two rows or columns |
| `sample_by_column` | Re-sample a dataset by column |
| `mutate_filter` | Mutate only a subset of a dataset |
| `add_key` (alias `indexed`) | Add index keys to a vector, list, data frame, or matrix |

### Variables & environment

| Function | Description |
|---|---|
| `init` | Initialize one or more variables/objects at once |
| `newSuperVar` | Create a variable accessible/mutable from any scope, with locking and edit limits |
| `setOnce` | Set a variable only once |
| `refresh` / `clean` / `libraryAll` | Clear environment/console, reset working directory, load files & libraries |
| `summarize.envobj` | Summarize environment objects and their sizes |
| `lastwd` | Return to the previous working directory |

### Statistics, outliers & distributions

| Function | Description |
|---|---|
| `zscore` / `zscore_outlier` / `zscore_outlier2` | Z-score calculation and z-score-based outlier flagging |
| `detect_outlier` / `detect_outlier2` / `iqr_outlier` | Outlier detection (including grouped) |
| `is.normal`, `is.lognormal`, `is.uniform`, `is.poisson`, `is.gamma`, `is.weibull`, `is.cauchy`, `is.logistic` | Test whether data fits a given distribution |
| `getDistribution` | Fit-check data against a distribution |
| `geo.mean`, `geo.sd`, `geo.cv` | Geometric mean, standard deviation, coefficient of variation |
| `mode.calc` | Mode of a numeric or character vector |
| `na.cumsum` | Cumulative sum with NA removal |
| `normalize.vector` | Normalize a numeric vector to [0, 1] |
| `sub.range` / `in.range` | Range difference / range membership checks |
| `unique_len` | Combine `unique()` and `length()` |
| `pairDist` | Distance of points from a cluster center |
| `compHist` | Compare histograms of two distributions |

### Machine learning & modeling utilities

| Function | Description |
|---|---|
| `cat_to_num` / `cat_to_num2` | Convert categorical values to numeric |
| `from_tensor_slices` | Create tensor-like slices from a data frame or matrix |
| `multihead_att` | Multi-head attention computation |
| `learn_rate_scheduler` | Learning rate scheduling utilities |
| `make_dosing_df` | Create subject-by-time dosing records with covariates |

### Strings & text

| Function | Description |
|---|---|
| `bionic_txt` | Generate bionic-reading formatted text |
| `randString` | Generate a random string |
| `strsplit.bool` / `strsplit.num` | Split a string into a boolean / numeric vector |
| `percent_match` | Percentage match between two strings |
| `ndecimal` | Count decimal places in a number |
| `as.boolean` / `yesNoBool` | Convert between boolean representations |
| `extract_comment` / `remove_comment` / `remove_content_in_quotes` | Text/comment extraction and cleanup |
| `getDate` | Extract all dates from a string |
| `extract_IP` | Extract IP addresses from a string |

### Dates

| Function | Description |
|---|---|
| `date1to3` / `date3to1` | Combine vectors into a Date, or split a Date into vectors |
| `getWeekSeq` | Convert dates into numeric week counts |
| `is.leap` | Check whether a year is a leap year |
| `fAddDate` | Append a date to a filename |
| `is.increasing` / `is.decreasing` | Check whether values in a vector are increasing/decreasing |

### Files & filesystem

| Function | Description |
|---|---|
| `duplicate` | Duplicate a file with global text replacement |
| `ai.duplicate` | Prompt-guided duplication and editing of files |
| `trim.file` | Remove empty lines from a file |
| `sort_file_type` / `sort_length` | Sort a vector by file type or content length |
| `read.csv.print` / `read.table.print` | Read and preview the first X rows/columns of a file |
| `insertInText` | Insert a string into the current RStudio file (Shiny helper) |

### RStudio addins & snippets

| Addin / function | Description |
|---|---|
| `add.header` | Add a header comment to the current R file |
| `header.rmd` | Add a header comment to the current Rmd file |
| `add.sect.comment` | Insert a custom section comment |
| `add.snippet.clear` | Insert a console-clear / set-directory snippet |

### Colors & shapes

| Function | Description |
|---|---|
| `rcolorconst` | Named R color constants |
| `mix.color` / `mix.cols.btw` | Blend two or more colors |
| `setDisAlpha` / `unsetDisAlpha` | Set/unset color transparency |
| `create_shape` | Create geometric shapes with optional text |

### Package & repo utilities

| Function | Description |
|---|---|
| `find_packages` | Search CRAN packages by keyword |
| `archivedPkg` | List all CRAN-archived R packages |
| `rDecomPkg` | Check whether a package has been decommissioned from CRAN |
| `getGitRepoStart` / `getGitRepoChange` | Fetch a GitHub repo's creation / last-updated date |
| `track_func` | Track function call frequency, timing, and usage patterns |

### Math & misc

| Function | Description |
|---|---|
| `plus` / `minus` / `inc` | Increment or decrement a vector by a value |
| `number` | Generate random integers |
| `const` | Common mathematical constants |
| `math.mm` / `math.qt` | Miscellaneous math and confidence-interval computations |
| `error.out` (`%eo%`) | Return an alternative value if an expression errors |
| `or` (`%or%`) | Return an alternative value if an expression is empty/NA/NULL |
| `chain_sep` (`%.%`) / `chain_func` | Simple function chaining |

This list covers the most commonly used functions; run `library(help = "quickcode")` or browse the `man/` directory for the complete, authoritative reference with full argument details and runnable examples.

## Vignettes

- `vignette("quickcode_r_introduction")` — general tour of the package
- `vignette("add_today_date_to_filenames_quickcode")` — appending dates to filenames with `fAddDate()`
- `vignette("nullish_coalescing_operator_r")` — using `%or%` and `%eo%`
- `vignette("track_function_usage_r")` — profiling function usage with `track_func()`

## Getting help

- Bug reports and feature requests: <https://github.com/oobianom/quickcode/issues>
- Package website: <https://quickcode.rpkg.net>

## Authors

- Obinna (OBI) Obianom — [idonshayo@gmail.com](mailto:idonshayo@gmail.com)
- Brice Richard — [wem3mew@msn.com](mailto:wem3mew@msn.com)

## License

MIT © Obinna Obianom. See [LICENSE](LICENSE) for details.

## -----------------------------------------------------------------------------
# quickcode::clean(clearPkgs = TRUE) #clear environment and clear previous package
quickcode::libraryAll(datasets) #load in-built dataset

## -----------------------------------------------------------------------------
data("USArrests")

filenames = paste0(row.names(USArrests), ".txt")

head(filenames,8) # before adding today's date

rev.filenames = fAddDate(filenames) # append today's date to the file names

rev.filenames

#...write further codes to use the file names to save new files.

#' Compare histograms of two distributions
#'
#' For comparing histograms of two data distributions.
#' Simply input the two distributions, and it generates a clear and
#' informative histogram that illustrates the differences between the data.
#'
#' @param x1 NUMERIC. the first distribution
#' @param x2 NUMERIC. the second distribution
#' @param title CHARACTER. title of the histogram plot
#' @param col1 CHARACTER. color fill for first distribution
#' @param col2 CHARACTER. color fill for second distribution
#' @param xlab CHARACTER. label of the x-axis
#' @param ylab CHARACTER. label of the y-axis
#' @param separate LOGICAL. whether to separate the plots
#' @return return histogram comparison using basic histogram plot
#'
#' @details
#' Users have the option to view individual histograms for each distribution
#' before initiating the comparison, allowing for a detailed examination of
#' each dataset's characteristics. This feature ensures a comprehensive
#' understanding of the data and enhances the user's ability to interpret
#' the results of the distribution comparison provided by this function.
#'
#' @section Some recommended color pairs:
#' col1 = 'dodgerblue4' (and) col2 = 'darksalmon' \cr
#' col1 = 'brown' (and) col2 = 'beige' \cr
#' col1 = 'pink' (and) col2 = 'royalblue4' \cr
#' col1 = 'red' (and) col2 = 'yellow' \cr
#' col1 = 'limegreen' (and) col2 = 'blue' \cr
#' col1 = 'darkred' (and) col2 = 'aquamarine4' \cr
#' col1 = 'purple' (and) col2 = 'yellow' \cr
#'
#' @note
#' - Hexadecimal values can also be passed \cr
#'   in for col1 and col2, see the example section
#' - For best visual results, \cr
#'   col1 should be a dark color and col2 should be passed as a light color. \cr
#'   For example, col1 = "black", col2 = "yellow"
#'
#'
#' @examples
#' # compare two normal distributions with means that differ a lot
#' # in this case, the overlap will not be observed
#' set.seed(123)
#' compHist(
#'   x1 = rnorm(1000, mean = 3),
#'   x2 = rnorm(1000, mean = 10),
#'   title = "Histogram of Distributions With Means 3 & 10",
#'   col1 = "yellow", col2 = "violet"
#' )
#'
#'
#' # compare two normal distributions with means that are close
#' # in this case, the overlap between the histograms will be observed
#' set.seed(123)
#' compHist(
#'   x1 = rnorm(1000, mean = 0),
#'   x2 = rnorm(1000, mean = 2),
#'   title = "Histogram of rnorm Distributions With Means 0 & 2",
#'   col1 = "lightslateblue", col2 = "salmon"
#' )
#'
#' set.seed(123)
#' # separate the plots for preview
#' compHist(
#'   x1 = rnorm(1000, mean = 0),
#'   x2 = rnorm(1000, mean = 2),
#'   title = c("Plot Means 0", "Plot Means 2"),
#'   col1 = "#F96167", col2 = "#CCF381",
#'   separate = TRUE
#' )
#'
#' @export

compHist <- function(x1, x2, title, col1 = "red", col2 = "yellow", xlab = "", ylab = "Frequency", separate = FALSE) {
  # compute means, min and max
  meanx1 <- round(mean(x1), 1)
  meanx2 <- round(mean(x2), 1)
  x1x2 <- c(x1, x2)
  minx <- min(x1x2) - 0.1 * min(x1x2)
  maxx <- max(x1x2) + 0.1 * max(x1x2)

  # close devices if open
  # if (.Device != "null device") grDevices::dev.off()

  # check if plots should be separated
  if (separate) graphics::par(mfrow = c(1, 2))
  if (separate & length(title) != 2) {
    stop("Title must contain two titles if the plots are to be separated")
  }


  # make plots
  cl1 <- grDevices::col2rgb(col1) / 255
  cl1b <- grDevices::rgb(cl1[1, 1], cl1[2, 1], cl1[3, 1], alpha = 0.6)
  graphics::hist(x1,
    main = ifelse(separate, title[1], title),
    xlab = xlab,
    ylab = ylab,
    col = cl1b,
    xlim = c(minx, maxx)
  )

  cl2 <- grDevices::col2rgb(col2) / 255
  cl2b <- grDevices::rgb(cl2[1, 1], cl2[2, 1], cl2[3, 1], alpha = 0.6)
  graphics::hist(x2,
    main = ifelse(separate, title[2], title),
    xlab = xlab,
    ylab = ylab,
    col = cl2b,
    xlim = c(minx, maxx),
    add = ifelse(separate, FALSE, TRUE)
  )

  # add legend if the plot is combined
  if (!separate) {
    graphics::legend("topright",
      legend = c(paste0("Mean: ", meanx1), paste0("Mean: ", meanx2), "Overlap"),
      fill = c(cl1b, cl2b, mix.color(c(col1, col2), 2, 1))
    )
  }
}


#' Mix or Blend two or more colors
#'
#' Combine colors to generate a new color
#'
#' @param color CHARACTER. color vector e.g see example
#' @param type NUMERIC. return type of the output
#' @param alpha NUMERIC. alpha or opacity of the resulting color
#' @return hex for the combined color
#' @examples
#' # color vector
#' colvec <- c("red", "blue", "violet", "green", "#ff0066")
#'
#' # just one color
#' mix.color(colvec[1], type = 1, alpha = 1)
#'
#' # add two colors
#' mix.color(colvec[1:2], type = 1, alpha = 1)
#'
#' # add three colors
#' mix.color(colvec[1:3], type = 1, alpha = 1)
#'
#'
#' # return type = 2
#'
#' # just one color
#' mix.color(colvec[1], type = 2, alpha = 1)
#'
#' # add two colors
#' mix.color(colvec[1:2], type = 2, alpha = 1)
#'
#' # add three colors
#' mix.color(colvec[1:3], type = 2, alpha = 1)
#'
#'
#' # opacity or alpha  0.5
#'
#' # just one color
#' mix.color(colvec[1], type = 1, alpha = 0.5)
#'
#' # add two colors
#' mix.color(colvec[1:2], type = 1, alpha = 0.5)
#'
#' # add three colors
#' mix.color(colvec[1:3], type = 1, alpha = 0.5)
#'
#' # add all colors
#' mix.color(colvec, type = 1, alpha = 0.5)
#'
#' @export
mix.color <- function(color, type = 2, alpha = 1) {
  stopifnot(alpha <= 1, alpha >= 0, type <= 3, type >= 1)
  vals <- apply(grDevices::col2rgb(color), 1, mean)
  switch(type,
    "1" = grDevices::rgb(vals[1], vals[2], vals[3], alpha * 255, maxColorValue = 255),
    "2" = grDevices::rgb(vals[1] / 255, vals[2] / 255, vals[3] / 255, alpha = alpha),
    "3" = vals
  )
}

#' Mix or Blend colors between two or more colors
#'
#' Mix or blend multiple colors between two colors
#' @param colors the vector of two colors
#' @param max maximum number of colors to blend between
#' @param alpha alpha for the new color blends
#' @param preview LOGICAL. preview all color generated
#' @return color hex for all generated colors
#'
#' @examples
#' # simply mix/blend two colors
#' mix.cols.btw(c("red","brown"))
#'
#' # simply mix/blend two colors, maximum number of colors at the end
#' mix.cols.btw(c("red","brown"), max = 8)
#'
#' # simply mix/blend two colors with alpha=0.2 (opacity=0.2)
#' mix.cols.btw(c("yellow","green"),alpha = 0.2)
#'
#' # also preview after mixing the two colors
#' mix.cols.btw(c("red","green"), preview = TRUE)
#' mix.cols.btw(c("blue","violet"),alpha = 0.2, preview = TRUE)
#'
#' mix.cols.btw(c("red","purple","yellow","gray"), preview = TRUE)
#'
#' mix.cols.btw(c("red","purple","yellow","gray"),alpha = 0.2, preview = TRUE)
#'
#' @export
mix.cols.btw <- function(colors, max = 20, alpha = 1, preview = FALSE) {
  bbcount <- new.env()
  bbcount$cnt <- length(colors)
  repeat{
    colors <- unlist(lapply(split(colors, ceiling(seq_along(colors) / 2)), function(ol) {
        if (length(ol[not.na(ol)]) > 1) {
          nwcol <- mix.color(ol, alpha = alpha)
          bbcount$cnt <- 1 + bbcount$cnt
          if(bbcount$cnt>max) ol else append(ol, nwcol, 1)
        } else {
          ol
        }
    }))
    if (length(colors) >= max) break
  }
  colors <- colors[not.na(colors)]

  # preview the colors generated sing swatch
  if (preview) {
    cls <- as.character(colors)
    names(cls) <- cls
    custom_swatch(cls, main = "Preview of color mix")
    #Polychrome::swatch(cls, main = "Preview of color mix")
  }

  # return color
  as.character(colors)
}

#' Not an environment
#'
#' Check if entry is not an environment object
#'
#' @param x vector entry
#' @return a boolean value to indicate if entry is an environment
#' @examples
#' test.env <- new.env()
#' test.notenv <- list(t=1)
#' not.environment(test.env) # FALSE
#' not.environment(test.notenv) # TRUE
#' if(not.environment(test.notenv)) message("yes") # yes
#'
#' @export

not.environment <- function(x) !is.environment(x)

#' Not a data
#'
#' Opposite of is.data.frame(). Check if entry is not a data object
#'
#' @param x vector entry
#' @return a boolean value to indicate if entry is a data table
#' @examples
#' test.dt <- data.frame(ID=1:200,Type="RPKG.net")
#' test.notenv <- list(t=1)
#'
#' is.data.frame(test.dt) # TRUE
#' not.data(test.dt) # FALSE
#'
#' not.data(test.notenv) # TRUE
#' if(not.data(test.dt)) message("yes") # NULL
#'
#' @export

not.data <- function(x)!{
  is.data.frame(x)
}

#' Fetch GitHub Repository Creation & Last Updated Date
#'
#' @description
#' The GitHub REST API is a powerful tool that allows developers
#' to interact with GitHub programmatically. It provides a set of
#' endpoints that allows a user to create integration, retrieve data,
#' and automate workflows related to GitHub repositories. It is a means
#' by which users can interact with GitHub without directly using a web interface.
#'
#' @details
#' The two functions utilize the GitHub REST API to extract important temporal information about a GitHub repository. \cr\cr
#'  - the getGitRepoStart function is used to retrieve the date a GitHub repository was first created.\cr\cr
#'  - the getGitRepoChange function retrieves the date a GitHub repository was last updated.
#'
#' @rdname github-tweaks
#' @param repo_name full path of the repository eg. "cran/quickcode"
#' @param out.format date output format eg. "%m %d %y"
#' @return date of creation of repository as a character
#'
#' @examples
#' \donttest{
#' # Use default date format
#' getGitRepoStart(repo_name = "oobianom/quickcode")
#'
#' # Specify date format
#' getGitRepoStart(repo_name = "oobianom/quickcode", out.format = "%j|%Y")
#' getGitRepoStart(repo_name = "oobianom/quickcode", out.format = "%D|%j")
#' }
#'
#' @export
getGitRepoStart = function(repo_name,out.format = "%Y-%m-%d"){
  if(!grepl("/",repo_name))
    stop("The format of the repo_name is incorrect. It should include user name and repository eg. 'oobianom/r2social'")
  str = paste0(git.api, repo_name)
  read = readLines(str, warn = FALSE)
  pat = unlist(gregexpr("created_at", read)) + 13
  format(as.POSIXct(substr(read, start = pat, stop = pat + 18), format = "%Y-%m-%dT%H:%M:%S"),out.format)
}



#' @rdname github-tweaks
#' @param repo_name name of the repository
#' @param out.format date output format
#' @return date of the last update of repository as a character
#'
#' @examples
#' \donttest{
#' getGitRepoChange(repo_name = "oobianom/shinyStorePlus", out.format = "%d-%b-%Y")
#' getGitRepoChange(repo_name = "oobianom/r2social", out.format = "%Y/%m/%d")
#' }
#'
#' @export
getGitRepoChange = function(repo_name,out.format = "%Y-%m-%d"){
  if(!grepl("/",repo_name))
    stop("The format of the repo_name is incorrect. It should include user name and repository eg. 'oobianom/r2social'")
  str = paste0(git.api, repo_name)
  read = readLines(str, warn = FALSE)
  pat = unlist(gregexpr("updated_at", read)) + 13
  format(as.POSIXct(substr(read, start = pat, stop = pat + 18), format = "%Y-%m-%dT%H:%M:%S"),out.format)
}

#' Add index  keys to a vector or data frame or list or matrix
#'
#' Index a vector or lists and convert to a list of objects
#'
#' @param vector vector or data frame to transform
#' @param key variable name for keys
#' @param value variable name for values
#' @return a transformed list containing keys along with vector values
#' @details
#' This function takes a vector and turns it into a list containing 'key' and 'value' for each vector.
#' This allows the output to be used in loops such as for loops or lapply or other functions to track
#' the index of the list content e.g. 1,2,3...\cr\cr
#'
#' This function also contains a validator to ensure that a vector had not been previously 'keyed',
#' which prevents the user from inadvertently calling the function twice on a vector. Helps especially
#' because the function keys the vector, and sets the new list to the variable name of the original
#' vector.
#'
#' @note
#' add_key - resaves the keys and value pairs to original variable\cr\cr
#' indexed - return the keys and value pairs
#'
#' @section Use case:
#' Efficient for loops and for tracking various steps through a vector contents
#' @rdname addkey
#' @examples
#' # EXAMPLES for add_key()
#'
#'
#' #ex1 simple conversion of a vector
#' rti2 <- c("rpkg","obinna", "obianom")
#' add_key(rti2)
#' rti2
#'
#' #ex2 add keys to a vector content for use in downstream processes
#' ver1 <- c("Test 1","Test 2","Test 3")
#' add_key(ver1)
#'
#' #ex3 use keyed ver1 in for loop
#' for(i in ver1){
#'   message(sprintf("%s is the key for this %s", i$key, i$value))
#' }
#'
#' #ex4 use keyed ver1 in lapply loop
#' xl1 <- lapply(ver1,function(i){
#'   message(sprintf("lapply - %s is the key for this %s", i$key, i$value))
#' })
#'
#'
#' @export
#'
add_key <- function(vector){
  if(inherits(vector,"klist")) stop("Key already added to object.")

  #create list and add keys
  . = list()
  iky = 1
  for(i in vector){
    .[[length(.)+1]] <- list(key = iky, value = i)
    inc(iky)
  }
  #resave to vector name
  .. <- substitute(vector)
  class(.) <- c('klist','list')
  assign(as.character(..), ., envir = parent.frame())
}




#' @rdname addkey
#' @details
#' This function takes a vector and turns it into a list containing 'key' and 'value' for each vector.
#' This allows the output to be used in loops such as for loops or lapply or other functions to track
#' the index of the list content e.g. 1,2,3...\cr\cr
#'
#' This function also contains a validator to ensure that a vector had not been previously 'keyed',
#' which prevents the user from inadvertently calling the function twice on a vector. Helps especially
#' because the function keys the vector, and sets the new list to the variable name of the original
#' vector.
#'
#'
#'
#' @examples
#' # EXAMPLES for indexed()
#'
#' #ex1 simple conversion of a vector
#' rti2 <- c("rpkg","obinna", "obianom")
#' indexed(rti2)
#'
#' #ex2 add keys to a vector content for use in downstream processes
#' ver1 <- c("Test 1","Test 2","Test 3")
#'
#'
#' #ex3 use keyed ver1 in for loop
#' for(i in indexed(ver1)){
#'   message(sprintf("%s is the key for this %s", i$key, i$value))
#' }
#'
#' #ex4 use keyed ver1 in for loop
#' #specify name for key and value
#' for(i in indexed(ver1,k,v)){
#'   message(
#'   sprintf("%s is the new key for this value %s",
#'   i$k, i$v))
#' }
#'
#' #ex5 use keyed ver1 in lapply loop
#' xl1 <- lapply(indexed(ver1),function(i){
#'   message(sprintf("lapply - %s is the key for this %s", i$key, i$value))
#' })
#'
#'
#' @export
#'
indexed <- function(vector,key = key,value = value){
  #create list and add keys
  .. = list()
  count = 1
  for(i in vector){
    num <- length(..)+1
    ..[[num]] <- list()
    ..[[num]][[as.character(substitute(key))]] = count
    ..[[num]][[as.character(substitute(value))]] = i
    inc(count) #increment count
  }
  ..
}

#' Remove last n rows or column or specified elements from a data frame like array_pop in PHP
#'
#' Shorthand to remove elements from a data frame and save as the same name
#'
#' @param . parent data
#' @param n number of elements to remove
#' @param which whether to remove from row or from column
#' @param ret TRUE or FALSE. whether to return value instead of setting it to the parent data
#' @return data with elements removed
#' @examples
#' #basic example: pop off 1 row from sample data
#' data.frame(ID=1:10,N=number(10)) #before
#' data_pop(data.frame(ID=1:10,N=number(10))) #after
#'
#' #using data objects
#' data.01 <- mtcars[1:7,]
#'
#' #task: remove 1 element from the end of the data and set it to the data name
#' data.01 #data.01 data before pop
#' data_pop(data.01) #does not return anything
#' data.01 #data.01 data updated after pop
#'
#' #task: remove 3 columns from the end of the data and set it to the data name
#' data.01 #data.01 data before pop
#' data_pop(data.01, n = 3, which = "cols") #does not return anything, but updates data
#' data.01 #data.01 data updated after pop
#'
#' #task: remove 5 elements from the end, but do not set it to the data name
#' data.01 #data.01 data before pop
#' data_pop(data.01,5, ret = TRUE) #return modified data
#' data.01 #data.01 data remains the same after pop
#'
#'
#' @export
#'
data_pop <- function(., n = 1, which = c("rows", "cols"), ret = FALSE) {
  .. <- substitute(.)
  wh <- match.arg(which)
  if (typeof(..) == "language"){
    switch(wh,
           "rows" = {
             d05 <- .[1:(nrow(.)-n),]
           },
           "cols" = {
             d05 <- .[,1:(ncol(.)-n)]
           }
    )
    return(d05)
  }
  if (typeof(..) != "symbol") stop(paste0(.., " must be an object."))
  data <- as.data.frame(get(as.character(..), envir = parent.frame()))
  if(nrow(data) & ncol(data)){
    switch(wh,
           "rows" = {
             data <- data[1:(nrow(data)-n),]
           },
           "cols" = {
             data <- data[,1:(ncol(data)-n)]
           }
    )
  }
  if(!ret) assign(as.character(..), data, envir = parent.frame())
  else data
}

#' Clear environment, clear console, set work directory and load files
#'
#' Shorthand to quickly clear console, clear environment, set working directory, load files
#' @rdname clearenvironment
#' @param setwd OPTIONAL. set working directory
#' @param source OPTIONAL. source in file(s)
#' @param load OPTIONAL. load in Rdata file(s)
#' @param clearPkgs Clear previous loaded packages, TRUE or FALSE
#' @return cleared environment and set directory
#' @details
#' The purpose of this function is provide a one-line code to clear the console, clear the environment,
#' set working directory to a specified path, source in various files into the current file, and
#' load RData files into the current environment. The first process in the sequence of events is to clear the
#' environment. Then the working directory is set, prior to inclusion of various files and RData. With the directory
#' being set first, the path to the sourced in or RData files will not need to be appended to the file name. See examples.
#'
#' @examples
#' \donttest{
#' if(interactive()){
#' #simply clear environment, clear console and devices
#' quickcode::clean()
#'
#' #clear combined with additional arguments
#' quickcode::clean(
#'   clearPkgs = FALSE
#' ) #also clear all previously loaded packages if set to true
#'
#' quickcode::clean(
#'   setwd = "/home/"
#' ) #clear env and also set working directory
#'
#'
#' quickcode::clean(
#'   source = c("/home/file1.R","file2")
#' ) #clear environment and source two files into current document
#'
#'
#' quickcode::clean(
#'   setwd = "/home/",
#'   source = c("file1","file2")
#' ) #clear environment, set working directory and source 2 files into environment
#'
#'
#' quickcode::clean(
#'   setwd = "/home/",
#'   source="file1.R",
#'   load="obi.RData"
#' ) #clear environment, set working directory, source files and load RData
#' }
#' }
#' @export
#'



clean <- function(setwd = NULL, source = c(), load = c(), clearPkgs = FALSE) {
  # clear console, clean garbage and shut devices
  erase() #clear console
  rm(list = setdiff(ls(envir = parent.frame(),all.names = TRUE),
                    c("setwd", "source", "load", "clearPkgs")), envir = parent.frame())
  #graphics off
  if(.Device !="null device" | length(grDevices::dev.list())) graphics.off()
  #close any open connections
  options(NULL)
  close(file())
  #garbage cleanup to free memory
  gc()


  # set directory if it exists
  prevwd <- getwd()
  options("last.used.wd" = prevwd)
  #on.exit(setwd(prevwd))

  if (not.null(setwd)) {
    if (dir.exists(setwd)) {
      setwd(setwd)
    }
  }

  # remove previous loaded packages
  if (clearPkgs) {
    clearPreviouslyLoadedPkgs()
  }

  # load quickcode if not loaded
  if ("quickcode" %nin% (.packages()))
    library(quickcode, quietly = TRUE)

  # source in any required files
  if (length(source)) {
    for (sourced in source) {
      message("Importing ",sourced)
      if (file.exists(sourced)) source(sourced)
      else warning(sourced," does not exist.\n")
    }
  }

  # load in any required data
  if (length(load)) {
    for (loaded in load) {
      message("Loading ",loaded)
      if (file.exists(loaded)) load(loaded, envir = parent.frame())
      else warning(loaded," does not exist.\n")
    }
  }
}

#' Go back to previous directory
#'
#' Navigate to previous working directory if the setwd was called by clean() or refresh() function
#'
#' @note
#' In order to use this function to retrieve the last/previous working directory, you must use the quickcode::clean() or quickcode::refresh() function to set the working directory.
#' @examples
#' lastwd()
#' @return the previous directory
#' @export

lastwd <- function() {
  .lastdir <- options()$last.used.wd
  if (is.null(.lastdir)) {
    message("In order to use this function to retrieve the last/previous working directory, you must use the quickcode::clean() or quickcode::refresh() function to set the working directory.")
  } else {
    options("last.used.wd" = getwd())
    if(dir.exists(.lastdir)) setwd(.lastdir)
    else message("The last directory (",.lastdir,") no longer exists.")
  }
}
#' @export
#' @rdname clearenvironment
clear <- clean

#' Clear environment, clear console, set work directory and load files
#'
#' Shorthand to quickly clear console, clear environment, set working directory, load files
#'
#' @param setwd OPTIONAL. set working directory
#' @param source OPTIONAL. source in file(s)
#' @param load OPTIONAL. load in Rdata file(s)
#' @param clearPkgs clear previously loaded packages
#' @return cleared environment and set directory
#' @details
#' The purpose of this function is provide a one-line code to clear the console, clear the environment,
#' set working directory to a specified path, source in various files into the current file, and
#' load RData files into the current environment. The first process in the sequence of events is to clear the
#' environment. Then the working directory is set, prior to inclusion of various files and RData. With the directory
#' being set first, the path to the sourced in or RData files will not need to be appended to the file name. See examples.
#' @examples
#' \donttest{
#' if(interactive()){
#' #exactly like the clean function
#' #simply clear environment, clear console and devices
#' quickcode::refresh()
#'
#' #clear combined with additional arguments
#' quickcode::refresh(
#'   clearPkgs = FALSE
#' ) #also clear all previously loaded packages if set to TRUE
#'
#' quickcode::refresh(
#'   setwd = "/home/"
#' ) #clear env and also set working directory
#'
#'
#' quickcode::refresh(
#'   source = c("/home/file1.R","file2")
#' ) #clear environment and source two files into current document
#'
#'
#' quickcode::refresh(
#'   setwd = "/home/",
#'   source = c("file1","file2")
#' ) #clear environment, set working directory and source 2 files into environment
#'
#'
#' quickcode::refresh(
#'   setwd = "/home/",
#'   source="file1.R",
#'   load="obi.RData"
#' ) #clear environment, set working directory, source files and load RData
#'
#' }
#' }
#' @export
#'
refresh <- clean


clearPreviouslyLoadedPkgs <- function(){
  deftPkg <- c("base", "quickcode", getOption("defaultPackages"))
  for (i in grep("package:", search(), value = TRUE)) {
    curr <- strsplit(i, ":")[[1]][2]
    if (curr %nin% deftPkg){
      tryCatch({
        detach(name = i, character.only = TRUE, force = TRUE)
      }, warning = function(w) {},
      error = function(e) {},
      finally = {})

    }
  }
}

#' Create and use a super variable with unique capabilities
#'
#' Create a variable that supersedes other variables and has various functionalities
#'
#' @param variable variable name for super variable
#' @param value value of the variable
#' @param lock lock variable to change
#' @param editn number of times the super variable may be set to a new value using .set(). \cr
#'  - Set to NULL to allow unlimited value change \cr
#'  - Set to 0 to prevent editing the super variable \cr
#'
#' @note
#' \strong{What you should know about the functionality:} \cr\cr
#' This function ensures that a variable is created and may not easily be altered.
#' It helps preserve the original variable by providing only limited access to the variable.\cr\cr
#' Creation of this super variable automatically attached some key functions to it,
#' such that the user is able to call the function like \code{.set()}, \code{.rm()}.\cr\cr
#' Super variable value may be set from any scope using the \code{.set()} function, which
#' means that it is granted global variable features without being present within the
#' global environment of the current section.\cr\cr The variable name of the super variable may
#' be overwritten in the local environment, but this would not alter the super variable.
#' It means that once the local variable is removed, the super variable remains available
#' for use.\cr\cr
#'
#' \strong{Use cases:} \cr\cr
#'  - Preserve originality of variable within an R session. Avoid inadvertent deletion.\cr\cr
#'  - Widely accessible from any scope e.g functions, lapply, loops, local environment etc\cr\cr
#'  - Restricted mutability of variable using set function e.g varname\code{.set()}\cr\cr
#'  - Variable with easy function calls by attaching '.'\cr\cr
#'  - Variable with un-mutable class when changing its value \cr\cr
#'  - Variable with restricted number of times it can be changed \cr\cr
#'
#'
#' @return no visible return, but variable is created and stored with various functionalities
#'
#' @examples
#' # Task: create a super variable to
#' # store dataset that should not be altered
#' newSuperVar(mtdf, value = austres) # create a super variable
#' head(mtdf) # view it
#' mtdf.class # view the store class of the variable, it cannot be changed
#' # it means that when the super variable is edited, the new value MUST have the same class "ts"
#'
#' # create and lock super variable by default
#' # extra security to prevent changing
#' newSuperVar(mtdf3, value = beaver1, lock = TRUE)
#' head(mtdf3) # view
#' mtdf3.round(1) # round to 1 decimal places
#' head(mtdf3) # view
#' mtdf3.signif(2) # round to 2 significant digits
#' head(mtdf3) # view
#'
#' # Task: create a new super variable to store numbers
#' # edit the numbers from various scopes
#' newSuperVar(edtvec, value = number(5))
#' edtvec # view content of the vector
#'
#' # edtvec.set(letters) #ERROR: Cannot set to value with different class than initial value
#'
#' edtvec.set(number(20)) # set to new numbers
#' edtvec # view output
#'
#' for (pu in 1:8) {
#'   print(edtvec) # view output within loop
#'   edtvec.set(number(pu)) # set to new numbers within for loop
#' }
#'
#' lc <- lapply(1:8, function(pu) {
#'   print(edtvec) # view output within loop
#'   edtvec.set(number(pu)) # set to new numbers within lapply loop
#' })
#'
#' # see that the above changed the super variable easily.
#' # local variable will not be altered by the loop
#' # example
#' bim <- 198
#' lc <- lapply(1:8, function(j) {
#'   print(bim)
#'   bim <- j # will not alter the value of bim in next round
#' })
#'
#'
#' # Task: create and search data.frame
#' # create a new super variable with value as mtcars
#' # search if it contains the numeric value 21
#' newSuperVar(lon2, value = mtcars) # declares lon2
#' lon2 # view content of lon2
#' lon2.contains("21.0") # WRONG - since df.col is not specified,
#' # only the first column is search for the character "21.0"
#' lon2.contains("21.0", df.col = "mpg") # WRONG - searches mpg column
#' # for the character "21.0"
#' lon2.contains(21.0, df.col = "mpg") # CORRECT - search mpg column for the
#' # numeric value 21.0
#'
#' # remove lon2 as a super variable
#' exists("lon2") # before removal
#' lon2.rm()
#' exists("lon2") # after removal
#'
#' # Task: create and search vector
#' # create a new super variable with value as 10 random numbers
#' # search if it contains the numeric value 72
#' newSuperVar(lon3, value = number(10, seed = 12)) # declares lon3
#' lon3 # view content of lon3
#' lon3.contains(72) # should give TRUE or false if the vector contains the value 45
#' lon3.contains(72, fixed = TRUE) # should give TRUE or false if the vector contains the value 45
#'
#' # remove lon3 as a super variable
#' lon3.rm()
#'
#'
#' #Task: create a super variable that can only be edited 3 times
#' newSuperVar(man1, value = number(5), editn = 3)
#' man1 # view value
#'
#' man1.set(number(10)) # change value first time
#' man1 # view value
#'
#' man1.set(number(2)) # change value second time
#' man1 # view value
#'
#' man1.set(number(1)) # change value third time
#' man1 # view value
#'
#' man1.set(number(5)) # change value forth time,
#' # should not change because max change times exceeded
#' man1 # view value
#'
#' @export

newSuperVar <- function(variable, value = 0L, lock = FALSE, editn = NULL) {
  .v <- as.list(substitute(args(variable))[-1L])
  .spkg <- new.env()
  .r231 = paste0("att",frt6,"(.spkg, name = super.)")
  .r232 = paste0("unl",frt5,"ing(ioo, env = .pos80cbca8022ece6174797e10bb8aebf18)")
  classi <- class(value)
  if (!is.attached(super.)) {
    eval(parse(text = .r231))
  }
  if (not.exists(".pos80cbca8022ece6174797e10bb8aebf18")) {
    .pos80cbca8022ece6174797e10bb8aebf18 <- as.environment(super.)
  }
  # sub functions
  .join <- function(ij4) {
    eval(parse(text = deparse(ij4)))
  }

  # FUN
  els <- c("", ".rm", ".set", ".contains", ".round", ".signif", ".class",".head",".tail",".push",".pop")

  # remove
  rmv <- function() {
    rm(list = paste0(i, els), envir = .pos80cbca8022ece6174797e10bb8aebf18)
  }
  # set
  setv <- function(value) {
    continue = TRUE
    if(!is.null(editn)){
      if(editn)
        editn <<- editn - 1
      else continue = FALSE
    }
    if(continue){
    ioo <- as.character(i)
    if (not.inherits(value,classi)) stop("Class of new value must be ", classi, ", the same as the original value of ", i)
    eval(parse(text = as.character(.r232)))
    assign(ioo, value, envir = .pos80cbca8022ece6174797e10bb8aebf18)
    lockBinding(ioo, env = .pos80cbca8022ece6174797e10bb8aebf18)
    }
  }
  setl <- function(value) {
    continue = TRUE
    if(!is.null(editn)){
      if(editn)
        editn <<- editn - 1
      else continue = FALSE
    }
    if(continue){
    if (not.inherits(value,classi)) stop("Class of new value must be ", classi, ", the same as the original value of ", i)
    assign(as.character(i), value, envir = .pos80cbca8022ece6174797e10bb8aebf18)
    }
  }

  # round
  rldd <- function(digits = 0) {
    ioo <- as.character(i)
    if (lock){
      eval(parse(text = as.character(.r232)))
    }
    bv <- round(get(as.character(i), envir = .pos80cbca8022ece6174797e10bb8aebf18), digits)
    assign(ioo, bv, envir = .pos80cbca8022ece6174797e10bb8aebf18)
    if (lock) lockBinding(ioo, env = .pos80cbca8022ece6174797e10bb8aebf18)
  }

  # head
  headd <- function(n = 10) {
    .l = {get(as.character(i), envir = .pos80cbca8022ece6174797e10bb8aebf18)}
    stopifnot(inherits(.l,"data.frame")) # data must be a data frame
    print(.l[1:n,])
  }
  # tail
  taill <- function(n = 10) {
    .l = {get(as.character(i), envir = .pos80cbca8022ece6174797e10bb8aebf18)}
    stopifnot(inherits(.l,"data.frame")) # data must be a data frame
    print(.l[(nrow(.l)-n+1):nrow(.l),])
  }

  # push
  pushl <- function(add, .df = c("row", "col")) {
    .df <- match.arg(.df)
    .l = {get(as.character(i), envir = .pos80cbca8022ece6174797e10bb8aebf18)}
    if(inherits(.l,"data.frame")){
      if(.df == "row").l <- rbind(.l, add) else .l <- cbind(.l, add)
    }
    else .l <- c(.l, add)
    .l
  }

  # pop
  popl <- function(n = 10, .df = c("row", "col")) {
    .df <- match.arg(.df)
    .l = {get(as.character(i), envir = .pos80cbca8022ece6174797e10bb8aebf18)}
    if(inherits(.l,"data.frame")){
      if(.df == "row").l <- .l[1:{nrow(.l)-n},] else .l <- .l[,1:{ncol(.l)-n}]
    }
    else{
      .l <- .l[-{length(.l):{length(.l) - {n-1}}}]
    }
    .l
  }

  # significant figures
  sgif <- function(digits = 0) {
    ioo <- as.character(i)
    if (lock){
      eval(parse(text = as.character(.r232)))
    }
    bv <- signif(get(as.character(i), envir = .pos80cbca8022ece6174797e10bb8aebf18), digits)
    assign(ioo, bv, envir = .pos80cbca8022ece6174797e10bb8aebf18)
    if (lock) lockBinding(ioo, env = .pos80cbca8022ece6174797e10bb8aebf18)
  }

  # contains
  cntsa <- function(pattern, ignore.case = FALSE, perl = FALSE,
                    fixed = FALSE, useBytes = FALSE, invert = FALSE, df.col) {
    if (classi[1] %in% c("matrix", "array", "data.frame")) {
      z <- data.frame(get(as.character(i), envir = .pos80cbca8022ece6174797e10bb8aebf18))
      res <- z[grep(
        pattern = pattern, x = z[, df.col], ignore.case = ignore.case, perl = perl, value = FALSE,
        fixed = fixed, useBytes = useBytes, invert = invert
      ), ]
    } else {
      res <- grep(
        pattern = pattern, x = get(as.character(i), envir = .pos80cbca8022ece6174797e10bb8aebf18), ignore.case = ignore.case, perl = perl, value = FALSE,
        fixed = fixed, useBytes = useBytes, invert = invert
      )
    }

    as.logical(length(unlist(res)))
  }
  # editable
  editn1 <- 1
  if(length(editn)){
    if (editn == 0) editn1 <- editn
  }

  # assign to variable
  for (i in .v) {
    if (exists(as.character(i), envir = .pos80cbca8022ece6174797e10bb8aebf18)) {
      stop("The variable '", i, "' already exists as a superVar. In order to redeclare it,
           you need to remove the existing object by using '", i, ".rm()'")
    }
    assign(as.character(i), value, envir = .pos80cbca8022ece6174797e10bb8aebf18)
    if (lock) lockBinding(i, env = .pos80cbca8022ece6174797e10bb8aebf18)
    assign(paste0(i, els[7]), classi, envir = .pos80cbca8022ece6174797e10bb8aebf18)
    assign(paste0(i, els[2]), .join(rmv), envir = .pos80cbca8022ece6174797e10bb8aebf18)
    assign(paste0(i, els[4]), .join(cntsa), envir = .pos80cbca8022ece6174797e10bb8aebf18)
    assign(paste0(i, els[5]), .join(rldd), envir = .pos80cbca8022ece6174797e10bb8aebf18)
    assign(paste0(i, els[6]), .join(sgif), envir = .pos80cbca8022ece6174797e10bb8aebf18)
    assign(paste0(i, els[10]), .join(pushl), envir = .pos80cbca8022ece6174797e10bb8aebf18)
    assign(paste0(i, els[11]), .join(popl), envir = .pos80cbca8022ece6174797e10bb8aebf18)

    if(inherits(value,"data.frame")){
      #if data frame, add the head and tail functions
      assign(paste0(i, els[8]), .join(headd), envir = .pos80cbca8022ece6174797e10bb8aebf18)
      assign(paste0(i, els[9]), .join(taill), envir = .pos80cbca8022ece6174797e10bb8aebf18)
    }else{
      els <- els[els %nin% c(".head",".tail")]
    }

    if(editn1 != 0)
      assign(paste0(i, els[3]), .join(ifelse(lock, setv, setl)), envir = .pos80cbca8022ece6174797e10bb8aebf18)
    else els <- els[els!=".set"]
    for(k3i in els[-1])
    lockBinding(paste0(i, k3i), env = .pos80cbca8022ece6174797e10bb8aebf18)
  }
  rm(rmv, setv, setl, rldd, cntsa, sgif, .r231)
}


# will be exported in future release
# `%-%` <- function(item1,item2,return=FALSE){
#   if(!all(class(item1) == class(item2))) stop("Both variables must have the same class")
#   res <- "The variables have to be either two lists or two character vectors"
#   if(inherits(item1,"character"))
#   res <- item1[item1%nin%item2]
#   if(inherits(item1,"list"))
#   res <- mapply('-', item1, item2, SIMPLIFY = FALSE)
#   res
# }

#' Check if a call or expression produces errors
#'
#' Whether a function or series of calls results in error
#'
#' @param ... the expression or function calls
#' @note
#' More information, check: https://rpkg.net/package/quickcode
#'
#' @examples
#' # this should not produce error
#' # so the function result should be FALSE
#' has.error({
#'   x = 8
#'   y = number(10)
#'   res = x + y
#' })
#'
#' # this should produce the following error
#' # Error in x + y : non-numeric argument to binary operator
#' # so the function result should be TRUE
#' has.error({
#'   x = 8
#'   y = "random"
#'   res = x + y
#' })
#'
#' # this should result in error because
#' # the dataset does not contain a "rpkg.net" column
#' # the result should be TRUE
#' df1 = mtcars
#' has.error(df1[,"rpkg.net"])
#'
#' @return boolean value to indicate if the expression produces errors
#' @export

has.error <- function(...) {
  .error <- FALSE
  tryCatch(...,
           error = function(e) {
             .error <<- TRUE
           }
  )
  .error
}

#' Check whether an R package has been decommissioned in CRAN
#'
#' Designed to assist users in checking the decommission status
#' of an R package on CRAN. In the context of R language,
#' CRAN stands for the Comprehensive R Archive Network.
#'
#' @param package package name to query
#'
#' @return the decommissioned status of a particular package based on the available packages using the utils package
#'
#' @details
#' CRAN is a network of servers around the world that store R packages
#' and their documentation, providing a centralized repository for
#' the R community. With the current function, users can quickly and
#' easily determine whether a specific R package has been decommissioned
#' on CRAN, ensuring they stay informed about the availability and
#' support status of the packages they rely on for their R
#' programming projects. This tool simplifies the process of package
#' management, helping users maintain up-to-date and reliable
#' dependencies in their R code.
#'
#'
#' @examples
#' \dontrun{
#' # check if cattonum package is decommissioned
#' # the current package is expected to be decommissioned
#' rDecomPkg("cattonum")
#'
#' # check if dplyr is decommissioned
#' # the current package is expected NOT to be decommissioned
#' rDecomPkg("dplyr")
#'
#' # when a package never existed in CRAN
#' # the result of the function call should be NA
#' rDecomPkg("printy")
#' rDecomPkg("package0002312122312")
#' }
#' @export

rDecomPkg <- function(package){
  stopifnot(nchar(package) > 1) #stop if the package name length is not greater than 1

  #check if the package ever existed in cran
  if(pkg.existed.cran(package)){
    #if it existed, check if it is active
    unlist(lapply(package, function(.p) .p %nin% allCRANpkg()))
  }else{
    #if it did not exist, return a warning and NA
    warning(paste0("The package '",package,"' never existed in CRAN."))
    return(NA)
  }

}

#' Not an integer
#'
#' Opposite of is.integer(). Check if entry is not an integer
#'
#' @param x vector entry
#' @return a boolean value to indicate if entry is an integer
#' @examples
#' is.integer(78L) #TRUE
#' not.integer(78L) #FALSE
#'
#' not.integer(23.43) # TRUE
#' not.integer(45L) # FALSE
#' if(not.integer(4L)) message("yes") # NULL
#'
#' @export

not.integer <- function(x)!{
  is.integer(x)
}

#' Track Function Usage and Performance
#'
#' Up count the number of times a particular function is called. This function tracker is a higher-order function or decorator that wraps around other functions to monitor and record their execution patterns, including metrics like call frequency, execution time, argument patterns, and return values. It acts as a transparent layer that doesn't modify the original function's behavior but collects valuable metadata about its usage.
#'
#' @param output.dest destination of csv file to store outputs
#' @return the numeric count of a function usage
#' @note
#' The usefulness of function tracking spans several critical areas:\cr\cr
#' \strong{Performance Optimization}: By measuring execution times and frequency, developers can identify bottlenecks and frequently called functions that need optimization\cr
#' \strong{Debugging}: Tracking argument patterns and function call sequences helps pinpoint issues in complex applications\cr
#' \strong{Usage Analytics}: Understanding which features (functions) are most commonly used helps guide development priorities and API design decisions\cr
#' \strong{Resource Management}: Monitoring function behavior helps identify memory leaks, resource consumption patterns, and potential optimization opportunities\cr
#' \strong{Testing}: Usage patterns can inform test case design and coverage requirements, ensuring critical paths are well-tested\cr
#' \strong{Documentation}: Automatically gathering real-world usage examples helps maintain accurate and relevant documentation\cr
#' \strong{Compliance}: In regulated environments, function tracking can help maintain audit trails of system behavior\cr
#' @examples
#' \dontrun{
#' library(quickcode)
#' # Track usage of type2 and type1 functions
#' store.usage.file <- tempfile()
#' type5 <- function(x) type2(x)
#' type4 <- function(x) type3(x)
#' type3 <- function(x) type1(x)
#' type1 <- function(x) {
#'   mean(x)
#'   sd(x)
#'   track_func(store.usage.file)
#' }
#' type2 <- function(x) {
#'   type1(x)
#'   track_func(store.usage.file)
#' }
#'
#' # add usage counts to store.usage.file
#' type1(number(10))
#' type2(number(10))
#' type3(number(10))
#' type4(number(10))
#' type5(number(10))
#'
#' # check the stored function usage file
#' print(read.csv(store.usage.file))
#' }
#' @export
track_func <- function(output.dest = "output_tracking.csv") {
  getCall <- as.character(sys.calls()[[length(sys.calls()) - 1]])
  getFuncName <- strsplit(getCall, "\\(")[[1]][1]
  appenddf <- data.frame(Function = getFuncName, Usage = 1)
  if (file.exists(output.dest)) {
    out.f <- read.csv(output.dest, header = T)
    match.f <- subset(out.f, Function == getFuncName)
    if (nrow(out.f[out.f$Function == getFuncName, ])) {
      out.f[out.f$Function == getFuncName, ]$Usage <- out.f[out.f$Function == getFuncName, ]$Usage + 1
    } else {
      out.f <- rbind(out.f, appenddf)
    }
    utils::write.csv(out.f, output.dest, quote = FALSE, row.names = FALSE)
  } else {
    utils::write.csv(appenddf, output.dest, quote = FALSE, row.names = FALSE)
  }
}

#' Extract all comments or functions from a file
#'
#' Vectorize all comments from the file
#' @rdname comments
#' @param file path of file to use for processes
#' @return vector of all comments within a file
#' @examples
#' \dontrun{
#' ex_file1 <- "path/file1.R"
#' # get all comments
#' cmmts <- extract_comment(ex_file1)
#' cmmts
#' }
#'
#' @param file file to parse
#' @export

extract_comment <- function(file) {
  # Read the file line by line
  lines <- readLines(file)

  # Initialize a vector to store comments
  comments <- c()

  # Loop through each line to extract comments
  for (line in lines) {
    # Extract comments from the beginning of the line
    line <- remove_content_in_quotes(line)
    if (grepl("^\\s*#", line)) {
      comments <- c(comments, line)
    }
    # Extract inline comments
    inline_comments <- unlist(regmatches(line, gregexpr("#.*", line)))
    if (length(inline_comments) > 0 && !grepl("^\\s*#", line)) {
      comments <- c(comments, inline_comments)
    }
  }

  # Return the comments
  return(comments)
}

#' @rdname comments
#' @param line string vector to remove contents within quotes or comments
#' @export
remove_content_in_quotes <- function(line) {
  # Remove content within single quotes
  line <- gsub("'[^']*'", "", line)
  # Remove content within double quotes
  line <- gsub('"[^"]*"', "", line)
  return(line)
}


#' @rdname comments
#' @export
remove_comment <- function(line) {
  stopifnot(length(line) == 1)
  comment_index <- regexpr("#", line)
  if (comment_index == -1) line else gsub("#.*$", "", line)
}


# Function to numb internal comments
hide_int_cmts <- function(string) {
  string <- gsub('("[^"]*)#+([^"]*)"', '\\10x5&9%80x\\2"', string, perl = TRUE)
  string <- gsub("('[^']*)#+([^']*)'", "\\10x5&9%80x\\2'", string, perl = TRUE)
  string
}


#' @rdname comments
#' @param output_file file path to write the output of the new file
#' @export
#' @examples
#' \dontrun{
#' # Ex to clean out comments from file
#' file_path <- ".testR"
#' output_file_path <- ".cleaned_script.R"
#' clean_file(file_path, output_file_path)
#' }
#'
clean_file <- function(file, output_file) {
  # Read the file line by line
  lines <- readLines(file)

  # Process each line: remove comments and empty lines
  cleaned_lines <- sapply(lines, function(line) {
    line <- hide_int_cmts(line)
    line <- gsub(master_file_clean_sep, "#", remove_comment(line))
    line <- trimws(line) # Remove leading and trailing whitespace
    if (nchar(line) > 0) {
      return(line)
    } else {
      return(NULL)
    }
  })
  res <- as.character(cleaned_lines[!sapply(cleaned_lines, is.null)])
  if (missing(output_file)) res else writeLines(res, output_file)
}


#' Get all defined functions within a file
#'
#' @examples
#' # example code
#'
#' \dontrun{
#' # Ex to get all defined functions
#' # within a file
#' file_path <- ".testR"
#' get_func_def(file_path)
#' }
#' @rdname comments
#' @export
get_func_def <- function(file) {
  code <- clean_file(file)
  .envi <- new.env()
  eval(parse(text = code), envir = .envi)
  ls(envir = .envi)
}

#' Check if a data fits the distribution
#'
#' @description
#' Check whether a vector of data contains values that fit a distribution
#' @details
#' This function takes a numeric vector as its input. This vector contains the dataset that will be analyzed.
#' \cr\cr
#' \strong{For Normal and LogNormal:}\cr\cr
#' - Method 1: we perform the Shapiro-Wilk test on the (log-transformed) data to test for normality.
#' The null hypothesis of the Shapiro-Wilk test is that the data are normally distributed.
#' If the p-value is greater than the chosen significance level (typically 0.05),
#' we fail to reject the null hypothesis, indicating that the data may follow a log-normal distribution.\cr\cr
#' - Method 2: we perform the Kolmogorov-Smirnov test on the log-transformed data, comparing it to a normal distribution with the same mean and standard deviation. Again, if the p-value is greater than the chosen significance level, it suggests that the data may follow a log-normal distribution.
#' These tests provide a statistical assessment of whether your data follows a log-normal distribution.
#'
#'
#' @rdname distribution_check
#' @param values vector of values
#' @param alpha significance level to test p-value against
#' @param method method for calculation, where 1 = Shapiro-Wilk test and 2 = Kolmogorov-Smirnov test
#' @return boolean value if lognormal distributed
#' @examples
#' # Set global alpha for testing significance
#' setDisAlpha(alpha = 0.05)
#'
#' # Prepare all data to test
#' # Set the seed for reproducibility
#' set.seed(13200323)
#' lognormal_data <- stats::rlnorm(n = 4000, meanlog = 1, sdlog = 1) #lognormal data
#' normal_data <- stats::rnorm(n = 4000, mean = 10, sd = 3) #normal data
#' uniform_data <- stats::runif(4000,min=0,max=10) #uniform data
#' poisson_data <- stats::rpois(4000, lambda = 5) #poisson data
#' gamma_data <- stats::rgamma(4000,shape = 5, rate = 2) #gamma data
#' logis_data <- stats::rlogis(4000, location = 4, scale = 2)#logistic values
#' weibull_data <- stats::rweibull(4000, shape = 4, scale = 2) #weibull data
#' cauchy_data <- stats::rcauchy(4000, location = 8, scale = 5) #cauchy data
#'
#' # EXAMPLE FOR is.lognormal
#'
#' # Test if the data is lognormal
#' is.lognormal(lognormal_data)
#' is.lognormal(normal_data)
#' is.lognormal(uniform_data)
#' is.lognormal(poisson_data)
#' is.lognormal(gamma_data)
#' is.lognormal(logis_data)
#' is.lognormal(weibull_data)
#' is.lognormal(cauchy_data)
#' is.lognormal(1:4000)
#'
#' @export
is.lognormal <- function(values, alpha = 0.05, method = 1)!{
  #override alpha if global significance level set
  if(not.null(options()$qc.sig.alpha.level))
    alpha = options()$qc.sig.alpha.level

  #test for lognormal
  values <- values[values>0]
  # error check
  stopifnot(length(values) > 3, method %in% 1:2)

  if (method == 2) {
    # Test for log-normal distribution using Kolmogorov-Smirnov test
    # message("Kolmogorov-Smirnov test for log-normal distribution")
    {
      stats::ks.test(log(values), "pnorm", mean = mean(log(values)), sd = sd(log(values)))
    }$p.value < alpha
  } else {
    # Test for log-normal distribution using Shapiro-Wilk test
    if (method == 1) {
      # message("Shapiro-Wilk test for log-normal distribution")
      {
        stats::shapiro.test(log(values))
      }$p.value < alpha
    } else {
      TRUE
    }
  }
}

#' @rdname distribution_check
#' @param values vector of values
#' @param alpha significance level to test p-value against
#' @return boolean value if normal distributed
#'
#' @examples
#' # EXAMPLE FOR is.normal
#'
#' # Test if the data fits a normal distribution
#' is.normal(lognormal_data)
#' is.normal(normal_data)
#' is.normal(uniform_data)
#' is.normal(poisson_data)
#' is.normal(gamma_data)
#' is.normal(logis_data)
#' is.normal(weibull_data)
#' is.normal(cauchy_data)
#' is.normal(1:4000)
#'
#' @export
is.normal <- function(values, alpha = 0.05, method = 1)!{
  #override alpha if global significance level set
  if(not.null(options()$qc.sig.alpha.level))
    alpha = options()$qc.sig.alpha.level
  #test for normality
  # error check
  stopifnot(length(values) > 3, method %in% 1:2)

  if (method == 2) {
    # Test for normal distribution using Kolmogorov-Smirnov test
    # message("Kolmogorov-Smirnov test for normal distribution")
    {
      stats::ks.test(values, "pnorm", mean = mean(values), sd = sd(values))
    }$p.value < alpha
  } else {
    # Test for normal distribution using Shapiro-Wilk test
    if (method == 1) {
      # message("Shapiro-Wilk test for normal distribution")
      {
        stats::shapiro.test(values)
      }$p.value < alpha
    } else {
      TRUE
    }
  }
}


#' @rdname distribution_check
#' @param values vector of values
#' @param alpha significance level to test p-value against
#' @return boolean value if uniform distributed
#'
#' @examples
#' \dontrun{
#' # EXAMPLES for is.uniform
#'
#' # Test if the data fits a uniform distribution
#' is.uniform(lognormal_data)
#' is.uniform(normal_data)
#' is.uniform(uniform_data)
#' is.uniform(poisson_data)
#' is.uniform(gamma_data)
#' is.uniform(logis_data)
#' is.uniform(weibull_data)
#' is.uniform(cauchy_data)
#' is.uniform(1:4000)
#' }
#' @export
is.uniform <- function(values,alpha = 0.05)!{
  #override alpha if global significance level set
  if(not.null(options()$qc.sig.alpha.level))
    alpha = options()$qc.sig.alpha.level
  stopifnot(length(values) > 3)
  #test for uniform
  {stats::ks.test(values, "punif", min(values), max(values))}$p.value < alpha
}


#' @rdname distribution_check
#' @param values vector of values
#'
#' @param alpha significance level to test p-value against
#' @return boolean value if poisson distributed
#' @examples
#' \dontrun{
#' # EXAMPLE for is.poisson
#'
#' # Test if the data fits a poisson distribution
#' is.poisson(lognormal_data)
#' is.poisson(normal_data)
#' is.poisson(uniform_data)
#' is.poisson(poisson_data)
#' is.poisson(gamma_data)
#' is.poisson(logis_data)
#' is.poisson(weibull_data)
#' is.poisson(cauchy_data)
#' is.poisson(1:4000)
#' }
#' @export
is.poisson <-function(values,alpha = 0.05){
  #override alpha if global significance level set
  if(not.null(options()$qc.sig.alpha.level))
    alpha = options()$qc.sig.alpha.level
  stopifnot(length(values) > 3)
  #test for poisson
  {suppressWarnings(stats::chisq.test(table(values)))}$p.value < alpha
}



#' @rdname distribution_check
#' @param values vector of values
#' @param alpha significance level to test p-value against
#' @return boolean value if gamma distributed
#'
#' @examples
#' \dontrun{
#' # EXAMPLE for is.gamma
#'
#' # Test if the data fits a gamma distribution
#' is.gamma(lognormal_data)
#' is.gamma(normal_data)
#' is.gamma(uniform_data)
#' is.gamma(poisson_data)
#' is.gamma(gamma_data)
#' is.gamma(logis_data)
#' is.gamma(weibull_data)
#' is.gamma(cauchy_data)
#' is.gamma(1:4000)
#' }
#' @export
is.gamma <- function(values, alpha = 0.05) {
  # override alpha if global significance level set
  if (not.null(options()$qc.sig.alpha.level)) {
    alpha <- options()$qc.sig.alpha.level
  }
  stopifnot(length(values) > 3)
  # test for gamma
  tryCatch(
    {
      .sr <- fitdistrplus::fitdist(values, "gamma")
      shape <- .sr$estimate["shape"]
      rate <- .sr$estimate["rate"]
      {
        stats::ks.test(values, "pgamma", shape = shape, rate = rate)
      }$p.value >= alpha
    },
    error = function(msg) {
      return(FALSE)
    }
  )
}

#' @rdname distribution_check
#' @param values vector of values
#' @param alpha significance level to test p-value against
#' @return boolean value if logistic distributed
#' @examples
#' \dontrun{
#' # EXAMPLE for is.logistic
#'
#' # Test if the data fits a logistic distribution
#' is.logistic(lognormal_data)
#' is.logistic(normal_data)
#' is.logistic(uniform_data)
#' is.logistic(poisson_data)
#' is.logistic(gamma_data)
#' is.logistic(logis_data)
#' is.logistic(weibull_data)
#' is.logistic(cauchy_data)
#' is.logistic(1:4000)
#' }
#' @export
is.logistic <- function(values,alpha = 0.05) {
  #override alpha if global significance level set
  if(not.null(options()$qc.sig.alpha.level))
    alpha = options()$qc.sig.alpha.level
  #test for logistic
  stopifnot(length(values) > 3)
  .sr <- fitdistrplus::fitdist(values, "logis")
  location <- .sr$estimate['location']
  scale <- .sr$estimate['scale']
  {stats::ks.test(values, "plogis",
                  location = location,
                  scale = scale)}$p.value >= alpha
}



#' @rdname distribution_check
#' @param values vector of values
#' @examples
#' \dontrun{
#' # Test if the data fits a weibull distribution
#' is.weibull(lognormal_data)
#' is.weibull(normal_data)
#' is.weibull(uniform_data)
#' is.weibull(poisson_data)
#' is.weibull(gamma_data)
#' is.weibull(logis_data)
#' is.weibull(weibull_data)
#' is.weibull(cauchy_data)
#' is.weibull(1:4000)
#' }
#' @param alpha significance level to test p-value against
#' @return boolean value if logistic distributed
#' @export
is.weibull <- function(values, alpha = 0.05) {
  # override alpha if global significance level set
  if (not.null(options()$qc.sig.alpha.level)) {
    alpha <- options()$qc.sig.alpha.level
  }
  values[values<=0] = 0
  stopifnot(length(values) > 3)
  # test for weibull
  tryCatch(
    {
      .sr <- fitdistrplus::fitdist(values, "weibull")
      shape <- .sr$estimate["shape"]
      scale <- .sr$estimate["scale"]
      {
        stats::ks.test(values, "pweibull", shape = shape, scale = scale)
      }$p.value >= alpha
    },
    error = function(msg) {
      return(FALSE)
    }
  )
}






#' @rdname distribution_check
#' @param values vector of values
#' @param alpha significance level to test p-value against
#' @return boolean value if cauchy distributed
#'
#' @examples
#' \dontrun{
#' # EXAMPLES for is.cauchy
#'
#' # Test if the data fits a cauchy distribution
#' is.cauchy(lognormal_data)
#' is.cauchy(normal_data)
#' is.cauchy(uniform_data)
#' is.cauchy(poisson_data)
#' is.cauchy(gamma_data)
#' is.cauchy(logis_data)
#' is.cauchy(weibull_data)
#' is.cauchy(cauchy_data)
#' is.cauchy(1:4000)
#' }
#' @export
is.cauchy <- function(values,alpha = 0.05){
  #override alpha if global significance level set
  if(not.null(options()$qc.sig.alpha.level))
    alpha = options()$qc.sig.alpha.level
  #test for cachy
  stopifnot(length(values) > 3)
  .sr <- fitdistrplus::fitdist(values, "cauchy")
  location <- .sr$estimate['location']
  scale <- .sr$estimate['scale']
  {stats::ks.test(values,"pcauchy",location = location,scale = scale)}$p.value >= alpha
}

#' @rdname distribution_check
#' @param alpha significance level to test p-value against
#' @return setDisAlpha sets global significance level for testing of distribution
#'
#' @examples
#' \dontrun{
#' # set global distribution alpha
#'
#' # default setting
#' setDisAlpha()
#'
#' # set to 0.001
#' setDisAlpha(alpha = 0.01)
#' }
#' @export
setDisAlpha <- function(alpha = 0.05){
  stopifnot(is.numeric(alpha) == TRUE)
  options(qc.sig.alpha.level = alpha)
}
#' @rdname distribution_check
#' @return unsetDisAlpha removes global significance level for testing of distribution
#'
#' @examples
#' \dontrun{
#' # unset global distribution alpha
#'
#' unsetDisAlpha()
#' }
#' @export
unsetDisAlpha <- function(){
  options(qc.sig.alpha.level = NULL)
}

# Future functions
# checkDistribution <- function(data,alpha,...){
#   # data.frame(Distributions = c(
#   #   "Beta",
#   #   "Binomial",
#   #   "Cauchy",
#   #   "Chi-Square",
#   #   "Exponential",
#   #   "F",
#   #   "Gamma",
#   #   "Geometric",
#   #   "Hypergeometric",
#   #   "Logistic",
#   #   "Log Normal",
#   #   "Negative Binomial",
#   #   "Normal",
#   #   "Poisson",
#   #   "Student",
#   #   "Studentized Range",
#   #   "Uniform",
#   #   "Weibull",
#   #   "Wilcoxon Rank Sum Statistic",
#   #   "Wilcoxon Signed Rank Statistic"
#   # ))
#
#
#
#   if (missing(distr))
#     stop("You must provide a distribution name")
#   distr <- match.arg(distr, c("norm", "lnorm", "gamma", "weibull",
#                               "exp", "cauchy", "logis", "rayleigh", "hyper", "beta",
#                               "llogis", "frechet", "pareto", "burr", "chi", "chisq",
#                               "f", "t", "binom", "geom", "pois", "nbinom"))
#   if (distr == "norm")
#     return(fitdist_norm(data, method, start, ...))
#   if (distr == "lnorm")
#     return(fitdist_lnorm(data, method, start, ...))
#   if (distr == "gamma")
#     return(fitdist_gamma(data, method, start, ...))
#   if (distr == "weibull")
#     return(fitdist_weibull(data, method, start, ...))
#   if (distr == "exp")
#     return(fitdist_exp(data, method, start, ...))
#   if (distr == "cauchy")
#     return(fitdist_cauchy(data, method, start, ...))
#   if (distr == "logis")
#     return(fitdist_logis(data, method, start, ...))
#   if (distr == "rayleigh")
#     return(fitdist_rayleigh(data, method, start, ...))
#   if (distr == "hyper")
#     return(fitdist_hyper(data, method, start, ...))
#   if (distr == "beta")
#     return(fitdist_beta(data, method, start, ...))
#   if (distr == "llogis")
#     return(fitdist_llogis(data, method, start, ...))
#   if (distr == "frechet")
#     return(fitdist_frechet(data, method, start, ...))
#   if (distr == "pareto")
#     return(fitdist_pareto(data, method, start, ...))
#   if (distr == "burr")
#     return(fitdist_burr(data, method, start, ...))
#   if (distr == "chi")
#     return(fitdist_chi(data, method, start, ...))
#   if (distr == "chisq")
#     return(fitdist_chisq(data, method, start, ...))
#   if (distr == "f")
#     return(fitdist_f(data, method, start, ...))
#   if (distr == "t")
#     return(fitdist_t(data, method, start, ...))
#   if (distr == "binom")
#     return(fitdist_binom(data, method, start, ...))
#   if (distr == "geom")
#     return(fitdist_geom(data, method, start, ...))
#   if (distr == "pois")
#     return(fitdist_pois(data, method, start, ...))
#   if (distr == "nbinom")
#     return(fitdist_nbinom(data, method, start, ...))
#
# }
# https://www.stat.umn.edu/geyer/old/5101/rlook.html

# Distribution	Functions
# Beta	pbeta	qbeta	dbeta	rbeta
# Binomial	pbinom	qbinom	dbinom	rbinom
# Cauchy	pcauchy	qcauchy	dcauchy	rcauchy
# Chi-Square	pchisq	qchisq	dchisq	rchisq
# Exponential	pexp	qexp	dexp	rexp
# F	pf	qf	df	rf
# Gamma	pgamma	qgamma	dgamma	rgamma
# Geometric	pgeom	qgeom	dgeom	rgeom
# Hypergeometric	phyper	qhyper	dhyper	rhyper
# Logistic	plogis	qlogis	dlogis	rlogis
# Log Normal	plnorm	qlnorm	dlnorm	rlnorm
# Negative Binomial	pnbinom	qnbinom	dnbinom	rnbinom
# Normal	pnorm	qnorm	dnorm	rnorm
# Poisson	ppois	qpois	dpois	rpois
# Student t	pt	qt	dt	rt
# Studentized Range	ptukey	qtukey	dtukey	rtukey
# Uniform	punif	qunif	dunif	runif
# Weibull	pweibull	qweibull	dweibull	rweibull
# Wilcoxon Rank Sum Statistic	pwilcox	qwilcox	dwilcox	rwilcox
# Wilcoxon Signed Rank Statistic	psignrank	qsignrank	dsignrank	rsignrank

#' Remove elements from a data matching filter
#'
#' Shorthand to remove elements from a data frame based on filter and save as the same name
#'
#' @param . data object
#' @param remove expression for filter
#' @return data filtered out based on the expression
#' @examples
#' # this function removes rows matching the filter expression
#' data.01 <- mtcars
#' data.02 <- airquality
#'
#' #task: remove all mpg > 20
#' data.01 #data.01 data before pop
#' data_pop_filter(data.01,mpg > 15) #computes and resaves to variable
#' #note: this is different from subset(data.01,data.01$mpg > 15)
#' data.01 #modified data after pop based on filter
#'
#' #task: remove all multiple. remove all elements where Month   == 5 or Solar.R > 50
#' data.02 #data.02 data before pop
#' data_pop_filter(data.02,Month   == 5 | Solar.R > 50) #computes and resaves to variable
#' data.02 #modified data after pop based on filter
#'
#' @export
#'

data_pop_filter <- function(.,remove){
  .. <- substitute(.)
  .... <- substitute(remove)
  if (typeof(..) != "symbol") stop(paste0(.., " must be an object."))
  data <- as.data.frame(get(as.character(..), envir = parent.frame()))
  filt <- as.character(list(....))
  eval(parse(text = paste0("data = subset(data,!(",filt,"))")))
  assign(as.character(..), data, envir = parent.frame())
}


# .. <- substitute(.)
# .... <- substitute(remove)
# if (typeof(..) != "symbol") stop(paste0(.., " must be an object."))
# data <- as.data.frame(get(as.character(..), envir = parent.frame()))
# filt <- as.character(list(....))
# for(x in names(data))
#   filt <- gsub(x,paste0("data$",x),filt)
# eval(parse(text = paste0("data = data[!(",filt,"),]")))
# #alternatively: eval(parse(text = paste0("data = subset(data,!(",filt,"))")))
# assign(as.character(..), data, envir = parent.frame())

#' Not empty
#'
#' Check if entry is not empty
#'
#' @rdname empty
#' @param x vector entry
#' @return a boolean value to indicate if entry is empty
#' @examples
#' not.empty("empty") # TRUE
#' not.empty('') # FALSE
#' not.empty(y<-NULL) # FALSE
#' if(not.empty('')) message("yes") # NULL
#' @export

not.empty <- function(x){
  eval <- not.null(x) & (x != '')
  if(!length(eval)) eval <- FALSE
  eval
}

#' @rdname empty
#' @export

is.empty <- function(x)!{
  not.empty(x)
}

#' Not exists
#'
#' Check if object does not exists
#'
#' @param x object
#' @return a boolean value to indicate if entry does not exists
#' @examples
#' go = 7
#' not.exists("exis") # TRUE
#' not.exists("go") # FALSE
#' if(not.exists('hallo')) message("yes") # NULL
#' @export

not.exists <- function(x){
  !exists(x)
}

#' Listing of all CRAN archived R packages
#'
#' Retrieve a list of all currently archived R packages and their archive date
#'
#' @param startsWith one letter that the package name starts with eg. a, e, f
#' @param after packages archived after a specific date eg. 2011-05-10
#' @param inc.date should archive date be included in the result
#' @param as return result as data frame or as list
#'
#' @return a data frame or list containing listing of all archived R packages
#'
#' @section Use case:
#' This function allows the retrieval of various R packages archived by CRAN along with
#' the respective latest archive date. The packages retrieved include both active and
#' inactive R projects submitted to CRAN. When a new version of an active R package is published,
#' the older versions of the package gets archived. In the same way, when a package is
#' decommissioned from CRAN active projects for one reason or another, it gets archived.
#'
#' @note
#' * The "startsWith" argument should be one letter and should be in lowercase \cr
#' * If no argument is provided for "startsWith", all the packages will be retrieved\cr
#' * The format of the "after" argument must be YYYY-MM-DD e.g. 2022-04-11
#'
#'
#' @examples
#' \donttest{
#' # Task 1: get archived R packages with names beginning with C or All
#' head(archivedPkg(startsWith = "all"), n= 10) #retrieves all packages
#' head(archivedPkg(startsWith = "c"), n= 10) #retrieves only packages beginning with a
#'
#' # Task 2: return the packages from Task 1 without including latest archive date
#' res.dt2 <- archivedPkg(startsWith = "b", inc.date = FALSE)
#' res.dt2[1:10,]
#'
#' # Task 3: return the results from Task 2 as a list
#' res.dt3 <- archivedPkg(startsWith = "c", inc.date = FALSE, as = "list")
#' res.dt3$name[1:10]
#'
#' res.dt3 <- archivedPkg(startsWith = "e", as = "list")
#' res.dt3$name[1:10]
#'
#' # Task 4: return the archived packages beginning with Y archived after 2022-08-12
#' # Note that startsWith should be lowercase
#'
#' #without archive date
#' yRPkg <- archivedPkg(startsWith = "y", after= NULL)
#' nrow(yRPkg) #number of rows returned
#' head(yRPkg, n = 15) #show first 15 rows
#'
#' #with archive date
#' yRPkg2 <- archivedPkg(startsWith = "y", after= "2022-08-12")
#' nrow(yRPkg2) #number of rows returned
#' head(yRPkg2, n = 15) #show first 15 rows, notice no archive date before 2022-08-12
#' }
#' @export
#'

archivedPkg <- function(startsWith = c("all",letters), after = NULL, inc.date = TRUE, as =c("data.frame","list")) {
  startsWith <- match.arg(startsWith)
  if(startsWith == "all") startsWith <- letters
  as <- match.arg(as)
  res <- data.frame()

  #fetch data and update res
  for(i in startsWith)
    data_push(res,utils::read.csv(
    file = paste0(
      "https://quickcode.obi.obianom.com/CRAN/archiveddata_",tolower(i),".txt?count=0&auth=1"),
    header = TRUE, quote = "'",
    skip = 1),which = "rows")

  if(not.null(after)){
    if(is.na(as.Date(after, "%Y-%m-%d")))
      stop("Date format must be YYYY-MM-DD e.g 2022-05-15")
    # filter by time
    res$difftime <- as.numeric(as.POSIXct(res$latest.archive) - as.POSIXct(after))
    res <- res[res$difftime >= 0,]
    res$difftime <- NULL
  }


  if(!inc.date) res$latest.archive <- NULL
  else res$latest.archive <- as.Date(res$latest.archive)
  if(as == "list") res <- as.list(res)
  res
}

#' Detect Outliers in a Numeric Vector
#'
#' This function identifies outliers in a numeric vector using either the interquartile range
#' (IQR) method or the z-score method. The IQR method defines outliers as values below
#' Q1 - multiplier * IQR or above Q3 + multiplier * IQR, where Q1 and Q3 are the first and
#' third quartiles. The z-score method identifies outliers as values with an absolute
#' z-score exceeding a specified threshold.
#'
#' The function returns a list containing the outliers, their indices, detection bounds or
#' thresholds, and a logical vector indicating which elements are outliers. If a grouping
#' vector is provided via `groups`, outlier detection is performed separately for each group,
#' and results are returned as a nested list by group. If `na.rm = TRUE` (default), missing
#' values (`NA`) are removed before computation. If `na.rm = FALSE` and `NA` values are
#' present, the function stops with an error. The function also stops for non-numeric input,
#' insufficient valid data, or mismatched group lengths.
#' @rdname outlierdetect1
#' @param x A numeric vector in which to detect outliers.
#' @param method A character string specifying the outlier detection method. Options are
#'   `"iqr"` (default) for the interquartile range method or `"zscore"` for the z-score method.
#' @param multiplier A positive numeric value specifying the multiplier for the IQR method.
#'   Default is `1.5`, typically used for moderate outliers; `3` is common for extreme outliers.
#'   Ignored if `method = "zscore"`.
#' @param z_threshold A positive numeric value specifying the z-score threshold for the
#'   `method = "zscore"` option. Default is `3`, meaning values with an absolute z-score
#'   greater than 3 are flagged as outliers. Ignored if `method = "iqr"`.
#' @param na.rm A logical value indicating whether to remove `NA` values before computation.
#'   Default is `TRUE`. If `FALSE` and `NA` values are present, the function stops with an error.
#' @param groups An optional vector of group names or labels corresponding to each value in
#'   `x`. If provided, must be the same length as `x`. Outlier detection is performed separately
#'   for each unique group, and results are returned as a nested list. Default is `NULL` (no grouping).
#' @param summary A logical value indicating whether to include a summary in the output.
#'   Default is `FALSE`. If `TRUE`, the output list includes a `summary` element with
#'   descriptive statistics and outlier counts, either overall or by group if `groups` is provided.
#'
#' @return If `groups = NULL` (default), a list with the following components:
#'   - `outliers`: A numeric vector of the outlier values.
#'   - `indices`: An integer vector of the indices where outliers occur in the input vector.
#'   - `bounds` (if `method = "iqr"`): A named numeric vector with the `lower` and `upper`
#'     bounds for outliers.
#'   - `threshold` (if `method = "zscore"`): A named numeric vector with the `lower` and
#'     `upper` z-score thresholds.
#'   - `is_outlier`: A logical vector of the same length as `x`, where `TRUE` indicates an
#'     outlier.
#'   - `summary` (if `summary = TRUE`): A list with summary statistics including the mean,
#'     median, standard deviation (for z-score), quartiles (for IQR), and number of outliers.
#'
#'   If `groups` is provided, a named list where each element corresponds to a unique group,
#'   containing the same components as above but computed separately for that group’s values.
#'
#' @details
#' The function requires at least two non-`NA` values per group (if `groups` is provided) or
#' overall (if `groups = NULL`) to compute meaningful statistics when `na.rm = TRUE`. If
#' `na.rm = FALSE`, the presence of `NA` values triggers an error. If all values in a group
#' are identical or there are insufficient data points, an error is thrown for that group.
#' The IQR method is robust to non-normal data, while the z-score method assumes approximate
#' normality and is sensitive to extreme values.
#'
#' @examples
#' # Example 1: Basic IQR method without groups
#' x <- c(1, 2, 3, 4, 100)
#' detect_outlier(x)
#'
#' # IQR method with summary
#' detect_outlier(x, summary = TRUE)
#'
#' # Z-score method with custom threshold
#' y <- c(-10, 1, 2, 3, 4, 5, 20)
#' detect_outlier(y, method = "zscore", z_threshold = 2.5)
#'
#' # Handling missing values
#' z <- c(1, 2, NA, 4, 5, 100)
#' detect_outlier(z, method = "iqr", multiplier = 3)
#'
#' # Example 2: IQR method with groups
#' x2 <- c(1, 2, 3, 100, 5, 6, 7, 200)
#' groups2 <- c("A", "A", "A", "A", "B", "B", "B", "B")
#' detect_outlier(x2, groups = groups2)
#'
#' # Example 3: Z-score method with groups and summary
#' x3 <- c(-10, 1, 2, 20, 3, 4, 5, 50)
#' groups3 <- c("X", "X", "X", "X", "Y", "Y", "Y", "Y")
#' detect_outlier(x3, method = "zscore", z_threshold = 2, groups = groups3, summary = TRUE)
#'
#' # Example 4: IQR method with groups and NA values
#' x4 <- c(1, 2, NA, 100, 5, 6, 7, 200,1000)
#' groups4 <- c("G1", "G1", "G1", "G1", "G2", "G2", "G2", "G2","G1")
#' detect_outlier(x4, groups = groups4)
#'
#' # Error cases
#' \dontrun{
#' detect_outlier(c("a", "b"))  # Non-numeric input
#' detect_outlier(c(1), groups = c("A"))  # Insufficient data
#' detect_outlier(c(1, 2), groups = c("A"))  # Mismatched group length
#' detect_outlier(c(1, NA, 3), na.rm = FALSE)  # NA with na.rm = FALSE
#' }
#'
#' @export
detect_outlier <- function(x, method = "iqr", multiplier = 1.5, z_threshold = 3, na.rm = TRUE, groups = NULL, summary = FALSE) {
  # Check if input is numeric
  if (!is.numeric(x)) {
    stop("Input must be a numeric vector")
  }

  # Validate groups if provided
  if (!is.null(groups)) {
    if (length(groups) != length(x)) {
      stop("Length of 'groups' must match length of 'x'")
    }
  }

  # Handle NA values based on na.rm
  if (!na.rm && any(is.na(x))) {
    stop("NA values are present and na.rm = FALSE")
  }

  # Validate method
  method <- tolower(method)
  if (!method %in% c("iqr", "zscore")) {
    stop("Method must be either 'iqr' or 'zscore'")
  }

  # Process by group if groups is provided
  if (!is.null(groups)) {
    # Split data by groups
    group_list <- split(data.frame(x = x, index = seq_along(x)), groups)
    result <- lapply(names(group_list), function(g) {
      x_group <- group_list[[g]]$x
      indices <- group_list[[g]]$index

      # Remove NA values if na.rm = TRUE
      x_clean <- if (na.rm) x_group[!is.na(x_group)] else x_group
      if (length(x_clean) < 2) {
        stop(sprintf("Group '%s' has fewer than 2 non-NA values", g))
      }

      # Initialize group result
      group_result <- list()

      if (method == "iqr") {
        q1 <- quantile(x_clean, 0.25, na.rm = TRUE)
        q3 <- quantile(x_clean, 0.75, na.rm = TRUE)
        iqr <- q3 - q1
        if (iqr == 0) {
          stop(sprintf("All values in group '%s' are identical; IQR is zero", g))
        }
        lower_bound <- q1 - multiplier * iqr
        upper_bound <- q3 + multiplier * iqr
        group_result$outliers <- x_clean[x_clean < lower_bound | x_clean > upper_bound]
        group_result$indices <- indices[x_group < lower_bound | x_group > upper_bound]
        group_result$bounds <- c(lower = lower_bound, upper = upper_bound)
        group_result$is_outlier <- x_group < lower_bound | x_group > upper_bound
        if (summary) {
          group_result$summary <- list(
            mean = mean(x_clean),
            median = median(x_clean),
            q1 = q1,
            q3 = q3,
            iqr = iqr,
            n_outliers = length(group_result$outliers)
          )
        }
      } else if (method == "zscore") {
        mean_x <- mean(x_clean)
        sd_x <- sd(x_clean)
        if (sd_x == 0) {
          stop(sprintf("All values in group '%s' are identical; standard deviation is zero", g))
        }
        z_scores <- (x_group - mean_x) / sd_x
        lower_z <- -z_threshold
        upper_z <- z_threshold
        group_result$outliers <- x_clean[abs(z_scores[!is.na(x_group)]) > z_threshold]
        group_result$indices <- indices[abs(z_scores) > z_threshold]
        group_result$threshold <- c(lower = lower_z, upper = upper_z)
        group_result$is_outlier <- abs(z_scores) > z_threshold
        if (summary) {
          group_result$summary <- list(
            mean = mean_x,
            median = median(x_clean),
            sd = sd_x,
            n_outliers = length(group_result$outliers)
          )
        }
      }
      group_result
    })
    names(result) <- names(group_list)
    return(result)
  }

  # Process without groups (original behavior)
  x_clean <- if (na.rm) x[!is.na(x)] else x
  if (length(x_clean) < 2) {
    stop("At least two non-NA values are required")
  }


  skewness = if(length(x_clean) > 2) {
    sum((x_clean - mean(x_clean))^3) /
      (length(x_clean) * sd(x_clean)^3)
  } else NA
  kurtosis = if(length(x_clean) > 2) {
    sum((x_clean - mean(x_clean))^4) /
      (length(x_clean) * sd(x_clean)^4) - 3
  } else NA

  result <- list()

  if (method == "iqr") {
    q1 <- quantile(x_clean, 0.25, na.rm = TRUE)
    q3 <- quantile(x_clean, 0.75, na.rm = TRUE)
    iqr <- q3 - q1
    if (iqr == 0) {
      stop("All values are identical; IQR is zero")
    }
    lower_bound <- q1 - multiplier * iqr
    upper_bound <- q3 + multiplier * iqr
    result$outliers <- x_clean[x_clean < lower_bound | x_clean > upper_bound]
    result$indices <- which(x < lower_bound | x > upper_bound)
    result$bounds <- c(lower = lower_bound, upper = upper_bound)
    result$is_outlier <- x < lower_bound | x > upper_bound
    if (summary) {
      result$summary <- list(
        n = length(x_clean),
        mean = mean(x_clean),
        median = median(x_clean),
        q1 = q1,
        q3 = q3,
        iqr = iqr,
        n_outliers = length(result$outliers),
        skewness = skewness,
        kurtosis = kurtosis
      )
    }
  } else if (method == "zscore") {
    mean_x <- mean(x_clean)
    sd_x <- sd(x_clean)
    if (sd_x == 0) {
      stop("All values are identical; standard deviation is zero")
    }
    z_scores <- (x - mean_x) / sd_x
    lower_z <- -z_threshold
    upper_z <- z_threshold
    result$outliers <- x_clean[abs(z_scores[!is.na(x)]) > z_threshold]
    result$indices <- which(abs(z_scores) > z_threshold)
    result$threshold <- c(lower = lower_z, upper = upper_z)
    result$is_outlier <- abs(z_scores) > z_threshold
    if (summary) {
      result$summary <- list(
        n = length(x_clean),
        mean = mean_x,
        median = median(x_clean),
        sd = sd_x,
        n_outliers = length(result$outliers),
        skewness = skewness,
        kurtosis = kurtosis
      )
    }
  }

  return(result)
}









#' Advanced Outlier Detection in Numeric Data with Optional Grouping
#'
#' @description
#' Detects outliers in numeric data using multiple statistical methods and provides
#' comprehensive analysis including visualizations and summary statistics. Supports
#' grouped analysis when group vector is provided. The function supports three
#' detection methods: IQR, Z-score, and Modified Z-score.
#'
#' @param x A numeric vector containing the data to analyze
#' @param groups Optional vector of group labels, must be same length as x
#' @param method Character string specifying the outlier detection method.
#'   Options are "iqr", "zscore", "modified_zscore", or "all". Default is "iqr"
#' @param multiplier Numeric value specifying the IQR multiplier for
#'   outlier detection. Default is 1.5
#' @param z_threshold Numeric value specifying the Z-score threshold.
#'   Default is 3
#' @param modified_z_threshold Numeric value specifying the modified Z-score
#'   threshold. Default is 3.5
#' @param plot Logical indicating whether to generate visualization plots.
#'   Default is TRUE
#' @param plot_groups Logical indicating whether to create separate plots for each
#'   group. Only used when groups are provided. Default is TRUE
#'
#' @return A list of class "outlier_analysis" containing:
#'   \itemize{
#'     \item overall: Overall analysis results
#'     \item by_group: Group-specific results (if groups provided)
#'     \item plots: List of generated plots
#'   }
#'
#' @examples
#' # Example 1: Basic grouped analysis
#' set.seed(123)
#' data <- c(rnorm(50), rnorm(50, 2), rnorm(50, 4))
#' groups <- rep(c("A", "B", "C"), each = 50)
#' resultA <- detect_outlier2(data) # no groups
#' resultB <- detect_outlier2(data, groups = groups) # with groups
#'
#' # Example 2: Custom thresholds by group
#' test_scores <- c(65, 70, 75, 72, 68, 73, 78, 71, 69, 74,
#'                  90, 85, 92, 88, 95, 87, 91, 89, 86, 93)
#' class_groups <- rep(c("Morning", "Afternoon"), each = 10)
#' result <- detect_outlier2(test_scores,
#'                                    groups = class_groups,
#'                                    method = "all",
#'                                    z_threshold = 2)
#' result$overall
#' result$by_group
#' result$plots$overall$boxplot()
#' result$plots$overall$density()
#' result$plots$overall$comparison()
#' result$plots$by_group$Morning$boxplot()
#' result$plots$by_group$Morning$density()
#' result$plots$by_group$Afternoon$boxplot()
#' result$plots$by_group$Afternoon$density()
#'
#' @export
detect_outlier2 <- function(x,
                            groups = NULL,
                            method = c("iqr", "zscore", "modified_zscore", "all"),
                            multiplier = 1.5,
                            z_threshold = 3,
                            modified_z_threshold = 3.5,
                            plot = TRUE,
                            plot_groups = TRUE) {
  # Input validation
  if (!is.numeric(x)) {
    stop("Input must be a numeric vector")
  }

  if (!is.null(groups) && length(groups) != length(x)) {
    stop("groups must be the same length as x")
  }

  method <- match.arg(method)

  # Initialize results list
  results <- list()

  # Analyze overall data
  results$overall <- analyze_dataset(x, m = multiplier, m2 = method, zt = z_threshold,mzt = modified_z_threshold)

  # Analyze by group if groups provided
  if (!is.null(groups)) {
    results$by_group <- lapply(split(x, groups), function(data) analyze_dataset(data, m = multiplier, m2 = method,zt = z_threshold,mzt = modified_z_threshold))
  }

  # Generate plots
  if (plot) {
    results$plots <- list()

    # Overall plots
    results$plots$overall <- list()

    # Boxplot
    results$plots$overall$boxplot <- function() {
      if (!is.null(groups)) {
        boxplot(split(x, groups), main = "Boxplot by Group",
                ylab = "Value", col = "lightblue",
                outline = TRUE)
      } else {
        boxplot(x, main = "Boxplot with Outliers",
                ylab = "Value", col = "lightblue",
                outline = TRUE)
        points(rep(1, length(x)), x, col = "red", pch = 20)
      }
    }

    # Density plot
    results$plots$overall$density <- function() {
      if (!is.null(groups)) {
        # Setup plot
        plot(NULL, xlim = range(x, na.rm = TRUE),
             ylim = c(0, max(sapply(split(x, groups), function(g) {
               max(density(g, na.rm = TRUE)$y)
             }))),
             main = "Density Plot by Group",
             xlab = "Value", ylab = "Density")

        # Plot each group with different colors
        colors <- rainbow(length(unique(groups)))
        for (i in seq_along(unique(groups))) {
          group_data <- x[groups == unique(groups)[i]]
          d <- density(group_data, na.rm = TRUE)
          lines(d, col = colors[i])
          rug(group_data, col = colors[i])
        }
        legend("topright", legend = unique(groups),
               col = colors, lty = 1)
      } else {
        d <- density(x, na.rm = TRUE)
        plot(d, main = "Density Plot of Data",
             xlab = "Value", ylab = "Density")
        polygon(d, col = "lightblue", border = "blue")
        rug(x, col = "red")
      }
    }

    # Method comparison plot
    if (method == "all") {
      results$plots$overall$comparison <- function() {
        par(mfrow = c(3, 1), mar = c(4, 4, 2, 2))
        methods <- c("IQR", "Z-score", "Modified Z-score")

        if (!is.null(groups)) {
          # Plot for each method with groups
          for(i in 1:3) {
            plot(NULL, xlim = range(x, na.rm = TRUE), ylim = c(0, 1),
                 main = paste(methods[i], "Method"),
                 xlab = "Value", ylab = "Group")

            unique_groups <- unique(groups)
            for(j in seq_along(unique_groups)) {
              group_data <- results$by_group[[unique_groups[j]]]
              outliers <- switch(i,
                                 group_data$iqr$is_outlier,
                                 group_data$zscore$is_outlier,
                                 group_data$modified_zscore$is_outlier)

              points(x[groups == unique_groups[j]],
                     rep(j/length(unique_groups), sum(groups == unique_groups[j])),
                     pch = ifelse(outliers, 16, 1),
                     col = rainbow(length(unique_groups))[j])
            }
            axis(2, at = seq_along(unique_groups)/length(unique_groups),
                 labels = unique_groups)
          }
        } else {
          # Original comparison plot for ungrouped data
          outliers_list <- list(
            results$overall$iqr$is_outlier,
            results$overall$zscore$is_outlier,
            results$overall$modified_zscore$is_outlier
          )

          for(i in 1:3) {
            plot(x[!is.na(x)], rep(0, sum(!is.na(x))),
                 pch = ifelse(outliers_list[[i]], 16, 1),
                 col = ifelse(outliers_list[[i]], "red", "black"),
                 main = paste(methods[i], "Method"),
                 xlab = "Value", ylab = "",
                 yaxt = "n")
            abline(h = 0, lty = 2, col = "gray")
          }
        }
        par(mfrow = c(1, 1))
      }
    }

    # Generate separate plots for each group if requested
    if (!is.null(groups) && plot_groups) {
      results$plots$by_group <- list()
      unique_groups <- unique(groups)

      for (group in unique_groups) {
        group_data <- x[groups == group]
        results$plots$by_group[[group]] <- list()

        # Boxplot for group
        results$plots$by_group[[group]]$boxplot <- function() {
          boxplot(group_data, main = paste("Boxplot -", group),
                  ylab = "Value", col = "lightblue",
                  outline = TRUE)
          points(rep(1, length(group_data)), group_data,
                 col = "red", pch = 20)
        }

        # Density plot for group
        results$plots$by_group[[group]]$density <- function() {
          d <- density(group_data, na.rm = TRUE)
          plot(d, main = paste("Density Plot -", group),
               xlab = "Value", ylab = "Density")
          polygon(d, col = "lightblue", border = "blue")
          rug(group_data, col = "red")
        }
      }
    }
  }

  class(results) <- "outlier_analysis"
  return(results)
}


#' @rdname outlierdetect1
#' @export
iqr_outlier <- function(x, multiplier) {
  q1 <- quantile(x, 0.25, na.rm = TRUE)
  q3 <- quantile(x, 0.75, na.rm = TRUE)
  iqr <- q3 - q1
  lower_bound <- q1 - multiplier * iqr
  upper_bound <- q3 + multiplier * iqr

  list(
    outliers = x[x < lower_bound | x > upper_bound],
    indices = which(x < lower_bound | x > upper_bound),
    bounds = c(lower = lower_bound, upper = upper_bound),
    is_outlier = x < lower_bound | x > upper_bound
  )
}
#' @rdname outlierdetect1
#' @export
zscore_outlier <- function(x, z_threshold) {
  z_scores <- scale(x)
  list(
    outliers = x[abs(z_scores) > z_threshold],
    indices = which(abs(z_scores) > z_threshold),
    z_scores = z_scores,
    is_outlier = abs(z_scores) > z_threshold
  )
}
#' @rdname outlierdetect1
#' @export
zscore_outlier2 <- function(x, z_threshold) {
  median_x <- median(x, na.rm = TRUE)
  mad_x <- mad(x, na.rm = TRUE)
  modified_z_scores <- 0.6745 * (x - median_x) / mad_x

  list(
    outliers = x[abs(modified_z_scores) > z_threshold],
    indices = which(abs(modified_z_scores) > z_threshold),
    modified_z_scores = modified_z_scores,
    is_outlier = abs(modified_z_scores) > z_threshold
  )
}

# Function to compute summary statistics
compute_summary <- function(x) {
  list(
    n = length(x),
    na_count = sum(is.na(x)),
    mean = mean(x, na.rm = TRUE),
    median = median(x, na.rm = TRUE),
    sd = sd(x, na.rm = TRUE),
    skewness = if(length(x) > 2) {
      sum((x - mean(x, na.rm = TRUE))^3, na.rm = TRUE) /
        (length(x[!is.na(x)]) * sd(x, na.rm = TRUE)^3)
    } else NA,
    kurtosis = if(length(x) > 2) {
      sum((x - mean(x, na.rm = TRUE))^4, na.rm = TRUE) /
        (length(x[!is.na(x)]) * sd(x, na.rm = TRUE)^4) - 3
    } else NA
  )
}

# Function to analyze a single dataset
analyze_dataset <- function(data, m,zt,mzt,m2) {
  data_clean <- data[!is.na(data)]
  result <- list()
  result$summary <- compute_summary(data)

  # Apply selected m(s)
  if (m2 %in% c("iqr", "all")) {
    result$iqr <- iqr_outlier(data_clean, m)
  }
  if (m2 %in% c("zscore", "all")) {
    result$zscore <- zscore_outlier(data_clean, zt)
  }
  if (m2 %in% c("modified_zscore", "all")) {
    result$modified_zscore <- zscore_outlier2(data_clean, mzt)
  }

  # Create comparison if multiple ms used
  if (m2 == "all") {
    result$comparison <- data.frame(
      value = data_clean,
      iqr_outlier = result$iqr$is_outlier,
      zscore_outlier = result$zscore$is_outlier,
      modified_zscore_outlier = result$modified_zscore$is_outlier,
      methods_agreeing = rowSums(cbind(
        result$iqr$is_outlier,
        result$zscore$is_outlier,
        result$modified_zscore$is_outlier
      ))
    )
  }

  return(result)
}

#' Not logical
#'
#' Opposite of is.logical(). Check if entry is a logical object
#'
#' @param x vector entry
#' @return a boolean value to indicate if entry is logical
#' @examples
#' test.env <- TRUE
#' test.notenv <- 0
#' not.logical(test.env) # FALSE
#' not.logical(test.notenv) # TRUE
#' if(not.logical(test.notenv)) message("yes") # yes
#'
#' @export

not.logical <- function(x)!{
  is.logical(x)
}

#' Calculates Z-Scores of a distribution
#'
#' Calculates Z-Scores based on data
#' @rdname zscore
#' @param .data data object
#' @param round round output to how many decimal place description
#' @param na.rm remove NA values before calculating z-scores
#'
#' @return zscore calculated based on data object or parameters
#'
#' @examples
#' # Capture z-score from the following distribution x
#' x = c(6, 7, 7, 12, 13, 13, 15, 16, 19, 22)
#' z_scores = zscore(x, round = 2) # limit to 2 decimal place
#' z_scores = zscore(x) # no decimal place limit
#'
#' df = data.frame(val = x, zscore = z_scores)
#' head(df)
#' @export

zscore <- function(.data, round, na.rm = TRUE) {
  .x <- (.data - mean(.data, na.rm = na.rm)) / sd(.data, na.rm = na.rm)
  if(!missing(round)){
   .x <-  round(.x, round)
  }
  .x
}



#' @rdname zscore
#' @param Xi physical measurement (e.g. weight, length, head circumference, stature or calculated BMI value)
#' @param Mi values from the table (see reference) corresponding to the age in months of the child
#' @param Si values from the table (see reference) corresponding to the age in months of the child
#' @param Li values from the table (see reference) corresponding to the age in months of the child
#'
#' @examples
#' #EXAMPLE for zscore based on CDC growth chart
#'
#' # Calculate the zscore for a patient weighing 50kg
#' Li=-0.1600954
#' Mi=9.476500305
#' Si=0.11218624
#' Xi=50
#' zscoreGrowthCurve(Xi,Mi,Si,Li)
#'
#' @references CDC growth chart Z score calculation: https://www.cdc.gov/growthcharts/cdc-data-files.htm
#' @export

zscoreGrowthCurve <- function(Xi,Mi,Si,Li=!0){
  stopifnot(is.numeric(Li))
  if(Li){
   (((Xi/Mi)**Li) - 1)/(Li*Si)
  }else{
   log(Xi/Mi)/Si
  }
}

#' R Color Constant
#'
#' This function provides information that describes the color constants that exist in R
#'
#' @param title title of the output
#' @return returns color constant
#'
#' @details
#' In addition to the color palette in R that can be represented as either color literals or hexadecimal values,
#' numeric values can also be used to add colorization to a plot. Numeric values ranging from 1 to 8 provide 8 basic colors that can be deployed. The rcolorconst
#' function returns both a Named Vector and a color palette plot that connects these numeric values with their corresponding color.
#'
#' @examples
#' # Without title
#' ex1 <- rcolorconst()
#'
#' # With title
#' ex2 <- rcolorconst("My new color constant")
#'
#' # More detailed example
#' set.seed(200)
#' x = data.frame(
#'   meas = rnorm(100),
#'   grp = sample(1:8, size = 100,
#'   replace = TRUE))
#' plot(x, pch = 16, col = x$grp)
#' colnums = rcolorconst()
#'
#' @export

rcolorconst <- function(title = "R Color Constants"){
  rcolorconst = c("1" ="black",
                  "2" ="red",
                  "3" = "green",
                  "4" = "blue",
                  "5" = "cyan",
                  "6" = "magenta",
                  "7" = "orange",
                  "8" = "gray")
  custom_swatch(rcolorconst, main = title)
  #Polychrome::swatch(rcolorconst, main = title)
  stats::setNames(as.numeric(names(rcolorconst)), rcolorconst)
}

#' Not NA
#'
#' Opposite of is.na(). Check if entry is not NA
#'
#' @param x vector entry
#' @return a boolean value to indicate if entry is NA
#' @examples
#' not.na(NA) # FALSE
#' not.na(NULL) # logical(0)
#' if(not.na(45)) message("something") # TRUE
#'
#' @export

not.na <- function(x)!{
  is.na(x)
}

#' Check is numbers in a vector are decreasing or increasing
#'
#' Test if numeric values of a vector are decreasing or increasing using the radix method
#'
#' @param . a numeric vector
#' @param na.last for controlling the treatment of NAs. If TRUE, missing values in the data are put last; if FALSE, they are put first; if NA, they are removed.
#' @rdname descasc
#' @return boolean value to indicate if the values are increasing or decreasing
#'
#' @examples
#' # example code
#' doy1 <- rnorm(1e3)
#' doy2 <- sort(doy1, decreasing = FALSE)
#' doy3 <- sort(doy1, decreasing = TRUE)
#'
#' is.increasing(doy1)
#' is.decreasing(doy1)
#'
#' is.increasing(doy2)
#' is.decreasing(doy2)
#'
#' is.increasing(doy3)
#' is.decreasing(doy3)
#' @export

is.increasing  <- function(., na.last = TRUE)!{
  is.unsorted(sort(as.numeric(.), decreasing = FALSE, na.last = na.last, method = "radix", index.return = TRUE)$ix)
}


#' @rdname descasc
#' @export

is.decreasing <- function(., na.last = TRUE){
  . <- as.numeric(.)
  .. <- sort(., decreasing = TRUE, na.last = na.last, method = "radix")
  all(.. == .)
}
#is.unsorted(rev(sort(as.numeric(.), decreasing = FALSE, na.last = na.last, method = "radix", index.return = TRUE)$ix))

#' Duplicate a file with global text replacement
#'
#' Shorthand to return a re-sample number of rows in a data frame by unique column
#'
#' @param file data frame to re-sample
#' @param new.name column to uniquely re-sample
#' @param pattern number of rows to return
#' @param replacement unique numeric value for reproducibility
#' @param open description
#' @return data frame containing re-sampled rows from an original data frame
#'
#' @examples
#' \donttest{
#' if(interactive()){
#' # example to duplicate a file, and replace text1 within it
#' # NOTE that, by default, this function will also open the file within RStudio
#'
#' #create sample file
#' file1s <- paste0(tempfile(),".R")
#' writeLines("message(
#' 'Sample items: eggs, coke, fanta, book'
#' )", file1s)
#'
#' file2s <- paste0(tempfile(),".R")
#' file3s <- paste0(tempfile(),".R")
#'
#' duplicate(
#'   file = file1s,
#'   new.name = file2s,
#'   pattern = 'text1',
#'   replacement = 'replacement1'
#'  )
#'
#' # duplicate the file, with multiple replacements
#' # replace 'book' with 'egg' and 'coke' with 'fanta'
#' duplicate(
#'   file1s, file2s,
#'   pattern = c('book','coke'),
#'   replacement = c('egg','fanta')
#' )
#'
#'
#'
#' # duplicate the file with no replacement
#' duplicate(file1s,file3s) # this simply performs file.copy, and opens the new file
#'
#'
#'
#' # duplicate the file but do not open for editing
#' duplicate(file1s,file3s, open = FALSE) # this does not open file after duplication
#' }
#' }
#' @export
#'
duplicate <- function(file, new.name,pattern = NULL, replacement = NULL,open = TRUE){
  #exit if the file does not exist
  if(!file.exists(file)) stop("The file you are trying to duplicate does not exist.")

  #get initial file
  .file.1 <- readLines(file)

  #substitute text
  if(not.null(pattern) & not.null(replacement)){
    for(.i in 1:length(pattern)){
      .file.1 <- gsub(pattern[.i],replacement[.i],.file.1)
    }
  }

  #write to new file
  writeLines(.file.1,new.name)

  message(sprintf("The new file '%s' was successfully created!",new.name))

  if(open) rstudioapi::navigateToFile(new.name)
}


#' Prompt guided duplication and editing of files
#'
#' AI like duplication and editing of files
#'
#' @param file file to duplicate
#' @param new.name OPTIONAL.name of new file
#' @param open open file after duplication
#'
#' @return duplicated files with edited texts
#'
#' @examples
#' \donttest{
#' if(interactive()){
#' file1s <- paste0(tempfile(),".R")
#' writeLines("message(
#' 'Sample items: farm, shinyappstore, rpkg'
#' )", file1s)
#' ai.duplicate(file1s,'file2.R')
#' }
#' }
#'
#'
#' @export
#'
ai.duplicate <- function(file = NULL, new.name = NULL , open = TRUE) {
  #declare initial pattern and replacement
  init(pattern, replacement)

  if (is.null(file)) {
    file <- readline(prompt = "What file are you trying to duplicate?")
  }

  if(!file.exists(file)) stop("The file you are trying to duplicate does not exist.")

  if (is.null(new.name)) {
    new.name <- readline(prompt = "What is the new file name?")
  }

  #strings to replace
  repeat{
    vector_push(pattern,readline(prompt = "What string would you like to replace?"))
    vector_push(replacement,readline(prompt = "What will you like to replace it with?"))

    #check if more replacements are needed
    if(!as.logical(toupper(readline(prompt = "Want to replace more (T = Yes, F = No) ?"))))
      break
  }
  #duplicate file with entered parameters
  duplicate(file, new.name, pattern, replacement, open = open)
  invisible(file)
}

#' Not NULL
#'
#' Opposite of is.null(). Check if entry is not NULL
#'
#' @param x vector entry
#' @return a boolean value to indicate if entry is NULL
#' @examples
#' is.null("") # FALSE
#' not.null("") # TRUE
#' not.null(NULL) # FALSE
#' if(not.null(45)) message("something") # yes
#'
#' @export

not.null <- function(x)!{
  is.null(x)
}

#' Count the number of decimal places
#'
#' @param num a numeric value
#'
#' @examples
#' #example 1
#' ndecimal(9.000322)
#'
#' #example 2
#' ndecimal(34)
#'
#' #example 3
#' ndecimal(45.)
#'
#' @export

ndecimal<-function(num){
  stopifnot(inherits(num,"numeric")|inherits(num,"integer"))
  unlist(lapply(num,function(x){
    if ((x %% 1) != 0) {
    nchar(strsplit(sub('0+$', '', as.character(x)), ".", fixed=TRUE)[[1]][[2]])
  } else {
    return(0)
  }
  }))
}
#

#' Calculate the distance of points from the center of a cluster
#'
#' This function operates on multivariate data and
#' calculates the distance of points from the centroid of one or more clusters.
#'
#' @rdname machine_learning
#' @param data data frame object or a matrix/array object
#' @param scale Boolean used for scaling the dataset
#' @param roundit Round x number of decimal places; min value is 0; max value is 6
#'
#' @section Function utility:
#' Used to generate the computations needed to model pair-distance measures in three dimensions
#'
#' @section More information about this function:
#' The pairDist function is used to quantify how far each data point (row)
#' is from the overall mean across all columns. It’s commonly used in multivariate statistics,
#' machine learning, and data analysis to assess the variability or similarity of data points
#' relative to their mean. More specifically, the function is used in outlier detection and
#' cluster analysis to evaluate the dispersion of data. Used in conjunction with other
#' calculations, pairDist output can also be used to model data in three dimensions.
#'
#'
#' @references the current function was adapted from one of the examples in the svgViewR package, \cr
#' https://cran.r-project.org/web/packages/svgViewR/svgViewR.pdf
#' @examples
#'
#' data(airquality)
#' airq <- airquality[, 1:4]
#'
#' # example 1:
#' x <- pairDist(airq, roundit = 3)
#'
#' # example 2:
#' y <- pairDist(airq, scale = TRUE, roundit = 5)
#'
#' # example 3
#'
#' data <- matrix(
#'   c(1, 5, NA, 4, 5, 6, NA, 8, 9, NA, 12, 23, 6, 0, 3, 3, 8, 15),
#'   ncol = 3,
#'   byrow = TRUE
#' )
#'
#' dist <- pairDist(data, roundit = 2)
#' dist_scaled <- pairDist(data, scale = TRUE, roundit = 6)
#'
#' @export

pairDist <- function(data, scale = FALSE, roundit = NULL) {
  if (!all(unique(unlist(sapply(data, class))) %in% c(
    "numeric",
    "double", "integer"
  ))) {
    stop("The data should only include numeric values and NAs")
  }

  if (roundit > 6) {
    stop("The roundit argument cannot be a value larger than 6 or smaller than 0")
  }

  if (!scale) {
    mean_matrix <- matrix(colMeans(data, na.rm = TRUE),
      nrow = nrow(data),
      ncol = ncol(data), byrow = TRUE
    )
    squared_diff <- (data - mean_matrix)^2
    row_sums <- rowSums(squared_diff, na.rm = TRUE)
    result <- sqrt(row_sums)
    result <- round(result, roundit)
  }

  if (scale) {
    datascale <- scale(data)
    mean_matrix <- matrix(colMeans(datascale, na.rm = TRUE),
      nrow = nrow(datascale),
      ncol = ncol(datascale), byrow = TRUE
    )
    squared_diff <- (datascale - mean_matrix)^2
    row_sums <- rowSums(squared_diff, na.rm = TRUE)
    result <- sqrt(row_sums)
    result <- round(result, roundit)
  }

  data.frame(pdist = result, isNA = apply(data, 1, function(row) any(is.na(row))))
}

#' Shuffle a list object just like shuffle in PHP
#'
#' Shorthand to shuffle a list and save
#'
#' @param . list to shuffle
#' @param seed apply seed if indicated for reproducibility
#' @return shuffled list of items store to the list name
#'
#' @examples
#' list001 <- list("a" = 1:5,
#'            "b" = letters[1:5],
#'            c = LETTERS[1:10],
#'            "2" = number(5,5),
#'            "e" = randString(5,5))
#' list001 #show initial list
#'
#' #illustrate basic functionality
#' list_shuffle(list001)
#' list001 #shuffle and resaved to variable
#'
#' list.f2<-list001
#' list_shuffle(list.f2)
#' list.f2 #first output
#'
#' list.f2<-list001
#' list_shuffle(list.f2)
#' list.f2 # different output from first output top
#'
#' list.f2<-list001
#' list_shuffle(list.f2,seed = 344L)
#' list.f2 #second output
#'
#' list.f2<-list001
#' list_shuffle(list.f2,seed = 344L)
#' list.f2 #the same output as second output top
#'
#' @export
#'

list_shuffle <- function(., seed = NULL) {
  .. <- substitute(.)

  if (typeof(..) != "symbol") stop(paste0(.., " must be an object."))

  list1 <- get(as.character(..), envir = parent.frame())

  if(not.null(seed))set.seed(seed)

  assign(as.character(..), sample(names(list1)), envir = parent.frame())
}

#' Shuffle a vector just like shuffle in PHP
#'
#' Shorthand to shuffle a vector and save
#'
#' @param . vector to shuffle
#' @param replace replace selected value
#' @param prob probability of occurrence
#' @param seed apply seed if indicated for reproducibility
#' @return shuffled vector of items store to the vector name
#'
#' @examples
#'
#' #basic example
#' vector_shuffle(c(3,45,23,3,2,4,1))
#'
#'
#' #using objects
#' v1<-c(3,45,23,3,2,4,1)
#'
#' #demonstrate vector_shuffle
#' vector_shuffle(v1)
#' v1 # show outputs
#'
#' #demonstrate reproducibility in shuffle with seed
#' v0<-v1
#' vector_shuffle(v0)
#' v0 #first output
#'
#' v0<-v1
#' vector_shuffle(v0)
#' v0 # different output from first output top
#'
#' v0<-v1
#' vector_shuffle(v0,seed = 232L)
#' v0 #second output
#'
#' v0<-v1
#' vector_shuffle(v0,seed = 232L)
#' v0 #the same output as second output top
#' @export
#'

vector_shuffle <- function(., replace = FALSE, prob = NULL, seed = NULL) {
  if(not.vector(.)) stop("The first argument must be a vector")
  .. <- substitute(.)
  if(not.null(seed))set.seed(seed)
  if (typeof(..) == "language")
    return(sample(., replace = replace, prob = prob))

  if (typeof(..) != "symbol") stop(paste0(.., " must be an object."))

  val <- get(as.character(..), envir = parent.frame())

  assign(as.character(..), sample(val, length(val), replace = replace, prob = prob), envir = parent.frame())
}

#' Not duplicated elements
#'
#' Opoosite of duplicated(). Checks which elements of a vector or data frame are NOT duplicates of elements with smaller subscripts
#'
#' @param x a vector or a data frame or an array or NULL.
#' @param incomparables a vector of values that cannot be compared. FALSE is a special value, meaning that all values can be compared, and may be the only value accepted for methods other than the default. It will be coerced internally to the same type as x
#' @param ... arguments for particular methods.
#' @return elements of a vector or data frame that are NOT duplicates
#' @examples
#'
#' set.seed(08082023)
#' dtf <- sample(1:10,15, replace = TRUE)
#' dtf # 3  9 10  3  8  9  6 10  5  1  2  2  2  9  8
#' dtf[ dtf > 4 & duplicated(dtf) ] # 9 10  9  8
#' dtf[ dtf > 4 & not.duplicated(dtf) ] # 9 10  8  6  5
#'
#' @export

not.duplicated <- function(x, incomparables = FALSE, ...)!{
  duplicated(x = x, incomparables = incomparables, ...)
}

#' Convert Dates into Numeric Week Counts
#'
#' Convert the range of date to number of weeks
#' @rdname date-topic
#' @param dates vector of dates that need not be sequential but that reference values of class Date;
#' Note that if this argument is populated, the start_date and end_date arguments must be empty
#' @param start_date A scaler of class Date (if this argument is populated, the date arg must be empty)
#' @param end_date A scaler of class Date; must be later than the start_date (if this argument is populated, the date arg must be empty)
#' @param in.format date input format
#' @return data frame of the dates along with their corresponding week
#' @examples
#' # simple example with start and end date
#' getWeekSeq(start_date="12/29/25",end_date="1/8/26")
#'
#' # enter specific dates instead
#' # specify format
#' getWeekSeq(
#' dates = c(
#'   '2025-12-29',
#'   '2025-12-30',
#'   '2025-12-31',
#'   '2026-01-01',
#'   '2026-01-04',
#'   '2026-01-05',
#'   '2026-01-06',
#'   '2026-01-07',
#'   '2026-01-08'),
#'   in.format = "%Y-%m-%d"
#' )
#'
#' getWeekSeq(
#' dates = c(
#'   '12/29/25',
#'   '12/30/25',
#'   '12/31/25',
#'   '01/01/26',
#'   '01/02/26',
#'   '01/03/26',
#'   '01/06/26',
#'   '01/07/26',
#'   '01/08/26'
#'   ),
#'   in.format = "%m/%d/%y"
#' )
#'
#' @export
#'

getWeekSeq <- function(start_date, end_date, dates, in.format = "%m/%d/%y") {
  if(!missing(dates)){
    date_sequence <- as.Date(dates, format=in.format)
    if(!missing(start_date) | !missing(end_date))
      stop("'start_date' and 'end_date' should not be provided if 'dates' is provided")
  }else{
    # Convert input dates to Date objects
    start_date <- as.Date(start_date, format=in.format)
    end_date <- as.Date(end_date, format=in.format)
    # Generate a sequence of dates
    date_sequence <- seq(from = start_date, to = end_date, by = "day")
  }

  # Create a data frame
  week_df <- data.frame(DATE = date_sequence)

  # Add a column for the week's sequence starting with the numeric
  # week of the year as provided by lubridate's week function
  week_df$WEEK <- unlist(lapply(week_df$DATE, function(h){
    (yday2(h) - 1)%/%7 + 1
  }))
  week_df
}



yday2 = function(start_date){
  .year <- as.numeric(format(start_date,"%Y"))
  .month <- as.numeric(format(start_date,"%m"))
  .day <- as.numeric(format(start_date,"%d"))
  dinm <- c(0,31,28,31,30,31,30,31,31,30,31,30,31)
  if(is.leap(.year)) dinm[2] <- 29
  cumsum(dinm)[.month]+.day
}


#' @rdname date-topic
#' @param x date item to check
#' @export
is.Date <- function(x){
  inherits(x, "Date")
}
#' @rdname date-topic
#' @export
not.Date <- function(x)!{
  inherits(x, "Date")
}

#' @rdname date-topic
#' @param yyyy year numeric value eg 1989
#' @export
is.leap <- function(yyyy){
  if(missing(yyyy)) yyyy = as.numeric(format(Sys.Date(),"%Y"))
  yyyy = as.numeric(yyyy)
  unlist(lapply(yyyy, function(y1){
    y1%%4==0 && !(y1%%100==0 && y1%%400!=0)
  }))
}

#' Snippet function to add header to a current Rmd opened file
#'
#' Shorthand to add Rmd header
#'
#' @return Inserts header content for Rmd file
#'
#' @examples
#' if(interactive())
#' header.rmd()
#' @export
#'

header.rmd <- function() {
  insertInText(paste0("
<!---

Document Path: ", rstudioapi::getActiveDocumentContext()$path, "

Author:

Date: ", Sys.Date(), "

Title:

Description:

Required Files:

Exported Files:

R Version: ", version$version.string, "


--->"))
}

#' Combine unique() and length()
#'
#' Combine two frequently used function together to return the length of the unique items of a vector
#'
#' @param . object such as vector or names(dataframe)
#' @return length of the unique items in a vector
#'
#' @examples
#' frenchnames1 = c("Léa","Obinna","Bastien","Léa","Obinna", "Hugo", "Amélie","Louis")
#' unique_len(frenchnames1)
#'
#' @export

unique_len <- function(.){
  length(unique(.))
}

#' Set a variable only once
#'
#' Facilitates the one-time setting of a variable in R, ensuring its immutability thereafter.
#'
#' @param . variable to set
#' @param val the value to set for the variable
#' @param envir environment where variables resides
#'
#' @return the variable set to the new variable, along with a class of once added to the output
#'
#' @details
#' With this function, users can establish the change to the initial value of a variable,
#' and it guarantees that any subsequent attempts to modify the variable are ignored.
#' This feature ensures that the variable remains constant and immutable once it has been set,
#' preventing unintentional changes and promoting code stability. This function simplifies the process
#' of managing immutable variables in R, providing a reliable mechanism for enforcing
#' consistency in data throughout the course of a program or script.
#'
#' @examples
#' # set the value of vector_x1, vector_y1, vector_z1
#' init(vector_x1, vector_y1, vector_z1, value = 85)
#'
#' # view the initial values of the variables
#' vector_x1
#' vector_y1
#' vector_z1
#'
#' # task 1: change the value vector_x1 and prevent further changes
#' vector_x1 # check value of unchanged
#' vector_x1 * 0.56 # check value when x 0.56
#'
#' setOnce(vector_x1, val = 4500) # set vector_x1
#' vector_x1 # check value
#' vector_x1 * 0.56 # check value when x 0.56
#'
#' setOnce(vector_x1, val = 13) # set vector_x1 AGAIN, should not change
#' vector_x1 # check value
#' vector_x1 * 0.56 # check value when x 0.56
#'
#' # task 2: In for loop, change vector_y1 and use later
#' vector_y1 # check value of unchanged
#'
#' for(i in 1:20){
#' setOnce(vector_y1,as.numeric(Sys.time()))
#' # now let's see the difference between vector_y1
#' # and the current time as it changes
#' message("current vector_y1: ",vector_y1,"; subtraction res: ",as.numeric(Sys.time()) - vector_y1)
#' }
#'
#' # task 3: In for lapply, change vector_z1 and use later
#' vector_z1 # check value of unchanged
#'
#' invisible(
#' lapply(1:20, function(i){
#' setOnce(vector_z1,as.numeric(Sys.time()))
#' # now let's see the difference between vector_z1
#' # and the current time as it changes
#' message("current vector_z1: ",vector_z1,"; subtraction res: ",as.numeric(Sys.time()) - vector_z1)
#' })
#' )
#'
#' # result of all the tasks
#' vector_x1
#' vector_y1
#' vector_z1
#'
#'
#' @export

setOnce <- function(., val = 1L, envir = NULL) {
  .. <- substitute(.)
  if(is.null(envir)) envir <- getEnvir(as.character(..))
  if(!inherits(get(as.character(..),envir = envir),"once")){
  if (typeof(..) != "symbol") stop(paste0(.., " must be an object."))
  res <- val
  class(res) <- c('once',class(res))
  assign(as.character(..), res, envir = envir)
  }
}

# Future release 1.0
#' Extract all dates from a string
#' @param out.format date output format
#' @param str1 string to parse
#' @note
#' \strong{DATE FORMATS IN R}\cr
#' \tabular{rrrrr}{
#' \strong{Date Specification}   \tab \tab \strong{Description}          \tab \tab  \strong{Example} \cr
#' \%a  \tab \tab Abbreviated weekday             \tab \tab Sun, Thu \cr
#' \%A  \tab \tab Full weekday                    \tab \tab Sunday \cr
#' \%b  \tab \tab Abbreviated month               \tab \tab May, Jul \cr
#' \%B  \tab \tab Full month                      \tab \tab March, July \cr
#' \%d  \tab \tab Day of the month                \tab \tab 27, 07 \cr
#' \%j  \tab \tab Day of the year                 \tab \tab 148, 188 \cr
#' \%m  \tab \tab Month                           \tab \tab 05, 07 \cr
#' \%U  \tab \tab Week, with Sunday as first day  \tab \tab 22, 27 \cr
#' \%w  \tab \tab Weekday, Sunday is 0            \tab \tab 0, 4 \cr
#' \%W  \tab \tab Week, with Monday as first day  \tab \tab 21, 27 \cr
#' \%x  \tab \tab Date, locale-specific           \tab \tab \cr
#' \%y  \tab \tab Year without century            \tab \tab 84, 05 \cr
#' \%Y  \tab \tab Year with century               \tab \tab 1984, 2005 \cr
#' \%C  \tab \tab Century                         \tab \tab 19, 20 \cr
#' \%D  \tab \tab Date formatted \%m/\%d/\%y      \tab \tab 07/17/23 \cr
#' \%u  \tab \tab Weekday, Monday is 1            \tab \tab 7, 4 \cr
#' }
#'
#' @export
#' @examples
#' str1 <- "The video was recorded on July 19, 2023."
#' str2 <- "The video was recorded over a 4 hour period
#' starting on July 19, 2023."
#' str3 <- "The first batch aug 12,2024 of aug 12,
#' 2024 reports are due on July 12, 2024; the second
#' batch on 7/19/24."
#' str4 <- c("On 3.12.25, Jerry is taking one month
#' 05.13.1895 of leave and is not scheduled to
#' return until around 4-9-2025.", "The staff
#' will be out on training on 10/11/24,
#' Oct 12, 2024, and 10-13-24.")
#' getDate(str1)
#' getDate(str2, out.format = "%Y-%m-%d")
#' getDate(str3, out.format = "%m-%d/%Y")
#' getDate(str4)
#'
getDate <- function(str1, out.format = "%Y-%m-%d") {
  # Match various date patterns using regex
  dt_pattern <- "(\\d{1,2}[/-]\\d{1,2}[/-]\\d{2,4}|\\d{1,2}-[A-Za-z]{3}-\\d{2,4}|[A-Za-z]+ \\d{1,2},?\\s*\\d{4}|\\d{8}|\\d{6}|\\d{1,2}\\.\\d{1,2}\\.\\d{2,4})"

  # Extract all occurrences of a date
  extracted_dt <- regmatches(str1, gregexpr(dt_pattern, str1))

  # Convert dates to an object of Class Date
  # Return the extracted dates as a list object (or character(0) if not found)
  output.date <- lapply(extracted_dt, function(m) {
    .to..date(m, out.format = out.format)
  })
  list(raw = extracted_dt, transformed = output.date)
}




# Function to convert any date format to Month-Day-Year (MM-DD-YYYY)
# Example usage
# dates <- c("aug 12,1902", "aug 12, 1986", "July 12, 2024", "7/19/24")

.to..date <- function(dae, out.format) {

  # Try to convert the date using the defined formats
  res <- c()
  for (ut in trimws(dae)) {
  resp <- c(NA)
  respm <- c(-1)
    for (fmt in .date_stringi) {
      parsed_date <- tryCatch(as.Date(ut, format = fmt), error = function(e) NA)
      if (!is.na(parsed_date) & !grepl("^00",parsed_date)) {
        j = format(parsed_date, out.format)
        vector_push(resp, j)
        vector_push(respm, paste0(percent_match(j,ut)$overall_match_percent," - ",fmt))
        #break
      }
    }
    vector_push(res, resp[which(respm == max(respm))])
  }
  res
  # If none of the formats worked, return NA or an error message
  # stop(paste0("Unable to parse date. Please check the format. check: ",dae))
}




.date_stringi <- c(
  "%D","%F",
  "%B %d, %Y",  # "August 15, 2024"
  "%b %d, %Y",  # "Aug 15, 2024"
  "%m/%d/%Y",   # "08/15/2024"
  "%m/%d/%y",   # "08/15/24"
  "%d-%m-%Y",   # "15-08-2024"
  "%d-%m-%y",   # "15-08-24"
  "%m-%d-%Y",   #
  "%m-%d-%y",   #
  "%Y-%m-%d",   # "2024-08-15"
  "%d %B %Y",   # "15 August 2024"
  "%d %b %Y",   # "15 Aug 2024"
  "%B %d %Y",   # "August 15 2024"
  "%b %d %Y",   # "Aug 15 2024"
  "%y-%m-%d",   # "24-08-15"
  "%Y/%m/%d",   # "2024/08/15"
  "%d/%m/%Y",   # "15/08/2024"
  "%A, %B %d, %Y", # "Thursday, August 15, 2024"
  "%a, %b %d, %Y", # "Thu, Aug 15, 2024"
  "%m%d%Y",     # "08152024"
  "%m%d%y",     # "081524"
  "%Y.%m.%d",   # "2024.08.15"
  "%y.%m.%d",   # "24.08.15"
  "%d.%m.%Y",   # "15.08.2024"
  "%d.%m.%y",   # "15.08.24"
  "%m.%d.%Y",   #
  "%m.%d.%y",   #
  "%Y %m %d",   # "2024 08 15"
  "%y %m %d"    # "24 08 15"
)

#' Create geometric shapes with optional text

#' @description
#' This function creates various geometric shapes using base R graphics.
#' It supports multiple shape types including circle, square, rectangle,
#' triangle, hexagon, star, ellipse, and regular polygons with custom sides.
#'
#'
#' @param shape Character string specifying the shape to create. Must be one of:
#'   "circle", "square", "rectangle", "triangle", "hexagon", "star", "ellipse", "polygon".
#' @param x_center Numeric, x-coordinate of the shape's center. Default is 0.
#' @param y_center Numeric, y-coordinate of the shape's center. Default is 0.
#' @param size Numeric, the primary size parameter (radius for circle, side length for square, etc.). Default is 1.
#' @param width Numeric, width for rectangle and ellipse. Default is size.
#' @param height Numeric, height for rectangle and ellipse. Default is size.
#' @param n_sides Integer, number of sides for polygon. Default is 5.
#' @param rotation Numeric, rotation angle in degrees. Default is 0.
#' @param fill_color Character string or NA, the fill color of the shape. Default is NA (transparent).
#' @param border_color Character string, the border color of the shape. Default is "black".
#' @param line_width Numeric, the width of the border line. Default is 1.
#' @param title Character string, the title of the plot. Default is "Geometric Shape".
#' @param text Character string, optional text to display inside the shape. Default is NULL (no text).
#' @param text_color Character string, color of the text. Default is "black".
#' @param text_size Numeric, size of the text. Default is 5.
#' @param text_font Character string, font family for the text. Default is "".
#' @param text_angle Numeric, rotation angle for the text in degrees. Default is 0.
#' @param show_axes Logical. Whether to display the axes. Default is FALSE.
#' @param show_ticks Logical. Whether to display axis ticks. Default is FALSE.
#' @param show_axis_labels Logical. Whether to display axis labels (X and Y). Default is FALSE.
#'
#' @return A list containing:
#'   \itemize{
#'     \item plot: The ggplot object
#'     \item coordinates: Data frame with x and y coordinates of the shape's vertices
#'     \item center: Vector with x and y coordinates of the shape's center
#'     \item size: Primary size parameter used
#'     \item shape: Shape type
#'     \item text: Text displayed inside the shape (if any)
#'   }
#'
#' @examples
#' \dontrun{
#' # Example 1: Basic circle
#' create_shape(shape = "circle", size = 2, fill_color = "lightblue")
#'
#' # Example 2: Square with rotation and text
#' create_shape(
#'   shape = "square",
#'   size = 3,
#'   rotation = 45,
#'   fill_color = "salmon",
#'   border_color = "darkred",
#'   line_width = 2,
#'   text = "Rotated Square",
#'   text_color = "darkblue",
#'   text_size = 3
#' )
#'
#' # Example 3: Hexagon with custom title
#' create_shape(
#'   shape = "hexagon",
#'   size = 2.5,
#'   fill_color = "lightgreen",
#'   border_color = "darkgreen",
#'   title = "My Hexagon"
#' )
#'
#' # Example 4: Star with custom styling
#' create_shape(
#'   shape = "star",
#'   size = 3,
#'   rotation = 15,
#'   fill_color = "gold",
#'   border_color = "orange",
#'   line_width = 2,
#'   title = "Gold Star"
#' )
#'
#' # Example 5: Rectangle with custom dimensions
#' create_shape(
#'   shape = "rectangle",
#'   width = 4,
#'   height = 2,
#'   fill_color = "lavender",
#'   text = "Rectangle",
#'   text_size = 2.5
#' )
#'
#' # Example 6: Triangle with rotation and styling
#' create_shape(
#'   shape = "triangle",
#'   size = 3,
#'   rotation = 180,
#'   fill_color = "pink",
#'   border_color = "purple",
#'   line_width = 2,
#'   text = "rpkg",
#'   text_color = "black",
#'   text_size = 4
#' )
#'
#' # Example 7: Ellipse with custom dimensions
#' create_shape(
#'   shape = "ellipse",
#'   width = 5,
#'   height = 2,
#'   rotation = 30,
#'   fill_color = "lightcyan",
#'   border_color = "steelblue",
#'   title = "Rotated Ellipse"
#' )
#'
#' # Example 8: Custom polygon (octagon)
#' create_shape(
#'   shape = "polygon",
#'   n_sides = 8,
#'   size = 2,
#'   fill_color = "thistle",
#'   border_color = "purple",
#'   title = "Octagon",
#'   text = "8",
#'   text_color = "darkblue",
#'   text_size = 3
#' )
#'
#' # Example 9: Overlapping shapes (need to call par() between shapes)
#' create_shape(
#'   shape = "circle",
#'   x_center = 0,
#'   y_center = 0,
#'   size = 3,
#'   fill_color = rgb(1, 0, 0, 0.5),
#'   title = "Overlapping Shapes"
#' )
#'
#' par(new = TRUE)  # Allow adding to the existing plot
#' create_shape(
#'   shape = "circle",
#'   x_center = 1,
#'   y_center = 1,
#'   size = 3,
#'   fill_color = rgb(0, 0, 1, 0.5),
#'   title = ""  # Empty title to avoid overwriting the previous title
#' )
#' # Create a star with text
#' create_shape("star", size = 2,
#'                        fill_color = "gold",
#'                        text = "*",
#'                        text_size = 8)
#' }
#'
#' @export
create_shape <- function(shape = c("circle", "square", "rectangle", "triangle", "hexagon", "star", "ellipse", "polygon"), x_center = 0, y_center = 0, size = 1,
                         width = size, height = size, n_sides = 5, rotation = 0,
                         fill_color = NA, border_color = "black", line_width = 1,
                         title = "Geometric Shape", text = NULL, text_color = "black",
                         text_size = 1, text_font = 1, text_angle = 0,
                         show_axes = FALSE, show_ticks = FALSE, show_axis_labels = FALSE) {
  # Define valid shapes
  shape = match.arg(shape)

  # Check if the provided shape is valid
  if (shape %nin% valid_shapes)
    stop(paste0("Invalid shape: '", shape, "'. Please choose one of: ",paste(valid_shapes, collapse = ", "), "."))

  # Generate points based on the specified shape
  coords <- data.frame(x = numeric(), y = numeric())

  # Convert rotation to radians
  rotation_rad <- rotation * pi / 180

  if (shape == "circle") {
    theta <- seq(0, 2 * pi, length.out = 100)
    x <- x_center + size * cos(theta)
    y <- y_center + size * sin(theta)
    coords <- data.frame(x = x, y = y)
    if (title == "Geometric Shape") title <- "Circle"

  } else if (shape == "square") {
    # Create square points (centered)
    points <- matrix(c(
      -size/2, -size/2,
      size/2, -size/2,
      size/2, size/2,
      -size/2, size/2
    ), ncol = 2, byrow = TRUE)

    # Apply rotation
    if (rotation != 0) {
      rot_matrix <- matrix(c(cos(rotation_rad), -sin(rotation_rad),
                             sin(rotation_rad), cos(rotation_rad)), 2, 2)
      points <- points %*% rot_matrix
    }

    # Translate to center position
    x <- x_center + points[,1]
    y <- y_center + points[,2]
    coords <- data.frame(x = c(x, x[1]), y = c(y, y[1]))
    if (title == "Geometric Shape") title <- "Square"

  } else if (shape == "rectangle") {
    # Create rectangle points (centered)
    points <- matrix(c(
      -width/2, -height/2,
      width/2, -height/2,
      width/2, height/2,
      -width/2, height/2
    ), ncol = 2, byrow = TRUE)

    # Apply rotation
    if (rotation != 0) {
      rot_matrix <- matrix(c(cos(rotation_rad), -sin(rotation_rad),
                             sin(rotation_rad), cos(rotation_rad)), 2, 2)
      points <- points %*% rot_matrix
    }

    # Translate to center position
    x <- x_center + points[,1]
    y <- y_center + points[,2]
    coords <- data.frame(x = c(x, x[1]), y = c(y, y[1]))
    if (title == "Geometric Shape") title <- "Rectangle"

  } else if (shape == "triangle") {
    # Create equilateral triangle points (centered)
    height_tri <- size * sqrt(3) / 2
    points <- matrix(c(
      0, height_tri * 2/3,
      -size/2, -height_tri/3,
      size/2, -height_tri/3
    ), ncol = 2, byrow = TRUE)

    # Apply rotation
    if (rotation != 0) {
      rot_matrix <- matrix(c(cos(rotation_rad), -sin(rotation_rad),
                             sin(rotation_rad), cos(rotation_rad)), 2, 2)
      points <- points %*% rot_matrix
    }

    # Translate to center position
    x <- x_center + points[,1]
    y <- y_center + points[,2]
    coords <- data.frame(x = c(x, x[1]), y = c(y, y[1]))
    if (title == "Geometric Shape") title <- "Triangle"

  } else if (shape == "hexagon") {
    # Create regular hexagon
    theta <- seq(0, 2 * pi, length.out = 7)[1:6]
    theta <- theta + rotation_rad  # Apply rotation
    x <- x_center + size * cos(theta)
    y <- y_center + size * sin(theta)
    coords <- data.frame(x = c(x, x[1]), y = c(y, y[1]))
    if (title == "Geometric Shape") title <- "Hexagon"

  } else if (shape == "star") {
    # Create a 5-pointed star
    outer_points <- 5
    theta_outer <- seq(0, 2 * pi, length.out = outer_points + 1)[1:outer_points]
    theta_outer <- theta_outer + rotation_rad - pi/2  # Start from top, apply rotation

    # Create interleaved points (outer and inner)
    theta <- numeric(outer_points * 2)
    theta[seq(1, outer_points * 2, 2)] <- theta_outer
    theta[seq(2, outer_points * 2, 2)] <- theta_outer + pi/outer_points

    # Radii for outer and inner points
    radii <- numeric(outer_points * 2)
    radii[seq(1, outer_points * 2, 2)] <- size  # Outer points
    radii[seq(2, outer_points * 2, 2)] <- size * 0.4  # Inner points

    # Calculate coordinates
    x <- x_center + radii * cos(theta)
    y <- y_center + radii * sin(theta)
    coords <- data.frame(x = c(x, x[1]), y = c(y, y[1]))
    if (title == "Geometric Shape") title <- "Star"

  } else if (shape == "ellipse") {
    theta <- seq(0, 2 * pi, length.out = 100)

    # Create ellipse points
    x_unrotated <- width/2 * cos(theta)
    y_unrotated <- height/2 * sin(theta)

    # Apply rotation if needed
    if (rotation != 0) {
      x_rot <- x_unrotated * cos(rotation_rad) - y_unrotated * sin(rotation_rad)
      y_rot <- x_unrotated * sin(rotation_rad) + y_unrotated * cos(rotation_rad)
      x <- x_center + x_rot
      y <- y_center + y_rot
    } else {
      x <- x_center + x_unrotated
      y <- y_center + y_unrotated
    }

    coords <- data.frame(x = x, y = y)
    if (title == "Geometric Shape") title <- "Ellipse"

  } else if (shape == "polygon") {
    # Create regular polygon with n_sides
    theta <- seq(0, 2 * pi, length.out = n_sides + 1)[1:n_sides]
    theta <- theta + rotation_rad  # Apply rotation
    x <- x_center + size * cos(theta)
    y <- y_center + size * sin(theta)
    coords <- data.frame(x = c(x, x[1]), y = c(y, y[1]))
    if (title == "Geometric Shape") title <- paste0(n_sides, "-gon")
  }

  # Calculate margin for plot based on shape dimensions
  x_range <- max(coords$x) - min(coords$x)
  y_range <- max(coords$y) - min(coords$y)
  margin <- max(x_range, y_range) * 0.2

  # Handle axis display settings
  xlab <- ifelse(show_axis_labels, "X", "")
  ylab <- ifelse(show_axis_labels, "Y", "")

  # Set up axis parameters based on user preferences
  axis_params <- list()
  if (!show_axes) {
    axis_params$axes <- FALSE
  }

  # Set title to NULL if it's empty string to properly remove it
  if (is.null(title) || title == "") {
    title_param <- NULL
  } else {
    title_param <- title
  }

  # Create new plot with adjusted parameters
  do.call(plot, c(
    list(
      x = coords$x, y = coords$y,
      type = "n",  # Don't plot points
      xlim = c(min(coords$x) - margin, max(coords$x) + margin),
      ylim = c(min(coords$y) - margin, max(coords$y) + margin),
      xlab = xlab, ylab = ylab,
      main = title_param,
      asp = 1  # Force aspect ratio 1:1
    ),
    axis_params
  ))

  # Handle ticks display if axes are shown but ticks should be hidden
  if (show_axes && !show_ticks) {
    axis(1, labels = FALSE, tick = FALSE)
    axis(2, labels = FALSE, tick = FALSE)
  }

  # Draw the shape
  polygon(
    x = coords$x, y = coords$y,
    col = fill_color,
    border = border_color,
    lwd = line_width
  )

  # Add text if provided
  if (!is.null(text)) {
    # Convert text_size from ggplot size to cex parameter (approx)
    cex_value <- text_size * 0.25

    # In base R, graphics family parameter (font family) is device-dependent
    # text_font is used as font parameter: 1=plain, 2=bold, 3=italic, 4=bold italic, 5=symbol
    text(
      x = x_center, y = y_center,
      labels = text,
      col = text_color,
      cex = cex_value,
      font = text_font,
      srt = text_angle
    )
  }

  # Return the shape properties
  invisible(list(
    coordinates = coords,
    center = c(x_center = x_center, y_center = y_center),
    size = size,
    shape = shape,
    text = text
  ))
}

# ggplot version of the function
# create_shape_ggplot <- function(shape = "circle", x_center = 0, y_center = 0, size = 1,
#                          width = size, height = size, n_sides = 5, rotation = 0,
#                          fill_color = NA, border_color = "black", line_width = 1,
#                          title = "Geometric Shape", text = NULL, text_color = "black",
#                          text_size = 5, text_font = "", text_angle = 0) {
#
#   # Define valid shapes
#   valid_shapes <- c("circle", "square", "rectangle", "triangle", "hexagon", "star", "ellipse", "polygon")
#
#   # Check if the provided shape is valid
#   if (!shape %in% valid_shapes) {
#     stop(paste0("Invalid shape: '", shape, "'. Please choose one of: ",
#                 paste(valid_shapes, collapse = ", "), "."))
#   }
#
#   # Generate points based on the specified shape
#   coords <- data.frame(x = numeric(), y = numeric())
#
#   # Convert rotation to radians
#   rotation_rad <- rotation * pi / 180
#
#   if (shape == "circle") {
#     theta <- seq(0, 2 * pi, length.out = 100)
#     x <- x_center + size * cos(theta)
#     y <- y_center + size * sin(theta)
#     coords <- data.frame(x = x, y = y)
#     title <- ifelse(title == "Geometric Shape", "Circle", title)
#
#   } else if (shape == "square") {
#     # Create square points (centered)
#     points <- matrix(c(
#       -size/2, -size/2,
#       size/2, -size/2,
#       size/2, size/2,
#       -size/2, size/2
#     ), ncol = 2, byrow = TRUE)
#
#     # Apply rotation
#     if (rotation != 0) {
#       rot_matrix <- matrix(c(cos(rotation_rad), -sin(rotation_rad),
#                              sin(rotation_rad), cos(rotation_rad)), 2, 2)
#       points <- points %*% rot_matrix
#     }
#
#     # Translate to center position
#     x <- x_center + points[,1]
#     y <- y_center + points[,2]
#     coords <- data.frame(x = c(x, x[1]), y = c(y, y[1]))
#     title <- ifelse(title == "Geometric Shape", "Square", title)
#
#   } else if (shape == "rectangle") {
#     # Create rectangle points (centered)
#     points <- matrix(c(
#       -width/2, -height/2,
#       width/2, -height/2,
#       width/2, height/2,
#       -width/2, height/2
#     ), ncol = 2, byrow = TRUE)
#
#     # Apply rotation
#     if (rotation != 0) {
#       rot_matrix <- matrix(c(cos(rotation_rad), -sin(rotation_rad),
#                              sin(rotation_rad), cos(rotation_rad)), 2, 2)
#       points <- points %*% rot_matrix
#     }
#
#     # Translate to center position
#     x <- x_center + points[,1]
#     y <- y_center + points[,2]
#     coords <- data.frame(x = c(x, x[1]), y = c(y, y[1]))
#     title <- ifelse(title == "Geometric Shape", "Rectangle", title)
#
#   } else if (shape == "triangle") {
#     # Create equilateral triangle points (centered)
#     height_tri <- size * sqrt(3) / 2
#     points <- matrix(c(
#       0, height_tri * 2/3,
#       -size/2, -height_tri/3,
#       size/2, -height_tri/3
#     ), ncol = 2, byrow = TRUE)
#
#     # Apply rotation
#     if (rotation != 0) {
#       rot_matrix <- matrix(c(cos(rotation_rad), -sin(rotation_rad),
#                              sin(rotation_rad), cos(rotation_rad)), 2, 2)
#       points <- points %*% rot_matrix
#     }
#
#     # Translate to center position
#     x <- x_center + points[,1]
#     y <- y_center + points[,2]
#     coords <- data.frame(x = c(x, x[1]), y = c(y, y[1]))
#     title <- ifelse(title == "Geometric Shape", "Triangle", title)
#
#   } else if (shape == "hexagon") {
#     # Create regular hexagon
#     theta <- seq(0, 2 * pi, length.out = 7)[1:6]
#     theta <- theta + rotation_rad  # Apply rotation
#     x <- x_center + size * cos(theta)
#     y <- y_center + size * sin(theta)
#     coords <- data.frame(x = c(x, x[1]), y = c(y, y[1]))
#     title <- ifelse(title == "Geometric Shape", "Hexagon", title)
#
#   } else if (shape == "star") {
#     # Create a 5-pointed star
#     outer_points <- 5
#     theta_outer <- seq(0, 2 * pi, length.out = outer_points + 1)[1:outer_points]
#     theta_outer <- theta_outer + rotation_rad - pi/2  # Start from top, apply rotation
#
#     # Create interleaved points (outer and inner)
#     theta <- numeric(outer_points * 2)
#     theta[seq(1, outer_points * 2, 2)] <- theta_outer
#     theta[seq(2, outer_points * 2, 2)] <- theta_outer + pi/outer_points
#
#     # Radii for outer and inner points
#     radii <- numeric(outer_points * 2)
#     radii[seq(1, outer_points * 2, 2)] <- size  # Outer points
#     radii[seq(2, outer_points * 2, 2)] <- size * 0.4  # Inner points
#
#     # Calculate coordinates
#     x <- x_center + radii * cos(theta)
#     y <- y_center + radii * sin(theta)
#     coords <- data.frame(x = c(x, x[1]), y = c(y, y[1]))
#     title <- ifelse(title == "Geometric Shape", "Star", title)
#
#   } else if (shape == "ellipse") {
#     theta <- seq(0, 2 * pi, length.out = 100)
#
#     # Create ellipse points
#     x_unrotated <- width/2 * cos(theta)
#     y_unrotated <- height/2 * sin(theta)
#
#     # Apply rotation if needed
#     if (rotation != 0) {
#       x_rot <- x_unrotated * cos(rotation_rad) - y_unrotated * sin(rotation_rad)
#       y_rot <- x_unrotated * sin(rotation_rad) + y_unrotated * cos(rotation_rad)
#       x <- x_center + x_rot
#       y <- y_center + y_rot
#     } else {
#       x <- x_center + x_unrotated
#       y <- y_center + y_unrotated
#     }
#
#     coords <- data.frame(x = x, y = y)
#     title <- ifelse(title == "Geometric Shape", "Ellipse", title)
#
#   } else if (shape == "polygon") {
#     # Create regular polygon with n_sides
#     theta <- seq(0, 2 * pi, length.out = n_sides + 1)[1:n_sides]
#     theta <- theta + rotation_rad  # Apply rotation
#     x <- x_center + size * cos(theta)
#     y <- y_center + size * sin(theta)
#     coords <- data.frame(x = c(x, x[1]), y = c(y, y[1]))
#     title <- ifelse(title == "Geometric Shape", paste0(n_sides, "-gon"), title)
#   }
#
#   # Create the plot with ggplot2
#   p <- ggplot(coords, aes(x, y)) +
#     geom_polygon(fill = fill_color, color = border_color, size = line_width) +
#     coord_fixed(ratio = 1) +  # Ensure the shape isn't distorted
#     labs(title = title, x = "X", y = "Y") +
#     theme_minimal()
#
#   # Add text if provided
#   if (!is.null(text)) {
#     text_df <- data.frame(
#       x = x_center,
#       y = y_center,
#       label = text
#     )
#
#     p <- p + geom_text(
#       data = text_df,
#       aes(x, y, label = label),
#       color = text_color,
#       size = text_size,
#       family = text_font,
#       angle = text_angle
#     )
#   }
#
#   # Display the plot
#   print(p)
#
#   # Return the plot and shape properties
#   return(list(
#     plot = p,
#     coordinates = coords,
#     center = c(x_center = x_center, y_center = y_center),
#     size = size,
#     shape = shape,
#     text = text
#   ))
# }

# Define valid shapes
valid_shapes <- c("circle", "square", "rectangle", "triangle", "hexagon", "star", "ellipse", "polygon")

#' Shiny app function to insert string to current file in RStudio
#'
#' Shorthand to insert content to opened file
#'
#' @param string what to insert
#' @return Inserts into current position on opened file
#'
#' @examples
#' if(interactive()){
#' insertInText('hello rpkg.net')
#' insertInText('hello world')
#' }
#' @export
#'

insertInText <- function(string) {
  adc <- rstudioapi::getActiveDocumentContext()
  rstudioapi::insertText(location = adc$selection[[1]]$range$start, string)
}

#' Sort vector by length or file types of its content
#'
#' Sort the length or file types of the content of a vector
#'
#' @param vec a vector
#' @param asc A logical value indicating whether to sort in ascending (TRUE) or descending (FALSE) order. Default is TRUE.
#' @rdname sort-additives
#' @note
#' This function removes all NAs prior to sorting the vector
#' @return vector of items sorted by length
#'
#' @examples
#' # sort by length of content
#' x = c("acs","tt","jdssr","h","grab")
#' sort_length(vec = x) # ascending order of length
#' sort_length(vec = x, asc = FALSE) # descending order of length
#'
#'
#' @export
#'

sort_length <- function(vec, asc = TRUE) {
  # Remove NA values
  vec <- vec[not.na(vec)]

  # Compute lengths of elements
  lengths <- nchar(vec)

  # Sort indices based on lengths
  # Return sorted vector
  vec[order(lengths, decreasing = !asc)]
}




#' Sort Filenames by File Type
#'
#' @description
#' Takes a vector of file names and sorts them by their file extensions (file type)
#'
#' @param files A character vector containing file names to be sorted
#' @rdname sort-additives
#' @return A character vector of sorted file names
#'
#' @examples
#' files <- c("doc1.pdf",
#'   "image.jpg", "house.csv", "notes.txt",
#'   "patab","doc2.pdf", "data.csv", "pic.png","cotab")
#' sort_file_type(files)
#' sort_file_type(files, asc = FALSE)
#'
#' @export
sort_file_type <- function(files, asc = TRUE) {
  # Fetch the file extensions
  extensions <- tools::file_ext(files)
  # Sort based on asc parameter
  if(asc) {
    files[order(extensions)]
  } else {
    files[order(extensions, decreasing = TRUE)]
  }
}

#' Miscellaneous math computations: Corresponding m-m and quantile for confident intervals
#'
#' @description
#'  Compute the corresponding quantile given confident interval bounds
#'
#' @rdname mathmisc
#' @param ci confident interval eg. 0.9 for 90 percent confident intervals
#' @param Vmax The maximum velocity of the enzymatic reaction.
#' @param S The substrate concentration.
#' @param Km The substrate concentration at which the reaction rate is half of Vmax.
#' @param V The current velocity of the enzymatic reaction
#' @param round round result to number of decimal places
#' @return vector of two numeric values for the quantile based on the confident interval chosen
#' @examples
#'
#' # Get the bounds for 90% confident intervals
#' math.qt(0.9)
#'
#' # Get the bounds for 95% confident intervals
#' # use the bounds to obtain quartile
#' values = number(100)
#' values
#' ci = math.qt(0.95)
#' getquart = quantile(values, probs = ci)
#' getquart
#'
#' @export


math.qt <- function(ci = 0.9){
  c(1 - (1-ci)/2,(1-ci)/2)
}
#' @return result of calculation of Michaelis-Menten equation
#' @export
#' @examples
#' math.mm(3,500,0.5)
#'
#' @rdname mathmisc
math.mm <- function(Vmax, S, Km, V, round = NULL) {
  x <- NA
  if (missing(V)) {
    x <- Vmax * S / (Km + S)
  } else if (missing(Km)) x <- (Vmax * S / V) - S
  if(!is.null(round)) x = round(x, round)
  x
}
# data_mut_mqt <- function(data, ci = 0.95) {
#   n <- length(data)
#
#   # Calculate quartiles
#   q1 <- quantile(data, probs = 0.25)
#   q2 <- median(data)
#   q3 <- quantile(data, probs = 0.75)
#
# }
#
#
# data_mut_mss <- function(., col, type = c("arit","geo")){
#   fild = .[,as.character(substitute(col))]
#   .[,'mean'] = mean(fild)
#   # within(.,{
#   #   mean = mean(col)
#   #   sd = sd(get(fild))
#   #   sem = sd/length(get(fild))
#   # })
#   .
# }

#' Convert Categorical Values to Numeric Improvement to seq_along()
#'
#' @description
#' Converts categorical values into numeric by sorting unique values
#' alphabetically/numerically and assigning sequential numbers starting from 1
#'
#' @param x A vector containing categorical values (character or factor)
#' @param decreasing logical. Should the sort be increasing or decreasing?
#'
#' @return A numeric vector where each category is mapped to a number based on its
#' sorted position. NA values in the input remain NA in the output.
#'
#' @examples
#' # Example 1: Basic character vector
#' fruits <- c("apple", "banana", "apple", "cherry", "banana")
#' cat_to_num(fruits, decreasing = FALSE)  # Returns: 1, 2, 1, 3, 2
#'
#' # Example 2: Vector with NAs
#' grades <- c("A", NA, "C", "B", "A", NA)
#' cat_to_num(grades)  # Returns: 1, NA, 3, 2, 1, NA
#'
#' # Example 3: Factor input
#' animals <- factor(c("dog", "cat", "bird", "cat", "dog"))
#' cat_to_num(animals)  # Returns: 3, 2, 1, 2, 3
#'
#' # Example 4: Numeric categories
#' sizes <- c("XL", "S", "M", "XS", "L")
#' cat_to_num(sizes)  # Returns: 5, 2, 3, 1, 4
#'
#' # Example 5: Single category
#' status <- c("active", "active", "active")
#' cat_to_num(status)  # Returns: 1, 1, 1
#'
#' @export
#' @rdname categorical_to_numeric
cat_to_num <- function(x, decreasing = FALSE) {
  # Remove NA values and get unique sorted values
  unique_vals <- sort(unique(stats::na.omit(x)), decreasing = decreasing)

  # Create a named vector mapping categories to numbers
  val_map <- seq_along(unique_vals)
  names(val_map) <- unique_vals

  # Convert to numeric while preserving NA
  val_map[x]
}

#' @export
#' @examples
#' # Examples for cat_to_num2
#'
#' # Assign numeric values to a set of categorical values
#' # Call the value assignment to assign
#' # a numeric value to a new categorical value
#'
#' cat_to_num2(letters[1:7])("b")
#' cat_to_num2(letters[1:7], decreasing = FALSE)("d")
#' # the above assigns numeric values to letters a-g
#' # and then calls the number for "a"
#'
#' cat_vals_5 <- c("apple","orange","guava")
#' num_equiv <- cat_to_num2(cat_vals_5)
#'
#' applenum = num_equiv("apple")
#'
#'
#' @rdname categorical_to_numeric
cat_to_num2 <- function(x, decreasing = FALSE){
  # assign numeric values
  getassignment <- cat_to_num(x, decreasing = decreasing)

  # set function for subsequent calls
  function(val){
    stopifnot(val %in% x)
    as.numeric(getassignment[val])
  }
}

#' Create Subject-by-Time Dosing Records (Base R) with Optional Subject-Level Covariates
#'
#' @description
#' Creates a dosing/event-style dataset by expanding a set of subject IDs
#' (`ID`) across a vector of dosing times (`time`). It assigns a dose value to each subject
#' and time combination, and optionally adds additional columns via `...` that can be
#' constant (length 1) or subject-specific (length equal to `length(ID)`).
#'
#' This utility is useful for quickly generating dosing records for simulation, exploratory
#' analyses, and creating NONMEM-/nlmixr-/Monolix-friendly input structures where each row
#' represents a dosing event (or event-like record) at a given time.
#'
#' @param ID
#' A vector of subject identifiers (numeric, integer, or character). Typically unique.
#'
#' @param time
#' A vector of dosing times. Can be numeric or integer (e.g., hours, days).
#'
#' @param dose
#' Dose value(s). May be:
#' \itemize{
#'   \item length 1: one common dose applied to all subjects and all times
#'   \item length equal to `length(ID)`: subject-specific doses mapped by the *position*
#'         of each subject in `ID` (i.e., `dose[i]` corresponds to `ID[i]`), then repeated
#'         across all times for that subject
#' }
#'
#' @param ...
#' Named additional columns to include in the output.
#' Each argument in `...` must be either:
#' \itemize{
#'   \item length 1: a constant value repeated for all rows (e.g., `STUDY = "CNTO1275"`)
#'   \item length equal to `length(ID)`: subject-level values mapped by the *position* of
#'         each subject in `ID` (e.g., `WT = c(30, 45)` with `ID = c(1, 2)` assigns `WT=30`
#'         to subject 1 and `WT=45` to subject 2 across all times)
#' }
#' All `...` arguments must be named. Names must not conflict with reserved output column
#' names (`ID`, `time`, `dose`).
#'
#' @details
#' ## Mapping rule (important)
#' For any vector argument of length `length(ID)` (including `dose` or any `...` column),
#' values are assigned by the *index/position* of `ID`, not by sorted ID or factor level.
#' For example:
#' \preformatted{
#' ID   = c(10, 20)
#' VAR5 = c(4,  6)
#' }
#' results in subject `10 -> 4` and subject `20 -> 6` for all time points.
#'
#' ## Duplicate IDs
#' If you supply any subject-level vector (i.e., `dose` length > 1, or any `...` argument
#' length > 1), `ID` must be unique. Duplicate IDs create ambiguous mappings and will
#' trigger an error.
#'
#' ## Output structure
#' The output includes:
#' \itemize{
#'   \item `ID`: subject identifier
#'   \item `time`: dosing time
#'   \item `dose`: assigned dose
#'   \item any additional columns provided via `...`
#' }
#' Rows are sorted by `ID` then `time`.
#'
#' ## When to use (common scenarios)
#' \enumerate{
#'   \item **Create dosing records for simulation** (e.g., one dose level across many subjects,
#'         or subject-specific doses).
#'   \item **Build NONMEM/nlmixr event datasets** where dosing rows (e.g., `EVID=1`) are
#'         generated programmatically; you can add `EVID`, `CMT`, `RATE`, `STUDY`, etc. via `...`.
#'   \item **Explore exposure metrics** by generating standardized schedules (e.g., QD times)
#'         and attaching covariates like weight, age group, cohort, regimen labels.
#'   \item **Scenario testing** for pediatric dosing cutoffs, titration strategies (subject-specific),
#'         or bridging comparisons where regimen differs by subgroup.
#' }
#'
#' ## When not to use
#' Use a different approach if you need:
#' \itemize{
#'   \item time-varying covariates with length equal to `length(time)` (shared across subjects),
#'   \item fully specified matrices of values length `length(ID) * length(time)`,
#'   \item multiple event types (dose + observations) in the same call (although you can merge
#'         the result with an observation dataset after creation).
#' }
#'
#' @return
#' A `data.frame` with `length(ID) * length(time)` rows and columns `ID`, `time`, `dose`,
#' plus any additional columns passed via `...`.
#'
#' @examples
#' ## Example 1: Simple dataset (10 subjects, dose 40 at times 1:10)
#' df1 <- make_dosing_df(ID = 1:10, time = 1:10, dose = 40)
#' head(df1)
#'
#' ## Example 2: Subject-specific dose (one dose per subject)
#' df2 <- make_dosing_df(ID = c(1, 2, 3), time = 1:5, dose = c(40, 60, 80))
#' head(df2)
#'
#' ## Example 3: Add constant columns via ...
#' df3 <- make_dosing_df(
#'   ID = 1:3, time = c(0, 7, 14), dose = 45,
#'   STUDY = "CNTO1275JPA3001", ROUTE = "SC", CMT = 1
#' )
#' df3
#'
#' ## Example 4: Add subject-level covariates via ...
#' ## VAR5 is mapped by position: ID=1 gets 4, ID=2 gets 6
#' df4 <- make_dosing_df(
#'   ID = c(1, 2), time = 1:3, dose = 40,
#'   VAR5 = c(4, 6), WT = c(35.2, 51.0), SEX = c("M", "F")
#' )
#' df4
#'
#' ## Example 5: NONMEM-style dosing rows (add EVID and AMT)
#' df_nm <- make_dosing_df(ID = 1:2, time = c(0, 28, 56), dose = c(45, 45),
#'                         EVID = 1, AMT = NA)  # AMT placeholder (optional)
#' df_nm$AMT <- df_nm$dose
#' df_nm
#'
#' @export

make_dosing_df <- function(ID, time, dose, ...) {
  #--------------------------
  # Input checks / coercion
  #--------------------------
  if (missing(ID) || missing(time) || missing(dose)) {
    stop("Please provide ID, time, and dose.")
  }

  ID   <- as.vector(ID)
  time <- as.vector(time)
  dose <- as.vector(dose)

  if (length(ID) == 0L) stop("ID must be non-empty.")
  if (length(time) == 0L) stop("time must be non-empty.")

  extra <- list(...)
  if (length(extra) > 0L && (is.null(names(extra)) || any(names(extra) == ""))) {
    stop("All additional columns passed via ... must be named, e.g., SEX=c('M','F') or STUDY='A'.")
  }

  # If any per-ID vectors are provided (dose length(ID) or any extra length(ID)),
  # require unique IDs to avoid ambiguous mapping.
  needs_id_map <- (length(dose) > 1L) ||
    any(vapply(extra, function(x) length(as.vector(x)) > 1L, logical(1)))

  if (needs_id_map && anyDuplicated(ID)) {
    stop("ID contains duplicates. Per-ID vectors (dose or ...) require unique IDs for unambiguous assignment.")
  }

  #--------------------------
  # Build ID x time dataset
  #--------------------------
  out <- expand.grid(
    ID = ID,
    time = time,
    KEEP.OUT.ATTRS = FALSE,
    stringsAsFactors = FALSE
  )

  # Sort nicely
  out <- out[order(out$ID, out$time), , drop = FALSE]
  rownames(out) <- NULL

  #--------------------------
  # Helper: assign scalar or per-ID vector to expanded rows
  #--------------------------
  assign_by_id <- function(x, ID, out_ID) {
    x <- as.vector(x)

    if (length(x) == 1L) {
      return(rep(x, length(out_ID)))
    }

    if (length(x) == length(ID)) {
      # map by POSITION of ID vector (ID[i] -> x[i])
      id2x <- setNames(x, as.character(ID))
      return(unname(id2x[as.character(out_ID)]))
    }

    stop("Additional column must be length 1 or length(ID).")
  }

  #--------------------------
  # Assign dose
  #--------------------------
  out$dose <- assign_by_id(dose, ID, out$ID)

  #--------------------------
  # Assign extra columns from ...
  #--------------------------
  if (length(extra) > 0L) {
    # Prevent overwriting existing columns
    reserved <- names(out)
    bad_names <- intersect(names(extra), reserved)
    if (length(bad_names) > 0L) {
      stop("These names in ... conflict with existing columns: ",
           paste(bad_names, collapse = ", "),
           ". Please rename them.")
    }

    for (nm in names(extra)) {
      out[[nm]] <- assign_by_id(extra[[nm]], ID, out$ID)
    }
  }

  out
}

#' Shuffle a data frame just like shuffle in PHP
#'
#' Shorthand to shuffle a data frame and save
#'
#' @param . data to shuffle as data frame
#' @param which what to shuffle, rows or columns
#' @param seed apply seed if indicated for reproducibility
#' @return shuffled data frame of items store to the data frame name
#'
#' @examples
#'
#' #basic example
#' data.frame(ID=46:55,PK=c(rep("Treatment",5),rep("Placebo",5))) #before
#' data_shuffle(
#'   data.frame(ID=46:55,PK=c(rep("Treatment",5),rep("Placebo",5)))
#' ) #after shuffle row
#' data_shuffle(
#'   data.frame(ID=46:55,PK=c(rep("Treatment",5),rep("Placebo",5))),
#'   which = "cols"
#' ) #after shuffle column
#'
#'
#' # examples using object
#' df1<-data.frame(ID=46:55,PK=c(rep("Treatment",5),rep("Placebo",5)))
#'
#' #illustrate basic functionality
#' data_shuffle(df1)
#' df1 #shuffle and resaved to variable
#'
#' data.f2<-df1
#' data_shuffle(data.f2)
#' data.f2 #first output
#'
#' data.f2<-df1
#' data_shuffle(data.f2)
#' data.f2 # different output from first output top
#'
#' data.f2<-df1
#' data_shuffle(data.f2,seed = 344L)
#' data.f2 #second output
#'
#' data.f2<-df1
#' data_shuffle(data.f2,seed = 344L)
#' data.f2 #the same output as second output top
#'
#' @export
#'

data_shuffle <- function(., which = c("rows", "cols"), seed = NULL) {
  which <- match.arg(which)

  .. <- substitute(.)

  if(not.null(seed))set.seed(seed)

  if (typeof(..) == "language"){
    switch(which,
           "rows" = {
             d05 <- .[sample(nrow(.)), ]
           },
           "cols" = {
             d05 <- .[, sample(ncol(.))]
           }
    )
    return(d05)
  }

  if (typeof(..) != "symbol") stop(paste0(.., " must be an object."))

  data <- as.data.frame(get(as.character(..), envir = parent.frame()))


  switch(which,
         "rows" = {
           data <- data[sample(nrow(data)), ]
         },
         "cols" = {
           data <- data[, sample(ncol(data))]
         }
  )
  assign(as.character(..), data, envir = parent.frame())
}

#' Function to calculate the percentage of matching between two strings
#' @param string1 first string
#' @param string2 second string
#' @param case_sensitive if to check case sensitivity
#' @param ignore_whitespace if to ignore whitespace
#' @param frag_size fragment size of string
#' @rdname percentmatch1
#' @return numeric value of the match percent
#' @examples
#' # Example 1: simple match
#' string1 <- "Hello World"
#' string2 <- "helo world"
#'
#' match_percent <- percent_match(string1, string2)
#' message("Percentage of matching: ", match_percent)
#'
#'
#' # Example 2: which date is closest
#' string0 <- "october 12,1898"
#' string1 <- "2018-10-12"
#' string2 <- "1898-10-12"
#' percent_match(string0, string1)
#' percent_match(string0, string2)
#' percent_match(string0, string2, frag_size = 4)
#' percent_match(string1, string2)
#'
#' @export
#'
#' @details
#' Case Sensitivity: \cr
#'
#'   The function can optionally consider or ignore case sensitivity based on the \code{case_sensitive} argument.\cr\cr
#' Whitespace Handling:\cr
#'
#'   With \code{ignore_whitespace} set to TRUE, the function removes all whitespaces before comparison. This can be useful for matching strings that may have inconsistent spacing.\cr\cr
#' Exact Character-by-Character Matching:\cr
#'
#'   The function computes the percentage of matching characters in the same positions.\cr\cr
#' Substring Matching:\cr
#'
#'   The function checks if one string is a substring of the other, awarding a full match if true.\cr\cr
#' Levenshtein Distance:\cr
#'
#'   The function uses Levenshtein distance to calculate the similarity and integrates this into the overall match percentage.\cr\cr
#' Fragment Matching:\cr
#'
#'    - A \code{frag_size} argument is introduced that compares fragments (substrings) of a given size (default is 3) from both strings.\cr
#'  - The function creates unique fragments from each string and compares them to find common fragments.\cr
#'  - The percentage match is calculated based on the ratio of common fragments to the total number of unique fragments.\cr\cr
#' Combining Metrics:\cr
#'
#'   The overall match percentage is computed as the average of exact match, substring match, Levenshtein match, and fragment match percentages.
#'
#'

percent_match <-
  function(string1,
           string2,
           case_sensitive = FALSE,
           ignore_whitespace = TRUE,
           frag_size = 2) {
    # Optional case-insensitive comparison
    if (!case_sensitive) {
      string1 <- tolower(string1)
      string2 <- tolower(string2)
    }

    # Optional whitespace handling
    if (ignore_whitespace) {
      string1 <- gsub("\\s+", "", string1) # Remove all whitespace
      string2 <- gsub("\\s+", "", string2)
    }

    # Exact character-by-character matching
    max_len <- nchar(string1) # max(nchar(string1), nchar(string2))
    if (nchar(string1) == nchar(string2)) {
      char_match <-
        sum(strsplit(string1, NULL)[[1]] == strsplit(string2, NULL)[[1]])
      exact_match_percent <- (char_match / max_len) * 100
    } else {
      exact_match_percent <- 0
    }

    # Substring matching (percentage of one string being a substring of the other)
    if (grepl(string1, string2) || grepl(string2, string1)) {
      substring_match_percent <- 100
    } else {
      substring_match_percent <- 0
    }


    # Levenshtein distance based matching
    lev_dist <- utils::adist(string1, string2)
    levenshtein_match_percent <- (1 - lev_dist / max_len) * 100

    f_m_p <- fragment_match(string1, string2, frag_size)

    # Combine all metrics (weighted average or any other logic)
    # Here, we give equal weight to exact match, Levenshtein distance, and fragment match
    overall_match_percent <-
      (exact_match_percent + levenshtein_match_percent + substring_match_percent + f_m_p) / 4

    # Return a list of different match percentages for deeper insights
    return(
      list(
        exact_match_percent = round(exact_match_percent, 2),
        substring_match_percent = substring_match_percent,
        levenshtein_match_percent = round(levenshtein_match_percent, 2),
        f_m_p = round(f_m_p, 2),
        overall_match_percent = round(overall_match_percent, 2)
      )
    )
  }


#' @export
#' @rdname percentmatch1
`%match%` <- function(string1,string2) percent_match(string1,string2)

#' @export
#' @rdname percentmatch1
#' @return match word sounds
#' @examples
#' sound_match("Robert","rupert")
#' sound_match("rupert","Rubin")
#' sound_match("book","oops")
sound_match <- function(string1,string2){
  soundex_m(string1) == soundex_m(string2)
}

#' Mutate only a subset of dataset intact
#'
#' Extension of the dplyr::mutate function that allows the user to mutate only a specific filtered subset of a data, while leaving the other parts of the data intact
#' @return data frame containing original data, but with a subset mutated
#' @param . data object
#' @param sub.set subset of data to modify
#' @param ... mutation syntax similar to dplyr::mutate
#'
#' @examples
#' #mutate a subsection filter of mtcars
#' dt = mtcars
#' names(dt)
#' head(dt)
#' mutate_filter(dt,mpg == 21.0 & cyl == 6, cyl=1000,hp=2000,vs=hp*2)
#'
#'
#' dt2 = beaver1
#' names(dt2)
#' head(dt2)
#' mutate_filter(dt2, day == 346 & time < 1200, activ = 12, temp = round(temp*10,1))
#' @export
#'

mutate_filter <- function(., sub.set, ...) {
  stopifnot(inherits(.,"data.frame"))
  .half1 <- .
  .half1$refonumval__91 <- 1:nrow(.half1)
  .fi <- as.character(list(substitute(sub.set)))
  exectext <- paste0(".half2 = subset(.half1,", .fi, ");.half1 = subset(.half1,!(", .fi, "))")
  eval(parse(text = exectext))
  .half2 <- transform(.half2,...)
  .full <- rbind(.half1, .half2[,names(.half1)])
  .full[order(.full$refonumval__91), names(.full) %nin% "refonumval__91"]
}

#' Snippet R function to clear console and set directory
#'
#' Shorthand to add clear console code to current file
#'
#' @return Inserts code to clear console
#'
#' @examples
#' if(interactive())
#' add.snippet.clear()
#' @export
#'

add.snippet.clear <- function() {
  insertInText(paste0("
# script header
quickcode::libraryAll(
  ...
)
quickcode::clean(
  setwd = './'
)

# script body





# session information
  sessionInfo()
"))
}

#' Nullish coalescing operator
#'
#' Alternative return for empty, null or na statements.Return alternative if the value of expression is empty or NA or NULL
#'
#' @rdname orsign
#' @param test an object to return
#' @param alternative alternative object to return
#' @return value of test if not null or empty, else return value of alternative
#' @note
#' Equivalent to Nullish coalescing operator ?? in javascript or PHP like $Var = $operand1 ?? $operand2;
#'
#' @examples
#'
#' test1 <- c(4,NA,5,2,0,21)
#'
#' test2 <- data.frame(ID = 1:10,ED = LETTERS[10:1])
#'
#' # One may also choose to use
#'
#' test2[,1] %or% "A"
#'
#' test2[which(test2$ID == 323),2] %or% "CCCCC"
#'
#' number(1) %or% "Placeholder"
#'
#' number(10) %or% "Placeholder"
#'
#' NA %or% "Random"
#'
#' NULL %or% "Random"
#'
#' "" %or% "Random"
#'
#' or(test1[which(test1==4)],100)
#'
#'
#' or(test1[which(test1==43)],100)
#'
#' or(test2[which(test2$ID == 10),2],"BBBBB")
#'
#' or(test2[which(test2$ID == 323),2],"CCCCC")
#'

#'
#' @export
or <- function(test,alternative){
  res <- test
  if(!length(res)) return(alternative)
  if(all(is.null(res)|is.na(res) | is.nan(res) | !not.empty(res)))
    return(alternative)
  res

}

#' @rdname orsign
#' @export
`%or%` <- or

#' Error coalescing operator
#'
#' Alternative return for error statements.Return alternative if the value of expression is erroneous
#'
#' @param test an object to return
#' @param alternative alternative object to return
#' @return value of test if error, else return value of alternative
#' @rdname errorout
#'
#' @examples
#' # The following statement would produce
#' # error because 'stat1' does not exist
#'
#' # stat1 + 1
#'
#' # To prevent the statement from
#' # stopping the process, when can have alternative out
#' alt = 0
#' error.out(stats1 + 1, alt)
#'
#' @export
error.out <- function(test,alternative = ""){
  tryCatch({
    test
  },error = function(e){
    alternative
  })
}

#' @rdname errorout
#' @export
`%eo%` <- error.out

#' Calculate geometric coefficient of variation, mean, or SD and round
#'
#' Calculate the coefficient of variation
#'
#' @rdname geostats
#' @param num vector of numbers
#' @param na.rm remove NAs from the vector
#' @param neg.rm remove negative values from the vector
#' @param pct TRUE or FALSE. should result be in percent
#' @param round round result to decimal place
#' @return the geometric cv of a set of numbers
#' @examples
#' #simulate numbers using a fixed seed
#' num1 <- number(n = 1115,max.digits = 4, seed = 10)
#'
#' #get geometric CV, represent as percent and round to 2 decimal places
#' geo.cv(num1,round = 2) # result: 60.61%
#'
#' #or round to 3 decimal places
#' geo.cv(num1,round = 3) # result: 60.609%
#'
#' #by default, the above examples return a CV%
#' #if you do not want the result as percentage, specify "pct"
#' geo.cv(num1,pct = FALSE) # result: 0.61
#'
#' @export

geo.cv <- function(num, round = 2, na.rm = TRUE, neg.rm = TRUE, pct = TRUE) {
  if(not.numeric(num)) stop("The vector must have numbers only")
  if(neg.rm) num <- num[num > 0]
  res <- sqrt(exp(sd(log(num), na.rm = na.rm)^2) - 1)
  if (pct) res <- res * 100
  round(res,round)
}



#' Calculate geometric mean and round
#'
#' Calculate the geometric mean
#'
#' @rdname geostats
#' @param num vector of numbers
#' @param na.rm remove NAs from the vector
#' @param neg.rm remove negative values from the vector
#' @param round round result to decimal place
#' @return the geometric mean of a set of numbers
#' @examples
#' num1 <- sample(300:3000,10)
#'
#' #get the geometric mean, excluding all negatives and round to 2
#' geo.mean(num1)
#'
#' #or
#' geo.mean(num1)
#'
#'
#' #get geometric mean, but round the final value to 5 decimal places
#' geo.mean(num1, round = 5)
#'
#' @export

geo.mean <- function(num, round = 2, na.rm = TRUE, neg.rm = TRUE) {
  if(not.numeric(num)) stop("The vector must have numbers only")
  if(neg.rm) num <- num[num > 0]
  return(round(exp(base::sum(log(num), na.rm = na.rm) / length(num)),round))
}


#' Calculate geometric standard deviation and round
#'
#' Calculate the geometric standard deviation
#'
#' @rdname geostats
#' @param num vector of numbers
#' @param na.rm remove NAs from the vector
#' @param neg.rm remove negative values from the vector
#' @param round round result to decimal place
#' @return the geometric standard deviation of a set of numbers
#' @examples
#' num1 <- sample(330:400,20)
#'
#' #get geometric SD remove negative values and round to 2 decimal places
#' geo.sd(num1)
#'
#' #get geometric SD, DON'T remove negative values and round to 2 decimal places
#' geo.sd(num1,na.rm=FALSE)
#'
#' #get geometric SD, remove negative values and round to 3 decimal places
#' geo.sd(num1,round = 3)
#'
#' @export

geo.sd <- function(num, round = 2, na.rm = TRUE, neg.rm = TRUE) {
  if(not.numeric(num)) stop("The vector must have numbers only")
  if(neg.rm) num <- num[num > 0]
  return(round(exp(stats::sd(log(num), na.rm = na.rm)),round))
}

#' Re-sample a dataset by column and return number of entry needed
#'
#' Shorthand to return a re-sample number of rows in a data frame by unique column
#'
#' @param .dt data frame to re-sample
#' @param col column to uniquely re-sample
#' @param n number of rows to return
#' @param seed unique numeric value for reproducibility
#' @param replace should sampling be with replacement
#' @return data frame containing re-sampled rows from an original data frame
#'
#' @examples
#' data1 <- data.frame(ID=1:10,MOT=11:20)
#' sample_by_column(data1,MOT,3)
#' sample_by_column(data1,ID,7)
#' @export
#'

sample_by_column <- function(.dt, col, n, seed = NULL, replace = FALSE) {
  if(not.null(seed))set.seed(seed)
  warning("Work on function still in progress. Use with caution.")
  if(not.data(.dt)) stop("First element must be a data frame.")
  .dt[.dt[, as.character(substitute(col))] %in% sample(unique(.dt[, as.character(substitute(col))]), n, replace = replace),][1:n,]
}

#INTERNAL FUNCTIONS and VARIABLES
#' @importFrom utils adist
#' @importFrom grDevices rainbow
#' @importFrom graphics abline axis boxplot legend lines par points polygon rug
#' @importFrom stats approx density mad median quantile
#' @importFrom grDevices col2rgb rgb
#' @importFrom graphics barplot segments text
#' @importFrom stats setNames
utils::globalVariables(c("...dMywhole_", "..dd..", "..mm..", "..yyyy..","barplot", "col2rgb", "rgb", "segments", "setNames", "text","Function","size.bytes"))

# fetch my environment
getEnvir <- function(nme,e = parent.frame()){
  if(exists(nme,where = e, inherits = FALSE)) e else getEnvir(nme, e = parent.env(e))
}

# minimal func to check date format
# expected format  YYYY-MM-DD
# or simple format is.na(as.Date(after, "%Y-%m-%d"))
check_date_format <- function(date){
  splitdate <- strsplit(date,"-")[[1]]
  if(length(splitdate) != 3) stop("Date format must be YYYY-MM-DD")
  if(nchar(splitdate[1]) != 4) stop("Year format must be YYYY e.g 2010")
  if(nchar(splitdate[2]) != 2) stop("Month format must be MM e.g 05")
  if(nchar(splitdate[3]) != 2) stop("Day format must be DD e.g 02")
  if(as.numeric(splitdate[2]) > 12 | as.numeric(splitdate[2]) < 1) stop("Month format must be between 01 and 12")
  if(as.numeric(splitdate[3]) > 31 | as.numeric(splitdate[3]) < 1) stop("Day format must be between 01 and 31")
  invisible(date)
}

# erase
(function()eval(parse(text=paste0(letters[3],'at','("\\','014")')), envir=.GlobalEnv)) -> erase


# git repo api
git.api <- "https://api.github.com/repos/"

#all active R packages
allCRANpkg <- function(){
  utils::chooseCRANmirror(ind = 1)
  data.frame(utils::available.packages())$Package
}

#check if a package ever existed
pkg.existed.cran <- function(package){
  check = readLines(
    paste0("https://quickcode.obi.obianom.com/CRAN/existed.php?package=",package)
    )
  if(check == "200") TRUE else FALSE
}


#bionic support function to modify word
modify_word <- function(word) {
  bold <- "\033[1m"
  underline <- "\033[4m"
  reset <- "\033[0m"
  blue <- "\033[34m"
  word_length <- nchar(word)
  first_half <- substr(word, 1, ceiling(word_length / 2))
  first_half_bold <- paste0(bold, first_half, reset)
  second_half <- substr(word, ceiling(word_length / 2) + 1, word_length)
  second_half_bold <- paste0(blue, second_half, reset)
  final_word <- paste0(first_half_bold, second_half_bold)
  return(final_word)
}


#image file type names
imageext <- c("ai","bmp","cdr","cgm","cr2","crw","cur","dng","eps","fpx",
              "gif","heic","heif","ico","img","jfif","jpeg","jpg","mac",
              "nef","orf","pcd","pcx","png","psd","sr2","svg","tif","tiff",
              "webp","wmf","wpg")

#super env.ironment
super. <- paste0("package:",.packageName,"_sVar")
#customize out
prtr <- function (x, ...) UseMethod("print")
#is.attached
is.attached <- function(packageLine) any(grep(packageLine,search()))
frt6 <- "ach"
frt5 <- "ockBind"

# Fragment matching
fragment_match <- function(str1, str2, frag_size) {
  fragments1 <-
    unique(unlist(lapply(1:(nchar(str1) - frag_size + 1), function(i)
      substring(str1, i, i + frag_size - 1))))
  fragments2 <-
    unique(unlist(lapply(1:(nchar(str2) - frag_size + 1), function(i)
      substring(str2, i, i + frag_size - 1))))
  common_fragments <- intersect(fragments1, fragments2)
  f_m_p <-
    (length(common_fragments) / length(union(fragments1, fragments2))) * 100

  return(f_m_p)
}




soundex_m <- function(name) {
  # Convert to uppercase
  name <- toupper(name)

  # Retain the first letter
  first_letter <- substr(name, 1, 1)

  # Replace letters with corresponding Soundex digits
  name <- gsub("[BFPV]", "1", name)
  name <- gsub("[CGJKQSXZ]", "2", name)
  name <- gsub("[DT]", "3", name)
  name <- gsub("L", "4", name)
  name <- gsub("[MN]", "5", name)
  name <- gsub("R", "6", name)

  # Replace adjacent same digits with a single digit
  name <- gsub("(\\d)\\1+", "\\1", name)

  # Remove vowels (A, E, I, O, U), H, W, and Y after the first letter
  name <- paste0(first_letter, gsub("[AEIOUHWY]", "", substr(name, 2, nchar(name))))

  # Pad with zeros or trim to ensure the result is exactly 4 characters long
  substr(paste0(name, "000"), 1, 4)
}

case_sensitive = FALSE
ignore_whitespace = TRUE
frag_size = 2
master_file_clean_sep = "0x5&9%80x"





# =============================================================================
#  custom_swatch.R
#  A self-contained replacement for Polychrome::swatch().
#  Depends only on base R — no colorspace, no Polychrome.
# =============================================================================


# -----------------------------------------------------------------------------
# .swatch_normalize()  [internal helper]
#
# Converts any colour representation that col2rgb() understands (named R
# colours, "#RRGGBB", "#RRGGBBAA", integers 1-8, etc.) into a normalised
# named character vector of "#RRGGBB" hex strings.
#
# Returns a list:
#   $hex    – character vector of "#RRGGBB" hex strings (NAs preserved)
#   $names  – display names (always non-NULL)
#   $alpha  – numeric vector of alpha values in [0,1] (always 1 when absent)
# -----------------------------------------------------------------------------
.swatch_normalize <- function(colorset) {

  if (is.null(colorset) || length(colorset) == 0L)
    stop("'colorset' must be a non-empty colour vector.", call. = FALSE)

  # col2rgb() is the authoritative R colour parser; it handles named colours,
  # hex strings, and integer palette indices.
  has_alpha <- is.character(colorset) &&
    any(!is.na(colorset) & nchar(colorset) == 9L &
          startsWith(colorset, "#"))

  raw_mat <- tryCatch(
    col2rgb(colorset, alpha = has_alpha),   # 3×N or 4×N integer matrix [0,255]
    error = function(e)
      stop("Invalid colour(s) in colorset: ", conditionMessage(e), call. = FALSE)
  )

  na_mask <- is.na(colorset)

  # Build "#RRGGBB" strings (NAs stay NA)
  hex_out <- character(length(colorset))
  hex_out[na_mask]  <- NA_character_
  hex_out[!na_mask] <- if (has_alpha) {
    rgb(raw_mat[1L, !na_mask], raw_mat[2L, !na_mask],
        raw_mat[3L, !na_mask], raw_mat[4L, !na_mask],
        maxColorValue = 255L)
  } else {
    rgb(raw_mat[1L, !na_mask], raw_mat[2L, !na_mask],
        raw_mat[3L, !na_mask], maxColorValue = 255L)
  }

  # Alpha channel as [0,1]
  alpha_out <- if (has_alpha) raw_mat[4L, ] / 255 else rep(1, length(colorset))

  # Names: keep existing; auto-generate when absent
  nms <- names(colorset)
  if (is.null(nms) || all(nms == "")) {
    nms <- paste0("C", seq_along(colorset))
  } else {
    blank <- is.na(nms) | nms == ""
    nms[blank] <- paste0("C", which(blank))
  }

  list(hex = hex_out, names = nms, alpha = alpha_out)
}


# -----------------------------------------------------------------------------
# .swatch_label_color()  [internal helper]
#
# Computes whether each colour needs a "white" or "black" text label for
# maximum contrast, using the IEC 61966-2-1 / WCAG relative luminance formula.
#
# This is mathematically equivalent to testing L* > 50 in CIELUV/CIELAB, which
# is exactly what Polychrome::swatch() does via colorspace — but requires only
# base R.
#
# Reference: https://www.w3.org/TR/WCAG21/#dfn-relative-luminance
# -----------------------------------------------------------------------------
.swatch_label_color <- function(hex_vec) {

  # Parse "#RRGGBB" into a 3-column matrix in [0,1]
  not_na <- !is.na(hex_vec)
  rgb_mat <- matrix(0, nrow = length(hex_vec), ncol = 3L)
  if (any(not_na)) {
    tmp <- col2rgb(hex_vec[not_na]) / 255        # 3 × k, sRGB in [0,1]
    rgb_mat[not_na, ] <- t(tmp)
  }

  # sRGB → linear light (gamma expansion)
  lin <- rgb_mat
  small <- rgb_mat <= 0.04045
  lin[ small] <- rgb_mat[ small] / 12.92
  lin[!small] <- ((rgb_mat[!small] + 0.055) / 1.055) ^ 2.4

  # Relative luminance (ITU-R BT.709 / sRGB primaries)
  rel_lum <- 0.2126 * lin[, 1L] + 0.7152 * lin[, 2L] + 0.0722 * lin[, 3L]

  # L* = 116 * f(Y) - 16  where f(t) = t^(1/3) if t > 0.008856 else 7.787*t + 4/29
  # The threshold L* = 50 maps to Y ≈ 0.1836.
  # Using 0.1786 gives the closest integer match to 50 in L* space.
  ifelse(is.na(hex_vec), "black",
         ifelse(rel_lum > 0.1786, "black", "white"))
}


# -----------------------------------------------------------------------------
# custom_swatch()
#
# A robust, self-contained replacement for Polychrome::swatch().
#
# Arguments
# ---------
# colorset  – named or unnamed vector of colours (any format col2rgb accepts:
#             R colour names, "#RRGGBB", "#RRGGBBAA", palette integers, NA).
# main      – plot title (defaults to the deparsed expression like the original).
# border    – bar border colour; NA = no border (cleaner look).
# label     – "auto" | "name" | "hex" | "none"
#               "auto"  – colour name when colorset has names, else hex value
#               "name"  – always show the name (falls back to hex for unnamed)
#               "hex"   – always show the "#RRGGBB" hex string
#               "none"  – no labels at all
# cex.names – character expansion for bar labels (default 0.75).
# srt       – label rotation in degrees (default 90, matches Polychrome).
# show.hex  – logical; append " #RRGGBB" to each bar label (default FALSE).
# na.color  – replacement colour rendered for NA entries (default "grey80").
# mar       – graphics margin vector passed to par(); NULL leaves par() alone.
# ...       – additional arguments forwarded to barplot().
#
# Returns
# -------
# Invisibly, the numeric bar midpoint positions from barplot() — same as the
# original Polychrome::swatch() — so callers can overlay additional graphics.
# -----------------------------------------------------------------------------
custom_swatch <- function(colorset,
                          main      = deparse(substitute(colorset)),
                          border    = NA,
                          label     = c("auto", "name", "hex", "none"),
                          cex.names = 0.75,
                          srt       = 90,
                          show.hex  = FALSE,
                          na.color  = "grey80",
                          mar       = c(3, 1, 3, 1),
                          ...) {

  label <- match.arg(label)

  # ---- 1. Normalise input --------------------------------------------------
  norm     <- .swatch_normalize(colorset)
  hex_vec  <- norm$hex
  disp_nms <- norm$names
  L        <- length(hex_vec)

  # ---- 2. Substitute NA colours for plotting -------------------------------
  plot_hex <- hex_vec
  plot_hex[is.na(hex_vec)] <- na.color

  # ---- 3. Build bar labels -------------------------------------------------
  bar_labels <- switch(label,
                       auto = {
                         has_names <- !is.null(names(colorset)) &&
                           !all(names(colorset) == "" | is.na(names(colorset)))
                         if (has_names) disp_nms else hex_vec
                       },
                       name = disp_nms,
                       hex  = hex_vec,
                       none = rep("", L)
  )
  bar_labels[is.na(bar_labels)] <- "(NA)"

  if (show.hex && label != "none" && label != "hex") {
    bar_labels <- ifelse(is.na(hex_vec),
                         paste0(bar_labels, " (NA)"),
                         paste0(bar_labels, " ", hex_vec))
  }

  # ---- 4. Label contrast (white vs black text) ------------------------------
  label_cols <- .swatch_label_color(plot_hex)

  # ---- 5. Truncate very long labels to avoid clipping ----------------------
  max_chars  <- max(5L, floor(30 / max(1, L / 10)))
  too_long   <- nchar(bar_labels) > max_chars
  bar_labels[too_long] <- paste0(substr(bar_labels[too_long], 1, max_chars - 1), "\u2026")

  # ---- 6. Plot -------------------------------------------------------------
  old_par <- if (!is.null(mar)) par(mar = mar) else list()
  on.exit(if (length(old_par)) par(old_par), add = TRUE)

  pts <- barplot(rep(1, L),
                 col    = plot_hex,
                 border = border,
                 main   = main,
                 yaxt   = "n",
                 xaxt   = "n",
                 ...)

  if (label != "none") {
    text(pts, 0.5, bar_labels,
         srt = srt,
         col = label_cols,
         cex = cex.names,
         adj = 0.5)
  }

  # Mark NA slots with a small diagonal cross so they are obvious
  if (any(is.na(hex_vec))) {
    na_pts <- pts[is.na(hex_vec)]
    segments(na_pts - 0.4, 0.1, na_pts + 0.4, 0.9, col = "red", lwd = 1.5)
    segments(na_pts - 0.4, 0.9, na_pts + 0.4, 0.1, col = "red", lwd = 1.5)
  }

  invisible(pts)
}

#' Generate a random string
#'
#' Create a random string of specified length
#'
#' @param n number of strings to create
#' @param length length of string to create
#' @return one more random string of specific length
#'
#' @examples
#' # Task 1: create 1 random string string of length 5
#' randString(n = 1, length = 5)
#'
#' # Task 2: create 5 random string string of length 10
#' randString(n = 5, length = 10)
#'
#'
#' # Task 3: create 4 random string string of length 16
#' randString(n = 4, length = 16)
#'
#' @export

randString <- function(n, length) {
  .combo <- c(0:9, letters, 0:9, LETTERS)
  .all <- matrix(sample(.combo, n * length, replace = TRUE), ncol = n)
  unlist(lapply(1:ncol(.all), function(i)paste(.all[, i], collapse = "")))
}

test_that("Generate one random string", {
  expect_equal(length(randString(1,5)),1)
})

test_that("Generate one random string with 5 characters", {
  expect_equal(length(randString(5,5)),5)
})


test_that("Generate five random string", {
  expect_equal(nchar(randString(1,5)),5)
})


test_that("Generate five random string and 10 characters each", {
  expect_equal(nchar(randString(5,10)),rep(10,5))
})

#' Switch the index of two rows in a data set
#'
#' Allows the user to choose precisely which two rows they want to swap places,
#' while optionally preventing some columns from being altered in the process.
#' Excluded columns within the rows act as anchors that are immune from the switching operation
#' on the selected rows.
#'
#' @param data dataset object
#' @param row1 numeric. the first row number
#' @param row2 numeric. the second row number
#' @param keep.cols numeric or character. column number or name to keep
#' @examples
#'
#' # Example using mtcars
#' data100 <- mtcars[1:7,]
#'
#' head(data100) # preview overall data
#'
#' # task 1: basic result of switching rows 5 and 6
#' head(switch_rows(data100, 5, 6))
#'
#' # task 2: switch rows, but retain some columns
#' data100[5:6,2:10] # preview the portion that is to be changed
#'
#' # lets switch 2 rows, but keep content of columns 7, 8, 9 10 within the changed rows
#' res1 <- switch_rows(data100, row1 = 5, row2 = 6, keep.cols = 7:10) # use column numbers
#' res1[5:6,] # check result, pay attention to columns 9 and 10 as well
#' res2 <- switch_rows(data100,
#' row1 = 5,
#' row2 = 6,
#' keep.cols = c("disp","cyl")) # use column names
#' res2[5:6,] # check result, pay attention to columns "disp","cyl" as well
#'
#' @export
switch_rows <- function(data, row1, row2, keep.cols = NULL) {
  # check
  stopifnot(row1 != row2)

  # update row names
  rownames(data)[c(row1, row2)] <- rownames(data)[c(row2, row1)]
  .x2 <- data[row2, ]

  # account for keep.cols
  if (not.null(keep.cols)) {
    if (inherits(keep.cols,"character"))
      keep.cols <- which(names(data) %in% keep.cols)
    .x2[, keep.cols] <- data[row1, keep.cols]
    data[row1, keep.cols] <- data[row2, keep.cols]
  }
  # switch rows
  data[row2, ] <- data[row1, ]
  data[row1, ] <- .x2

  # return output
  data
}


#' Switch the index of two columns in a data set
#'
#' Allows the user to choose precisely which two columns they want to swap places,
#' while optionally preventing some rows within the columns from being altered in the process.
#' Excluded rows within the columns act as anchors that are immune from the switching operation
#' on the selected columns.
#'
#' @param data dataset object
#' @param col1 numeric or character the first column name or number
#' @param col2 numeric or character the second column name or number
#' @param keep.rows numeric. row number to keep
#' @examples
#'
#' # Example using mtcars
#' data101 <- mtcars[1:7,]
#'
#' head(data101) # preview overall data
#'
#' # task 1: basic result of switching columns 5 and 6
#' head(switch_cols(data101, 5, 6))
#'
#' # task 1: basic result of switching columns number 5 and name "gear"
#' head(switch_cols(data101, 5, "gear"))
#'
#' # task 1: basic result of switching columns "qsec" and "carb"
#' head(switch_cols(data101, "qsec", "carb"))
#'
#'
#' # task 2: switch columns, but retain some rows with the switched columns
#'
#'
#' # lets exchange some columns, but keep content of row 4, 5 intact
#' data101[1:6,4:7] # preview the portion that is to be changed
#' res1 <- switch_cols(data101, col1 = 5, col2 = 6, keep.rows = 4:5) # use column numbers
#' res1[1:6,4:7] # check result, pay attention to rows 4, 5 of columns 5, 6 as well
#'
#' data101[1:6,6:11] # preview the portion that is to be changed
#' res2 <- switch_cols(data101,
#' col1 = "qsec",
#' col2 = "carb",
#' keep.rows = c(1,2,3)) # keep 1, 2, 3
#' res2[1:6,6:11] # check result
#'
#' @export
switch_cols <- function(data, col1, col2, keep.rows = NULL) {
  # check
  stopifnot(col1 != col2, not.inherits(keep.rows,"character"))

  # update column names
  if (inherits(col1,"character")) col1 <- which(names(data) %in% col1)
  if (inherits(col2,"character")) col2 <- which(names(data) %in% col2)

  colnames(data)[c(col1, col2)] <- colnames(data)[c(col2, col1)]

  .x2 <- data[,col2, drop=FALSE]

  # account for keep.rows
  if (not.null(keep.rows)) {
    .x2[keep.rows, ] <- data[keep.rows,col1]
    data[keep.rows,col1] <- data[keep.rows,col2]
  }
  # switch columns
  data[, col2] <- data[, col1, drop=FALSE]
  data[, col1] <- .x2

  # return output
  data
}

#' Compute NONMEM S1 scaling from concentration, amount, and volume units
#'
#' @title Compute S1 scaling expression for NONMEM
#' @description
#' Given the units used in your dataset/model for the dependent variable (DV,
#' a concentration like "ng/mL"), the dosing amount (AMT, e.g. "mg") and the
#' model volume parameter V (e.g. "L" or "mL"), this function returns a robust
#' NONMEM-ready S1 expression so that \code{A(n)/S1} has the same units as DV.
#'
#' The function:
#' \itemize{
#'   \item parses common unit notations and synonyms (e.g. "ug", "mcg"),
#'   \item validates input and provides informative errors,
#'   \item computes the numeric factor such that \code{S1 = V * factor},
#'   \item returns a human-readable explanation and an optional numeric example.
#' }
#'
#' @param dv_unit Character scalar. DV concentration unit, e.g. "ng/mL", "mg/L", "ug per mL".
#' @param amt_unit Character scalar. AMT unit (mass), e.g. "mg", "ug", "ng".
#' @param v_unit Character scalar. Unit for model volume parameter V, e.g. "L", "mL".
#' @param sig_digits Integer scalar. Significant digits for formatted output (default = 8).
#' @param example Logical scalar. If TRUE, include a numeric example (default = FALSE).
#' @param example_amt Numeric scalar. Example AMT value (default = 1).
#' @param example_v Numeric scalar. Example V value (default = 1).
#'
#' @return A list with elements:
#' \describe{
#'   \item{factor}{Numeric. The multiplicative factor such that S1 = V * factor.}
#'   \item{S1_expression}{Character. NONMEM-ready expression (e.g. "S1 = V/1000").}
#'   \item{explanation}{Character. Human-readable explanation of the derivation.}
#'   \item{example}{Named list (present only when \code{example = TRUE}) with numeric example.}
#' }
#'
#' @examples
#' # Typical PK: DV in ng/mL, AMT in mg, V in L -> S1 = V * 1e+06 / 1000 = V * 1000
#' nonmem_scaling("ng/mL", "mg", "L")
#'
#' # If V is stored in mL:
#' nonmem_scaling("ng/mL", "mg", "mL")
#'
#' # With numeric example
#' nonmem_scaling("ng/mL", "mg", "L", example = TRUE, example_amt = 100, example_v = 40)
#'
#' # Microgram dosing
#' nonmem_scaling("pg/mL", "ug", "L")
#'
#' @export
nonmem_scaling <- function(dv_unit,
                          amt_unit,
                          v_unit,
                          sig_digits = 8L,
                          example = FALSE,
                          example_amt = 1,
                          example_v = 1) {


  # ===========================================================================

  # Input validation

  # ===========================================================================

  .assert_scalar_char <- function(x, arg_name) {
    if (missing(x) || is.null(x) || length(x) != 1L || !is.character(x) || is.na(x) || nchar(trimws(x)) == 0L) {
      stop(sprintf("'%s' must be a non-empty character scalar.", arg_name), call. = FALSE)
    }
  }
  .assert_scalar_char(dv_unit, "dv_unit")
  .assert_scalar_char(amt_unit, "amt_unit")
  .assert_scalar_char(v_unit, "v_unit")


  if (!is.numeric(sig_digits) || length(sig_digits) != 1L || sig_digits < 1L) {
    stop("'sig_digits' must be a positive integer.", call. = FALSE)
  }
  sig_digits <- as.integer(sig_digits)


  if (!is.logical(example) || length(example) != 1L || is.na(example)) {
    stop("'example' must be TRUE or FALSE.", call. = FALSE)
  }
  if (isTRUE(example)) {
    if (!is.numeric(example_amt) || length(example_amt) != 1L || !is.finite(example_amt) || example_amt <= 0) {
      stop("'example_amt' must be a positive finite number.", call. = FALSE)
    }
    if (!is.numeric(example_v) || length(example_v) != 1L || !is.finite(example_v) || example_v <= 0) {
      stop("'example_v' must be a positive finite number.", call. = FALSE)
    }
  }


  # ===========================================================================

  # Normalization helper
  # ===========================================================================
  .normalize_unit <- function(x) {
    x <- tolower(trimws(as.character(x)))
    # Remove all whitespace
    x <- gsub("\\s+", "", x, perl = TRUE)
    # Normalize unicode micro signs to 'u'
    x <- gsub("\u00B5", "u", x, fixed = TRUE)
    x <- gsub("\u03BC", "u", x, fixed = TRUE)
    # Normalize 'mcg' -> 'ug'
    x <- sub("^mcg", "ug", x)
    # Remove trailing dots
    x <- gsub("\\.$", "", x)
    x
  }

  dv_unit_norm  <- .normalize_unit(dv_unit)
  amt_unit_norm <- .normalize_unit(amt_unit)
  v_unit_norm   <- .normalize_unit(v_unit)

  # ===========================================================================
  # Parse DV into mass/volume components
  # ===========================================================================
  .split_concentration <- function(x, original) {
    # Try separators in priority order: "/", " per ", "per"
    sep <- NULL
    if (grepl("/", x, fixed = TRUE)) {
      sep <- "/"
    } else if (grepl("per", x, fixed = TRUE)) {
      sep <- "per"
    }
    if (is.null(sep)) {
      stop(sprintf(
        "Cannot parse DV unit '%s' as a concentration. Expected format: 'mass/volume' (e.g. 'ng/mL', 'mg per L').",
        original
      ), call. = FALSE)
    }
    parts <- strsplit(x, sep, fixed = TRUE)[[1L]]
    parts <- trimws(parts)
    parts <- parts[nchar(parts) > 0L]
    if (length(parts) != 2L) {
      stop(sprintf(
        "Cannot parse DV unit '%s' into exactly two components (mass and volume). Got: [%s].",
        original, paste(parts, collapse = ", ")
      ), call. = FALSE)
    }
    list(mass = parts[1L], volume = parts[2L])
  }

  dv_parsed <- .split_concentration(dv_unit_norm, dv_unit)
  dv_mass_str <- dv_parsed$mass
  dv_vol_str  <- dv_parsed$volume

  # ===========================================================================
  # Canonical unit maps
  # Mass: value = grams per 1 unit
  # Volume: value = milliliters per 1 unit
  # ===========================================================================
  mass_map <- c(
    kg = 1e3,
    g  = 1,
    mg = 1e-3,
    ug = 1e-6,
    ng = 1e-9,
    pg = 1e-12,
    fg = 1e-15

  )

  vol_map <- c(
    kl = 1e6,
    l  = 1e3,
    dl = 1e2,
    cl = 10,
    ml = 1,
    ul = 1e-3,
    nl = 1e-6,
    cc = 1        # 1 cc = 1 mL
  )

  # ===========================================================================

  # Resolve unit strings to canonical keys
  # ===========================================================================
  .resolve_mass <- function(x, original_input) {
    # Direct match
    if (x %in% names(mass_map)) return(x)
    # Common synonyms / long forms
    x2 <- x
    x2 <- sub("^micrograms?$", "ug", x2)
    x2 <- sub("^milligrams?$", "mg", x2)
    x2 <- sub("^nanograms?$", "ng", x2)
    x2 <- sub("^picograms?$", "pg", x2)
    x2 <- sub("^femtograms?$", "fg", x2)
    x2 <- sub("^kilograms?$", "kg", x2)
    x2 <- sub("^grams?$", "g", x2)
    if (x2 %in% names(mass_map)) return(x2)
    # Strip trailing 's' for plurals
    x3 <- sub("s$", "", x2)
    if (x3 %in% names(mass_map)) return(x3)
    stop(sprintf(
      "Unknown mass unit: '%s' (from input '%s'). Supported: %s.",
      x, original_input, paste(names(mass_map), collapse = ", ")
    ), call. = FALSE)
  }

  .resolve_volume <- function(x, original_input) {
    # Direct match
    if (x %in% names(vol_map)) return(x)
    # Common synonyms / long forms
    x2 <- x
    x2 <- sub("^lit(er|re)s?$", "l", x2)
    x2 <- sub("^millilit(er|re)s?$", "ml", x2)
    x2 <- sub("^microlit(er|re)s?$", "ul", x2)
    x2 <- sub("^decilit(er|re)s?$", "dl", x2)
    x2 <- sub("^centilit(er|re)s?$", "cl", x2)
    x2 <- sub("^nanolit(er|re)s?$", "nl", x2)
    x2 <- sub("^kilolit(er|re)s?$", "kl", x2)
    if (x2 %in% names(vol_map)) return(x2)
    # Strip trailing 's'
    x3 <- sub("s$", "", x2)
    if (x3 %in% names(vol_map)) return(x3)
    stop(sprintf(
      "Unknown volume unit: '%s' (from input '%s'). Supported: %s.",
      x, original_input, paste(names(vol_map), collapse = ", ")
    ), call. = FALSE)
  }

  amt_key        <- .resolve_mass(amt_unit_norm, amt_unit)
  dv_mass_key    <- .resolve_mass(dv_mass_str, dv_unit)
  v_key          <- .resolve_volume(v_unit_norm, v_unit)
  dv_vol_key     <- .resolve_volume(dv_vol_str, dv_unit)

  # ===========================================================================
  # Compute scaling factor
  # ---------------------------------------------------------------------------
  # In NONMEM, A(n) is in AMT units. We need:
  #   A(n) / S1 = concentration in DV units
  #   => S1 = A(n) / C_dv
  #
  # If A = 1 amt_unit in compartment with volume V (in v_units):

  #   True concentration = (1 amt_unit) / (V v_units)
  #   Convert to DV units:
  #     mass conversion: 1 amt_unit = (g_per_amt / g_per_dv_mass) dv_mass_units
  #     vol conversion:  1 v_unit   = (mL_per_v / mL_per_dv_vol) dv_vol_units
  #
  #   C_dv = f_mass / (V * f_vol)   ... per unit AMT
  #   => S1 = V * f_vol / f_mass
  # ===========================================================================
  g_per_amt     <- mass_map[[amt_key]]
  g_per_dv_mass <- mass_map[[dv_mass_key]]
  mL_per_v      <- vol_map[[v_key]]
  mL_per_dv_vol <- vol_map[[dv_vol_key]]

  # f_mass: how many DV-mass-units per 1 AMT-unit
  f_mass <- g_per_amt / g_per_dv_mass
  # f_vol: how many DV-volume-units per 1 V-unit
  f_vol  <- mL_per_v / mL_per_dv_vol

  factor <- f_vol / f_mass

  if (!is.finite(factor) || factor <= 0) {
    stop("Computed scaling factor is non-positive or non-finite. Please verify unit inputs.", call. = FALSE)
  }

  # ===========================================================================
  # Format the S1 expression for NONMEM
  # ===========================================================================
  .is_power_of_10 <- function(x, tol = 1e-12) {
    if (x <= 0) return(list(yes = FALSE))
    log_val <- log10(x)
    if (abs(log_val - round(log_val)) < tol) {
      return(list(yes = TRUE, exponent = as.integer(round(log_val))))
    }
    list(yes = FALSE)
  }

  .format_expression <- function(fac) {
    # Check if factor is exactly 1
    if (abs(fac - 1) < .Machine$double.eps^0.5) {
      return("S1 = V")
    }

    p10 <- .is_power_of_10(fac)
    if (p10$yes) {
      exp_val <- p10$exponent
      if (exp_val == 0L) return("S1 = V")
      if (exp_val > 0L) {
        # e.g. factor = 1000 -> "S1 = V * 1000"
        return(sprintf("S1 = V * %s", format(10^exp_val, scientific = FALSE)))
      } else {
        # e.g. factor = 0.001 -> "S1 = V / 1000"
        return(sprintf("S1 = V / %s", format(10^(-exp_val), scientific = FALSE)))
      }
    }

    # Check if 1/factor is a nice integer (for division form)
    inv <- 1 / fac
    p10_inv <- .is_power_of_10(inv)
    if (p10_inv$yes && p10_inv$exponent > 0L) {
      return(sprintf("S1 = V / %s", format(10^(p10_inv$exponent), scientific = FALSE)))
    }

    # Check if factor itself is a nice integer
    if (abs(fac - round(fac)) < 1e-9 && fac < 1e12) {
      return(sprintf("S1 = V * %s", format(round(fac), scientific = FALSE)))
    }

    # General case: use signif
    fstr <- formatC(fac, digits = sig_digits, format = "g")
    sprintf("S1 = V * %s", fstr)
  }

  expr_str <- .format_expression(factor)

  # ===========================================================================
  # Build explanation
  # ===========================================================================
  explanation <- paste0(
    "Goal: A(n)/S1 yields concentration in [", dv_mass_str, "/", dv_vol_str, "].\n",
    "\n",
    "Unit conversions:\n",
    "  - 1 ", amt_unit, " = ", signif(f_mass, sig_digits), " ", dv_mass_str, "\n",
    "  - 1 ", v_unit,   " = ", signif(f_vol, sig_digits),  " ", dv_vol_str, "\n",
    "\n",
    "Derivation:\n",
    "  C [", dv_mass_str, "/", dv_vol_str, "] = A [", amt_unit, "] * f_mass / (V [", v_unit, "] * f_vol)\n",
    "  => S1 = V * f_vol / f_mass = V * ", signif(f_vol, sig_digits), " / ", signif(f_mass, sig_digits), "\n",
    "  => S1 = V * ", signif(factor, sig_digits), "\n",
    "\n",
    "NONMEM code:\n",
    "  ", expr_str, "\n"
  )

  # ===========================================================================
  # Optional numeric example
  # ===========================================================================
  example_out <- NULL
  if (isTRUE(example)) {
    # C_dv = AMT * f_mass / (V * f_vol)
    conc <- (example_amt * f_mass) / (example_v * f_vol)
    # Verify via S1: A/S1 = AMT / (V * factor) should equal conc
    conc_via_s1 <- example_amt / (example_v * factor)
    example_out <- list(
      AMT            = example_amt,
      AMT_unit       = amt_unit,
      V              = example_v,
      V_unit         = v_unit,
      S1             = example_v * factor,
      concentration  = conc,
      conc_via_S1    = conc_via_s1,
      DV_unit        = paste0(dv_mass_str, "/", dv_vol_str),
      note           = sprintf(
        "A dose of %s %s distributed in V = %s %s gives C = %s %s/%s.",
        signif(example_amt, sig_digits), amt_unit,
        signif(example_v, sig_digits), v_unit,
        signif(conc, sig_digits), dv_mass_str, dv_vol_str
      )
    )
  }

  # ===========================================================================
  # Return
  # ===========================================================================
  structure(
    list(
      factor        = factor,
      S1_expression = expr_str,
      explanation   = explanation,
      example       = example_out
    ),
    class = "nonmem_scaling"
  )
}


#' Print method for s1_scaling objects
#' @param x An object of class \code{s1_scaling}.
#' @param ... Additional arguments (ignored).
print.nonmem_scaling <- function(x, ...) {
  cat("--- NONMEM S1 Scaling ---\n\n")
  cat(x$explanation)
  if (!is.null(x$example)) {
    cat("\nExample:\n")
    cat("  ", x$example$note, "\n")
  }
  cat("\n")
  invisible(x)
}

#' Calculate the Mode of a Numeric or Character Vector
#'
#' This function calculates the mode (most frequently occurring value(s)) of a numeric or character vector.
#'
#' @param x A numeric or character vector for which the mode is to be calculated.
#' @return The mode(s) of the input vector. If multiple values have the same highest frequency, all modes are returned.
#'         Returns `NA` if the input vector is empty.
#' @examples
#' # Example with a numeric vector
#' numeric_vector <- c(1, 2, 2, 3, 3, 3, 4, 5)
#' mode.calc(numeric_vector)
#'
#' # Example with a character vector
#' character_vector <- c("apple", "banana", "apple", "orange", "banana", "banana")
#' mode.calc(character_vector)
#'
#' @export
mode.calc <- function(x) {
  # Check for empty input
  if (length(x) == 0) {
    return(NA)
  }

  # Calculate frequency of each element
  freq_table <- table(x)

  # Find the maximum frequency
  max_freq <- max(freq_table)

  # Return the element(s) with the highest frequency
  mode_values <- names(freq_table[freq_table == max_freq])

  # If numeric, convert to numeric type
  if (is.numeric(x)) {
    mode_values <- as.numeric(mode_values)
  }

  return(mode_values)
}


#' Calculate the Range Difference of a Numeric Vector
#'
#' This function calculates the difference between the maximum and minimum values of a numeric vector.
#'
#' @param x A numeric vector for which the range difference is to be calculated.
#' @return A numeric value representing the difference between the maximum and minimum values of the input vector.
#'         Returns `NA` if the input is empty or contains only `NA` values.
#' @examples
#' # Example with a numeric vector
#' numeric_vector <- c(1, 5, 3, 8, 2)
#' sub.range(numeric_vector)
#'
#' # Example with missing values
#' sub.range(c(NA, 4, 7, NA, 10))
#'
#' @export
sub.range <- function(x) {
  # Check if input is valid
  if (length(x) == 0 || all(is.na(x))) {
    return(NA)
  }

  # Calculate the range difference
 max(x, na.rm = TRUE) - min(x, na.rm = TRUE)
}


#' Cumulative Sum with NA Removal
#'
#' This function calculates the cumulative sum of a numeric vector, removing any `NA` values before computation.
#'
#' @param x A numeric vector for which the cumulative sum is to be calculated.
#' @return A numeric vector representing the cumulative sum with `NA` values removed.
#'         Returns an empty vector if the input is empty or contains only `NA` values.
#' @examples
#' # Example with numeric vector
#' numeric_vector <- c(1, 2, NA, 4, 5)
#' na.cumsum(numeric_vector)
#'
#' # Example with all NAs
#' na.cumsum(c(NA, NA))
#'
#' @export
na.cumsum <- function(x) {
  # Check if input is valid
  if (length(x) == 0 || all(is.na(x))) {
    return(numeric(0)) # Return an empty numeric vector if all values are NA
  }

  # Remove NA values
  # Calculate the cumulative sum
  cumsum(x[!is.na(x)])
}





#' Replicate Rows in a Data Frame
#'
#' This function replicates each row in a data frame a specified number of times.
#'
#' @param data A data frame whose rows are to be replicated.
#' @param n An integer specifying the number of times each row should be replicated.
#' @return A data frame with each row replicated `n` times. If `n` is less than or equal to 0, an empty data frame is returned.
#' @examples
#' # Example with a simple data frame
#' df <- data.frame(A = c(1, 2), B = c("x", "y"))
#' rows.rep(df, 3)
#'
#' # Example with no replication (n = 0)
#' rows.rep(df, 0)
#'
#' @export
rows.rep <- function(data, n) {
  # Validate inputs
  if (!is.data.frame(data))stop("Input 'data' must be a data frame.")
  if (missing(n) | n <= 0) return(data)
  # Replicate rows
  replicated_data <- data[rep(1:nrow(data), each = n), ]
  rownames(replicated_data) <- NULL # Reset row names for clarity
  replicated_data
}



#' Replicate Columns in a Data Frame
#'
#' This function replicates each column in a data frame a specified number of times.
#'
#' @param data A data frame whose columns are to be replicated.
#' @param n An integer specifying the number of times each column should be replicated.
#' @return A data frame with each column replicated `n` times. If `n` is less than or equal to 0, an empty data frame is returned.
#' @examples
#' # Example with a simple data frame
#' df <- data.frame(A = c(1, 2), B = c(3, 4))
#' cols.rep(df, 3)
#'
#' # Example with no replication (n = 0)
#' cols.rep(df, 0)
#'
#' @export
cols.rep <- function(data, n) {
  # Validate inputs
  if (!is.data.frame(data)) stop("Input 'data' must be a data frame.")
  if (missing(n) | n <= 0) return(data)
  # Replicate each column `n` times
  replicated_data <- do.call(cbind, replicate(n, data, simplify = FALSE))
  # Optionally reset column names to indicate replication
  colnames(replicated_data) <- make.names(rep(colnames(data), each = n), unique = TRUE)
  replicated_data
}



#' Normalize a Numeric Vector to the Range [0, 1]
#'
#' This function normalizes a numeric vector so that all values are scaled to the range [0, 1].
#'
#' @param x A numeric vector to be normalized.
#' @return A numeric vector with values scaled to the range [0, 1].
#'         If all values are identical, all values are set to 0.
#' @examples
#' # Example with a numeric vector
#' normalize.vector(c(1, 2, 3, 4, 5))
#'
#' # Example with a vector containing identical values
#' normalize.vector(c(3, 3, 3))
#'
#' @export
normalize.vector <- function(x) {
  # Check if the input is numeric
  if (!is.numeric(x)) {
    stop("Input 'x' must be a numeric vector.")
  }

  # Handle cases with no variation (all values identical or single element)
  if (length(unique(x)) == 1) {
    return(rep(0, length(x)))
  }

  # Calculate min and max
  x_min <- min(x, na.rm = TRUE)
  x_max <- max(x, na.rm = TRUE)

  # Normalize to the range [0, 1]
  (x - x_min) / (x_max - x_min)
}



#' Create Tensor-Like Slices from a Data Frame or Matrix
#'
#' This function converts a data frame or matrix into row-wise slices, similar to TensorFlow's `from_tensor_slices()`.
#'
#' @param data A data frame or matrix to be converted into slices.
#' @return A list of slices, each corresponding to a row of the input data.
#' @examples
#' df <- data.frame(A = 1:3, B = c("x", "y", "z"))
#' from_tensor_slices(df)
#'
#' @export
from_tensor_slices <- function(data) {
  if (!is.data.frame(data) && !is.matrix(data)) {
    stop("Input must be a data frame or matrix.")
  }
  split(data, seq(nrow(data)))
}



#' Multi-Head Attention
#'
#' This function implements a simplified version of multi-head attention for sequence data.
#'
#' @param query A numeric matrix representing the query.
#' @param key A numeric matrix representing the key.
#' @param value A numeric matrix representing the value.
#' @param num_heads An integer specifying the number of attention heads.
#' @return A numeric matrix representing the attention output.
#' @examples
#' query <- matrix(rnorm(12), nrow = 3)
#' key <- matrix(rnorm(12), nrow = 3)
#' value <- matrix(rnorm(12), nrow = 3)
#' multihead_att(query, key, value, num_heads = 2)
#'
#' @export
multihead_att <- function(query, key, value, num_heads) {
  if (ncol(query) != ncol(key) || ncol(key) != ncol(value)) {
    stop("Query, key, and value must have the same number of columns.")
  }
  # Simple dot-product attention
  attention_scores <- query %*% t(key)
  attention_weights <- exp(attention_scores) / rowSums(exp(attention_scores))
  attention_weights %*% value
}



#' Learning Rate Scheduler
#'
#' This function adjusts the learning rate at each epoch according to a custom schedule.
#'
#' @param initial_lr A numeric value representing the initial learning rate.
#' @param schedule A function that takes the epoch number as input and returns the adjusted learning rate.
#' @param epochs An integer specifying the total number of epochs.
#' @return A numeric vector representing the learning rate for each epoch.
#' @examples
#' schedule <- function(epoch) { 0.01 * (0.9 ^ epoch) }
#' learn_rate_scheduler(0.01, schedule, epochs = 10)
#'
#' @export
learn_rate_scheduler <- function(initial_lr, schedule, epochs) {
  if (!is.numeric(initial_lr) || initial_lr <= 0) {
    stop("Initial learning rate must be a positive number.")
  }
  sapply(0:(epochs - 1), schedule)
}

#' Remove Empty Lines from a File
#'
#' This function reads a text file, removes all empty lines, and saves the
#' modified content back to the same file.
#'
#' @param file_path A character string specifying the path to the text file.
#' @param resave A logical value indicating if the file content should be resaved or returned
#'
#' @return NULL This function modifies the file in place and does not return a value.
#'
#' @examples
#' \donttest{
#' if(interactive()){
#' # Remove empty lines from a file
#' trim.file("path/to/your/file.txt")
#' }
#' }
#' @export
trim.file <- function(file_path, resave = TRUE) {
  if(missing(file_path)) file_path = rstudioapi::getActiveDocumentContext()$path
  # Read the file
  lines <- readLines(file_path)
  # Remove empty lines
  non_empty_lines <- lines[nzchar(lines)]
  message(paste0("Empty lines have been removed from ",basename(file_path)))
  # Write the non-empty lines back to the file
  if(resave) writeLines(non_empty_lines, file_path) else non_empty_lines
}

#' Add elements to a vector like array_push in PHP
#'
#' Shorthand to add elements to a vector and save as the same name
#'
#' @param . first vector
#' @param add vector to add
#' @param unique remove duplicated entries
#' @param rm.na remove NA values
#' @param rm.empty remove empty values
#' @return vector combining fist and second vector, but have name set to the first
#'
#' @section Use case:
#' This function allows the combination of two vectors in one short line of code. It allows specification of further downstream
#' filtering of the resulting vector such as selecting only unique items, removing NA or empty values. It simplifies a code chunk
#' with many lines of code to concatenate and filter various vectors.
#'
#' @details
#' Note that two vectors are required in order to use this function. Also, note that the final result replaces the content of the
#' first vector. This means that the original content of the 'first vector' will no longer exist after this function executes.
#'
#' @examples
#' num1 <- number(10, seed = 45)
#' num2 <-"rpkg.net"
#'
#' num1
#' num2
#'
#' #Task: add num2 to num1 and re-save as num1
#' vector_push(num1,num2)
#' num1 #updated with num2
#' num2 #not updated
#'
#'
#' #Task: concatenate two vectors and remove duplicates
#' vector1 = number(4,seed = 5)
#' vector2 = number(8,seed = 5)
#' vector3 = number(12,seed = 5)
#'
#' vector1 #length is 4
#' vector2 #length is 8
#' vector3 #length is 12
#'
#' # with duplicated
#' vector_push(vector1,vector2, unique = FALSE)
#' vector1 #return modified vector
#' length(vector1) #length is 12 because nothing was removed
#' #duplicates in vector1 is 886905927 100040083 293768998  54080431
#'
#' # without duplicated
#' vector_push(vector2,vector3, unique = TRUE)
#' vector2 #return modified vector
#' length(vector2) #length is 12 instead of 20
#' #Total of 8 duplicated numbers were removed
#'
#'
#' #Task: concatenate two vector and remove NA values
#' vector1 = number(5)
#' vector2 = c(4,NA,5,NA)
#' vector3 = number(5)
#'
#' # with NA
#' vector_push(vector1,vector2, rm.na = FALSE)
#' vector1 #return modified vector
#'
#' # without NA
#' vector_push(vector3,vector2, rm.na = TRUE)
#' vector3 #return modified vector
#'
#'
#' #Task: concatenate two vector and remove empty values
#' vector1 = number(5)
#' vector2 = c(4,'',5,'',NULL,' ')
#' vector3 = number(5)
#'
#' # with empty
#' vector_push(vector1,vector2, rm.empty = FALSE)
#' vector1 #return modified vector
#'
#' # without empty
#' vector_push(vector3,vector2, rm.empty = TRUE)
#' vector3 #return modified vector
#'
#' @export
#'
vector_push <- function(., add, unique = FALSE, rm.na = FALSE, rm.empty = FALSE) {
  .. <- substitute(.)
  if (typeof(..) != "symbol") stop(paste0(.., " must be an object."))
  .res <- c(get(as.character(..), envir = parent.frame()), add)
  if(unique) .res <- unique(.res)
  if(rm.na) .res <- .res[not.na(.res)]
  if(rm.empty) .res <- .res[not.empty(.res)]
  assign(as.character(..), .res , envir = parent.frame())
}

#' Not inherit from any of the classes specified
#'
#' Opposite of base::inherits().
#' Indicates whether its first argument inherits from any of the classes specified in the what argument. If which is TRUE then an integer vector of the same length as what is returned. Each element indicates the position in the class(x) matched by the element of what; zero indicates no match. If which is FALSE then TRUE is returned by inherits if any of the names in what match with any class.
#'
#' @inheritParams base::inherits
#' @param what a character vector naming classes.
#' @return a boolean value to indicate if !inherits
#' @examples
#' keep.cols = "a character"
#' class(keep.cols) # class is character
#' not.inherits(keep.cols,"character")
#'
#' num.var = 1L
#' class(num.var) # class is integer
#' not.inherits(num.var,"double")
#'
#' @export

not.inherits <- function(x, what, which = FALSE)!{
  inherits(x = x, what = what, which = which)
  }

#' Increment vector by value
#'
#' Increment the content of a vector and re-save as the vector
#'
#' @param . vector of number(s)
#' @param add number to add
#' @return a vector incremented by a number
#' @details
#' This function is very useful when writing complex codes involving loops.
#' Apart from the for loop, this can be useful to quickly increment a variable located outside the loop
#' by simply incrementing the variable by 1 or other numbers. Check in the example section for a specific use.
#' Nonetheless, one may also choose to use this function in any other instance, as it's simple purpose is
#' to increase the value of a variable by a number and then re-save the new value to that variable.
#'
#' @examples
#' num1 <- sample(330:400,10)
#' num1#before increment
#'
#' # increment num1 by 1
#' inc(num1)
#' num1 #after increment
#'
#' # increment num1 by 5
#' num1 #before increment
#' inc(num1, add= 10)
#' num1 #after increment
#'
#' #when used in loops
#'
#' #add and compare directly
#' rnum = 10
#' inc(rnum) == 11 #returns TRUE
#' rnum #the variable was also updated
#'
#' # use in a for loop
#' ynum = 1
#' for( i in c("scientist","dancer","handyman","pharmacist")){
#' message("This is the item number ")
#' message(ynum)
#' message(". For this item, I am a ")
#' message(i)
#'
#' #decrement easily at each turn
#' plus(ynum)
#' }
#'
#'
#' #use in a repeat loop
#' xnum = 1
#' repeat{ #repeat until xnum is 15
#' message(xnum)
#' if(inc(xnum) == 15) break
#' }
#' @export
#'
inc <- function(., add = 1L) {
  .. <- substitute(.)
  if (typeof(..) != "symbol") stop(paste0(.., " must be an object."))
  assign(as.character(..), (get(as.character(..), envir = parent.frame()) + add), envir = parent.frame())
}

#' @inherit inc
#' @export
plus <- inc


#' Decrease vector by value
#'
#' decrease the content of a vector and re-save as the vector
#'
#' @param . vector of number(s)
#' @param minus number to minus
#' @return a vector decreased by a number
#' @details
#' Similar to the inc and plus functions, the minus function is very useful when writing complex codes involving loops.
#' Apart from the for loop, minus can be useful to quickly decrease the value of a variable located outside the loop
#' by simply decreement the variable by 1 or other numbers. Check in the example section for a specific use.
#' Given the scope, one may also choose to use this function in any other instances, as it's simple purpose is
#' to decrease the value of a variable by a number and then re-save the new value to that variable.
#' @examples
#' num1 <- sample(5:150,10)
#' num1
#'
#' # decrease num1 by 1
#' num1 #before decrease
#' minus(num1)
#' num1 #after decrease
#'
#' # decrease num1 by 5
#' num1 #before decrease
#' minus(num1, minus = 5)
#' num1 #after decrease
#'
#'
#'
#' #when used in loops
#'
#' #add and compare directly
#' rnum = 23
#' minus(rnum) == 220 #returns FALSE
#' rnum #the variable was also updated
#'
#'
#' # use in a for loop
#' ynum = 100
#'
#' for( i in c("teacher","student","lawyer","pharmacist")){
#' message("This is the item number ")
#' message(ynum)
#' message(". For this item, I am a ")
#' message(i)
#'
#' #decrement easily at each turn
#' minus(ynum,3)
#' }
#'
#' #use in a repeat loop
#' xnum = 100
#' repeat{ #repeat until xnum is 85
#' message(xnum)
#' if(minus(xnum) == 85) break
#' }
#' @export
#'
minus <- function(., minus = 1L) {
  .. <- substitute(.)
  if (typeof(..) != "symbol") stop(paste0(.., " must be an object."))
  assign(as.character(..), (get(as.character(..), envir = parent.frame()) - minus), envir = parent.frame())
}

#' Not in vector or array
#'
#' Check if entry is in vector
#'
#' @rdname nin
#' @param x vector entry
#' @param table table of items to check
#' @return a boolean value to indicate if entry is present
#' @examples
#' 5 %nin% c(1:10) #FALSE
#' 5 %nin% c(11:20) #TRUE
#'
#' x = "a"
#' if(x %nin% letters) x
#'
#' # let's say we are trying to exclude numbers from a vector
#' vector_num1 <- number(9, max.digits = 5, seed = 1) #simulate 9 numbers
#' vector_num1 #values
#' vector_num1[vector_num1 %nin% c(83615,85229)]#return values not 83615 or 85229
#' @export

`%nin%` <- function(x, table)!{
  (x %in% table)
}

`%nin%` -> `%!in%`

#' Load specific R libraries and clear environment
#'
#' Load specific packages, print a list of the loaded packages along with versions.
#' Only include libraries, don't install if library doesn't exist
#'
#' @param ... multiple library names
#' @param lib.loc OPTIONAL. library store location
#' @param quietly OPTIONAL. attach library quietly
#' @param clear OPTIONAL. clear environment after attach
#' @param clearPkgs Clear previous loaded packages, TRUE or FALSE
#' @return loaded libraries and clear environment
#' @examples
#' \donttest{
#' # load packages and print their versions to the console
#' libraryAll(base) #one package
#'
#' libraryAll(
#'   base,
#'   tools,
#'   stats
#' ) #multiple packages
#'
#' libraryAll("grDevices") #with quotes
#'
#' libraryAll(
#'   stats,
#'   utils,
#'   quietly = TRUE
#' ) #load quietly
#'
#' libraryAll(
#'   base,
#' clear = FALSE) #do not clear console after load
#'
#' # clear previously loaded packages, then load r2resize and r2social
#' libraryAll(
#'   stats,
#'   utils,
#'   clearPkgs = TRUE
#' )
#' }
#' @export

libraryAll <- function(..., lib.loc = NULL, quietly = FALSE, clear = TRUE, clearPkgs = FALSE) {
  # remove previous loaded packages
  if (clearPkgs) {
    clearPreviouslyLoadedPkgs()
  }
  # load quickcode if not loaded
  if("quickcode" %nin% (.packages())) library(quickcode, quietly = TRUE)
  # load user requested libraries
  lib.names <- as.list(substitute(args(...))[-1L])
  if(!length(lib.names)) stop("Packages must be included to execute this function")
  invisible(lapply(lib.names, function(lib) do.call("library", list(package = lib, lib.loc = lib.loc, quietly = quietly, warn.conflicts = FALSE))))
  if(clear){
    erase()
    message("Packages Loaded:")
    #return package versions
    invisible(lapply(lib.names, function(lib)
      message(paste0(lib,", version ",utils::packageVersion(lib)))))
  }
}

#' Add elements to a list like array_push in PHP
#'
#' Shorthand to add elements to a vector and save as the same name
#'
#' @param . first list
#' @param add list to add
#' @return vector combining fist and second vector, but have name set to the first
#' @examples
#' num1 <- list(sample(330:400,10))
#' num2 <-list("rpkg.net")
#' list_push(num1, add= num2)
#' @export
#'
list_push <- function(., add) {
  .. <- substitute(.)
  if (typeof(..) != "symbol") stop(paste0(.., " must be an object."))
  assign(as.character(..), list(get(as.character(..), envir = parent.frame()), add), envir = parent.frame())
}

#' Generate a bionic text
#'
#' This function serves as a mechanism
#' enabling the conversion of provided
#' text into a bionic form. Users input
#' the text, and the function, in turn,
#' delivers the text transformed into a
#' bionic format.
#'
#' @param text input text
#'
#' @details
#' A bionic text refers to a transformed version
#' of a given text achieved through a specialized
#' function designed to incorporate elements of
#' advanced technology, enhancing both the form
#' and content of the original input. This function
#' operates by infusing the text with a fusion of
#' various elements, resulting in a synthesis
#' that transcends traditional linguistic boundaries.
#' The function augments the text
#' with dynamic visual representations that adapt to
#' the reader's preferences.
#' The goal is to create a text that not only conveys
#' information but also engages the audience in a more
#' immersive and interactive manner, harnessing
#' the capabilities of modern technology to redefine
#' the traditional concept of textual communication.
#' An example of a bionic text could be a news article
#' that dynamically updates with real-time data,
#' incorporates multimedia elements, and adjusts its
#' presentation style based on the reader's preferences,
#' thereby offering a more enriched and personalized reading
#' experience.
#'
#' @references This idea stems from a blog article published at
#' https://www.r-bloggers.com/2023/10/little-useless-useful-r-functions-function-for-faster-reading-with-bionic-reading/
#' and the original source for bionic texts may be found at https://bionic-reading.com/
#'
#'
#' @return bionic text
#'
#' @examples
#' # simple example to show a text
#' # transformation to bionic text
#'
#' # text to transform
#' text1 <- "A tool for nonparametric
#' estimation and inference
#' of a non-decreasing
#' monotone hazard\nratio
#' from a right censored survival dataset."
#'
#' # transform text
#' genbt <- bionic_txt(text1)
#'
#' # print bionic text as message or cat
#' message(genbt)
#' cat(genbt)
#'
#' @export

bionic_txt <- function(text) {
  stopifnot(nchar(text)>1)
  words <- unlist(strsplit(text, " "))
  modified_words <- sapply(words, modify_word)
  formatted_text <- paste(modified_words, collapse = " ")
  paste0(formatted_text, "\n")
}

#' If number falls within a range of values and get closest values
#'
#'
#' @description With a defined range of values, the function systematically examines
#' each provided number to determine if it falls within the specified range. It may
#' also provide the values with the range that are closest to a desired number.
#'
#' @param value NUMERIC. the vector of numbers to check
#' @param range.min NUMERIC. OPTIONAL. the minimum value of the range
#' @param range.max NUMERIC. OPTIONAL. the maximum value of the range
#' @param range.vec NUMERIC. OPTIONAL. a vector of numbers to use for the range
#' @param closest BOOLEAN. OPTIONAL. return closest value
#' @param rm.na BOOLEAN. OPTIONAL. remove NA values from input
#'
#' @return boolean to indicate if the value or set of values are within the range
#' @note
#' The argument range.vec is utilized when users opt not to employ the range.min or
#' range.max arguments. If range.vec is specified, range.min and range.max are
#' disregarded. It's important to note that the use of range.vec is optional.
#'
#' @details
#' The described function serves the purpose of checking whether a given number
#' or set of numbers falls within a specified range. It operates by taking a range
#' of values as input and then systematically evaluates each provided number to
#' determine if it lies within the defined range. This function proves particularly
#' useful for scenarios where there is a need to assess numeric values
#' against predefined boundaries, ensuring they meet specific criteria or constraints.
#' In the same manner, this function allows the user to also retrieve values within
#' the range that are closest to each provided number.
#'
#' @examples
#' # Task 1: Check if a number is within specified range
#' in.range(5, range.min = 3, range.max = 10) # TRUE
#' in.range(25, range.min = 12, range.max = 20) # FALSE
#'
#' # Task 2: Check if a set of values are within a specified range
#' in.range(1:5, range.min = 2, range.max = 7) #
#' in.range(50:60, range.min = 16, range.max = 27) #
#'
#'
#' # Task 3: Check if a number is within the range of a set of numbers
#' in.range(5, range.vec = 1:10) # TRUE
#' in.range(345, range.vec = c(1001,1002,1003,1004,1005,
#' 1006,1007,1008,1009,1010,1011,1012,1013,1014)) # FALSE
#'
#' # Task 4: Check if a set of values are within the range of a set of numbers
#' in.range(1:5, range.vec = 4:19) #
#' in.range(50:60, range.vec = c(55,33,22,56,75,213,120)) #
#'
#' # Task 5: remove NAs prior to processing
#' in.range(c(1,3,NA,3,4,NA,8), range.min = 4, range.max = 6, rm.na = FALSE) # do not remove NA
#' in.range(c(1,3,NA,3,4,NA,8), range.min = 4, range.max = 6, rm.na = TRUE) # remove NA
#' #in.range(c(NA), range.min = 4, range.max = 6, rm.na = TRUE) #This will return error
#'
#' # Task 6: return the closest number to the value
#' in.range(5:23, range.vec = 7:19, closest = TRUE)
#' in.range(-5:10, range.vec = -2:19, closest = TRUE)
#' in.range(c(1:5,NA,6:9), range.vec = 4:19, closest = TRUE)
#' in.range(c(1:5,NA,6:9), range.vec = 4:19, closest = TRUE, rm.na = TRUE)
#'
#' @export

in.range <- function(value, range.min, range.max, range.vec = NULL, closest = FALSE, rm.na = FALSE){
  #remove NA if set to TRUE
  if(rm.na) value <- value[not.na(value)]

  #if closest is true, then range.vec must not be null
  if(closest){
    if(is.null(range.vec)) stop("If 'closest' is TRUE, then 'range.vec' must not be NULL, see examples.")
    if(length(range.vec)<2) stop("If 'closest' is TRUE, then 'range.vec' must have at least 2 values, see examples.")
  }

  #exit if value to check is not numeric
  stopifnot(all(is.numeric(value)))

  #if range.vec is set, use the value to set range.min and range.max
  if(not.null(range.vec) & length(range.vec) > 1){
  stopifnot(all(is.numeric(range.vec)))
    computRange <- range(range.vec)
    range.min <- computRange[1]
    range.max <- computRange[2]
  }
  #return range check or closest
  if(closest){
    stats::setNames(as.numeric(apply(data.frame(value),1,function(x){
      names(sort(stats::setNames(abs(x - range.vec),range.vec))[1])
    })),value)
  }else{
    with(data.frame(less = range.min <= value, more = range.max >= value),less & more)
  }
}

#' Convert boolean values between formats
#'
#' Convert Yes/No to 1/0 or to TRUE/FALSE or vice versa
#'
#' @param ds item to convert
#' @param type format to convert to, choices 1, 2 or 3

#' @details
#' Output various format of booleans into a specified format. Below are the options for the type argument.
#'
#' \strong{type}: options are as follows - \cr\cr
#'      1 - Yes/No \cr
#'      2 - TRUE/FALSE\cr
#'      3 - 1/0\cr
#'
#' @return output adhering to the format of the type provided

#' @examples
#'
#' # Task: convert "yes" or "no" to format of TRUE or FALSE
#' as.boolean("yes",2)
#' as.boolean("no",2)
#' as.boolean("YES",2)
#' as.boolean("NO",2)
#'
#' # Task: convert "yes" or "no" to format of 1 or 0
#' as.boolean("yes",3)
#' as.boolean("no",3)
#' as.boolean("YES",3)
#' as.boolean("NO",3)
#'
#'
#' # Task: convert 1 to format of Yes or No
#' as.boolean(1,1)
#'
#' # Task: convert "T" to format of Yes or No
#' as.boolean("T",1)
#'
#'
#' # Task: convert "f" to format of TRUE or FALSE
#' as.boolean("f",2)
#'
#'
#' # Task: convert 1 to format of TRUE or FALSE
#' as.boolean(1,2)
#'
#'
#' # Task: convert "Y" or "y" to format of Yes or No
#' as.boolean("Y",1) #uppercase Y
#' as.boolean("y",1) #lowercase y
#'
#'
#' # Task: convert TRUE/FALSE to format of 1 or 0
#' as.boolean(TRUE,3)
#' as.boolean(FALSE,3)
#'
#'
#' # Task: convert TRUE/FALSE to format of Yes or No
#' as.boolean(TRUE,1)
#' as.boolean(FALSE,1)
#'
#'
#'
#' # In case of error in argument
#' # as.boolean("tr",3) #NA
#' # as.boolean("ye",3) #NA
#'
#' # vector of mixed boolean to TRUE/FALSE or 1/0
#' multv <- c(TRUE,"y","n","YES","yes",FALSE,"f","F","T","t")
#' as.boolean(multv,1) # return vector as Yes/No
#' as.boolean(multv,2) # return vector as TRUE/FALSE
#' as.boolean(multv,3) # return vector as 1/0
#'
#' @export
#'
as.boolean <- function(ds, type = 3) {
  # lowercase of entry
  .d <- tolower(ds)
  # convert to 1/0
  .d[.d %in% c("t", "true", "yes", "y")] <- 1L
  .d[.d %in% c("f", "false", "no", "n")] <- 0L
  NULL # if type is not 1:3
  switch(type,
    "1" = as.factor(gsub("^0$", "No", gsub("^1$", "Yes", .d))),
    "2" = as.logical(as.numeric(.d)),
    "3" = as.factor(as.numeric(.d))
  )
}






#' Convert Yes/No to Binary or Logical
#'
#' Seamlessly convert a yes or no to either a binary or logical output
#'
#' @param table data frame
#' @param fldname field name in the data frame
#' @param out output form, choices - change, append, vector
#' @param type output type, choices - bin, log
#' @return converted Yes/No entries into 1/0 or TRUE/FALSE
#' @details
#' type - "bin" for binary, and "log" for logical
#'
#' @examples
#' \donttest{
#' # Declare data for example
#' usedata <- data.frame(ID = 1:32)
#' usedata #view the dataset
#'
#' usedata$yess = rep(c("yes","n","no","YES","No","NO","yES","Y"),4) #create a new column
#' usedata #view the modified dataset
#'
#' # Set all yess field as standardize boolean
#' # Task: convert the "yess" column content to 1/0 or TRUE/FALSE
#' # Notice that you have add the column name with or without quotes
#' yesNoBool(usedata,yess, type="bin") #set all as binary 1/0
#' yesNoBool(usedata,"yess", type="log") #set all as logical TRUE/FALSE
#'
#'
#' # Task: By default, the 'out' argument is set to "change"
#' # means that the original data field will be
#' # replaced with the results as above
#'
#' # In this example, set the out variable to
#' # append data frame with a new column name containing the result
#'
#' yesNoBool(usedata,yess,"append")
#' #or yesNoBool(usedata,"yess","append")
#'
#' # In this example, return as vector
#' yesNoBool(usedata,yess,"vector")
#' #or yesNoBool(usedata,"yess","vector")
#'
#' # Task: Return result as logical
#' yesNoBool(usedata,"yess",type = "log")
#' }
#' @export
#'
yesNoBool <- function(table,fldname, out = c("change","append","vector"), type = c("bin","log")){
  if(typeof(table) != "list") stop("A data frame must be used.")
  .tt <- switch (match.arg(type), "bin" = 3, "log" = 2  )
  .cook <- as.character(substitute(fldname))
  switch (match.arg(out),
    "change" = {table[,.cook] <- as.boolean(table[,.cook],.tt)},
    "append" = {table$new__col <- as.boolean(table[,.cook],.tt)},
    "vector" = {table <- as.boolean(table[,.cook],.tt)}
  )
  table
}

#' File name extension(s) is Not an image
#'
#' Check if one or multiple file name entry is not an image
#'
#' @param x vector entry
#' @return a boolean value to indicate if entry is not an image
#'
#' @details
#' This current function tests if the extension of the file name provided does NOT belongs to any of the image extensions listed below \cr
#' AI - Adobe Illustrator \cr
#' BMP - Bitmap Image \cr
#' CDR - Corel Draw Picture \cr
#' CGM - Computer Graphics Metafile \cr
#' CR2 - Canon Raw Version 2 \cr
#' CRW - Canon Raw \cr
#' CUR - Cursor Image \cr
#' DNG - Digital Negative \cr
#' EPS - Encapsulated PostScript \cr
#' FPX - FlashPix \cr
#' GIF - Graphics Interchange Format \cr
#' HEIC - High-Efficiency Image File Format \cr
#' HEIF - High-Efficiency Image File Format \cr
#' ICO - Icon Image \cr
#' IMG - GEM Raster Graphics \cr
#' JFIF - JPEG File Interchange Format \cr
#' JPEG - Joint Photographic Experts Group \cr
#' JPG - Joint Photographic Experts Group \cr
#' MAC - MacPaint Image \cr
#' NEF - Nikon Electronic Format \cr
#' ORF - Olympus Raw Format \cr
#' PCD - Photo CD \cr
#' PCX - Paintbrush Bitmap Image \cr
#' PNG - Portable Network Graphics \cr
#' PSD - Adobe Photoshop Document \cr
#' SR2 - Sony Raw Version 2 \cr
#' SVG - Scalable Vector Graphics \cr
#' TIF - Tagged Image File \cr
#' TIFF - Tagged Image File Format \cr
#' WebP - Web Picture Format \cr
#' WMF - Windows Metafile \cr
#' WPG - WordPerfect Graphics
#'
#'
#' @examples
#' img.1 <- "fjk.jpg"
#' not.image(img.1)
#'
#' img.2 <- "fjk.bmp"
#' not.image(img.2)
#'
#' img.3 <- "fjk.SVG"
#' not.image(img.3)
#'
#' # a vector of file names
#' v <- c("logo.png", "business process.pdf",
#' "front_cover.jpg", "intro.docx",
#' "financial_future.doc", "2022 buybacks.xlsx")
#' not.image(v)
#'
#' # when the file name has no extension
#' # the function returns NA
#' v2 <- c("img2.jpg",NA,"northbound.xlsx","landimg")
#' not.image(v2)
#' @export

not.image <- function(x)!{
  is.image(x)
}




#' Is file name extension(s) an image
#'
#' Check if one or multiple file name entry is an image
#'
#' @param x vector entry
#' @return a boolean value to indicate if entry is an image
#'
#' @details
#' This current function tests if the extension of the file name provided belongs to any of the image extensions listed below \cr
#' AI - Adobe Illustrator \cr
#' BMP - Bitmap Image \cr
#' CDR - Corel Draw Picture \cr
#' CGM - Computer Graphics Metafile \cr
#' CR2 - Canon Raw Version 2 \cr
#' CRW - Canon Raw \cr
#' CUR - Cursor Image \cr
#' DNG - Digital Negative \cr
#' EPS - Encapsulated PostScript \cr
#' FPX - FlashPix \cr
#' GIF - Graphics Interchange Format \cr
#' HEIC - High-Efficiency Image File Format \cr
#' HEIF - High-Efficiency Image File Format \cr
#' ICO - Icon Image \cr
#' IMG - GEM Raster Graphics \cr
#' JFIF - JPEG File Interchange Format \cr
#' JPEG - Joint Photographic Experts Group \cr
#' JPG - Joint Photographic Experts Group \cr
#' MAC - MacPaint Image \cr
#' NEF - Nikon Electronic Format \cr
#' ORF - Olympus Raw Format \cr
#' PCD - Photo CD \cr
#' PCX - Paintbrush Bitmap Image \cr
#' PNG - Portable Network Graphics \cr
#' PSD - Adobe Photoshop Document \cr
#' SR2 - Sony Raw Version 2 \cr
#' SVG - Scalable Vector Graphics \cr
#' TIF - Tagged Image File \cr
#' TIFF - Tagged Image File Format \cr
#' WebP - Web Picture Format \cr
#' WMF - Windows Metafile \cr
#' WPG - WordPerfect Graphics
#'
#'
#' @examples
#' img.1 <- "fjk.jpg"
#' is.image(img.1)
#'
#' img.0 <- "fjk.bbVG"
#' is.image(img.0)
#'
#' img.2 <- "fjk.bmp"
#' is.image(img.2)
#'
#' img.3 <- "fjk.SVG"
#' is.image(img.3)
#'
#' # a vector of file names
#' v <- c("logo.png", "business process.pdf",
#' "front_cover.jpg", "intro.docx",
#' "financial_future.doc", "2022 buybacks.xlsx")
#'
#' is.image(v)
#'
#'
#' # when the file name has no extension
#' # the function returns NA
#' v2 <- c("img2.jpg","northbound.xlsx","landimg",NA)
#' is.image(v2)
#'
#' @export

is.image <- function(x){
  exts <- tools::file_ext(tolower(x))
  res <- unlist(lapply(exts, function(ext) ext %in% imageext  ))
  res[c(which(!nchar(exts) | is.na(exts)))] = NA
  res
}


# Future goal of the is.image is to be able to read into files and determine if they are image files !! More development to be done

# Functions that generally go together often

#' Split a string of numbers and return as numeric vector
#'
#' Given a sting, split by a separator into numbers
#'
#' @param x character vector, each element of which is to be split. Other inputs, including a factor, will give an error.
#' @param split character vector
#' @param perl logical. Should Perl-compatible regexps be used?
#' @param useBytes 	logical. If TRUE the matching is done byte-by-byte rather than character-by-character, and inputs with marked encodings are not converted.
#' @param fixed logical. If TRUE match split exactly, otherwise use regular expressions. Has priority over perl.
#' @return numeric values based on split string
#'
#' @description
#' The purpose of this function is combine the functionality of
#' \strong{strsplit}, \strong{unlist} and \strong{as.numeric}, which are often used together.\cr\cr
#'
#' @examples
#' # Example 1
#' # string of numbers with separator " "
#' num.01 = "5 3 2 3 5 2 33 23 5 32 432 42 23 554"
#'
#' # split a string of numbers and return as numeric
#' strsplit.num(num.01, split = " ")
#'
#'
#' # Example 2
#' # string of numbers with separator "|||"
#' num.02 = "0|||1|||4|||43|||6|||8|||00||| 1||| 0 1||| T |||F|||TRUE |||f"
#'
#' # split a string of numbers and return as numeric
#' strsplit.num(num.02, split = "[|||]")
#'
#' @export

strsplit.num <-
  function(x,
           split,
           fixed = FALSE,
           perl = FALSE,
           useBytes = FALSE) {
    as.numeric(unlist(strsplit(x, split, fixed, perl, useBytes)))
  }



#' Split a string of values and return as boolean vector
#'
#' Given a sting, split by a separator into boolean
#'
#' @param x character vector, each element of which is to be split. Other inputs, including a factor, will give an error.
#' @param split character vector
#' @param perl logical. Should Perl-compatible regexps be used?
#' @param useBytes 	logical. If TRUE the matching is done byte-by-byte rather than character-by-character, and inputs with marked encodings are not converted.
#' @param fixed logical. If TRUE match split exactly, otherwise use regular expressions. Has priority over perl.
#' @param type type of return, see the as.boolean function for more info
#' @return boolean values based on split string
#'
#' @description
#' The purpose of this function is combine the functionality of
#' \strong{strsplit}, \strong{unlist} and \strong{as.logical}, which are often used together.\cr\cr
#'
#' @examples
#' # string of numbers
#' num.01 = "0 1 0 0 1 0 1 T F TRUE FALSE t f"
#'
#' # split a string of numbers and return as boolean 1/0
#' strsplit.bool(num.01, split = " ", type = 3)
#'
#' # split a string of numbers and return as boolean TRUE/FALSE
#' strsplit.bool(num.01, split = " ", type = 2)
#'
#' # split a string of numbers and return as boolean Yes/No
#' strsplit.bool(num.01, split = " ", type = 1)
#'
#'
#' # string of numbers
#' num.02 = "0abc1abc0abc0abc1abc0abc1abcTabcFabcTRUEabcFALSEabcf"
#'
#' # split a string of numbers and return as boolean 1/0
#' strsplit.bool(num.02, split = "abc", type = 3)
#'
#' # split a string of numbers and return as boolean TRUE/FALSE
#' strsplit.bool(num.02, split = "abc", type = 2)
#'
#' # split a string of numbers and return as boolean Yes/No
#' strsplit.bool(num.02, split = "abc", type = 1)
#'
#' @export

strsplit.bool <-
  function(x,
           split,
           fixed = FALSE,
           perl = FALSE,
           useBytes = FALSE,
           type = 2) {
    as.boolean(unlist(strsplit(x, split, fixed, perl, useBytes)), type = type)
  }




#' Read a CSV and preview first X rows and columns
#'
#' Read a dataset of type csv and show x rows and y columns with
#' one function call
#'
#' @param dim dimension of CSV content to show
#' @inheritParams utils::read.csv
#' @importFrom utils read.csv
#' @return read csv content and a print out of the data head
#'
#' @description
#' The purpose of this function is combine the functionality of
#' \strong{read.csv} and \strong{print}, which are often used together.\cr\cr
#' The purpose of this function is to read data from a file into a variable and
#' simultaneously display a preview of the data, showing either the first few rows or
#' columns based on the user's specification. It is important to emphasize that the
#' function expects the user to assign the result of the read operation to a variable
#' in order to achieve its intended purpose. eg. Use \strong{var1 = read.csv.print(file1)} instead of
#' \strong{read.csv.print(file1)}
#'
#'
#' @examples
#' \dontrun{
#' # Example: read a csv file and print the first 10 lines
#' # declare file
#' new.file <- "test.csv"
#'
#' # read file and preview default
#' dth3 <- read.csv.print(file = new.file)
#'
#' # read file and preview 10 rows and all columns
#' dth1 <- read.csv.print(file = new.file, dim = 10)
#'
#' # read file and preview 10 rows and 5 columns
#' dth2 <- read.csv.print(file = new.file, dim = c(10,5))
#' }
#' @export

read.csv.print <-
  function(file,
           header = TRUE,
           sep = ",",
           quote = "\"",
           dec = ".",
           fill = TRUE,
           comment.char = "",
           ...,
           dim = c(10L, 5L)) {
    x <- read.table.print(
      file = file,
      header = header,
      sep = sep,
      quote = quote,
      dec = dec,
      fill = fill,
      comment.char = comment.char,
      dim = 0,
      ...
    )
    if (length(dim) > 1 & dim[1] != 0) {
      if (prod(dim) > 0) {
        prtr(x[1:dim[1],1:dim[2]])
      }
    } else {
      prtr(x[1:dim[1],])
    }
    x
  }

#' Read in a table and show first X rows and columns
#'
#' Read a dataset of type table and show x rows and y columns
#'
#' @param dim dimension of table content to show
#' @inheritParams utils::read.table
#' @importFrom utils read.table
#' @return read table content and a print out of the data head
#'
#' @description
#' The purpose of this function is combine the functionality of
#' \strong{read.table} and \strong{print}, which are often used together.\cr\cr
#' The purpose of this function is to read table from a file into a variable and
#' simultaneously display a preview of the data, showing either the first few rows or
#' columns based on the user's specification. It is important to emphasize that the
#' function expects the user to assign the result of the read operation to a variable
#' in order to achieve its intended purpose. eg. Use \strong{var1 = read.table.print(file1)} instead of
#' \strong{read.table.print(file1)}
#'
#'
#' @examples
#' \dontrun{
#' # Example: read a table file and print the first 10 lines
#' # declare file
#' new.file <- "test.csv"
#'
#' # read file and preview default
#' dth3 <- read.table.print(file = new.file, sep=",",quote = "\"",dec = ".",
#' fill = TRUE, comment.char = "", header = TRUE)
#'
#' # read file and preview 10 rows and all columns
#' dth1 <- read.table.print(file = new.file, sep=",",quote = "\"",dec = ".",
#' fill = TRUE, comment.char = "", header = TRUE, dim = 10)
#'
#' # read file and preview 10 rows and 5 columns
#' dth2 <- read.table.print(file = new.file, sep=",",quote = "\"",dec = ".",
#' fill = TRUE, comment.char = "", header = TRUE, dim = c(10,5))
#' }
#' @export

read.table.print <-
  function(file,
           header = FALSE,
           sep = "",
           quote = "\"'",
           dec = ".",
           numerals = c("allow.loss", "warn.loss", "no.loss"),
           row.names,
           col.names,
           as.is = TRUE,
           na.strings = "NA",
           colClasses = NA,
           nrows = -1,
           skip = 0,
           check.names = TRUE,
           fill = NULL,
           strip.white = FALSE,
           blank.lines.skip = TRUE,
           comment.char = "#",
           allowEscapes = FALSE,
           flush = FALSE,
           stringsAsFactors = FALSE,
           fileEncoding = "",
           encoding = "unknown",
           skipNul = FALSE,
           dim = c(10L, 5L),
           ...) {
    x <- read.table(
      file = file,
      header = header,
      sep = sep,
      quote = quote,
      dec = dec,
      numerals = match.arg(numerals),
      row.names = row.names,
      col.names = col.names,
      as.is = as.is,
      na.strings = na.strings,
      colClasses = colClasses,
      nrows = nrows,
      skip = skip,
      check.names = check.names,
      fill = fill,
      strip.white = strip.white,
      blank.lines.skip = blank.lines.skip,
      comment.char = comment.char,
      allowEscapes = allowEscapes,
      flush = flush,
      stringsAsFactors = stringsAsFactors,
      fileEncoding = fileEncoding,
      encoding = encoding,
      skipNul = skipNul,
      ...
    )
    if (length(dim) > 1 & dim[1] != 0) {
      if (prod(dim) > 0) {
        prtr(x[1:dim[1],1:dim[2]])
      }
    } else {
      if(dim[1] != 0) prtr(x[1:dim[1],])
    }

    x
  }

#' Add data to another data like array_push in PHP
#'
#' Shorthand to add data to a dataset and save as the same name
#'
#' @param . first data set
#' @param add data set to add
#' @param which where to append the new data e.g. rows or cols
#' @return the combined dataset store to a variable with the name of the first
#' @examples
#' # initialize p1 and p2
#' init(p1,p2)
#' p1
#' p2
#'
#' # declare p1 and p2 as data frame
#' p1 <- data.frame(PK=1:10,ID2=1:10)
#' p2 <- data.frame(PK=11:20,ID2=21:30)
#'
#' p1
#' p2
#'
#' #add p1 to p2 by row, and resave as p1
#' data_push(p1,p2,"rows")
#' # p2 # p2 remains the same
#' p1 #p1 has been updated
#'
#' # declare a new data frame called p3
#' p3 <- data.frame(Hindex=number(20),Rindex=number(20,seed=20))
#'
#' # add p3 to p1 as column, and resave as p1
#' data_push(p1,p3,"cols")
#' p1 # p1 has been updated
#' @export
#'
data_push <- function(., add, which = c("rows", "cols")) {
  which <- match.arg(which)
  .. <- substitute(.)

  if (typeof(..) != "symbol") stop(paste0(.., " must be an object."))

  data <- as.data.frame(get(as.character(..), envir = parent.frame()))
  add <- as.data.frame(add)
  switch(which,
         "rows" = {
           data <- rbind(data,add)
         },
         "cols" = {
           data <- cbind(data,add)
         }
  )
  assign(as.character(..), data, envir = parent.frame())
}

#' quickcode
#'
#' @name quickcode
#' @docType package
#' @importFrom grDevices graphics.off
#' @importFrom stats sd
#' @noRd
#' @keywords internal
"_PACKAGE"

.onLoad <- function(libname,pkgname){
  utils::globalVariables(c("graphics.off","vali", "sd", "x",'pattern','replacement'))
}

#' Remove last n elements or specified elements from a vector like array_pop in PHP
#'
#' Shorthand to remove elements from a vector and save as the same name
#'
#' @param . parent vector
#' @param n number of elements to remove
#' @param el vector to remove
#' @param ret TRUE or FALSE. whether to return value instead of setting it to the parent vector
#' @return vector with elements removed
#' @examples
#' # basic example: pop off the last 2 values from vector
#' c(0,3,"A","Apple", TRUE) #before
#' vector_pop(c(0,3,"A","Apple", TRUE)) #after 1 pop
#' vector_pop(c(0,3,"A","Apple", TRUE), n=3) #after 3 pop
#'
#' # using objects
#' num1 <- sample(330:400,10)
#' name1 <- "ObinnaObianomObiObianom"
#'
#' #task: remove 1 element from the end of the vector and set it to the vector name
#' num1 #num1 vector before pop
#' vector_pop(num1) #does not return anything
#' num1 #num1 vector updated after pop
#'
#' #task: remove 5 elements from the end, but do not set it to the vector name
#' num1 #num1 vector before pop
#' vector_pop(num1,5, ret = TRUE) #return modified vector
#' num1 #num1 vector remains the same after pop
#'
#'
#' #task: remove 6 elements from a word, set it back to vector name
#' name1 #name1 before pop
#' vector_pop(name1,6) #does not return anything
#' name1 #name updated after pop
#'
#' #task: remove 3 elements from a word, Do not set it back to vector name
#' name1 #name1 before pop
#' vector_pop(name1,3, ret = TRUE) #returns modified name1
#' name1 #name1 not updated after pop
#'
#' #task: remove 4 elements from the end of a vector and return both the removed content and remaining
#' v_f_num <- paste0(number(20),c("TI")) #simulate 20 numbers and add TI suffix
#' v_f_num #show simulated numbers
#' vector_pop(v_f_num, n = 4, ret = TRUE) #get the modified vector
#' vector_pop(v_f_num, n = 4, ret = "removed") #get the content removed
#'
#' #task: remove specific items from vector
#' #note that this aspect of the functionality ignores the 'n' argument
#' v_f_num_2 <- paste0(number(6, seed = 33),c("AB")) #simulate 6 numbers using seed and add AB suffix
#' v_f_num_2 #show numbers
#' vector_pop(v_f_num_2, el = c("403211378AB")) #remove 1 specific entries
#' v_f_num_2 #show results
#' vector_pop(v_f_num_2, el = c("803690460AB","66592309AB")) #remove 2 specific entries
#' v_f_num_2 #show results
#'
#' @export
#'
vector_pop <- function(., n = 1, el = NULL, ret = FALSE){
  .. <- substitute(.)
  if (typeof(..) == "language"){
    return(.[1:(length(.)-n)])
  }
  if (typeof(..) != "symbol") stop(paste0(.., " must be an object."))

  init(val,vali,value = get(as.character(..), envir = parent.frame()))

  if(length(val) > 1){
    if (n > length(val))
      stop(paste0("Value of n must not be greater than length of vector content"))
    if(all(not.empty(el))) val <- val[val %nin% el]
    else val <- val[1:(length(val)-n)]
  }else{
    val1 <- strsplit(val,"")[[1]]
    if(not.empty(el)) val <- val1[val1 %nin% el]
    else val <- paste(val1[1:(length(val1)-n)], collapse = "")
  }

  if(ret == FALSE){
    # if return is false, then set the changed vector to the first
    assign(as.character(..),val, envir = parent.frame())
  }else{
    # if return is set to TRUE, return changed vector
    if(ret == TRUE){
      return(val)
    }
    # if return is set to "removed", return removed elements
    if(ret == "removed"){
      return(setdiff(vali,val))
    }
  }
}

#' Append date to filename
#'
#' Add today's date to the filename
#'
#' @param ... file name or path to concat
#' @param format time format e.g. \%d-\%b-\%Y , refer to \code{\link{date3to1}} for date formats
#' @details
#' The present function enables users to add the current date to the file name,
#' facilitating the straightforward saving of files with their respective dates.
#' It accepts different file paths and names as arguments, as demonstrated
#' in the example section. This functionality simplifies the process of associating
#' a file's creation date with its name, aiding users in recalling when a file was
#' saved. Moreover, it serves as a preventive measure against unintentional
#' overwriting of files created on different dates.
#'
#' @return file name with the current date added as suffix
#' @examples
#'
#' # Task 1
#' fAddDate("path1/","path2/filepre","filemid","fileend.png")
#'
#' # Task 2
#' fAddDate(c("path1/","path2/"),"filepre","filemid","fileend.png")
#'
#' # Task 3
#' fAddDate("one_file_name_fileend.pdf")
#'
#' # Task 4
#' fAddDate(c("path1/","path2/"),"file1-","filemid",c("fileend.png",".pdf"))
#'
#' # Task 5
#' data("USArrests")
#' USArrests$fn = paste0(row.names(USArrests), ".txt")
#' head(fAddDate(USArrests$fn),10)
#'
#' # Task 6: format date - month.day.year
#' fAddDate("sample_file_name.pdf", format = "%B.%d.%YYYY")
#'
#' # Task 7: format date - year_month
#' fAddDate("sample_file_name.pdf", format = "%YYYY_%m")
#'
#'
#' @export

fAddDate <- function(...,format = "%d-%b-%Y"){
  stopifnot(...length()>0)
  combine <- paste0(...)
  apply(data.frame(f=combine,e=tools::file_ext(combine)), 1, function(e){
    gsub(paste0("\\.",e['e'],"$"),
         paste0("_",format(Sys.time(), format),
                paste0(".",e['e'])),e['f'])
  })
}

#' Extract all IP addresses from a string
#'
#' Parse a string and vectorize the IP addresses within it
#'
#' @rdname cybersecurity
#' @param input input string containing IP
#' @return extract_IP returns a vector of IP addresses
#'
#' @examples
#' \donttest{
#' # Extract all IP addresses from a string
#'
#' # Example 1
#' # This example demonstrates the separate
#' # extraction of an IP address one per vector element:
#' str1 = c("Two IP(66.544.33.54) addresses
#' were discovered in the scan.
#' One was at 92.234.1.0.",
#' "The other IP was 62.3.45.255.")
#' extract_IP(str1)
#'
#' # Example 2
#' # This example demonstrates the extraction
#' # of multiple IP addresses from a single vector element:
#' str2 = "Two IP addresses were discovered
#' in the scan. One was at 92.234.1.0.
#' The other IP was 62.3.45.255."
#' extract_IP(str2)
#' }
#' @export

extract_IP <- function(input) {
  # Match an IP address pattern
  ip_pattern <- "\\b(?:\\d{1,3}\\.){3}\\d{1,3}\\b"

  # Extract all occurrences of IP addresses
  extracted_ips <- gregexpr(ip_pattern, input)

  # Deconstruct the list object
  # Return the extracted IP addresses (or an empty list if not found)
  unlist(regmatches(input, extracted_ips))
}

#' Duplicate a data rows or columns X times
#'
#' Add a data to itself X times by rows or columns
#'
#' @param . data frame variable
#' @param n multiples of duplicate
#' @param which where to append the duplicated data e.g. rows or cols
#' @return the duplicated dataset store to a variable with the name of the first
#' @examples
#' # initialize p1 and p2
#' init(p1,p2)
#' p1
#' p2
#'
#' # declare p1 and p2 as data frame
#' p1 <- data.frame(PK=1:10,ID2=1:10)
#' p2 <- data.frame(PK=11:20,ID2=21:30)
#'
#' p1
#' p2
#'
#' #add p1  twice by row, and resave as p1
#' data_rep(p1,n=2,"rows")
#' p1 #p1 has been updated
#'
#'
#' #add p2  3 times by col, and resave as p2
#' data_rep(p2,n=3,"cols")
#' p2 #p2 has been updated
#' @export
#'
data_rep <- function(., n, which = c("rows", "cols")) {
  which <- match.arg(which)
  .. <- substitute(.)
  stopifnot(n>1)

  if (typeof(..) != "symbol"){
    message("Did you forget to declare the variable first? See package examples.??quickcode::data_rep")
    stop(paste0(.., " must be an object. "))
  }

  data <- as.data.frame(get(as.character(..), envir = parent.frame()))

  switch(which,
         "rows" = {
           data <- do.call(rbind, lapply(1:n,function(i)data))
         },
         "cols" = {
           data <- do.call(cbind, lapply(1:n,function(i)data))
         }
  )
  assign(as.character(..), data, envir = parent.frame())
}

# Future release

##' Check Multicollinearity Between Two Covariate Vectors
##'
##' This function calculates the Variance Inflation Factor (VIF) to assess multicollinearity
##' between two covariate vectors. A VIF value greater than 10 indicates high multicollinearity.
##'
##' @ param x A numeric vector representing the first covariate (e.g., AGE).
##' @ param y A numeric vector representing the second covariate (e.g., WEIGHT).
##' @ return A list containing the VIF value and a message about the level of multicollinearity.
##' @ importFrom car vif
##' @ export
##'
##' @ examples
##' # Example 1
##' age <- c(25, 30, 35, 40, 45)
##' weight <- c(60, 70, 80, 90, 100)
##' check_collinear(age, weight)
##'
##' # Example 2
##' age2 <- c(20, 22, 24, 26, 28)
##' weight2 <- c(50, 55, 53, 58, 60)
##' check_collinear(age2, weight2)
##'
##' # Example 3
##' age3 <- c(30, 35, 40, 45, 50)
##' weight3 <- c(70, 75, 80, 85, 90)
##' check_collinear(age3, weight3)
##'
##' # Example 4
##' age4 <- c(60, 62, 65, 67, 70)
##' weight4 <- c(80, 82, 85, 90, 95)
##' check_collinear(age4, weight4)
##'
##' # Example 5
##' age5 <- c(10, 15, 20, 25, 30)
##' weight5 <- c(30, 35, 40, 45, 50)
##' check_collinear(age5, weight5)
#'
#' check_collinear <- function(x, y) {
#'   # Combine variables into a data frame
#'   data <- data.frame(x = x, y = y)
#'   # Fit a linear model to check for multicollinearity
#'   model <- stats::lm(y ~ x, data = data)
#'   # Calculate VIF
#'   vif_value <- car::vif(model)
#'   # Check VIF value
#'   if (vif_value > 10) {
#'     message <- "High multicollinearity detected."
#'   } else {
#'     message <- "Multicollinearity is not a concern."
#'   }
#'   return(list(VIF = vif_value, Message = message))
#' }

#' Generate a random number (integer)
#'
#' Fetch n random integers between 1 and 1,000,000,000
#'
#' @param n how many numbers to generate
#' @param max.digits maximum number of digits in each number
#' @param seed set seed for sampling to maintain reproducibility
#' @return random numbers between 1 and 1 billion
#'
#' @examples
#' number(1)
#' number(10)
#' paste0(number(2),LETTERS)
#'
#' #set maximum number of digits
#' number(1,max.digits = 5)
#' number(10,max.digits = 4)
#'
#' #set seed for reproducibility
#' #without seed
#' number(6) #result 1
#' number(6) #result 2, different from result 1
#' #with seed
#' number(6,seed=1)#result 3
#' number(6,seed=1)#result 4, same as result 3
#' @export
#'
number <- function(n,max.digits=10,seed=NULL){
  if(not.null(seed))set.seed(seed)
  as.integer(substr(sample(1L:1000000000L, n),0,max.digits))
}

#' simple function chaining routine
#'
#' chain multiple function to a call
#'
#' @rdname simplechaining
#' @param obj data object to apply function
#' @param funcs function chains to apply to the data object
#' @param sep separator for functions argument values
#' @return the result of applying the chained functions to the data object
#' @note
#' chain_sep allows the user to preset the separator for the function chaining \cr\cr
#' e.g. you can call the function to set sep = "__" before using the %.% call to parse the chained function "__unique__sum"
#' @examples
#' #use default sep ".."
#' 1:3%.%unique..length
#' sample(1:1000,10,replace=TRUE) %.%unique..length
#' sample(1:10,10,replace=TRUE) %.%unique..cumsum
#'
#' # set sep before function chaining
#' chain_sep("__")
#' sample(1:10,10,replace=TRUE) %.%unique__cumsum
#' sample(1:10,10,replace=TRUE) %.%unique__cumsum__length
#'
#'
#' # set sep before function chaining
#' chain_sep("X")
#' sample(1:10,10,replace=TRUE) %.%uniqueXcumsum
#' @export

`%.%` <- function(obj, funcs){
  sep. <- options()$quickcode_chain_sep %or% "\\.\\."
  .pF <- as.character(substitute(funcs))
  .pF2 <- trimws(strsplit(.pF,sep.)[[1]])

  for(u in .pF2) obj<- do.call(u,list(obj))
  #lapply(.pF2, function(l) eval(parse(text = paste0("obj<<- ",l,"(obj)"))))
  obj
}


#' @export
#' @rdname simplechaining
chain_sep <- function(sep = "\\.\\."){
  options(quickcode_chain_sep = sep)
}





#' Combine specific functions as store as one function
#'
#' Use many functions in one call using predeclared calls
#'
#' @param ... functions to include. see example for how to use
#' @param otherargs other arguments for use in each function
#' @rdname simplechaining2
#' @return returns a function that combines multiple function
#' @export
#' @examples
#'
#' # Example 1 with base functions
#' combf1 <- chain_func(unique, cumsum, print)
#' result <- combf1(sample(1:5,10,replace = TRUE))
#' #or
#' u = sample(1:5,10,replace = TRUE)
#' result <- combf1(u)
#'
#' # Example 2 with base functions with arguments
#' combf2 <- chain_func(unique, print, otherargs = list(.= c(FALSE),.=c(2)))
#' result <- combf2(sample(1:3,10,replace = TRUE))
#'
#' # Example 3 with custom functions
#' r = function(a,b,c){
#'   if(!missing(a))print(a)
#'   if(!missing(b))print(b)
#'   if(!missing(c))print(c)
#'   return(a)
#' }
#'
#' r2 = function(a,b,c){
#'   if(!missing(a))message(a)
#'   if(!missing(b))message(b)
#'   if(!missing(c))message(c)
#'   return(a)
#' }
#'
#' combf3 <- chain_func(r,r2, otherargs =list(.=c("apple","cat"),.=c("rice")))
#' res <- combf3(head(mtcars))

chain_func <- function(...,otherargs = list()) {
  # Store the functions in a list
  functions <- list(...)
  # Create a function to apply the chained functions
  return(function(x) {
    result <- x
    for (func in indexed(functions)) {
      list1 = list(result)
      for (le in unlist(otherargs[func$key])) list1<- c(list1,list(le))
      result <- do.call(func$value,list1)
    }
    return(result)
  })
}

#' Combine vector to create Date, or split Date into vector
#'
#' Combine or split Date into a specified format
#'
#' @rdname date_tweaks
#'
#' @references
#' Adapted from Ecfun R package
#' @examples
#' # EXAMPLES FOR date3to1
#'
#' data0 <- data.frame(y=c(NA, -1, 2001:2009),
#' m=c(1:2, -1, NA, 13, 2, 12, 6:9),
#' d=c(0, 0:6, NA, -1, 32) )
#' head(data0)
#'
#' # combine and convert to date
#' # return as data frame
#' date3to1(data0)
#'
#' # combine and convert to date
#' # return as vector
#' date3to1(data0, as.vector = TRUE) #eg. 2004-02-04
#'
#'
#' # combine and convert to date in the format DD_MM_YYYY
#' date3to1(data0, out.format = "%d_%m_%Y") #eg. 04_02_1974
#'
#'
#' # combine and convert to date in the format MM_DD_YY
#' date3to1(data0, out.format = "%m_%d_%y") #eg. 02_04_74
#'
#' # combine and convert to date in the various date formats
#' date3to1(data0, out.format = "%B %d, %y") #eg. February 04, 74
#' date3to1(data0, out.format = "%a, %b %d, %Y") #eg. Mon, Feb 04, 1974
#' date3to1(data0, out.format = "%A, %B %d, %Y") #eg. Monday, February 04, 1974
#' date3to1(data0, out.format = "Day %j in Year %Y") #eg. Day 035 in Year 1974
#' date3to1(data0, out.format = "Week %U in %Y") #eg. Week 05 in 1974
#' date3to1(data0, out.format = "Numeric month %m in Year %Y") #eg. Numeric month 02 in Year 1974
#'
#' @param data data frame object
#' @param out.format date output format
#' @param col.YMD columns to combine for Year, Month and Day
#' @param as.vector return output as vector, or leave as data frame
#' @return date derived from combining values from three columns of a data frame
#'
#' @section Note:
#' \strong{DATE FORMATS IN R}\cr
#' \tabular{rrrrr}{
#' \strong{Date Specification}   \tab \tab \strong{Description}          \tab \tab  \strong{Example} \cr
#' \%a  \tab \tab Abbreviated weekday             \tab \tab Sun, Thu \cr
#' \%A  \tab \tab Full weekday                    \tab \tab Sunday \cr
#' \%b  \tab \tab Abbreviated month               \tab \tab May, Jul \cr
#' \%B  \tab \tab Full month                      \tab \tab March, July \cr
#' \%d  \tab \tab Day of the month                \tab \tab 27, 07 \cr
#' \%j  \tab \tab Day of the year                 \tab \tab 148, 188 \cr
#' \%m  \tab \tab Month                           \tab \tab 05, 07 \cr
#' \%U  \tab \tab Week, with Sunday as first day  \tab \tab 22, 27 \cr
#' \%w  \tab \tab Weekday, Sunday is 0            \tab \tab 0, 4 \cr
#' \%W  \tab \tab Week, with Monday as first day  \tab \tab 21, 27 \cr
#' \%x  \tab \tab Date, locale-specific           \tab \tab \cr
#' \%y  \tab \tab Year without century            \tab \tab 84, 05 \cr
#' \%Y  \tab \tab Year with century               \tab \tab 1984, 2005 \cr
#' \%C  \tab \tab Century                         \tab \tab 19, 20 \cr
#' \%D  \tab \tab Date formatted \%m/\%d/\%y      \tab \tab 07/17/23 \cr
#' \%u  \tab \tab Weekday, Monday is 1            \tab \tab 7, 4 \cr
#' }
#'
#' @details
#' \strong{NOTE for date3to1}\cr
#'
#' The three input columns corresponding to "Year Month Day" must be numeric values. \cr\cr
#' For example, Do not provide the month variable as non-numeric such as "Mar", "Jul", or "Jan". \cr\cr
#' If the values of the columns are non-numeric, the results will return an "NA" in the output.date column.
#'
#' @export

date3to1 <-
  function(data,
           out.format = "%Y-%m-%d",
           col.YMD = 1:3,
           as.vector = FALSE
  ){

    stopifnot(inherits(data,"data.frame")) # data must be a data frame
    if(has.error(data[,col.YMD]))
      stop("The columns for Year Month Day (col.YMD) does not exist in the dataset")
    or.names<- names(data)
    names(data)[col.YMD] = c("..yyyy..","..mm..","..dd..")

    b.out <- within(data,{
      output.date = as.POSIXct(paste0(..yyyy..,"-",..mm..,"-",..dd..),format="%Y-%m-%d")
      if(out.format!="%Y-%m-%d")
        output.date = format(output.date, out.format)
    })

    if(as.vector){
      as.character(b.out$output.date)
    }else{
      names(b.out) <- c(or.names,'output.date')
      b.out
    }
  }





#'
#' @examples
#'
#'
#'
#'
#' # EXAMPLES FOR date1to3
#'
#' data1 <- data.frame(Full.Dates =
#'                       c("2023-02-14",NA,NA,
#'                         "2002-12-04","1974-08-04",
#'                         "2008-11-10"))
#' head(data1)
#'
#' # split date with default settings
#' # return as data frame with columns
#' # for day(d), month(m) and year(Y)
#' date1to3(data1)
#'
#'
#' # split date in the format and only return year in YYYY
#' date1to3(data1, out.cols = "%Y") #eg. 2002, 2023
#'
#'
#' # split date in the format and only return month in m
#' date1to3(data1, out.cols = "%m") #eg. 02, 12, 08
#'
#' # split date in the format and return multiple date formats colums
#' date1to3(data1, out.cols = c("%B","%d") )
#' date1to3(data1, out.cols = c("%a","%b","%y") )
#' date1to3(data1, out.cols = c("%A","%B","%Y","%y") )
#' date1to3(data1, out.cols = c("%j","%Y","%y","%m") )
#' date1to3(data1, out.cols = c("%U","%Y","%y","%x") )
#' date1to3(data1, out.cols = c("%m","%Y","%y","%C") )
#'
#' @param data data frame object
#' @param in.format date input format
#' @param date.col numeric value of column within the dataset that contains the dates
#' @param out.cols cols to of date items to split. Make sure to conform to date formats. See "NOTE" section
#' for date formats
#' @rdname date_tweaks
#' @export
date1to3 <-
  function(data,
           in.format = "%Y-%m-%d",
           date.col = 1,
           out.cols = c("%Y", "%m", "%d")
  ){

    stopifnot(inherits(data,"data.frame"), length(out.cols) > 0) # data must be a data frame
    if(has.error(data[,date.col]))
      stop("The columns for Year Month Day (col.YMD) does not exist in the dataset")

    .prevn = names(data)[date.col]
    names(data)[date.col] = "...dMywhole_"

    b.out <- within(data,{
      output.date = as.POSIXct(...dMywhole_,format=in.format)
      for(iui in rev(out.cols))
        assign(paste0(".date_",iui),format(output.date,iui))
      rm(iui,output.date)
    })

    names(b.out)[date.col] = .prevn
    b.out
  }

#' @note
#' getDistribution() checks if data fits multiple distributions
#' @rdname distribution_check
#' @param data the data to check against distributions
#' @param alpha p-value alpha level
#' @export
#'

getDistribution <- function(data, alpha = 0.05) {
  setDisAlpha(alpha = alpha)
  dists <- list(
    lognormal = is.lognormal(data, alpha = alpha, method = 1),
    normal = is.normal(data, alpha = alpha, method = 1),
    uniform = is.uniform(data, alpha = alpha),
    poisson = is.poisson(data, alpha = alpha),
    gamma = is.gamma(data, alpha = alpha),
    logistic = is.logistic(data, alpha = alpha),
    weibull = is.weibull(data, alpha = alpha),
    cauchy = is.cauchy(data, alpha = alpha)
  )
  unsetDisAlpha()

  dists
}

#' Fetch R package based on keyword
#'
#' This function gets R packages based on a keyword in their description.
#'
#' @param keyword character, the keyword to search package descriptions for.
#'
#' @return character vector of matching package names or NULL if no matches.
#'
#' @note
#' the function in its current form only searches available.packages()
#'
#' @examples
#' \donttest{
#' # find the list of R packages for data or machine learning
#'
#' matched_pkgs <- suppressWarnings(find_packages("plotting"))
#' matched_pkgs
#'
#' matched_pkgs <- suppressWarnings(find_packages("machine learning"))
#' matched_pkgs
#' }
#' @export


find_packages <- function(keyword) {
  options(repos = structure(c(CRAN = "https://cran.r-project.org")))
  packages <- rownames(utils::available.packages())

  matched_pkgs <- c()

  for (pkg in packages) {
    tryCatch({
      desc <- utils::packageDescription(pkg)$Description

      if (grepl(keyword, desc, ignore.case = TRUE)) {
        matched_pkgs <- c(matched_pkgs, pkg)
      }
    },
    error = function(e) {
    })
  }

  if (length(matched_pkgs) == 0) {
    return(NULL)
  } else {
    return(matched_pkgs)
  }
}

#' Initialize new variables and objects
#'
#' Shorthand to initialize one or more objects
#'
#' @param ... variable names to initialize
#' @param value value to initialize them to
#' @return initialized objects set to the value specified
#'
#' @examples
#' init(t,u,v)
#' message(t) # t = NULL
#' message(u) # u = NULL
#' message(v) # v = NULL
#' init(j,k,m,value = 7)
#' message(j) # j = 7
#' message(k) # k = 7
#' message(m) # m = 7
#'
#' @export
#'
init <- function(...,value = NULL){
  .v <- as.list(substitute(args(...))[-1L])
  for(i in .v)
    assign(as.character(i), value , envir = parent.frame())
}

#' Not a vector
#'
#' Opposite of is.vector(). Check if entry is not vector
#'
#' @param x vector entry
#' @return a boolean value to indicate if entry is vector
#' @examples
#' vect1 = list(r=1,t=3:10)
#' vect2 = LETTERS
#' is.vector(vect1) # TRUE
#' not.vector(vect1) # FALSE
#' not.vector(vect2) # FALSE
#' if(not.vector(vect1)) message("yes") # NULL
#'
#' @export

not.vector <- function(x)!{
  is.vector(x)
}

# print all the environment object and sizes and connections to each other
#' Get all the environment objects and their sizes
#'
#' Retrieve the size contribution of all the available objects in the environment
#'
#' @param envir the environment to retrieve objects from
#' @return a dataframe of all the variables within the environment
#' @examples
#' # Get a data frame of all environment objects and their size
#' summarize.envobj()
#' @export
summarize.envobj <- function(envir = parent.frame()){
  envs = ls(all.names = TRUE, envir = envir)
  envs.size = c()
  for(b in  envs) vector_push(envs.size, eval(parse(text = paste0("object.size(",b,")"))))
  within(data.frame(objects = c(envs,"TOTAL"), size.bytes = c(envs.size,sum(envs.size))),{
    size.kbytes = size.bytes/1000
    size.mbytes = size.kbytes/1000
  })
}


# Future updates: version 2
# within(data.frame(size.bytes = unlist(
#   sapply(ls(all.names = TRUE, envir = envir), function(n) {
#     object.size(get(n))
#   }, simplify = FALSE)
# )),{
#   size.kbytes = size.bytes/1000
# })

#' Not numeric
#'
#' Check if entry is not numeric
#'
#' @param x vector entry
#' @return a boolean value to indicate if entry is numeric
#' @examples
#' not.numeric("45") # TRUE
#' not.numeric(45) # FALSE
#' if(not.numeric(45)) message("yes") # yes
#'
#' @export

not.numeric <- function(x) !is.numeric(x)

---
title: "Easily add today's date to filenames in R using quickcode package"
author: "Obinna (OBI) Obianom"
date: "`r Sys.Date()`"
output: rmarkdown::html_vignette
vignette: >
  %\VignetteIndexEntry{Easily add today's date to filenames in R using quickcode package}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
---

```{r}
# quickcode::clean(clearPkgs = TRUE) #clear environment and clear previous package
quickcode::libraryAll(datasets) #load in-built dataset
```

## fAddDate function

#### In R, data often gets saved to files for future use or subsequent processes. However, without a version control system in place, ensuring the consistency and reproducibility of saved data can pose challenges, particularly when scripts evolve over time. The "fAddDate" function addresses such concerns by automatically appending the current date to the file name, mitigating the risk of data loss and enhancing reproducibility. By simplifying the process with just a single line of code, users no longer need to manually update the date of each run, thereby streamlining workflow efficiency. Reproducibility in programming is crucial for maintaining the integrity of analyses and fostering trust in research outcomes. Ensuring that results can be consistently replicated not only enhances transparency but also facilitates collaboration and builds confidence in the reliability of findings. Therefore, by incorporating automated date stamping, this function not only safeguards against data inconsistency but also reinforces the fundamental principles of reproducible research in R programming.

![](https://quickcode.obi.obianom.com/fadddate.png)

## Usage

Let's load the USArrests data. And let's assume that we want to save data to the system using the content of the row names as our file names.

```{r}
data("USArrests")

filenames = paste0(row.names(USArrests), ".txt")

head(filenames,8) # before adding today's date

rev.filenames = fAddDate(filenames) # append today's date to the file names

rev.filenames

#...write further codes to use the file names to save new files.

```

## List of other cool functions to use

```

%nin%	Not in vector or array
add.header	Addin snippet function to add header comment to a current opened file
add.sect.comment	Addin snippet function to custom section comment
add.snippet.clear	Snippet R function to clear console and set directory
add_key	Add index keys to a vector or data frame or list or matrix
ai.duplicate	Prompt guided duplication of files
archivedPkg	Listing of all CRAN archived R packages
as.boolean	Convert boolean values between formats
bionic_txt	Generate a bionic text
clean	Clear environment, clear console, set work directory and load files
compHist	Compare histograms of two distributions
data_pop	Remove last n rows or column or specified elements from a data frame like array_pop in PHP
data_pop_filter	Remove elements from a data matching filter
data_push	Add data to another data like array_push in PHP
data_rep	Duplicate a data rows or columns X times
data_shuffle	Shuffle a data frame just like shuffle in PHP
date1to3	Combine vectors to create Date, or split Date into vectors
date3to1	Combine vectors to create Date, or split Date into vectors
duplicate	Duplicate a file with global text replacement
fAddDate	Append date to filename
genRandImg	Download random images from the web
geo.cv	Calculate geometric coefficient of variation, mean, or SD and round
geo.mean	Calculate geometric coefficient of variation, mean, or SD and round
geo.sd	Calculate geometric coefficient of variation, mean, or SD and round
has.error	Check if a call or expression produces errors
header.rmd	Snippet function to add header to a current Rmd opened file
in.range	If number falls within a range of values and get closest values
inc	Increment vector by value
init	Initialize new variables and objects
insertInText	Shiny app function to insert string to current file in RStudio
is.gamma	Check if a data fits a Normal or LogNormal or Uniform or Poisson or Gamma distribution
is.image	Is file name extension(s) an image
is.logistic	Check if a data fits a Normal or LogNormal or Uniform or Poisson or Gamma distribution
is.lognormal	Check if a data fits a Normal or LogNormal or Uniform or Poisson or Gamma distribution
is.normal	Check if a data fits a Normal or LogNormal or Uniform or Poisson or Gamma distribution
is.poisson	Check if a data fits a Normal or LogNormal or Uniform or Poisson or Gamma distribution
is.uniform	Check if a data fits a Normal or LogNormal or Uniform or Poisson or Gamma distribution
is.weibull	Check if a data fits a Normal or LogNormal or Uniform or Poisson or Gamma distribution
libraryAll	Load specific R libraries and clear environment
list_push	Add elements to a list like array_push in PHP
list_shuffle	Shuffle a list object just like shuffle in PHP
minus	Decrease vector by value
mix.color	Mix or Blend two or more colors
mix.cols.btw	Mix or Blend colors between two or more colors
multiply	Multiple a vector of numeric values
newSuperVar	Create and use a super variable with unique capabilities
not.data	Not a data
not.duplicated	Not duplicated elements
not.empty	Not empty
not.environment	Not an environment
not.exists	Not exists
not.image	File name extension(s) is Not an image
not.integer	Not an integer
not.logical	Not logical
not.na	Not NA
not.null	Not NULL
not.numeric	Not numeric
not.vector	Not a vector
number	Generate a random number (integer)
pairDist	Calculate the distance of points from the center of a cluster
plus	Increment vector by value
randString	Generate a random string
rcolorconst	R Color Constant
rDecomPkg	Check whether an R package has been decommissioned in CRAN
read.csv.print	Read a CSV and preview first X rows and columns
read.table.print	Read in a table and show first X rows and columns
refresh	Clear environment, clear console, set work directory and load files
sample_by_column	Re-sample a dataset by column and return number of entry needed
setOnce	Set a variable only once
sim.logistic	Generate logistic random values
strsplit.bool	Split a string of values and return as boolean vector
strsplit.num	Split a string of numbers and return as numeric vector
summarize.envobj	Get all the environment objects and their sizes
vector_pop	Remove last n elements or specified elements from a vector like array_pop in PHP
vector_push	Add elements to a vector like array_push in PHP
vector_shuffle	Shuffle a vector just like shuffle in PHP
yesNoBool	Convert Yes/No to Binary or Logical

```

---
title: "5. Introducing the Nullish coalescing operator in the quickcode R package"
author: "Obinna (OBI) Obianom"
date: "`r Sys.Date()`"
output: rmarkdown::html_vignette
vignette: >
  %\VignetteIndexEntry{5. Introducing the Nullish coalescing operator in the quickcode R package}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
---

```{r setup, include = FALSE}
knitr::opts_chunk$set(
  collapse = TRUE,
  comment = "#>"
)
```

## About the nullish coalescing operator

The nullish coalescing operator, also known as the nullish assignment operator, is a logical operator that was introduced in ECMAScript 2020. It has the form ?? and is used to define a default value for a variable if that variable is null or undefined.

In statistical programming and data analysis with R, dealing with missing or undefined values is very common. Oftentimes variables may be initialized without a value, or functions may return undefined if no suitable value can be determined. Previously in R, programmers would have to explicitly check if a value was NULL using if/else statements before using it or assigning a default.

The nullish coalescing operator with the Quickcode R package helps address this in a more succinct way. It allows defining a fallback value inline, without needing an additional check. This can make code more readable where default values are assigned.


## Example of usage

```

# Use the nullish coalescing operator using  "%or%"
ex.V1 <- 5
ex.V2 <- NA
ex.V3 <- NULL
ex.V1 <- ""
alternative <- 500

# Give an alternative result if the test is NULL NA or empty

ex.V1 %or% alternative # result will give 5 because ex.V1 is not NULL NA or empty

ex.V2 %or% alternative # result will give 500 because ex.V2 is NA

ex.V3 %or% alternative # result will give 500 because ex.V3 is NULL

ex.V4 %or% alternative # result will give 500 because ex.V4 is empty

# Further chaining

ex.V2 %or% ex.V1 %or% alternative # result will be 5 because ex.V2 is NA but ex.V1 is 5

ex.V2 %or% ex.V2 %or% alternative # result will be 500 because ex.V2 is NA and ex.V3 is NULL

```

---
title: "Download random HD images from various unsplash.com categories in R using the Quickcode package"
author: "Obi Obianom and Brice Richard"
date: "2023-10-20"
output: html_document
---

```{r}
# install quickcode if not available
# install.packages(quickcode)

# load quickcode library
library(quickcode)
#clean() #clear the environment

# let's load the r2resize to provide toolbars for the image resizing
library(r2resize)
add.resizer(line.color = "brown", line.height = "3", default.image.width = "30%")

```

## Download 3 random images from the school category and display

For this example, we will download 3 random png images from the web in the category of 'school' to a temp directory and then display it. The r2resize package provides the image resizer that lets us view the images nicely below.

```{r download school images}
storlocation <- tempdir()
genRandImg(fp = storlocation, n = 3, cat = "school", ext = "png")

getdownloadedfiles = list.files(path = storlocation, pattern = ".png", full.names = TRUE)
```

Display the images:

`r paste0("![](",getdownloadedfiles[1],")")`
`r paste0("![](",getdownloadedfiles[2],")")`
`r paste0("![](",getdownloadedfiles[3],")")`



## Download 6 random images from the animals category and display

For this example, we will download 6 random jpg images from the web in the category of 'school' to a temp directory and then display it. 

```{r download animals images}
storlocation2 <- tempdir()
genRandImg(fp = storlocation2, n = 6, cat = "animals", ext = "jpg")

getdownloadedfiles = list.files(path = storlocation2, pattern = ".jpg", full.names = TRUE)

```

Display the images:

`r paste0("![](",getdownloadedfiles[1],")")`
`r paste0("![](",getdownloadedfiles[2],")")`
`r paste0("![](",getdownloadedfiles[3],")")`
`r paste0("![](",getdownloadedfiles[4],")")`
`r paste0("![](",getdownloadedfiles[5],")")`
`r paste0("![](",getdownloadedfiles[6],")")`

## Quick links

 - quickcode github: https://github.com/oobianom/quickcode
 - quickcode CRAN: https://cran.r-project.org/web/packages/quickcode/index.html
 - raw Rmd file for this HTML: https://github.com/oobianom/quickcode/blob/main/inst/sample%20codes/downloadImages.Rmd

---
title: "A comprehensive guide to the NOT functions for data validation in quickcode"
author: "Obinna (OBI) Obianom"
date: "`r Sys.Date()`"
output: rmarkdown::html_vignette
vignette: >
  %\VignetteIndexEntry{A comprehensive guide to the NOT functions for data validation in quickcode}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
---

```{r setup, include = FALSE}
knitr::opts_chunk$set(
  collapse = TRUE,
  comment = "#>"
)
```

## Why NOT functions

Base R gives you plenty of `is.*()` checks — `is.null()`, `is.na()`, `is.numeric()`, `is.vector()` and so on — but it does not give you their opposites. In practice, negated checks are extremely common: "is this argument *not* missing", "is this object *not* a data frame", "is this value *not* already in my lookup vector". Without a dedicated function, every one of these checks ends up written as `!is.null(x)`, `!is.na(x)`, `!(x %in% table)`, scattered across a script with a leading `!` that is easy to miss when skimming code, and easy to accidentally drop or double up when editing.

The quickcode package's NOT functions exist to make these checks first-class, readable, and safe to compose. `not.null(x)` reads exactly like the English sentence it represents, so validation code and guard clauses stop requiring a mental "flip the meaning" step every time a reader hits a `!`.

This vignette walks through the full family of NOT functions, what each one actually checks, and some patterns for using them together to validate function inputs and clean up data.

## The full NOT function family

```

not.null          Opposite of is.null() - check if entry is NOT NULL
not.na            Opposite of is.na() - check if entry is NOT NA
not.empty         Check if entry is NOT empty (NULL, NA, or "")
is.empty          Check if entry IS empty (companion to not.empty)
not.numeric       Opposite of is.numeric()
not.integer       Opposite of is.integer()
not.logical       Opposite of is.logical()
not.vector        Opposite of is.vector()
not.data          Opposite of is.data.frame()
not.environment   Opposite of is.environment()
not.Date          Opposite of inherits(x, "Date")
not.duplicated    Opposite of duplicated()
not.image         Opposite of is.image() - file extension is NOT an image type
not.inherits      Opposite of inherits() - object does NOT inherit from given class(es)
not.exists        Opposite of exists() - object does NOT exist in scope
has.error         Whether an expression or function call produces an error
%nin%             Opposite of %in% - value is NOT in a vector/table (alias %!in%)

```

Every function above returns a logical value (or logical vector), so they drop directly into `if()`, `while()`, `stopifnot()`, `Filter()`, or logical indexing, exactly like their `is.*()` counterparts.

## Core object checks

### not.null

```

is.null("")          # FALSE
not.null("")         # TRUE
not.null(NULL)        # FALSE

if (not.null(45)) message("something") # prints "something"

```

`not.null()` is the one you will reach for constantly, especially at the top of a function to make sure a required argument was actually supplied.

### not.na

```

not.na(NA)                     # FALSE
not.na(NULL)                    # logical(0), since is.na(NULL) is also logical(0)

if (not.na(45)) message("something") # prints "something"

```

Note that, just like `is.na()`, `not.na()` is vectorized: passing a vector returns a logical vector of the same length, one result per element.

### not.empty / is.empty

```

not.empty("empty")     # TRUE
not.empty('')           # FALSE
not.empty(y <- NULL)    # FALSE

is.empty("")            # TRUE
is.empty("empty")       # FALSE

if (not.empty('')) message("yes") # nothing printed, condition is FALSE

```

`not.empty()` combines three checks in one call: it is FALSE whenever the value is NULL, is NA, or is the empty string `""`. This is convenient for validating text inputs, such as form fields or CLI arguments, where any of those three "nothing was provided" states should be treated the same way. `is.empty()` is simply the inverse, provided so both directions read naturally depending on context.

## Type and class checks

### not.numeric, not.integer, not.logical, not.vector, not.data, not.environment

```

not.numeric("45")   # TRUE  (a character string is not numeric)
not.numeric(45)      # FALSE

is.integer(78L)      # TRUE
not.integer(78L)     # FALSE
not.integer(23.43)   # TRUE, since 23.43 is a double, not an integer

test.env <- TRUE
test.notenv <- 0
not.logical(test.env)     # FALSE
not.logical(test.notenv)  # TRUE

vect1 <- list(r = 1, t = 3:10)
vect2 <- LETTERS
not.vector(vect1) # FALSE, a list is still a vector in R's type system
not.vector(vect2) # FALSE

test.dt <- data.frame(ID = 1:200, Type = "RPKG.net")
test.notdf <- list(t = 1)
not.data(test.dt)    # FALSE
not.data(test.notdf) # TRUE

test.env2 <- new.env()
test.notenv2 <- list(t = 1)
not.environment(test.env2)     # FALSE
not.environment(test.notenv2)  # TRUE

```

These map one-to-one onto their base R `is.*()` counterparts, so they carry the exact same semantics — `not.integer(45)` is TRUE for the same reason `is.integer(45)` is FALSE: an unsuffixed numeric literal in R is a double, not an integer, unless written as `45L`.

### not.Date

```

d1 <- as.Date("2024-01-01")
d2 <- "2024-01-01"

is.Date(d1)   # TRUE
not.Date(d1)  # FALSE
not.Date(d2)  # TRUE, a plain character string is not a Date object

```

### not.inherits

```

keep.cols <- "a character"
class(keep.cols) # "character"
not.inherits(keep.cols, "character") # FALSE

num.var <- 1L
class(num.var) # "integer"
not.inherits(num.var, "double") # TRUE

```

`not.inherits()` is the negated form of base R's `inherits()`, and accepts the same `what` (a character vector of class names) and `which` arguments, so you can check against several possible classes at once.

## Working with collections

### not.duplicated

```

set.seed(08082023)
dtf <- sample(1:10, 15, replace = TRUE)
dtf # 3  9 10  3  8  9  6 10  5  1  2  2  2  9  8

dtf[dtf > 4 & duplicated(dtf)]     # values that ARE duplicates: 9 10  9  8
dtf[dtf > 4 & not.duplicated(dtf)] # values that are NOT duplicates: 9 10  8  6  5

```

Because `not.duplicated()` mirrors `duplicated()` argument-for-argument (including `incomparables`), it slots directly into logical indexing anywhere you'd normally combine `!duplicated(x)`.

### %nin% (not in)

```

5 %nin% c(1:10)   # FALSE, 5 is in 1:10
5 %nin% c(11:20)  # TRUE, 5 is not in 11:20

x <- "a"
if (x %nin% letters) x # condition is FALSE, so nothing runs

# exclude specific values from a vector
vector_num1 <- number(9, max.digits = 5, seed = 1)
vector_num1[vector_num1 %nin% c(83615, 85229)] # keep everything except those two values

```

`%nin%` also has the alias `%!in%`, for readers who prefer an operator that visually echoes `%in%` with a `!` in front of it.

## File and error checks

### not.image / is.image

```

img.1 <- "fjk.jpg"
not.image(img.1) # FALSE

img.2 <- "fjk.bmp"
not.image(img.2) # FALSE

img.3 <- "fjk.SVG" # extension matching is case-insensitive
not.image(img.3)   # FALSE

v <- c("logo.png", "business process.pdf",
       "front_cover.jpg", "intro.docx",
       "financial_future.doc", "2022 buybacks.xlsx")
not.image(v) # FALSE TRUE FALSE TRUE TRUE TRUE

# when a file name has no extension, both is.image() and not.image()
# return NA rather than guessing
v2 <- c("img2.jpg", NA, "northbound.xlsx", "landimg")
not.image(v2) # FALSE NA TRUE NA

```

`is.image()`/`not.image()` recognize a broad list of common raster and vector image formats (PNG, JPG/JPEG, GIF, BMP, SVG, TIFF, HEIC/HEIF, WebP, PSD, AI, and several camera raw formats among others), so both functions are handy for filtering a directory listing down to (or away from) image files without hand-rolling a regular expression.

### has.error

```

# this should not produce an error, so the result is FALSE
has.error({
  x <- 8
  y <- number(10)
  res <- x + y
})

# this produces an error ("non-numeric argument to binary operator"),
# so the result is TRUE - but the error itself is caught, not thrown
has.error({
  x <- 8
  y <- "random"
  res <- x + y
})

# accessing a column that doesn't exist also errors, so this is TRUE
df1 <- mtcars
has.error(df1[, "rpkg.net"])

```

`has.error()` wraps its argument in `tryCatch()` and reports whether the expression raised an error, without letting that error propagate and stop your script. This makes it useful for validating that user-supplied code, formulas, or file paths behave as expected before you rely on them further downstream, without wrapping every call site in its own `tryCatch()` block.

## Putting it together: a validation guard clause

A common pattern is to combine several NOT functions at the top of a function to validate its inputs before doing any real work:

```

process_dataset <- function(df, id_col) {

  if (not.data(df)) stop("df must be a data.frame")
  if (is.empty(id_col)) stop("id_col must not be empty")
  if (id_col %nin% names(df)) stop("id_col must be one of the column names in df")
  if (not.numeric(df[[id_col]])) stop("the id_col column must be numeric")

  # ... continue processing, confident the inputs are valid
  df
}

```

Because every NOT function returns a plain logical value, they compose with `&&`, `||`, `all()`, and `any()` exactly like base R's own `is.*()` functions — the only difference is that your validation code reads the way you'd say it out loud.

## See also

- `vignette("quickcode_r_introduction")` for a broader tour of the package
- `vignette("nullish_coalescing_operator_r")` for the closely related `%or%` and `%eo%` operators, which supply a fallback value instead of just testing for one

---
title: "NOT functions, is.log, is.poisson, is.weibull, date3to1, genRandImg, vector_shuffle, data_pop,  and init function amongst many great new functions in the quickcode R package"
author: "Obinna (OBI) Obianom, Brice Richard"
date: "`r Sys.Date()`"
output: rmarkdown::html_vignette
vignette: >
  %\VignetteIndexEntry{NOT functions, is.log, is.poisson, is.weibull, date3to1, genRandImg, vector_shuffle, data_pop,  and init function amongst many great new functions in the quickcode R package}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
---

```{r setup, include = FALSE}
knitr::opts_chunk$set(
  collapse = TRUE,
  comment = "#>"
)
```


### A compilation of NOT functions + some useful functions to simply R programming, keep the console neat, and save memory usage 

## Install

```
devtools::install_github("oobianom/quickcode")

install.packages("quickcode") # when published

```

## List of many cool functions to use

```

%nin%	Not in vector or array
add.header	Addin snippet function to add header comment to a current opened file
add.sect.comment	Addin snippet function to custom section comment
add.snippet.clear	Snippet R function to clear console and set directory
add_key	Add index keys to a vector or data frame or list or matrix
ai.duplicate	Prompt guided duplication of files
archivedPkg	Listing of all CRAN archived R packages
as.boolean	Convert boolean values between formats
bionic_txt	Generate a bionic text
clean	Clear environment, clear console, set work directory and load files
compHist	Compare histograms of two distributions
data_pop	Remove last n rows or column or specified elements from a data frame like array_pop in PHP
data_pop_filter	Remove elements from a data matching filter
data_push	Add data to another data like array_push in PHP
data_rep	Duplicate a data rows or columns X times
data_shuffle	Shuffle a data frame just like shuffle in PHP
date1to3	Combine vectors to create Date, or split Date into vectors
date3to1	Combine vectors to create Date, or split Date into vectors
duplicate	Duplicate a file with global text replacement
fAddDate	Append date to filename
genRandImg	Download random images from the web
geo.cv	Calculate geometric coefficient of variation, mean, or SD and round
geo.mean	Calculate geometric coefficient of variation, mean, or SD and round
geo.sd	Calculate geometric coefficient of variation, mean, or SD and round
has.error	Check if a call or expression produces errors
header.rmd	Snippet function to add header to a current Rmd opened file
in.range	If number falls within a range of values and get closest values
inc	Increment vector by value
init	Initialize new variables and objects
insertInText	Shiny app function to insert string to current file in RStudio
is.gamma	Check if a data fits a Normal or LogNormal or Uniform or Poisson or Gamma distribution
is.image	Is file name extension(s) an image
is.logistic	Check if a data fits a Normal or LogNormal or Uniform or Poisson or Gamma distribution
is.lognormal	Check if a data fits a Normal or LogNormal or Uniform or Poisson or Gamma distribution
is.normal	Check if a data fits a Normal or LogNormal or Uniform or Poisson or Gamma distribution
is.poisson	Check if a data fits a Normal or LogNormal or Uniform or Poisson or Gamma distribution
is.uniform	Check if a data fits a Normal or LogNormal or Uniform or Poisson or Gamma distribution
is.weibull	Check if a data fits a Normal or LogNormal or Uniform or Poisson or Gamma distribution
libraryAll	Load specific R libraries and clear environment
list_push	Add elements to a list like array_push in PHP
list_shuffle	Shuffle a list object just like shuffle in PHP
minus	Decrease vector by value
mix.color	Mix or Blend two or more colors
mix.cols.btw	Mix or Blend colors between two or more colors
multiply	Multiple a vector of numeric values
newSuperVar	Create and use a super variable with unique capabilities
not.data	Not a data
not.duplicated	Not duplicated elements
not.empty	Not empty
not.environment	Not an environment
not.exists	Not exists
not.image	File name extension(s) is Not an image
not.integer	Not an integer
not.logical	Not logical
not.na	Not NA
not.null	Not NULL
not.numeric	Not numeric
not.vector	Not a vector
number	Generate a random number (integer)
pairDist	Calculate the distance of points from the center of a cluster
plus	Increment vector by value
randString	Generate a random string
rcolorconst	R Color Constant
rDecomPkg	Check whether an R package has been decommissioned in CRAN
read.csv.print	Read a CSV and preview first X rows and columns
read.table.print	Read in a table and show first X rows and columns
refresh	Clear environment, clear console, set work directory and load files
sample_by_column	Re-sample a dataset by column and return number of entry needed
setOnce	Set a variable only once
sim.logistic	Generate logistic random values
strsplit.bool	Split a string of values and return as boolean vector
strsplit.num	Split a string of numbers and return as numeric vector
summarize.envobj	Get all the environment objects and their sizes
vector_pop	Remove last n elements or specified elements from a vector like array_pop in PHP
vector_push	Add elements to a vector like array_push in PHP
vector_shuffle	Shuffle a vector just like shuffle in PHP
yesNoBool	Convert Yes/No to Binary or Logical

```

## Let me show you a few R Examples

***

```
#initialize one or more variables

print(g) # Error: object 'g' not found

init(g,h,i,o)
print(g) # g = NULL
print(h) # h = NULL

init(r,y,u,b,value = 5)
print(r) # r = 5
print(b) # b = 5
print(z) # Error: object 'z' not found

```

***

```
#add keys to a vector content for use in downstream processes

ver1 <- c("Test 1","Test 2","Test 3")
add_key(ver1)

for(i in ver1){
message(sprintf("%s is the key for this %s", i$key, i$value))
}

```

***

```

#check if the entry is not integer

not.integer(45) #returns TRUE
not.integer(45.) #returns TRUE
not.integer(45L) #returns FALSE

not.null(45L) #returns TRUE
not.null(h<-NULL) #returns FALSE


```

***

```

#clear R environment, set directory and load data
#note: the code below also automatically loads the quickcode library so that all other functions within package can be used easily


quickcode::refresh()
quickcode::clean()

#or combine with setwd and source and load

quickcode::clean(
  setwd = "/wd/",
  source = c(
  "file.R",
  "file2.R"
  ),
  load = c(
  "data.RData",
  "data2.RData"
  )
)


```

***

```

#shorthand for not in vector

p1 <- 4
p2 <- c(1:10)

p1 %nin% p2



```

***

```

#add to a vector in one code

p1 <- c(6,7,8)
p2 <- c(1,2,3)

vector_push(p1,p2)

print(p1)


```

***

```

#add to a data frame in one code

p1 <- data.frame(ID=1:10,ID2=1:10)
p2 <- data.frame(ID=11:20,ID2=21:30)

data_push(p1,p2,"rows")

print(p1)

```

***

```

#remove from a vector in one code

p1 <- c(6,7,8,1,2,3)

vector_pop(p1)

print(p1)


```

***

```



#remove from a data frame in one code

p1 <- data.frame(ID=1:10,ID2=1:10,CD=11:20,BD=21:30)

data_pop(p1) #remove last row

print(p1)

data_pop(p1,5) #remove last 5 rows

print(p1)

```

***

```


#remove columns from a data frame in one code

p1 <- data.frame(ID=1:10,ID2=1:10,ID4=1:10,CD=11:20,BD=21:30)

data_pop(p1,which = "cols") #remove last column

print(p1)

data_pop(p1,2,which = "cols") #remove last 2 columns

print(p1)

data_pop(p1,1,which = "cols") #remove last 1 column and vectorise

print(p1)

```

***

```
#load libraries

quickcode::libraryAll(
  dplyr,
  r2resize,
  ggplot2
)


```

***

```
```


Check out some examples at <u>quickcode.obi.obianom.com</u>

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/distributions.R, R/getdist.R
\name{is.lognormal}
\alias{is.lognormal}
\alias{is.normal}
\alias{is.uniform}
\alias{is.poisson}
\alias{is.gamma}
\alias{is.logistic}
\alias{is.weibull}
\alias{is.cauchy}
\alias{setDisAlpha}
\alias{unsetDisAlpha}
\alias{getDistribution}
\title{Check if a data fits the distribution}
\usage{
is.lognormal(values, alpha = 0.05, method = 1)

is.normal(values, alpha = 0.05, method = 1)

is.uniform(values, alpha = 0.05)

is.poisson(values, alpha = 0.05)

is.gamma(values, alpha = 0.05)

is.logistic(values, alpha = 0.05)

is.weibull(values, alpha = 0.05)

is.cauchy(values, alpha = 0.05)

setDisAlpha(alpha = 0.05)

unsetDisAlpha()

getDistribution(data, alpha = 0.05)
}
\arguments{
\item{values}{vector of values}

\item{alpha}{p-value alpha level}

\item{method}{method for calculation, where 1 = Shapiro-Wilk test and 2 = Kolmogorov-Smirnov test}

\item{data}{the data to check against distributions}
}
\value{
boolean value if lognormal distributed

boolean value if normal distributed

boolean value if uniform distributed

boolean value if poisson distributed

boolean value if gamma distributed

boolean value if logistic distributed

boolean value if logistic distributed

boolean value if cauchy distributed

setDisAlpha sets global significance level for testing of distribution

unsetDisAlpha removes global significance level for testing of distribution
}
\description{
Check whether a vector of data contains values that fit a distribution
}
\details{
This function takes a numeric vector as its input. This vector contains the dataset that will be analyzed.
\cr\cr
\strong{For Normal and LogNormal:}\cr\cr
- Method 1: we perform the Shapiro-Wilk test on the (log-transformed) data to test for normality.
The null hypothesis of the Shapiro-Wilk test is that the data are normally distributed.
If the p-value is greater than the chosen significance level (typically 0.05),
we fail to reject the null hypothesis, indicating that the data may follow a log-normal distribution.\cr\cr
- Method 2: we perform the Kolmogorov-Smirnov test on the log-transformed data, comparing it to a normal distribution with the same mean and standard deviation. Again, if the p-value is greater than the chosen significance level, it suggests that the data may follow a log-normal distribution.
These tests provide a statistical assessment of whether your data follows a log-normal distribution.
}
\note{
getDistribution() checks if data fits multiple distributions
}
\examples{
# Set global alpha for testing significance
setDisAlpha(alpha = 0.05)

# Prepare all data to test
# Set the seed for reproducibility
set.seed(13200323)
lognormal_data <- stats::rlnorm(n = 4000, meanlog = 1, sdlog = 1) #lognormal data
normal_data <- stats::rnorm(n = 4000, mean = 10, sd = 3) #normal data
uniform_data <- stats::runif(4000,min=0,max=10) #uniform data
poisson_data <- stats::rpois(4000, lambda = 5) #poisson data
gamma_data <- stats::rgamma(4000,shape = 5, rate = 2) #gamma data
logis_data <- stats::rlogis(4000, location = 4, scale = 2)#logistic values
weibull_data <- stats::rweibull(4000, shape = 4, scale = 2) #weibull data
cauchy_data <- stats::rcauchy(4000, location = 8, scale = 5) #cauchy data

# EXAMPLE FOR is.lognormal

# Test if the data is lognormal
is.lognormal(lognormal_data)
is.lognormal(normal_data)
is.lognormal(uniform_data)
is.lognormal(poisson_data)
is.lognormal(gamma_data)
is.lognormal(logis_data)
is.lognormal(weibull_data)
is.lognormal(cauchy_data)
is.lognormal(1:4000)

# EXAMPLE FOR is.normal

# Test if the data fits a normal distribution
is.normal(lognormal_data)
is.normal(normal_data)
is.normal(uniform_data)
is.normal(poisson_data)
is.normal(gamma_data)
is.normal(logis_data)
is.normal(weibull_data)
is.normal(cauchy_data)
is.normal(1:4000)

\dontrun{
# EXAMPLES for is.uniform

# Test if the data fits a uniform distribution
is.uniform(lognormal_data)
is.uniform(normal_data)
is.uniform(uniform_data)
is.uniform(poisson_data)
is.uniform(gamma_data)
is.uniform(logis_data)
is.uniform(weibull_data)
is.uniform(cauchy_data)
is.uniform(1:4000)
}
\dontrun{
# EXAMPLE for is.poisson

# Test if the data fits a poisson distribution
is.poisson(lognormal_data)
is.poisson(normal_data)
is.poisson(uniform_data)
is.poisson(poisson_data)
is.poisson(gamma_data)
is.poisson(logis_data)
is.poisson(weibull_data)
is.poisson(cauchy_data)
is.poisson(1:4000)
}
\dontrun{
# EXAMPLE for is.gamma

# Test if the data fits a gamma distribution
is.gamma(lognormal_data)
is.gamma(normal_data)
is.gamma(uniform_data)
is.gamma(poisson_data)
is.gamma(gamma_data)
is.gamma(logis_data)
is.gamma(weibull_data)
is.gamma(cauchy_data)
is.gamma(1:4000)
}
\dontrun{
# EXAMPLE for is.logistic

# Test if the data fits a logistic distribution
is.logistic(lognormal_data)
is.logistic(normal_data)
is.logistic(uniform_data)
is.logistic(poisson_data)
is.logistic(gamma_data)
is.logistic(logis_data)
is.logistic(weibull_data)
is.logistic(cauchy_data)
is.logistic(1:4000)
}
\dontrun{
# Test if the data fits a weibull distribution
is.weibull(lognormal_data)
is.weibull(normal_data)
is.weibull(uniform_data)
is.weibull(poisson_data)
is.weibull(gamma_data)
is.weibull(logis_data)
is.weibull(weibull_data)
is.weibull(cauchy_data)
is.weibull(1:4000)
}
\dontrun{
# EXAMPLES for is.cauchy

# Test if the data fits a cauchy distribution
is.cauchy(lognormal_data)
is.cauchy(normal_data)
is.cauchy(uniform_data)
is.cauchy(poisson_data)
is.cauchy(gamma_data)
is.cauchy(logis_data)
is.cauchy(weibull_data)
is.cauchy(cauchy_data)
is.cauchy(1:4000)
}
\dontrun{
# set global distribution alpha

# default setting
setDisAlpha()

# set to 0.001
setDisAlpha(alpha = 0.01)
}
\dontrun{
# unset global distribution alpha

unsetDisAlpha()
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/add_key.R
\name{add_key}
\alias{add_key}
\alias{indexed}
\title{Add index  keys to a vector or data frame or list or matrix}
\usage{
add_key(vector)

indexed(vector, key = key, value = value)
}
\arguments{
\item{vector}{vector or data frame to transform}

\item{key}{variable name for keys}

\item{value}{variable name for values}
}
\value{
a transformed list containing keys along with vector values
}
\description{
Index a vector or lists and convert to a list of objects
}
\details{
This function takes a vector and turns it into a list containing 'key' and 'value' for each vector.
This allows the output to be used in loops such as for loops or lapply or other functions to track
the index of the list content e.g. 1,2,3...\cr\cr

This function also contains a validator to ensure that a vector had not been previously 'keyed',
which prevents the user from inadvertently calling the function twice on a vector. Helps especially
because the function keys the vector, and sets the new list to the variable name of the original
vector.

This function takes a vector and turns it into a list containing 'key' and 'value' for each vector.
This allows the output to be used in loops such as for loops or lapply or other functions to track
the index of the list content e.g. 1,2,3...\cr\cr

This function also contains a validator to ensure that a vector had not been previously 'keyed',
which prevents the user from inadvertently calling the function twice on a vector. Helps especially
because the function keys the vector, and sets the new list to the variable name of the original
vector.
}
\note{
add_key - resaves the keys and value pairs to original variable\cr\cr
indexed - return the keys and value pairs
}
\section{Use case}{

Efficient for loops and for tracking various steps through a vector contents
}

\examples{
# EXAMPLES for add_key()


#ex1 simple conversion of a vector
rti2 <- c("rpkg","obinna", "obianom")
add_key(rti2)
rti2

#ex2 add keys to a vector content for use in downstream processes
ver1 <- c("Test 1","Test 2","Test 3")
add_key(ver1)

#ex3 use keyed ver1 in for loop
for(i in ver1){
  message(sprintf("\%s is the key for this \%s", i$key, i$value))
}

#ex4 use keyed ver1 in lapply loop
xl1 <- lapply(ver1,function(i){
  message(sprintf("lapply - \%s is the key for this \%s", i$key, i$value))
})


# EXAMPLES for indexed()

#ex1 simple conversion of a vector
rti2 <- c("rpkg","obinna", "obianom")
indexed(rti2)

#ex2 add keys to a vector content for use in downstream processes
ver1 <- c("Test 1","Test 2","Test 3")


#ex3 use keyed ver1 in for loop
for(i in indexed(ver1)){
  message(sprintf("\%s is the key for this \%s", i$key, i$value))
}

#ex4 use keyed ver1 in for loop
#specify name for key and value
for(i in indexed(ver1,k,v)){
  message(
  sprintf("\%s is the new key for this value \%s",
  i$k, i$v))
}

#ex5 use keyed ver1 in lapply loop
xl1 <- lapply(indexed(ver1),function(i){
  message(sprintf("lapply - \%s is the key for this \%s", i$key, i$value))
})


}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/not.numeric.R
\name{not.numeric}
\alias{not.numeric}
\title{Not numeric}
\usage{
not.numeric(x)
}
\arguments{
\item{x}{vector entry}
}
\value{
a boolean value to indicate if entry is numeric
}
\description{
Check if entry is not numeric
}
\examples{
not.numeric("45") # TRUE
not.numeric(45) # FALSE
if(not.numeric(45)) message("yes") # yes

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/not.na.R
\name{not.na}
\alias{not.na}
\title{Not NA}
\usage{
not.na(x)
}
\arguments{
\item{x}{vector entry}
}
\value{
a boolean value to indicate if entry is NA
}
\description{
Opposite of is.na(). Check if entry is not NA
}
\examples{
not.na(NA) # FALSE
not.na(NULL) # logical(0)
if(not.na(45)) message("something") # TRUE

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/add.header.R
\name{add.sect.comment}
\alias{add.sect.comment}
\title{Addin snippet function to custom section comment}
\usage{
add.sect.comment()
}
\value{
Inserts section comment content for file
}
\description{
Shorthand to add section comment to current file
}
\examples{
if(interactive())
add.sect.comment()
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/aided_chatgpt.R
\name{from_tensor_slices}
\alias{from_tensor_slices}
\title{Create Tensor-Like Slices from a Data Frame or Matrix}
\usage{
from_tensor_slices(data)
}
\arguments{
\item{data}{A data frame or matrix to be converted into slices.}
}
\value{
A list of slices, each corresponding to a row of the input data.
}
\description{
This function converts a data frame or matrix into row-wise slices, similar to TensorFlow's `from_tensor_slices()`.
}
\examples{
df <- data.frame(A = 1:3, B = c("x", "y", "z"))
from_tensor_slices(df)

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/aided_chatgpt.R
\name{normalize.vector}
\alias{normalize.vector}
\title{Normalize a Numeric Vector to the Range [0, 1]}
\usage{
normalize.vector(x)
}
\arguments{
\item{x}{A numeric vector to be normalized.}
}
\value{
A numeric vector with values scaled to the range [0, 1].
        If all values are identical, all values are set to 0.
}
\description{
This function normalizes a numeric vector so that all values are scaled to the range [0, 1].
}
\examples{
# Example with a numeric vector
normalize.vector(c(1, 2, 3, 4, 5))

# Example with a vector containing identical values
normalize.vector(c(3, 3, 3))

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/shapes.R
\name{create_shape}
\alias{create_shape}
\title{Create geometric shapes with optional text}
\usage{
create_shape(
  shape = c("circle", "square", "rectangle", "triangle", "hexagon", "star", "ellipse",
    "polygon"),
  x_center = 0,
  y_center = 0,
  size = 1,
  width = size,
  height = size,
  n_sides = 5,
  rotation = 0,
  fill_color = NA,
  border_color = "black",
  line_width = 1,
  title = "Geometric Shape",
  text = NULL,
  text_color = "black",
  text_size = 1,
  text_font = 1,
  text_angle = 0,
  show_axes = FALSE,
  show_ticks = FALSE,
  show_axis_labels = FALSE
)
}
\arguments{
\item{shape}{Character string specifying the shape to create. Must be one of:
"circle", "square", "rectangle", "triangle", "hexagon", "star", "ellipse", "polygon".}

\item{x_center}{Numeric, x-coordinate of the shape's center. Default is 0.}

\item{y_center}{Numeric, y-coordinate of the shape's center. Default is 0.}

\item{size}{Numeric, the primary size parameter (radius for circle, side length for square, etc.). Default is 1.}

\item{width}{Numeric, width for rectangle and ellipse. Default is size.}

\item{height}{Numeric, height for rectangle and ellipse. Default is size.}

\item{n_sides}{Integer, number of sides for polygon. Default is 5.}

\item{rotation}{Numeric, rotation angle in degrees. Default is 0.}

\item{fill_color}{Character string or NA, the fill color of the shape. Default is NA (transparent).}

\item{border_color}{Character string, the border color of the shape. Default is "black".}

\item{line_width}{Numeric, the width of the border line. Default is 1.}

\item{title}{Character string, the title of the plot. Default is "Geometric Shape".}

\item{text}{Character string, optional text to display inside the shape. Default is NULL (no text).}

\item{text_color}{Character string, color of the text. Default is "black".}

\item{text_size}{Numeric, size of the text. Default is 5.}

\item{text_font}{Character string, font family for the text. Default is "".}

\item{text_angle}{Numeric, rotation angle for the text in degrees. Default is 0.}

\item{show_axes}{Logical. Whether to display the axes. Default is FALSE.}

\item{show_ticks}{Logical. Whether to display axis ticks. Default is FALSE.}

\item{show_axis_labels}{Logical. Whether to display axis labels (X and Y). Default is FALSE.}
}
\value{
A list containing:
  \itemize{
    \item plot: The ggplot object
    \item coordinates: Data frame with x and y coordinates of the shape's vertices
    \item center: Vector with x and y coordinates of the shape's center
    \item size: Primary size parameter used
    \item shape: Shape type
    \item text: Text displayed inside the shape (if any)
  }
}
\description{
This function creates various geometric shapes using base R graphics.
It supports multiple shape types including circle, square, rectangle,
triangle, hexagon, star, ellipse, and regular polygons with custom sides.
}
\examples{
\dontrun{
# Example 1: Basic circle
create_shape(shape = "circle", size = 2, fill_color = "lightblue")

# Example 2: Square with rotation and text
create_shape(
  shape = "square",
  size = 3,
  rotation = 45,
  fill_color = "salmon",
  border_color = "darkred",
  line_width = 2,
  text = "Rotated Square",
  text_color = "darkblue",
  text_size = 3
)

# Example 3: Hexagon with custom title
create_shape(
  shape = "hexagon",
  size = 2.5,
  fill_color = "lightgreen",
  border_color = "darkgreen",
  title = "My Hexagon"
)

# Example 4: Star with custom styling
create_shape(
  shape = "star",
  size = 3,
  rotation = 15,
  fill_color = "gold",
  border_color = "orange",
  line_width = 2,
  title = "Gold Star"
)

# Example 5: Rectangle with custom dimensions
create_shape(
  shape = "rectangle",
  width = 4,
  height = 2,
  fill_color = "lavender",
  text = "Rectangle",
  text_size = 2.5
)

# Example 6: Triangle with rotation and styling
create_shape(
  shape = "triangle",
  size = 3,
  rotation = 180,
  fill_color = "pink",
  border_color = "purple",
  line_width = 2,
  text = "rpkg",
  text_color = "black",
  text_size = 4
)

# Example 7: Ellipse with custom dimensions
create_shape(
  shape = "ellipse",
  width = 5,
  height = 2,
  rotation = 30,
  fill_color = "lightcyan",
  border_color = "steelblue",
  title = "Rotated Ellipse"
)

# Example 8: Custom polygon (octagon)
create_shape(
  shape = "polygon",
  n_sides = 8,
  size = 2,
  fill_color = "thistle",
  border_color = "purple",
  title = "Octagon",
  text = "8",
  text_color = "darkblue",
  text_size = 3
)

# Example 9: Overlapping shapes (need to call par() between shapes)
create_shape(
  shape = "circle",
  x_center = 0,
  y_center = 0,
  size = 3,
  fill_color = rgb(1, 0, 0, 0.5),
  title = "Overlapping Shapes"
)

par(new = TRUE)  # Allow adding to the existing plot
create_shape(
  shape = "circle",
  x_center = 1,
  y_center = 1,
  size = 3,
  fill_color = rgb(0, 0, 1, 0.5),
  title = ""  # Empty title to avoid overwriting the previous title
)
# Create a star with text
create_shape("star", size = 2,
                       fill_color = "gold",
                       text = "*",
                       text_size = 8)
}

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/constants.R
\docType{data}
\name{const}
\alias{const}
\title{Mathematical constants}
\format{
An object of class \code{list} of length 2.
}
\usage{
const
}
\description{
List of mathematical functions
}
\details{
const list contains a list of mathematical constants below\cr\cr
- pi\cr
- e: Euler's number\cr
- golden_ratio\cr
- euler_mascheroni: Euler-Mascheroni constant\cr
- feigenbaum: Feigenbaum constant\cr
- champernowne: Champernowne constant\cr
- apery: Roger Apery's constant\cr
- gelfond_schneider: Gelfond-Schneider constant\cr
- khinchin: Aleksandr Khinchin constant\cr
- ramanujan_soldner: Ramanujan-Soldner constant\cr
}
\examples{
# definition of pi
const$defs$pi

# value of pi
const$values$pi


}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/not.empty.R
\name{not.empty}
\alias{not.empty}
\alias{is.empty}
\title{Not empty}
\usage{
not.empty(x)

is.empty(x)
}
\arguments{
\item{x}{vector entry}
}
\value{
a boolean value to indicate if entry is empty
}
\description{
Check if entry is not empty
}
\examples{
not.empty("empty") # TRUE
not.empty('') # FALSE
not.empty(y<-NULL) # FALSE
if(not.empty('')) message("yes") # NULL
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/bool.R
\name{as.boolean}
\alias{as.boolean}
\title{Convert boolean values between formats}
\usage{
as.boolean(ds, type = 3)
}
\arguments{
\item{ds}{item to convert}

\item{type}{format to convert to, choices 1, 2 or 3}
}
\value{
output adhering to the format of the type provided
}
\description{
Convert Yes/No to 1/0 or to TRUE/FALSE or vice versa
}
\details{
Output various format of booleans into a specified format. Below are the options for the type argument.

\strong{type}: options are as follows - \cr\cr
     1 - Yes/No \cr
     2 - TRUE/FALSE\cr
     3 - 1/0\cr
}
\examples{

# Task: convert "yes" or "no" to format of TRUE or FALSE
as.boolean("yes",2)
as.boolean("no",2)
as.boolean("YES",2)
as.boolean("NO",2)

# Task: convert "yes" or "no" to format of 1 or 0
as.boolean("yes",3)
as.boolean("no",3)
as.boolean("YES",3)
as.boolean("NO",3)


# Task: convert 1 to format of Yes or No
as.boolean(1,1)

# Task: convert "T" to format of Yes or No
as.boolean("T",1)


# Task: convert "f" to format of TRUE or FALSE
as.boolean("f",2)


# Task: convert 1 to format of TRUE or FALSE
as.boolean(1,2)


# Task: convert "Y" or "y" to format of Yes or No
as.boolean("Y",1) #uppercase Y
as.boolean("y",1) #lowercase y


# Task: convert TRUE/FALSE to format of 1 or 0
as.boolean(TRUE,3)
as.boolean(FALSE,3)


# Task: convert TRUE/FALSE to format of Yes or No
as.boolean(TRUE,1)
as.boolean(FALSE,1)



# In case of error in argument
# as.boolean("tr",3) #NA
# as.boolean("ye",3) #NA

# vector of mixed boolean to TRUE/FALSE or 1/0
multv <- c(TRUE,"y","n","YES","yes",FALSE,"f","F","T","t")
as.boolean(multv,1) # return vector as Yes/No
as.boolean(multv,2) # return vector as TRUE/FALSE
as.boolean(multv,3) # return vector as 1/0

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/add.snippet.clear.R
\name{add.snippet.clear}
\alias{add.snippet.clear}
\title{Snippet R function to clear console and set directory}
\usage{
add.snippet.clear()
}
\value{
Inserts code to clear console
}
\description{
Shorthand to add clear console code to current file
}
\examples{
if(interactive())
add.snippet.clear()
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/filename.R
\name{fAddDate}
\alias{fAddDate}
\title{Append date to filename}
\usage{
fAddDate(..., format = "\%d-\%b-\%Y")
}
\arguments{
\item{...}{file name or path to concat}

\item{format}{time format e.g. \%d-\%b-\%Y , refer to \code{\link{date3to1}} for date formats}
}
\value{
file name with the current date added as suffix
}
\description{
Add today's date to the filename
}
\details{
The present function enables users to add the current date to the file name,
facilitating the straightforward saving of files with their respective dates.
It accepts different file paths and names as arguments, as demonstrated
in the example section. This functionality simplifies the process of associating
a file's creation date with its name, aiding users in recalling when a file was
saved. Moreover, it serves as a preventive measure against unintentional
overwriting of files created on different dates.
}
\examples{

# Task 1
fAddDate("path1/","path2/filepre","filemid","fileend.png")

# Task 2
fAddDate(c("path1/","path2/"),"filepre","filemid","fileend.png")

# Task 3
fAddDate("one_file_name_fileend.pdf")

# Task 4
fAddDate(c("path1/","path2/"),"file1-","filemid",c("fileend.png",".pdf"))

# Task 5
data("USArrests")
USArrests$fn = paste0(row.names(USArrests), ".txt")
head(fAddDate(USArrests$fn),10)

# Task 6: format date - month.day.year
fAddDate("sample_file_name.pdf", format = "\%B.\%d.\%YYYY")

# Task 7: format date - year_month
fAddDate("sample_file_name.pdf", format = "\%YYYY_\%m")


}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/track.R
\name{extract_comment}
\alias{extract_comment}
\alias{remove_content_in_quotes}
\alias{remove_comment}
\alias{clean_file}
\alias{get_func_def}
\title{Extract all comments or functions from a file}
\usage{
extract_comment(file)

remove_content_in_quotes(line)

remove_comment(line)

clean_file(file, output_file)

get_func_def(file)
}
\arguments{
\item{file}{file to parse}

\item{line}{string vector to remove contents within quotes or comments}

\item{output_file}{file path to write the output of the new file}
}
\value{
vector of all comments within a file
}
\description{
Vectorize all comments from the file
}
\examples{
\dontrun{
ex_file1 <- "path/file1.R"
# get all comments
cmmts <- extract_comment(ex_file1)
cmmts
}

\dontrun{
# Ex to clean out comments from file
file_path <- ".testR"
output_file_path <- ".cleaned_script.R"
clean_file(file_path, output_file_path)
}

# example code

\dontrun{
# Ex to get all defined functions
# within a file
file_path <- ".testR"
get_func_def(file_path)
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/switch.R
\name{switch_rows}
\alias{switch_rows}
\title{Switch the index of two rows in a data set}
\usage{
switch_rows(data, row1, row2, keep.cols = NULL)
}
\arguments{
\item{data}{dataset object}

\item{row1}{numeric. the first row number}

\item{row2}{numeric. the second row number}

\item{keep.cols}{numeric or character. column number or name to keep}
}
\description{
Allows the user to choose precisely which two rows they want to swap places,
while optionally preventing some columns from being altered in the process.
Excluded columns within the rows act as anchors that are immune from the switching operation
on the selected rows.
}
\examples{

# Example using mtcars
data100 <- mtcars[1:7,]

head(data100) # preview overall data

# task 1: basic result of switching rows 5 and 6
head(switch_rows(data100, 5, 6))

# task 2: switch rows, but retain some columns
data100[5:6,2:10] # preview the portion that is to be changed

# lets switch 2 rows, but keep content of columns 7, 8, 9 10 within the changed rows
res1 <- switch_rows(data100, row1 = 5, row2 = 6, keep.cols = 7:10) # use column numbers
res1[5:6,] # check result, pay attention to columns 9 and 10 as well
res2 <- switch_rows(data100,
row1 = 5,
row2 = 6,
keep.cols = c("disp","cyl")) # use column names
res2[5:6,] # check result, pay attention to columns "disp","cyl" as well

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/track.R
\name{track_func}
\alias{track_func}
\title{Track Function Usage and Performance}
\usage{
track_func(output.dest = "output_tracking.csv")
}
\arguments{
\item{output.dest}{destination of csv file to store outputs}
}
\value{
the numeric count of a function usage
}
\description{
Up count the number of times a particular function is called. This function tracker is a higher-order function or decorator that wraps around other functions to monitor and record their execution patterns, including metrics like call frequency, execution time, argument patterns, and return values. It acts as a transparent layer that doesn't modify the original function's behavior but collects valuable metadata about its usage.
}
\note{
The usefulness of function tracking spans several critical areas:\cr\cr
\strong{Performance Optimization}: By measuring execution times and frequency, developers can identify bottlenecks and frequently called functions that need optimization\cr
\strong{Debugging}: Tracking argument patterns and function call sequences helps pinpoint issues in complex applications\cr
\strong{Usage Analytics}: Understanding which features (functions) are most commonly used helps guide development priorities and API design decisions\cr
\strong{Resource Management}: Monitoring function behavior helps identify memory leaks, resource consumption patterns, and potential optimization opportunities\cr
\strong{Testing}: Usage patterns can inform test case design and coverage requirements, ensuring critical paths are well-tested\cr
\strong{Documentation}: Automatically gathering real-world usage examples helps maintain accurate and relevant documentation\cr
\strong{Compliance}: In regulated environments, function tracking can help maintain audit trails of system behavior\cr
}
\examples{
\dontrun{
library(quickcode)
# Track usage of type2 and type1 functions
store.usage.file <- tempfile()
type5 <- function(x) type2(x)
type4 <- function(x) type3(x)
type3 <- function(x) type1(x)
type1 <- function(x) {
  mean(x)
  sd(x)
  track_func(store.usage.file)
}
type2 <- function(x) {
  type1(x)
  track_func(store.usage.file)
}

# add usage counts to store.usage.file
type1(number(10))
type2(number(10))
type3(number(10))
type4(number(10))
type5(number(10))

# check the stored function usage file
print(read.csv(store.usage.file))
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/aided_chatgpt.R
\name{mode.calc}
\alias{mode.calc}
\title{Calculate the Mode of a Numeric or Character Vector}
\usage{
mode.calc(x)
}
\arguments{
\item{x}{A numeric or character vector for which the mode is to be calculated.}
}
\value{
The mode(s) of the input vector. If multiple values have the same highest frequency, all modes are returned.
        Returns `NA` if the input vector is empty.
}
\description{
This function calculates the mode (most frequently occurring value(s)) of a numeric or character vector.
}
\examples{
# Example with a numeric vector
numeric_vector <- c(1, 2, 2, 3, 3, 3, 4, 5)
mode.calc(numeric_vector)

# Example with a character vector
character_vector <- c("apple", "banana", "apple", "orange", "banana", "banana")
mode.calc(character_vector)

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/vector_pop.R
\name{vector_pop}
\alias{vector_pop}
\title{Remove last n elements or specified elements from a vector like array_pop in PHP}
\usage{
vector_pop(., n = 1, el = NULL, ret = FALSE)
}
\arguments{
\item{.}{parent vector}

\item{n}{number of elements to remove}

\item{el}{vector to remove}

\item{ret}{TRUE or FALSE. whether to return value instead of setting it to the parent vector}
}
\value{
vector with elements removed
}
\description{
Shorthand to remove elements from a vector and save as the same name
}
\examples{
# basic example: pop off the last 2 values from vector
c(0,3,"A","Apple", TRUE) #before
vector_pop(c(0,3,"A","Apple", TRUE)) #after 1 pop
vector_pop(c(0,3,"A","Apple", TRUE), n=3) #after 3 pop

# using objects
num1 <- sample(330:400,10)
name1 <- "ObinnaObianomObiObianom"

#task: remove 1 element from the end of the vector and set it to the vector name
num1 #num1 vector before pop
vector_pop(num1) #does not return anything
num1 #num1 vector updated after pop

#task: remove 5 elements from the end, but do not set it to the vector name
num1 #num1 vector before pop
vector_pop(num1,5, ret = TRUE) #return modified vector
num1 #num1 vector remains the same after pop


#task: remove 6 elements from a word, set it back to vector name
name1 #name1 before pop
vector_pop(name1,6) #does not return anything
name1 #name updated after pop

#task: remove 3 elements from a word, Do not set it back to vector name
name1 #name1 before pop
vector_pop(name1,3, ret = TRUE) #returns modified name1
name1 #name1 not updated after pop

#task: remove 4 elements from the end of a vector and return both the removed content and remaining
v_f_num <- paste0(number(20),c("TI")) #simulate 20 numbers and add TI suffix
v_f_num #show simulated numbers
vector_pop(v_f_num, n = 4, ret = TRUE) #get the modified vector
vector_pop(v_f_num, n = 4, ret = "removed") #get the content removed

#task: remove specific items from vector
#note that this aspect of the functionality ignores the 'n' argument
v_f_num_2 <- paste0(number(6, seed = 33),c("AB")) #simulate 6 numbers using seed and add AB suffix
v_f_num_2 #show numbers
vector_pop(v_f_num_2, el = c("403211378AB")) #remove 1 specific entries
v_f_num_2 #show results
vector_pop(v_f_num_2, el = c("803690460AB","66592309AB")) #remove 2 specific entries
v_f_num_2 #show results

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/pkg_by.R
\name{find_packages}
\alias{find_packages}
\title{Fetch R package based on keyword}
\usage{
find_packages(keyword)
}
\arguments{
\item{keyword}{character, the keyword to search package descriptions for.}
}
\value{
character vector of matching package names or NULL if no matches.
}
\description{
This function gets R packages based on a keyword in their description.
}
\note{
the function in its current form only searches available.packages()
}
\examples{
\donttest{
# find the list of R packages for data or machine learning

matched_pkgs <- suppressWarnings(find_packages("plotting"))
matched_pkgs

matched_pkgs <- suppressWarnings(find_packages("machine learning"))
matched_pkgs
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/has.error.R
\name{has.error}
\alias{has.error}
\title{Check if a call or expression produces errors}
\usage{
has.error(...)
}
\arguments{
\item{...}{the expression or function calls}
}
\value{
boolean value to indicate if the expression produces errors
}
\description{
Whether a function or series of calls results in error
}
\note{
More information, check: https://rpkg.net/package/quickcode
}
\examples{
# this should not produce error
# so the function result should be FALSE
has.error({
  x = 8
  y = number(10)
  res = x + y
})

# this should produce the following error
# Error in x + y : non-numeric argument to binary operator
# so the function result should be TRUE
has.error({
  x = 8
  y = "random"
  res = x + y
})

# this should result in error because
# the dataset does not contain a "rpkg.net" column
# the result should be TRUE
df1 = mtcars
has.error(df1[,"rpkg.net"])

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/dat2week.R
\name{getWeekSeq}
\alias{getWeekSeq}
\alias{is.Date}
\alias{not.Date}
\alias{is.leap}
\title{Convert Dates into Numeric Week Counts}
\usage{
getWeekSeq(start_date, end_date, dates, in.format = "\%m/\%d/\%y")

is.Date(x)

not.Date(x)

is.leap(yyyy)
}
\arguments{
\item{start_date}{A scaler of class Date (if this argument is populated, the date arg must be empty)}

\item{end_date}{A scaler of class Date; must be later than the start_date (if this argument is populated, the date arg must be empty)}

\item{dates}{vector of dates that need not be sequential but that reference values of class Date;
Note that if this argument is populated, the start_date and end_date arguments must be empty}

\item{in.format}{date input format}

\item{x}{date item to check}

\item{yyyy}{year numeric value eg 1989}
}
\value{
data frame of the dates along with their corresponding week
}
\description{
Convert the range of date to number of weeks
}
\examples{
# simple example with start and end date
getWeekSeq(start_date="12/29/25",end_date="1/8/26")

# enter specific dates instead
# specify format
getWeekSeq(
dates = c(
  '2025-12-29',
  '2025-12-30',
  '2025-12-31',
  '2026-01-01',
  '2026-01-04',
  '2026-01-05',
  '2026-01-06',
  '2026-01-07',
  '2026-01-08'),
  in.format = "\%Y-\%m-\%d"
)

getWeekSeq(
dates = c(
  '12/29/25',
  '12/30/25',
  '12/31/25',
  '01/01/26',
  '01/02/26',
  '01/03/26',
  '01/06/26',
  '01/07/26',
  '01/08/26'
  ),
  in.format = "\%m/\%d/\%y"
)

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/aided_chatgpt.R
\name{sub.range}
\alias{sub.range}
\title{Calculate the Range Difference of a Numeric Vector}
\usage{
sub.range(x)
}
\arguments{
\item{x}{A numeric vector for which the range difference is to be calculated.}
}
\value{
A numeric value representing the difference between the maximum and minimum values of the input vector.
        Returns `NA` if the input is empty or contains only `NA` values.
}
\description{
This function calculates the difference between the maximum and minimum values of a numeric vector.
}
\examples{
# Example with a numeric vector
numeric_vector <- c(1, 5, 3, 8, 2)
sub.range(numeric_vector)

# Example with missing values
sub.range(c(NA, 4, 7, NA, 10))

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/sample_by_column.R
\name{sample_by_column}
\alias{sample_by_column}
\title{Re-sample a dataset by column and return number of entry needed}
\usage{
sample_by_column(.dt, col, n, seed = NULL, replace = FALSE)
}
\arguments{
\item{.dt}{data frame to re-sample}

\item{col}{column to uniquely re-sample}

\item{n}{number of rows to return}

\item{seed}{unique numeric value for reproducibility}

\item{replace}{should sampling be with replacement}
}
\value{
data frame containing re-sampled rows from an original data frame
}
\description{
Shorthand to return a re-sample number of rows in a data frame by unique column
}
\examples{
data1 <- data.frame(ID=1:10,MOT=11:20)
sample_by_column(data1,MOT,3)
sample_by_column(data1,ID,7)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/not.environment.R
\name{not.environment}
\alias{not.environment}
\title{Not an environment}
\usage{
not.environment(x)
}
\arguments{
\item{x}{vector entry}
}
\value{
a boolean value to indicate if entry is an environment
}
\description{
Check if entry is not an environment object
}
\examples{
test.env <- new.env()
test.notenv <- list(t=1)
not.environment(test.env) # FALSE
not.environment(test.notenv) # TRUE
if(not.environment(test.notenv)) message("yes") # yes

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/0.7_combo_func1.R
\name{read.csv.print}
\alias{read.csv.print}
\title{Read a CSV and preview first X rows and columns}
\usage{
read.csv.print(
  file,
  header = TRUE,
  sep = ",",
  quote = "\\"",
  dec = ".",
  fill = TRUE,
  comment.char = "",
  ...,
  dim = c(10L, 5L)
)
}
\arguments{
\item{file}{the name of the file which the data are to be read from.
    Each row of the table appears as one line of the file.  If it does
    not contain an \emph{absolute} path, the file name is
    \emph{relative} to the current working directory,
    \code{\link{getwd}()}. Tilde-expansion is performed where supported.
    This can be a compressed file (see \code{\link{file}}).

    Alternatively, \code{file} can be a readable text-mode
    \link{connection} (which will be opened for reading if
    necessary, and if so \code{\link{close}}d (and hence destroyed) at
    the end of the function call).  (If \code{\link{stdin}()} is used,
    the prompts for lines may be somewhat confusing.  Terminate input
    with a blank line or an \abbr{EOF} signal, \code{Ctrl-D} on Unix and
    \code{Ctrl-Z} on Windows.  Any pushback on \code{stdin()} will be
    cleared before return.)

    \code{file} can also be a complete URL.  (For the supported URL
    schemes, see the \sQuote{URLs} section of the help for
    \code{\link{url}}.)
  }

\item{header}{a logical value indicating whether the file contains the
    names of the variables as its first line.  If missing, the value is
    determined from the file format: \code{header} is set to \code{TRUE}
    if and only if the first row contains one fewer field than the
    number of columns.}

\item{sep}{the field separator character.  Values on each line of the
    file are separated by this character.  If \code{sep = ""} (the
    default for \code{read.table}) the separator is \sQuote{white space},
    that is one or more spaces, tabs, newlines or carriage returns.}

\item{quote}{the set of quoting characters. To disable quoting
    altogether, use \code{quote = ""}.  See \code{\link{scan}} for the
    behaviour on quotes embedded in quotes.  Quoting is only considered
    for columns read as character, which is all of them unless
    \code{colClasses} is specified.}

\item{dec}{the character used in the file for decimal points.}

\item{fill}{logical. If \code{TRUE} then in case the rows have unequal
    length, blank fields are implicitly added.  See \sQuote{Details}.}

\item{comment.char}{character: a character vector of length one
    containing a single character or an empty string.  Use \code{""} to
    turn off the interpretation of comments altogether.}

\item{...}{Further arguments to be passed to \code{read.table}.}

\item{dim}{dimension of CSV content to show}
}
\value{
read csv content and a print out of the data head
}
\description{
The purpose of this function is combine the functionality of
\strong{read.csv} and \strong{print}, which are often used together.\cr\cr
The purpose of this function is to read data from a file into a variable and
simultaneously display a preview of the data, showing either the first few rows or
columns based on the user's specification. It is important to emphasize that the
function expects the user to assign the result of the read operation to a variable
in order to achieve its intended purpose. eg. Use \strong{var1 = read.csv.print(file1)} instead of
\strong{read.csv.print(file1)}
}
\details{
Read a dataset of type csv and show x rows and y columns with
one function call
}
\examples{
\dontrun{
# Example: read a csv file and print the first 10 lines
# declare file
new.file <- "test.csv"

# read file and preview default
dth3 <- read.csv.print(file = new.file)

# read file and preview 10 rows and all columns
dth1 <- read.csv.print(file = new.file, dim = 10)

# read file and preview 10 rows and 5 columns
dth2 <- read.csv.print(file = new.file, dim = c(10,5))
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/OR.R
\name{or}
\alias{or}
\alias{\%or\%}
\title{Nullish coalescing operator}
\usage{
or(test, alternative)

test \%or\% alternative
}
\arguments{
\item{test}{an object to return}

\item{alternative}{alternative object to return}
}
\value{
value of test if not null or empty, else return value of alternative
}
\description{
Alternative return for empty, null or na statements.Return alternative if the value of expression is empty or NA or NULL
}
\note{
Equivalent to Nullish coalescing operator ?? in javascript or PHP like $Var = $operand1 ?? $operand2;
}
\examples{

test1 <- c(4,NA,5,2,0,21)

test2 <- data.frame(ID = 1:10,ED = LETTERS[10:1])

# One may also choose to use

test2[,1] \%or\% "A"

test2[which(test2$ID == 323),2] \%or\% "CCCCC"

number(1) \%or\% "Placeholder"

number(10) \%or\% "Placeholder"

NA \%or\% "Random"

NULL \%or\% "Random"

"" \%or\% "Random"

or(test1[which(test1==4)],100)


or(test1[which(test1==43)],100)

or(test2[which(test2$ID == 10),2],"BBBBB")

or(test2[which(test2$ID == 323),2],"CCCCC")


}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/data_push.R
\name{data_push}
\alias{data_push}
\title{Add data to another data like array_push in PHP}
\usage{
data_push(., add, which = c("rows", "cols"))
}
\arguments{
\item{.}{first data set}

\item{add}{data set to add}

\item{which}{where to append the new data e.g. rows or cols}
}
\value{
the combined dataset store to a variable with the name of the first
}
\description{
Shorthand to add data to a dataset and save as the same name
}
\examples{
# initialize p1 and p2
init(p1,p2)
p1
p2

# declare p1 and p2 as data frame
p1 <- data.frame(PK=1:10,ID2=1:10)
p2 <- data.frame(PK=11:20,ID2=21:30)

p1
p2

#add p1 to p2 by row, and resave as p1
data_push(p1,p2,"rows")
# p2 # p2 remains the same
p1 #p1 has been updated

# declare a new data frame called p3
p3 <- data.frame(Hindex=number(20),Rindex=number(20,seed=20))

# add p3 to p1 as column, and resave as p1
data_push(p1,p3,"cols")
p1 # p1 has been updated
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/inc.R
\name{inc}
\alias{inc}
\title{Increment vector by value}
\usage{
inc(., add = 1L)
}
\arguments{
\item{.}{vector of number(s)}

\item{add}{number to add}
}
\value{
a vector incremented by a number
}
\description{
Increment the content of a vector and re-save as the vector
}
\details{
This function is very useful when writing complex codes involving loops.
Apart from the for loop, this can be useful to quickly increment a variable located outside the loop
by simply incrementing the variable by 1 or other numbers. Check in the example section for a specific use.
Nonetheless, one may also choose to use this function in any other instance, as it's simple purpose is
to increase the value of a variable by a number and then re-save the new value to that variable.
}
\examples{
num1 <- sample(330:400,10)
num1#before increment

# increment num1 by 1
inc(num1)
num1 #after increment

# increment num1 by 5
num1 #before increment
inc(num1, add= 10)
num1 #after increment

#when used in loops

#add and compare directly
rnum = 10
inc(rnum) == 11 #returns TRUE
rnum #the variable was also updated

# use in a for loop
ynum = 1
for( i in c("scientist","dancer","handyman","pharmacist")){
message("This is the item number ")
message(ynum)
message(". For this item, I am a ")
message(i)

#decrement easily at each turn
plus(ynum)
}


#use in a repeat loop
xnum = 1
repeat{ #repeat until xnum is 15
message(xnum)
if(inc(xnum) == 15) break
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/archivedR.R
\name{archivedPkg}
\alias{archivedPkg}
\title{Listing of all CRAN archived R packages}
\usage{
archivedPkg(
  startsWith = c("all", letters),
  after = NULL,
  inc.date = TRUE,
  as = c("data.frame", "list")
)
}
\arguments{
\item{startsWith}{one letter that the package name starts with eg. a, e, f}

\item{after}{packages archived after a specific date eg. 2011-05-10}

\item{inc.date}{should archive date be included in the result}

\item{as}{return result as data frame or as list}
}
\value{
a data frame or list containing listing of all archived R packages
}
\description{
Retrieve a list of all currently archived R packages and their archive date
}
\note{
* The "startsWith" argument should be one letter and should be in lowercase \cr
* If no argument is provided for "startsWith", all the packages will be retrieved\cr
* The format of the "after" argument must be YYYY-MM-DD e.g. 2022-04-11
}
\section{Use case}{

This function allows the retrieval of various R packages archived by CRAN along with
the respective latest archive date. The packages retrieved include both active and
inactive R projects submitted to CRAN. When a new version of an active R package is published,
the older versions of the package gets archived. In the same way, when a package is
decommissioned from CRAN active projects for one reason or another, it gets archived.
}

\examples{
\donttest{
# Task 1: get archived R packages with names beginning with C or All
head(archivedPkg(startsWith = "all"), n= 10) #retrieves all packages
head(archivedPkg(startsWith = "c"), n= 10) #retrieves only packages beginning with a

# Task 2: return the packages from Task 1 without including latest archive date
res.dt2 <- archivedPkg(startsWith = "b", inc.date = FALSE)
res.dt2[1:10,]

# Task 3: return the results from Task 2 as a list
res.dt3 <- archivedPkg(startsWith = "c", inc.date = FALSE, as = "list")
res.dt3$name[1:10]

res.dt3 <- archivedPkg(startsWith = "e", as = "list")
res.dt3$name[1:10]

# Task 4: return the archived packages beginning with Y archived after 2022-08-12
# Note that startsWith should be lowercase

#without archive date
yRPkg <- archivedPkg(startsWith = "y", after= NULL)
nrow(yRPkg) #number of rows returned
head(yRPkg, n = 15) #show first 15 rows

#with archive date
yRPkg2 <- archivedPkg(startsWith = "y", after= "2022-08-12")
nrow(yRPkg2) #number of rows returned
head(yRPkg2, n = 15) #show first 15 rows, notice no archive date before 2022-08-12
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/func_chain.R
\name{\%.\%}
\alias{\%.\%}
\alias{chain_sep}
\title{simple function chaining routine}
\usage{
obj \%.\% funcs

chain_sep(sep = "\\\\.\\\\.")
}
\arguments{
\item{obj}{data object to apply function}

\item{funcs}{function chains to apply to the data object}

\item{sep}{separator for functions argument values}
}
\value{
the result of applying the chained functions to the data object
}
\description{
chain multiple function to a call
}
\note{
chain_sep allows the user to preset the separator for the function chaining \cr\cr
e.g. you can call the function to set sep = "__" before using the %.% call to parse the chained function "__unique__sum"
}
\examples{
#use default sep ".."
1:3\%.\%unique..length
sample(1:1000,10,replace=TRUE) \%.\%unique..length
sample(1:10,10,replace=TRUE) \%.\%unique..cumsum

# set sep before function chaining
chain_sep("__")
sample(1:10,10,replace=TRUE) \%.\%unique__cumsum
sample(1:10,10,replace=TRUE) \%.\%unique__cumsum__length


# set sep before function chaining
chain_sep("X")
sample(1:10,10,replace=TRUE) \%.\%uniqueXcumsum
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/supervariable.R
\name{newSuperVar}
\alias{newSuperVar}
\title{Create and use a super variable with unique capabilities}
\usage{
newSuperVar(variable, value = 0L, lock = FALSE, editn = NULL)
}
\arguments{
\item{variable}{variable name for super variable}

\item{value}{value of the variable}

\item{lock}{lock variable to change}

\item{editn}{number of times the super variable may be set to a new value using .set(). \cr
- Set to NULL to allow unlimited value change \cr
- Set to 0 to prevent editing the super variable \cr}
}
\value{
no visible return, but variable is created and stored with various functionalities
}
\description{
Create a variable that supersedes other variables and has various functionalities
}
\note{
\strong{What you should know about the functionality:} \cr\cr
This function ensures that a variable is created and may not easily be altered.
It helps preserve the original variable by providing only limited access to the variable.\cr\cr
Creation of this super variable automatically attached some key functions to it,
such that the user is able to call the function like \code{.set()}, \code{.rm()}.\cr\cr
Super variable value may be set from any scope using the \code{.set()} function, which
means that it is granted global variable features without being present within the
global environment of the current section.\cr\cr The variable name of the super variable may
be overwritten in the local environment, but this would not alter the super variable.
It means that once the local variable is removed, the super variable remains available
for use.\cr\cr

\strong{Use cases:} \cr\cr
 - Preserve originality of variable within an R session. Avoid inadvertent deletion.\cr\cr
 - Widely accessible from any scope e.g functions, lapply, loops, local environment etc\cr\cr
 - Restricted mutability of variable using set function e.g varname\code{.set()}\cr\cr
 - Variable with easy function calls by attaching '.'\cr\cr
 - Variable with un-mutable class when changing its value \cr\cr
 - Variable with restricted number of times it can be changed \cr\cr
}
\examples{
# Task: create a super variable to
# store dataset that should not be altered
newSuperVar(mtdf, value = austres) # create a super variable
head(mtdf) # view it
mtdf.class # view the store class of the variable, it cannot be changed
# it means that when the super variable is edited, the new value MUST have the same class "ts"

# create and lock super variable by default
# extra security to prevent changing
newSuperVar(mtdf3, value = beaver1, lock = TRUE)
head(mtdf3) # view
mtdf3.round(1) # round to 1 decimal places
head(mtdf3) # view
mtdf3.signif(2) # round to 2 significant digits
head(mtdf3) # view

# Task: create a new super variable to store numbers
# edit the numbers from various scopes
newSuperVar(edtvec, value = number(5))
edtvec # view content of the vector

# edtvec.set(letters) #ERROR: Cannot set to value with different class than initial value

edtvec.set(number(20)) # set to new numbers
edtvec # view output

for (pu in 1:8) {
  print(edtvec) # view output within loop
  edtvec.set(number(pu)) # set to new numbers within for loop
}

lc <- lapply(1:8, function(pu) {
  print(edtvec) # view output within loop
  edtvec.set(number(pu)) # set to new numbers within lapply loop
})

# see that the above changed the super variable easily.
# local variable will not be altered by the loop
# example
bim <- 198
lc <- lapply(1:8, function(j) {
  print(bim)
  bim <- j # will not alter the value of bim in next round
})


# Task: create and search data.frame
# create a new super variable with value as mtcars
# search if it contains the numeric value 21
newSuperVar(lon2, value = mtcars) # declares lon2
lon2 # view content of lon2
lon2.contains("21.0") # WRONG - since df.col is not specified,
# only the first column is search for the character "21.0"
lon2.contains("21.0", df.col = "mpg") # WRONG - searches mpg column
# for the character "21.0"
lon2.contains(21.0, df.col = "mpg") # CORRECT - search mpg column for the
# numeric value 21.0

# remove lon2 as a super variable
exists("lon2") # before removal
lon2.rm()
exists("lon2") # after removal

# Task: create and search vector
# create a new super variable with value as 10 random numbers
# search if it contains the numeric value 72
newSuperVar(lon3, value = number(10, seed = 12)) # declares lon3
lon3 # view content of lon3
lon3.contains(72) # should give TRUE or false if the vector contains the value 45
lon3.contains(72, fixed = TRUE) # should give TRUE or false if the vector contains the value 45

# remove lon3 as a super variable
lon3.rm()


#Task: create a super variable that can only be edited 3 times
newSuperVar(man1, value = number(5), editn = 3)
man1 # view value

man1.set(number(10)) # change value first time
man1 # view value

man1.set(number(2)) # change value second time
man1 # view value

man1.set(number(1)) # change value third time
man1 # view value

man1.set(number(5)) # change value forth time,
# should not change because max change times exceeded
man1 # view value

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/clean.R
\name{clean}
\alias{clean}
\alias{clear}
\title{Clear environment, clear console, set work directory and load files}
\usage{
clean(setwd = NULL, source = c(), load = c(), clearPkgs = FALSE)

clear(setwd = NULL, source = c(), load = c(), clearPkgs = FALSE)
}
\arguments{
\item{setwd}{OPTIONAL. set working directory}

\item{source}{OPTIONAL. source in file(s)}

\item{load}{OPTIONAL. load in Rdata file(s)}

\item{clearPkgs}{Clear previous loaded packages, TRUE or FALSE}
}
\value{
cleared environment and set directory
}
\description{
Shorthand to quickly clear console, clear environment, set working directory, load files
}
\details{
The purpose of this function is provide a one-line code to clear the console, clear the environment,
set working directory to a specified path, source in various files into the current file, and
load RData files into the current environment. The first process in the sequence of events is to clear the
environment. Then the working directory is set, prior to inclusion of various files and RData. With the directory
being set first, the path to the sourced in or RData files will not need to be appended to the file name. See examples.
}
\examples{
\donttest{
if(interactive()){
#simply clear environment, clear console and devices
quickcode::clean()

#clear combined with additional arguments
quickcode::clean(
  clearPkgs = FALSE
) #also clear all previously loaded packages if set to true

quickcode::clean(
  setwd = "/home/"
) #clear env and also set working directory


quickcode::clean(
  source = c("/home/file1.R","file2")
) #clear environment and source two files into current document


quickcode::clean(
  setwd = "/home/",
  source = c("file1","file2")
) #clear environment, set working directory and source 2 files into environment


quickcode::clean(
  setwd = "/home/",
  source="file1.R",
  load="obi.RData"
) #clear environment, set working directory, source files and load RData
}
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/nm_scale.R
\name{nonmem_scaling}
\alias{nonmem_scaling}
\title{Compute S1 scaling expression for NONMEM}
\usage{
nonmem_scaling(
  dv_unit,
  amt_unit,
  v_unit,
  sig_digits = 8L,
  example = FALSE,
  example_amt = 1,
  example_v = 1
)
}
\arguments{
\item{dv_unit}{Character scalar. DV concentration unit, e.g. "ng/mL", "mg/L", "ug per mL".}

\item{amt_unit}{Character scalar. AMT unit (mass), e.g. "mg", "ug", "ng".}

\item{v_unit}{Character scalar. Unit for model volume parameter V, e.g. "L", "mL".}

\item{sig_digits}{Integer scalar. Significant digits for formatted output (default = 8).}

\item{example}{Logical scalar. If TRUE, include a numeric example (default = FALSE).}

\item{example_amt}{Numeric scalar. Example AMT value (default = 1).}

\item{example_v}{Numeric scalar. Example V value (default = 1).}
}
\value{
A list with elements:
\describe{
  \item{factor}{Numeric. The multiplicative factor such that S1 = V * factor.}
  \item{S1_expression}{Character. NONMEM-ready expression (e.g. "S1 = V/1000").}
  \item{explanation}{Character. Human-readable explanation of the derivation.}
  \item{example}{Named list (present only when \code{example = TRUE}) with numeric example.}
}
}
\description{
Given the units used in your dataset/model for the dependent variable (DV,
a concentration like "ng/mL"), the dosing amount (AMT, e.g. "mg") and the
model volume parameter V (e.g. "L" or "mL"), this function returns a robust
NONMEM-ready S1 expression so that \code{A(n)/S1} has the same units as DV.

The function:
\itemize{
  \item parses common unit notations and synonyms (e.g. "ug", "mcg"),
  \item validates input and provides informative errors,
  \item computes the numeric factor such that \code{S1 = V * factor},
  \item returns a human-readable explanation and an optional numeric example.
}
}
\details{
Compute NONMEM S1 scaling from concentration, amount, and volume units
}
\examples{
# Typical PK: DV in ng/mL, AMT in mg, V in L -> S1 = V * 1e+06 / 1000 = V * 1000
nonmem_scaling("ng/mL", "mg", "L")

# If V is stored in mL:
nonmem_scaling("ng/mL", "mg", "mL")

# With numeric example
nonmem_scaling("ng/mL", "mg", "L", example = TRUE, example_amt = 100, example_v = 40)

# Microgram dosing
nonmem_scaling("pg/mL", "ug", "L")

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/not.empty.R
\name{not.exists}
\alias{not.exists}
\title{Not exists}
\usage{
not.exists(x)
}
\arguments{
\item{x}{object}
}
\value{
a boolean value to indicate if entry does not exists
}
\description{
Check if object does not exists
}
\examples{
go = 7
not.exists("exis") # TRUE
not.exists("go") # FALSE
if(not.exists('hallo')) message("yes") # NULL
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/not.image.R
\name{is.image}
\alias{is.image}
\title{Is file name extension(s) an image}
\usage{
is.image(x)
}
\arguments{
\item{x}{vector entry}
}
\value{
a boolean value to indicate if entry is an image
}
\description{
Check if one or multiple file name entry is an image
}
\details{
This current function tests if the extension of the file name provided belongs to any of the image extensions listed below \cr
AI - Adobe Illustrator \cr
BMP - Bitmap Image \cr
CDR - Corel Draw Picture \cr
CGM - Computer Graphics Metafile \cr
CR2 - Canon Raw Version 2 \cr
CRW - Canon Raw \cr
CUR - Cursor Image \cr
DNG - Digital Negative \cr
EPS - Encapsulated PostScript \cr
FPX - FlashPix \cr
GIF - Graphics Interchange Format \cr
HEIC - High-Efficiency Image File Format \cr
HEIF - High-Efficiency Image File Format \cr
ICO - Icon Image \cr
IMG - GEM Raster Graphics \cr
JFIF - JPEG File Interchange Format \cr
JPEG - Joint Photographic Experts Group \cr
JPG - Joint Photographic Experts Group \cr
MAC - MacPaint Image \cr
NEF - Nikon Electronic Format \cr
ORF - Olympus Raw Format \cr
PCD - Photo CD \cr
PCX - Paintbrush Bitmap Image \cr
PNG - Portable Network Graphics \cr
PSD - Adobe Photoshop Document \cr
SR2 - Sony Raw Version 2 \cr
SVG - Scalable Vector Graphics \cr
TIF - Tagged Image File \cr
TIFF - Tagged Image File Format \cr
WebP - Web Picture Format \cr
WMF - Windows Metafile \cr
WPG - WordPerfect Graphics
}
\examples{
img.1 <- "fjk.jpg"
is.image(img.1)

img.0 <- "fjk.bbVG"
is.image(img.0)

img.2 <- "fjk.bmp"
is.image(img.2)

img.3 <- "fjk.SVG"
is.image(img.3)

# a vector of file names
v <- c("logo.png", "business process.pdf",
"front_cover.jpg", "intro.docx",
"financial_future.doc", "2022 buybacks.xlsx")

is.image(v)


# when the file name has no extension
# the function returns NA
v2 <- c("img2.jpg","northbound.xlsx","landimg",NA)
is.image(v2)

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/github.R
\name{getGitRepoStart}
\alias{getGitRepoStart}
\alias{getGitRepoChange}
\title{Fetch GitHub Repository Creation & Last Updated Date}
\usage{
getGitRepoStart(repo_name, out.format = "\%Y-\%m-\%d")

getGitRepoChange(repo_name, out.format = "\%Y-\%m-\%d")
}
\arguments{
\item{repo_name}{name of the repository}

\item{out.format}{date output format}
}
\value{
date of creation of repository as a character

date of the last update of repository as a character
}
\description{
The GitHub REST API is a powerful tool that allows developers
to interact with GitHub programmatically. It provides a set of
endpoints that allows a user to create integration, retrieve data,
and automate workflows related to GitHub repositories. It is a means
by which users can interact with GitHub without directly using a web interface.
}
\details{
The two functions utilize the GitHub REST API to extract important temporal information about a GitHub repository. \cr\cr
 - the getGitRepoStart function is used to retrieve the date a GitHub repository was first created.\cr\cr
 - the getGitRepoChange function retrieves the date a GitHub repository was last updated.
}
\examples{
\donttest{
# Use default date format
getGitRepoStart(repo_name = "oobianom/quickcode")

# Specify date format
getGitRepoStart(repo_name = "oobianom/quickcode", out.format = "\%j|\%Y")
getGitRepoStart(repo_name = "oobianom/quickcode", out.format = "\%D|\%j")
}

\donttest{
getGitRepoChange(repo_name = "oobianom/shinyStorePlus", out.format = "\%d-\%b-\%Y")
getGitRepoChange(repo_name = "oobianom/r2social", out.format = "\%Y/\%m/\%d")
}

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/aided_chatgpt.R
\name{learn_rate_scheduler}
\alias{learn_rate_scheduler}
\title{Learning Rate Scheduler}
\usage{
learn_rate_scheduler(initial_lr, schedule, epochs)
}
\arguments{
\item{initial_lr}{A numeric value representing the initial learning rate.}

\item{schedule}{A function that takes the epoch number as input and returns the adjusted learning rate.}

\item{epochs}{An integer specifying the total number of epochs.}
}
\value{
A numeric vector representing the learning rate for each epoch.
}
\description{
This function adjusts the learning rate at each epoch according to a custom schedule.
}
\examples{
schedule <- function(epoch) { 0.01 * (0.9 ^ epoch) }
learn_rate_scheduler(0.01, schedule, epochs = 10)

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/list_shuffle.R
\name{list_shuffle}
\alias{list_shuffle}
\title{Shuffle a list object just like shuffle in PHP}
\usage{
list_shuffle(., seed = NULL)
}
\arguments{
\item{.}{list to shuffle}

\item{seed}{apply seed if indicated for reproducibility}
}
\value{
shuffled list of items store to the list name
}
\description{
Shorthand to shuffle a list and save
}
\examples{
list001 <- list("a" = 1:5,
           "b" = letters[1:5],
           c = LETTERS[1:10],
           "2" = number(5,5),
           "e" = randString(5,5))
list001 #show initial list

#illustrate basic functionality
list_shuffle(list001)
list001 #shuffle and resaved to variable

list.f2<-list001
list_shuffle(list.f2)
list.f2 #first output

list.f2<-list001
list_shuffle(list.f2)
list.f2 # different output from first output top

list.f2<-list001
list_shuffle(list.f2,seed = 344L)
list.f2 #second output

list.f2<-list001
list_shuffle(list.f2,seed = 344L)
list.f2 #the same output as second output top

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/vector_push.R
\name{vector_push}
\alias{vector_push}
\title{Add elements to a vector like array_push in PHP}
\usage{
vector_push(., add, unique = FALSE, rm.na = FALSE, rm.empty = FALSE)
}
\arguments{
\item{.}{first vector}

\item{add}{vector to add}

\item{unique}{remove duplicated entries}

\item{rm.na}{remove NA values}

\item{rm.empty}{remove empty values}
}
\value{
vector combining fist and second vector, but have name set to the first
}
\description{
Shorthand to add elements to a vector and save as the same name
}
\details{
Note that two vectors are required in order to use this function. Also, note that the final result replaces the content of the
first vector. This means that the original content of the 'first vector' will no longer exist after this function executes.
}
\section{Use case}{

This function allows the combination of two vectors in one short line of code. It allows specification of further downstream
filtering of the resulting vector such as selecting only unique items, removing NA or empty values. It simplifies a code chunk
with many lines of code to concatenate and filter various vectors.
}

\examples{
num1 <- number(10, seed = 45)
num2 <-"rpkg.net"

num1
num2

#Task: add num2 to num1 and re-save as num1
vector_push(num1,num2)
num1 #updated with num2
num2 #not updated


#Task: concatenate two vectors and remove duplicates
vector1 = number(4,seed = 5)
vector2 = number(8,seed = 5)
vector3 = number(12,seed = 5)

vector1 #length is 4
vector2 #length is 8
vector3 #length is 12

# with duplicated
vector_push(vector1,vector2, unique = FALSE)
vector1 #return modified vector
length(vector1) #length is 12 because nothing was removed
#duplicates in vector1 is 886905927 100040083 293768998  54080431

# without duplicated
vector_push(vector2,vector3, unique = TRUE)
vector2 #return modified vector
length(vector2) #length is 12 instead of 20
#Total of 8 duplicated numbers were removed


#Task: concatenate two vector and remove NA values
vector1 = number(5)
vector2 = c(4,NA,5,NA)
vector3 = number(5)

# with NA
vector_push(vector1,vector2, rm.na = FALSE)
vector1 #return modified vector

# without NA
vector_push(vector3,vector2, rm.na = TRUE)
vector3 #return modified vector


#Task: concatenate two vector and remove empty values
vector1 = number(5)
vector2 = c(4,'',5,'',NULL,' ')
vector3 = number(5)

# with empty
vector_push(vector1,vector2, rm.empty = FALSE)
vector1 #return modified vector

# without empty
vector_push(vector3,vector2, rm.empty = TRUE)
vector3 #return modified vector

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/header.rmd.R
\name{header.rmd}
\alias{header.rmd}
\title{Snippet function to add header to a current Rmd opened file}
\usage{
header.rmd()
}
\value{
Inserts header content for Rmd file
}
\description{
Shorthand to add Rmd header
}
\examples{
if(interactive())
header.rmd()
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/getDate.R
\name{getDate}
\alias{getDate}
\title{Extract all dates from a string}
\usage{
getDate(str1, out.format = "\%Y-\%m-\%d")
}
\arguments{
\item{str1}{string to parse}

\item{out.format}{date output format}
}
\description{
Extract all dates from a string
}
\note{
\strong{DATE FORMATS IN R}\cr
\tabular{rrrrr}{
\strong{Date Specification}   \tab \tab \strong{Description}          \tab \tab  \strong{Example} \cr
\%a  \tab \tab Abbreviated weekday             \tab \tab Sun, Thu \cr
\%A  \tab \tab Full weekday                    \tab \tab Sunday \cr
\%b  \tab \tab Abbreviated month               \tab \tab May, Jul \cr
\%B  \tab \tab Full month                      \tab \tab March, July \cr
\%d  \tab \tab Day of the month                \tab \tab 27, 07 \cr
\%j  \tab \tab Day of the year                 \tab \tab 148, 188 \cr
\%m  \tab \tab Month                           \tab \tab 05, 07 \cr
\%U  \tab \tab Week, with Sunday as first day  \tab \tab 22, 27 \cr
\%w  \tab \tab Weekday, Sunday is 0            \tab \tab 0, 4 \cr
\%W  \tab \tab Week, with Monday as first day  \tab \tab 21, 27 \cr
\%x  \tab \tab Date, locale-specific           \tab \tab \cr
\%y  \tab \tab Year without century            \tab \tab 84, 05 \cr
\%Y  \tab \tab Year with century               \tab \tab 1984, 2005 \cr
\%C  \tab \tab Century                         \tab \tab 19, 20 \cr
\%D  \tab \tab Date formatted \%m/\%d/\%y      \tab \tab 07/17/23 \cr
\%u  \tab \tab Weekday, Monday is 1            \tab \tab 7, 4 \cr
}
}
\examples{
str1 <- "The video was recorded on July 19, 2023."
str2 <- "The video was recorded over a 4 hour period
starting on July 19, 2023."
str3 <- "The first batch aug 12,2024 of aug 12,
2024 reports are due on July 12, 2024; the second
batch on 7/19/24."
str4 <- c("On 3.12.25, Jerry is taking one month
05.13.1895 of leave and is not scheduled to
return until around 4-9-2025.", "The staff
will be out on training on 10/11/24,
Oct 12, 2024, and 10-13-24.")
getDate(str1)
getDate(str2, out.format = "\%Y-\%m-\%d")
getDate(str3, out.format = "\%m-\%d/\%Y")
getDate(str4)

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/nin.R
\name{\%nin\%}
\alias{\%nin\%}
\title{Not in vector or array}
\usage{
x \%nin\% table
}
\arguments{
\item{x}{vector entry}

\item{table}{table of items to check}
}
\value{
a boolean value to indicate if entry is present
}
\description{
Check if entry is in vector
}
\examples{
5 \%nin\% c(1:10) #FALSE
5 \%nin\% c(11:20) #TRUE

x = "a"
if(x \%nin\% letters) x

# let's say we are trying to exclude numbers from a vector
vector_num1 <- number(9, max.digits = 5, seed = 1) #simulate 9 numbers
vector_num1 #values
vector_num1[vector_num1 \%nin\% c(83615,85229)]#return values not 83615 or 85229
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/how_similar.R
\name{percent_match}
\alias{percent_match}
\alias{\%match\%}
\alias{sound_match}
\title{Function to calculate the percentage of matching between two strings}
\usage{
percent_match(
  string1,
  string2,
  case_sensitive = FALSE,
  ignore_whitespace = TRUE,
  frag_size = 2
)

string1 \%match\% string2

sound_match(string1, string2)
}
\arguments{
\item{string1}{first string}

\item{string2}{second string}

\item{case_sensitive}{if to check case sensitivity}

\item{ignore_whitespace}{if to ignore whitespace}

\item{frag_size}{fragment size of string}
}
\value{
numeric value of the match percent

match word sounds
}
\description{
Function to calculate the percentage of matching between two strings
}
\details{
Case Sensitivity: \cr

  The function can optionally consider or ignore case sensitivity based on the \code{case_sensitive} argument.\cr\cr
Whitespace Handling:\cr

  With \code{ignore_whitespace} set to TRUE, the function removes all whitespaces before comparison. This can be useful for matching strings that may have inconsistent spacing.\cr\cr
Exact Character-by-Character Matching:\cr

  The function computes the percentage of matching characters in the same positions.\cr\cr
Substring Matching:\cr

  The function checks if one string is a substring of the other, awarding a full match if true.\cr\cr
Levenshtein Distance:\cr

  The function uses Levenshtein distance to calculate the similarity and integrates this into the overall match percentage.\cr\cr
Fragment Matching:\cr

   - A \code{frag_size} argument is introduced that compares fragments (substrings) of a given size (default is 3) from both strings.\cr
 - The function creates unique fragments from each string and compares them to find common fragments.\cr
 - The percentage match is calculated based on the ratio of common fragments to the total number of unique fragments.\cr\cr
Combining Metrics:\cr

  The overall match percentage is computed as the average of exact match, substring match, Levenshtein match, and fragment match percentages.
}
\examples{
# Example 1: simple match
string1 <- "Hello World"
string2 <- "helo world"

match_percent <- percent_match(string1, string2)
message("Percentage of matching: ", match_percent)


# Example 2: which date is closest
string0 <- "october 12,1898"
string1 <- "2018-10-12"
string2 <- "1898-10-12"
percent_match(string0, string1)
percent_match(string0, string2)
percent_match(string0, string2, frag_size = 4)
percent_match(string1, string2)

sound_match("Robert","rupert")
sound_match("rupert","Rubin")
sound_match("book","oops")
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/not.null.R
\name{not.null}
\alias{not.null}
\title{Not NULL}
\usage{
not.null(x)
}
\arguments{
\item{x}{vector entry}
}
\value{
a boolean value to indicate if entry is NULL
}
\description{
Opposite of is.null(). Check if entry is not NULL
}
\examples{
is.null("") # FALSE
not.null("") # TRUE
not.null(NULL) # FALSE
if(not.null(45)) message("something") # yes

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/once.R
\name{setOnce}
\alias{setOnce}
\title{Set a variable only once}
\usage{
setOnce(., val = 1L, envir = NULL)
}
\arguments{
\item{.}{variable to set}

\item{val}{the value to set for the variable}

\item{envir}{environment where variables resides}
}
\value{
the variable set to the new variable, along with a class of once added to the output
}
\description{
Facilitates the one-time setting of a variable in R, ensuring its immutability thereafter.
}
\details{
With this function, users can establish the change to the initial value of a variable,
and it guarantees that any subsequent attempts to modify the variable are ignored.
This feature ensures that the variable remains constant and immutable once it has been set,
preventing unintentional changes and promoting code stability. This function simplifies the process
of managing immutable variables in R, providing a reliable mechanism for enforcing
consistency in data throughout the course of a program or script.
}
\examples{
# set the value of vector_x1, vector_y1, vector_z1
init(vector_x1, vector_y1, vector_z1, value = 85)

# view the initial values of the variables
vector_x1
vector_y1
vector_z1

# task 1: change the value vector_x1 and prevent further changes
vector_x1 # check value of unchanged
vector_x1 * 0.56 # check value when x 0.56

setOnce(vector_x1, val = 4500) # set vector_x1
vector_x1 # check value
vector_x1 * 0.56 # check value when x 0.56

setOnce(vector_x1, val = 13) # set vector_x1 AGAIN, should not change
vector_x1 # check value
vector_x1 * 0.56 # check value when x 0.56

# task 2: In for loop, change vector_y1 and use later
vector_y1 # check value of unchanged

for(i in 1:20){
setOnce(vector_y1,as.numeric(Sys.time()))
# now let's see the difference between vector_y1
# and the current time as it changes
message("current vector_y1: ",vector_y1,"; subtraction res: ",as.numeric(Sys.time()) - vector_y1)
}

# task 3: In for lapply, change vector_z1 and use later
vector_z1 # check value of unchanged

invisible(
lapply(1:20, function(i){
setOnce(vector_z1,as.numeric(Sys.time()))
# now let's see the difference between vector_z1
# and the current time as it changes
message("current vector_z1: ",vector_z1,"; subtraction res: ",as.numeric(Sys.time()) - vector_z1)
})
)

# result of all the tasks
vector_x1
vector_y1
vector_z1


}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/0.7_combo_func1.R
\name{strsplit.bool}
\alias{strsplit.bool}
\title{Split a string of values and return as boolean vector}
\usage{
strsplit.bool(
  x,
  split,
  fixed = FALSE,
  perl = FALSE,
  useBytes = FALSE,
  type = 2
)
}
\arguments{
\item{x}{character vector, each element of which is to be split. Other inputs, including a factor, will give an error.}

\item{split}{character vector}

\item{fixed}{logical. If TRUE match split exactly, otherwise use regular expressions. Has priority over perl.}

\item{perl}{logical. Should Perl-compatible regexps be used?}

\item{useBytes}{logical. If TRUE the matching is done byte-by-byte rather than character-by-character, and inputs with marked encodings are not converted.}

\item{type}{type of return, see the as.boolean function for more info}
}
\value{
boolean values based on split string
}
\description{
The purpose of this function is combine the functionality of
\strong{strsplit}, \strong{unlist} and \strong{as.logical}, which are often used together.\cr\cr
}
\details{
Given a sting, split by a separator into boolean
}
\examples{
# string of numbers
num.01 = "0 1 0 0 1 0 1 T F TRUE FALSE t f"

# split a string of numbers and return as boolean 1/0
strsplit.bool(num.01, split = " ", type = 3)

# split a string of numbers and return as boolean TRUE/FALSE
strsplit.bool(num.01, split = " ", type = 2)

# split a string of numbers and return as boolean Yes/No
strsplit.bool(num.01, split = " ", type = 1)


# string of numbers
num.02 = "0abc1abc0abc0abc1abc0abc1abcTabcFabcTRUEabcFALSEabcf"

# split a string of numbers and return as boolean 1/0
strsplit.bool(num.02, split = "abc", type = 3)

# split a string of numbers and return as boolean TRUE/FALSE
strsplit.bool(num.02, split = "abc", type = 2)

# split a string of numbers and return as boolean Yes/No
strsplit.bool(num.02, split = "abc", type = 1)

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/clean.R
\name{lastwd}
\alias{lastwd}
\title{Go back to previous directory}
\usage{
lastwd()
}
\value{
the previous directory
}
\description{
Navigate to previous working directory if the setwd was called by clean() or refresh() function
}
\note{
In order to use this function to retrieve the last/previous working directory, you must use the quickcode::clean() or quickcode::refresh() function to set the working directory.
}
\examples{
lastwd()
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/not.integer.R
\name{not.integer}
\alias{not.integer}
\title{Not an integer}
\usage{
not.integer(x)
}
\arguments{
\item{x}{vector entry}
}
\value{
a boolean value to indicate if entry is an integer
}
\description{
Opposite of is.integer(). Check if entry is not an integer
}
\examples{
is.integer(78L) #TRUE
not.integer(78L) #FALSE

not.integer(23.43) # TRUE
not.integer(45L) # FALSE
if(not.integer(4L)) message("yes") # NULL

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/switch.R
\name{switch_cols}
\alias{switch_cols}
\title{Switch the index of two columns in a data set}
\usage{
switch_cols(data, col1, col2, keep.rows = NULL)
}
\arguments{
\item{data}{dataset object}

\item{col1}{numeric or character the first column name or number}

\item{col2}{numeric or character the second column name or number}

\item{keep.rows}{numeric. row number to keep}
}
\description{
Allows the user to choose precisely which two columns they want to swap places,
while optionally preventing some rows within the columns from being altered in the process.
Excluded rows within the columns act as anchors that are immune from the switching operation
on the selected columns.
}
\examples{

# Example using mtcars
data101 <- mtcars[1:7,]

head(data101) # preview overall data

# task 1: basic result of switching columns 5 and 6
head(switch_cols(data101, 5, 6))

# task 1: basic result of switching columns number 5 and name "gear"
head(switch_cols(data101, 5, "gear"))

# task 1: basic result of switching columns "qsec" and "carb"
head(switch_cols(data101, "qsec", "carb"))


# task 2: switch columns, but retain some rows with the switched columns


# lets exchange some columns, but keep content of row 4, 5 intact
data101[1:6,4:7] # preview the portion that is to be changed
res1 <- switch_cols(data101, col1 = 5, col2 = 6, keep.rows = 4:5) # use column numbers
res1[1:6,4:7] # check result, pay attention to rows 4, 5 of columns 5, 6 as well

data101[1:6,6:11] # preview the portion that is to be changed
res2 <- switch_cols(data101,
col1 = "qsec",
col2 = "carb",
keep.rows = c(1,2,3)) # keep 1, 2, 3
res2[1:6,6:11] # check result

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/pairDist.R
\name{pairDist}
\alias{pairDist}
\title{Calculate the distance of points from the center of a cluster}
\usage{
pairDist(data, scale = FALSE, roundit = NULL)
}
\arguments{
\item{data}{data frame object or a matrix/array object}

\item{scale}{Boolean used for scaling the dataset}

\item{roundit}{Round x number of decimal places; min value is 0; max value is 6}
}
\description{
This function operates on multivariate data and
calculates the distance of points from the centroid of one or more clusters.
}
\section{Function utility}{

Used to generate the computations needed to model pair-distance measures in three dimensions
}

\section{More information about this function}{

The pairDist function is used to quantify how far each data point (row)
is from the overall mean across all columns. It’s commonly used in multivariate statistics,
machine learning, and data analysis to assess the variability or similarity of data points
relative to their mean. More specifically, the function is used in outlier detection and
cluster analysis to evaluate the dispersion of data. Used in conjunction with other
calculations, pairDist output can also be used to model data in three dimensions.
}

\examples{

data(airquality)
airq <- airquality[, 1:4]

# example 1:
x <- pairDist(airq, roundit = 3)

# example 2:
y <- pairDist(airq, scale = TRUE, roundit = 5)

# example 3

data <- matrix(
  c(1, 5, NA, 4, 5, 6, NA, 8, 9, NA, 12, 23, 6, 0, 3, 3, 8, 15),
  ncol = 3,
  byrow = TRUE
)

dist <- pairDist(data, roundit = 2)
dist_scaled <- pairDist(data, scale = TRUE, roundit = 6)

}
\references{
the current function was adapted from one of the examples in the svgViewR package, \cr
https://cran.r-project.org/web/packages/svgViewR/svgViewR.pdf
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/make_df.R
\name{make_dosing_df}
\alias{make_dosing_df}
\title{Create Subject-by-Time Dosing Records (Base R) with Optional Subject-Level Covariates}
\usage{
make_dosing_df(ID, time, dose, ...)
}
\arguments{
\item{ID}{A vector of subject identifiers (numeric, integer, or character). Typically unique.}

\item{time}{A vector of dosing times. Can be numeric or integer (e.g., hours, days).}

\item{dose}{Dose value(s). May be:
\itemize{
  \item length 1: one common dose applied to all subjects and all times
  \item length equal to `length(ID)`: subject-specific doses mapped by the *position*
        of each subject in `ID` (i.e., `dose[i]` corresponds to `ID[i]`), then repeated
        across all times for that subject
}}

\item{...}{Named additional columns to include in the output.
Each argument in `...` must be either:
\itemize{
  \item length 1: a constant value repeated for all rows (e.g., `STUDY = "CNTO1275"`)
  \item length equal to `length(ID)`: subject-level values mapped by the *position* of
        each subject in `ID` (e.g., `WT = c(30, 45)` with `ID = c(1, 2)` assigns `WT=30`
        to subject 1 and `WT=45` to subject 2 across all times)
}
All `...` arguments must be named. Names must not conflict with reserved output column
names (`ID`, `time`, `dose`).}
}
\value{
A `data.frame` with `length(ID) * length(time)` rows and columns `ID`, `time`, `dose`,
plus any additional columns passed via `...`.
}
\description{
Creates a dosing/event-style dataset by expanding a set of subject IDs
(`ID`) across a vector of dosing times (`time`). It assigns a dose value to each subject
and time combination, and optionally adds additional columns via `...` that can be
constant (length 1) or subject-specific (length equal to `length(ID)`).

This utility is useful for quickly generating dosing records for simulation, exploratory
analyses, and creating NONMEM-/nlmixr-/Monolix-friendly input structures where each row
represents a dosing event (or event-like record) at a given time.
}
\details{
## Mapping rule (important)
For any vector argument of length `length(ID)` (including `dose` or any `...` column),
values are assigned by the *index/position* of `ID`, not by sorted ID or factor level.
For example:
\preformatted{
ID   = c(10, 20)
VAR5 = c(4,  6)
}
results in subject `10 -> 4` and subject `20 -> 6` for all time points.

## Duplicate IDs
If you supply any subject-level vector (i.e., `dose` length > 1, or any `...` argument
length > 1), `ID` must be unique. Duplicate IDs create ambiguous mappings and will
trigger an error.

## Output structure
The output includes:
\itemize{
  \item `ID`: subject identifier
  \item `time`: dosing time
  \item `dose`: assigned dose
  \item any additional columns provided via `...`
}
Rows are sorted by `ID` then `time`.

## When to use (common scenarios)
\enumerate{
  \item **Create dosing records for simulation** (e.g., one dose level across many subjects,
        or subject-specific doses).
  \item **Build NONMEM/nlmixr event datasets** where dosing rows (e.g., `EVID=1`) are
        generated programmatically; you can add `EVID`, `CMT`, `RATE`, `STUDY`, etc. via `...`.
  \item **Explore exposure metrics** by generating standardized schedules (e.g., QD times)
        and attaching covariates like weight, age group, cohort, regimen labels.
  \item **Scenario testing** for pediatric dosing cutoffs, titration strategies (subject-specific),
        or bridging comparisons where regimen differs by subgroup.
}

## When not to use
Use a different approach if you need:
\itemize{
  \item time-varying covariates with length equal to `length(time)` (shared across subjects),
  \item fully specified matrices of values length `length(ID) * length(time)`,
  \item multiple event types (dose + observations) in the same call (although you can merge
        the result with an observation dataset after creation).
}
}
\examples{
## Example 1: Simple dataset (10 subjects, dose 40 at times 1:10)
df1 <- make_dosing_df(ID = 1:10, time = 1:10, dose = 40)
head(df1)

## Example 2: Subject-specific dose (one dose per subject)
df2 <- make_dosing_df(ID = c(1, 2, 3), time = 1:5, dose = c(40, 60, 80))
head(df2)

## Example 3: Add constant columns via ...
df3 <- make_dosing_df(
  ID = 1:3, time = c(0, 7, 14), dose = 45,
  STUDY = "CNTO1275JPA3001", ROUTE = "SC", CMT = 1
)
df3

## Example 4: Add subject-level covariates via ...
## VAR5 is mapped by position: ID=1 gets 4, ID=2 gets 6
df4 <- make_dosing_df(
  ID = c(1, 2), time = 1:3, dose = 40,
  VAR5 = c(4, 6), WT = c(35.2, 51.0), SEX = c("M", "F")
)
df4

## Example 5: NONMEM-style dosing rows (add EVID and AMT)
df_nm <- make_dosing_df(ID = 1:2, time = c(0, 28, 56), dose = c(45, 45),
                        EVID = 1, AMT = NA)  # AMT placeholder (optional)
df_nm$AMT <- df_nm$dose
df_nm

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/equations1.R
\name{geo.cv}
\alias{geo.cv}
\alias{geo.mean}
\alias{geo.sd}
\title{Calculate geometric coefficient of variation, mean, or SD and round}
\usage{
geo.cv(num, round = 2, na.rm = TRUE, neg.rm = TRUE, pct = TRUE)

geo.mean(num, round = 2, na.rm = TRUE, neg.rm = TRUE)

geo.sd(num, round = 2, na.rm = TRUE, neg.rm = TRUE)
}
\arguments{
\item{num}{vector of numbers}

\item{round}{round result to decimal place}

\item{na.rm}{remove NAs from the vector}

\item{neg.rm}{remove negative values from the vector}

\item{pct}{TRUE or FALSE. should result be in percent}
}
\value{
the geometric cv of a set of numbers

the geometric mean of a set of numbers

the geometric standard deviation of a set of numbers
}
\description{
Calculate the coefficient of variation

Calculate the geometric mean

Calculate the geometric standard deviation
}
\examples{
#simulate numbers using a fixed seed
num1 <- number(n = 1115,max.digits = 4, seed = 10)

#get geometric CV, represent as percent and round to 2 decimal places
geo.cv(num1,round = 2) # result: 60.61\%

#or round to 3 decimal places
geo.cv(num1,round = 3) # result: 60.609\%

#by default, the above examples return a CV\%
#if you do not want the result as percentage, specify "pct"
geo.cv(num1,pct = FALSE) # result: 0.61

num1 <- sample(300:3000,10)

#get the geometric mean, excluding all negatives and round to 2
geo.mean(num1)

#or
geo.mean(num1)


#get geometric mean, but round the final value to 5 decimal places
geo.mean(num1, round = 5)

num1 <- sample(330:400,20)

#get geometric SD remove negative values and round to 2 decimal places
geo.sd(num1)

#get geometric SD, DON'T remove negative values and round to 2 decimal places
geo.sd(num1,na.rm=FALSE)

#get geometric SD, remove negative values and round to 3 decimal places
geo.sd(num1,round = 3)

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/not.duplicated.R
\name{not.duplicated}
\alias{not.duplicated}
\title{Not duplicated elements}
\usage{
not.duplicated(x, incomparables = FALSE, ...)
}
\arguments{
\item{x}{a vector or a data frame or an array or NULL.}

\item{incomparables}{a vector of values that cannot be compared. FALSE is a special value, meaning that all values can be compared, and may be the only value accepted for methods other than the default. It will be coerced internally to the same type as x}

\item{...}{arguments for particular methods.}
}
\value{
elements of a vector or data frame that are NOT duplicates
}
\description{
Opoosite of duplicated(). Checks which elements of a vector or data frame are NOT duplicates of elements with smaller subscripts
}
\examples{

set.seed(08082023)
dtf <- sample(1:10,15, replace = TRUE)
dtf # 3  9 10  3  8  9  6 10  5  1  2  2  2  9  8
dtf[ dtf > 4 & duplicated(dtf) ] # 9 10  9  8
dtf[ dtf > 4 & not.duplicated(dtf) ] # 9 10  8  6  5

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/data_pop.R
\name{data_pop}
\alias{data_pop}
\title{Remove last n rows or column or specified elements from a data frame like array_pop in PHP}
\usage{
data_pop(., n = 1, which = c("rows", "cols"), ret = FALSE)
}
\arguments{
\item{.}{parent data}

\item{n}{number of elements to remove}

\item{which}{whether to remove from row or from column}

\item{ret}{TRUE or FALSE. whether to return value instead of setting it to the parent data}
}
\value{
data with elements removed
}
\description{
Shorthand to remove elements from a data frame and save as the same name
}
\examples{
#basic example: pop off 1 row from sample data
data.frame(ID=1:10,N=number(10)) #before
data_pop(data.frame(ID=1:10,N=number(10))) #after

#using data objects
data.01 <- mtcars[1:7,]

#task: remove 1 element from the end of the data and set it to the data name
data.01 #data.01 data before pop
data_pop(data.01) #does not return anything
data.01 #data.01 data updated after pop

#task: remove 3 columns from the end of the data and set it to the data name
data.01 #data.01 data before pop
data_pop(data.01, n = 3, which = "cols") #does not return anything, but updates data
data.01 #data.01 data updated after pop

#task: remove 5 elements from the end, but do not set it to the data name
data.01 #data.01 data before pop
data_pop(data.01,5, ret = TRUE) #return modified data
data.01 #data.01 data remains the same after pop


}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/detect_outlier.R
\name{detect_outlier}
\alias{detect_outlier}
\alias{iqr_outlier}
\alias{zscore_outlier}
\alias{zscore_outlier2}
\title{Detect Outliers in a Numeric Vector}
\usage{
detect_outlier(
  x,
  method = "iqr",
  multiplier = 1.5,
  z_threshold = 3,
  na.rm = TRUE,
  groups = NULL,
  summary = FALSE
)

iqr_outlier(x, multiplier)

zscore_outlier(x, z_threshold)

zscore_outlier2(x, z_threshold)
}
\arguments{
\item{x}{A numeric vector in which to detect outliers.}

\item{method}{A character string specifying the outlier detection method. Options are
`"iqr"` (default) for the interquartile range method or `"zscore"` for the z-score method.}

\item{multiplier}{A positive numeric value specifying the multiplier for the IQR method.
Default is `1.5`, typically used for moderate outliers; `3` is common for extreme outliers.
Ignored if `method = "zscore"`.}

\item{z_threshold}{A positive numeric value specifying the z-score threshold for the
`method = "zscore"` option. Default is `3`, meaning values with an absolute z-score
greater than 3 are flagged as outliers. Ignored if `method = "iqr"`.}

\item{na.rm}{A logical value indicating whether to remove `NA` values before computation.
Default is `TRUE`. If `FALSE` and `NA` values are present, the function stops with an error.}

\item{groups}{An optional vector of group names or labels corresponding to each value in
`x`. If provided, must be the same length as `x`. Outlier detection is performed separately
for each unique group, and results are returned as a nested list. Default is `NULL` (no grouping).}

\item{summary}{A logical value indicating whether to include a summary in the output.
Default is `FALSE`. If `TRUE`, the output list includes a `summary` element with
descriptive statistics and outlier counts, either overall or by group if `groups` is provided.}
}
\value{
If `groups = NULL` (default), a list with the following components:
  - `outliers`: A numeric vector of the outlier values.
  - `indices`: An integer vector of the indices where outliers occur in the input vector.
  - `bounds` (if `method = "iqr"`): A named numeric vector with the `lower` and `upper`
    bounds for outliers.
  - `threshold` (if `method = "zscore"`): A named numeric vector with the `lower` and
    `upper` z-score thresholds.
  - `is_outlier`: A logical vector of the same length as `x`, where `TRUE` indicates an
    outlier.
  - `summary` (if `summary = TRUE`): A list with summary statistics including the mean,
    median, standard deviation (for z-score), quartiles (for IQR), and number of outliers.

  If `groups` is provided, a named list where each element corresponds to a unique group,
  containing the same components as above but computed separately for that group’s values.
}
\description{
This function identifies outliers in a numeric vector using either the interquartile range
(IQR) method or the z-score method. The IQR method defines outliers as values below
Q1 - multiplier * IQR or above Q3 + multiplier * IQR, where Q1 and Q3 are the first and
third quartiles. The z-score method identifies outliers as values with an absolute
z-score exceeding a specified threshold.
}
\details{
The function returns a list containing the outliers, their indices, detection bounds or
thresholds, and a logical vector indicating which elements are outliers. If a grouping
vector is provided via `groups`, outlier detection is performed separately for each group,
and results are returned as a nested list by group. If `na.rm = TRUE` (default), missing
values (`NA`) are removed before computation. If `na.rm = FALSE` and `NA` values are
present, the function stops with an error. The function also stops for non-numeric input,
insufficient valid data, or mismatched group lengths.


The function requires at least two non-`NA` values per group (if `groups` is provided) or
overall (if `groups = NULL`) to compute meaningful statistics when `na.rm = TRUE`. If
`na.rm = FALSE`, the presence of `NA` values triggers an error. If all values in a group
are identical or there are insufficient data points, an error is thrown for that group.
The IQR method is robust to non-normal data, while the z-score method assumes approximate
normality and is sensitive to extreme values.
}
\examples{
# Example 1: Basic IQR method without groups
x <- c(1, 2, 3, 4, 100)
detect_outlier(x)

# IQR method with summary
detect_outlier(x, summary = TRUE)

# Z-score method with custom threshold
y <- c(-10, 1, 2, 3, 4, 5, 20)
detect_outlier(y, method = "zscore", z_threshold = 2.5)

# Handling missing values
z <- c(1, 2, NA, 4, 5, 100)
detect_outlier(z, method = "iqr", multiplier = 3)

# Example 2: IQR method with groups
x2 <- c(1, 2, 3, 100, 5, 6, 7, 200)
groups2 <- c("A", "A", "A", "A", "B", "B", "B", "B")
detect_outlier(x2, groups = groups2)

# Example 3: Z-score method with groups and summary
x3 <- c(-10, 1, 2, 20, 3, 4, 5, 50)
groups3 <- c("X", "X", "X", "X", "Y", "Y", "Y", "Y")
detect_outlier(x3, method = "zscore", z_threshold = 2, groups = groups3, summary = TRUE)

# Example 4: IQR method with groups and NA values
x4 <- c(1, 2, NA, 100, 5, 6, 7, 200,1000)
groups4 <- c("G1", "G1", "G1", "G1", "G2", "G2", "G2", "G2","G1")
detect_outlier(x4, groups = groups4)

# Error cases
\dontrun{
detect_outlier(c("a", "b"))  # Non-numeric input
detect_outlier(c(1), groups = c("A"))  # Insufficient data
detect_outlier(c(1, 2), groups = c("A"))  # Mismatched group length
detect_outlier(c(1, NA, 3), na.rm = FALSE)  # NA with na.rm = FALSE
}

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/number.R
\name{number}
\alias{number}
\title{Generate a random number (integer)}
\usage{
number(n, max.digits = 10, seed = NULL)
}
\arguments{
\item{n}{how many numbers to generate}

\item{max.digits}{maximum number of digits in each number}

\item{seed}{set seed for sampling to maintain reproducibility}
}
\value{
random numbers between 1 and 1 billion
}
\description{
Fetch n random integers between 1 and 1,000,000,000
}
\examples{
number(1)
number(10)
paste0(number(2),LETTERS)

#set maximum number of digits
number(1,max.digits = 5)
number(10,max.digits = 4)

#set seed for reproducibility
#without seed
number(6) #result 1
number(6) #result 2, different from result 1
#with seed
number(6,seed=1)#result 3
number(6,seed=1)#result 4, same as result 3
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/cybersec.R
\name{extract_IP}
\alias{extract_IP}
\title{Extract all IP addresses from a string}
\usage{
extract_IP(input)
}
\arguments{
\item{input}{input string containing IP}
}
\value{
extract_IP returns a vector of IP addresses
}
\description{
Parse a string and vectorize the IP addresses within it
}
\examples{
\donttest{
# Extract all IP addresses from a string

# Example 1
# This example demonstrates the separate
# extraction of an IP address one per vector element:
str1 = c("Two IP(66.544.33.54) addresses
were discovered in the scan.
One was at 92.234.1.0.",
"The other IP was 62.3.45.255.")
extract_IP(str1)

# Example 2
# This example demonstrates the extraction
# of multiple IP addresses from a single vector element:
str2 = "Two IP addresses were discovered
in the scan. One was at 92.234.1.0.
The other IP was 62.3.45.255."
extract_IP(str2)
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/not.inherits.R
\name{not.inherits}
\alias{not.inherits}
\title{Not inherit from any of the classes specified}
\usage{
not.inherits(x, what, which = FALSE)
}
\arguments{
\item{x}{an \R object.}

\item{what}{a character vector naming classes.}

\item{which}{logical affecting return value: see \sQuote{Details}.}
}
\value{
a boolean value to indicate if !inherits
}
\description{
Opposite of base::inherits().
Indicates whether its first argument inherits from any of the classes specified in the what argument. If which is TRUE then an integer vector of the same length as what is returned. Each element indicates the position in the class(x) matched by the element of what; zero indicates no match. If which is FALSE then TRUE is returned by inherits if any of the names in what match with any class.
}
\examples{
keep.cols = "a character"
class(keep.cols) # class is character
not.inherits(keep.cols,"character")

num.var = 1L
class(num.var) # class is integer
not.inherits(num.var,"double")

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/not.logical.R
\name{not.logical}
\alias{not.logical}
\title{Not logical}
\usage{
not.logical(x)
}
\arguments{
\item{x}{vector entry}
}
\value{
a boolean value to indicate if entry is logical
}
\description{
Opposite of is.logical(). Check if entry is a logical object
}
\examples{
test.env <- TRUE
test.notenv <- 0
not.logical(test.env) # FALSE
not.logical(test.notenv) # TRUE
if(not.logical(test.notenv)) message("yes") # yes

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/bionic.R
\name{bionic_txt}
\alias{bionic_txt}
\title{Generate a bionic text}
\usage{
bionic_txt(text)
}
\arguments{
\item{text}{input text}
}
\value{
bionic text
}
\description{
This function serves as a mechanism
enabling the conversion of provided
text into a bionic form. Users input
the text, and the function, in turn,
delivers the text transformed into a
bionic format.
}
\details{
A bionic text refers to a transformed version
of a given text achieved through a specialized
function designed to incorporate elements of
advanced technology, enhancing both the form
and content of the original input. This function
operates by infusing the text with a fusion of
various elements, resulting in a synthesis
that transcends traditional linguistic boundaries.
The function augments the text
with dynamic visual representations that adapt to
the reader's preferences.
The goal is to create a text that not only conveys
information but also engages the audience in a more
immersive and interactive manner, harnessing
the capabilities of modern technology to redefine
the traditional concept of textual communication.
An example of a bionic text could be a news article
that dynamically updates with real-time data,
incorporates multimedia elements, and adjusts its
presentation style based on the reader's preferences,
thereby offering a more enriched and personalized reading
experience.
}
\examples{
# simple example to show a text
# transformation to bionic text

# text to transform
text1 <- "A tool for nonparametric
estimation and inference
of a non-decreasing
monotone hazard\nratio
from a right censored survival dataset."

# transform text
genbt <- bionic_txt(text1)

# print bionic text as message or cat
message(genbt)
cat(genbt)

}
\references{
This idea stems from a blog article published at
https://www.r-bloggers.com/2023/10/little-useless-useful-r-functions-function-for-faster-reading-with-bionic-reading/
and the original source for bionic texts may be found at https://bionic-reading.com/
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/date.R
\name{date3to1}
\alias{date3to1}
\alias{date1to3}
\title{Combine vector to create Date, or split Date into vector}
\usage{
date3to1(data, out.format = "\%Y-\%m-\%d", col.YMD = 1:3, as.vector = FALSE)

date1to3(
  data,
  in.format = "\%Y-\%m-\%d",
  date.col = 1,
  out.cols = c("\%Y", "\%m", "\%d")
)
}
\arguments{
\item{data}{data frame object}

\item{out.format}{date output format}

\item{col.YMD}{columns to combine for Year, Month and Day}

\item{as.vector}{return output as vector, or leave as data frame}

\item{in.format}{date input format}

\item{date.col}{numeric value of column within the dataset that contains the dates}

\item{out.cols}{cols to of date items to split. Make sure to conform to date formats. See "NOTE" section
for date formats}
}
\value{
date derived from combining values from three columns of a data frame
}
\description{
Combine or split Date into a specified format
}
\details{
\strong{NOTE for date3to1}\cr

The three input columns corresponding to "Year Month Day" must be numeric values. \cr\cr
For example, Do not provide the month variable as non-numeric such as "Mar", "Jul", or "Jan". \cr\cr
If the values of the columns are non-numeric, the results will return an "NA" in the output.date column.
}
\section{Note}{

\strong{DATE FORMATS IN R}\cr
\tabular{rrrrr}{
\strong{Date Specification}   \tab \tab \strong{Description}          \tab \tab  \strong{Example} \cr
\%a  \tab \tab Abbreviated weekday             \tab \tab Sun, Thu \cr
\%A  \tab \tab Full weekday                    \tab \tab Sunday \cr
\%b  \tab \tab Abbreviated month               \tab \tab May, Jul \cr
\%B  \tab \tab Full month                      \tab \tab March, July \cr
\%d  \tab \tab Day of the month                \tab \tab 27, 07 \cr
\%j  \tab \tab Day of the year                 \tab \tab 148, 188 \cr
\%m  \tab \tab Month                           \tab \tab 05, 07 \cr
\%U  \tab \tab Week, with Sunday as first day  \tab \tab 22, 27 \cr
\%w  \tab \tab Weekday, Sunday is 0            \tab \tab 0, 4 \cr
\%W  \tab \tab Week, with Monday as first day  \tab \tab 21, 27 \cr
\%x  \tab \tab Date, locale-specific           \tab \tab \cr
\%y  \tab \tab Year without century            \tab \tab 84, 05 \cr
\%Y  \tab \tab Year with century               \tab \tab 1984, 2005 \cr
\%C  \tab \tab Century                         \tab \tab 19, 20 \cr
\%D  \tab \tab Date formatted \%m/\%d/\%y      \tab \tab 07/17/23 \cr
\%u  \tab \tab Weekday, Monday is 1            \tab \tab 7, 4 \cr
}
}

\examples{
# EXAMPLES FOR date3to1

data0 <- data.frame(y=c(NA, -1, 2001:2009),
m=c(1:2, -1, NA, 13, 2, 12, 6:9),
d=c(0, 0:6, NA, -1, 32) )
head(data0)

# combine and convert to date
# return as data frame
date3to1(data0)

# combine and convert to date
# return as vector
date3to1(data0, as.vector = TRUE) #eg. 2004-02-04


# combine and convert to date in the format DD_MM_YYYY
date3to1(data0, out.format = "\%d_\%m_\%Y") #eg. 04_02_1974


# combine and convert to date in the format MM_DD_YY
date3to1(data0, out.format = "\%m_\%d_\%y") #eg. 02_04_74

# combine and convert to date in the various date formats
date3to1(data0, out.format = "\%B \%d, \%y") #eg. February 04, 74
date3to1(data0, out.format = "\%a, \%b \%d, \%Y") #eg. Mon, Feb 04, 1974
date3to1(data0, out.format = "\%A, \%B \%d, \%Y") #eg. Monday, February 04, 1974
date3to1(data0, out.format = "Day \%j in Year \%Y") #eg. Day 035 in Year 1974
date3to1(data0, out.format = "Week \%U in \%Y") #eg. Week 05 in 1974
date3to1(data0, out.format = "Numeric month \%m in Year \%Y") #eg. Numeric month 02 in Year 1974





# EXAMPLES FOR date1to3

data1 <- data.frame(Full.Dates =
                      c("2023-02-14",NA,NA,
                        "2002-12-04","1974-08-04",
                        "2008-11-10"))
head(data1)

# split date with default settings
# return as data frame with columns
# for day(d), month(m) and year(Y)
date1to3(data1)


# split date in the format and only return year in YYYY
date1to3(data1, out.cols = "\%Y") #eg. 2002, 2023


# split date in the format and only return month in m
date1to3(data1, out.cols = "\%m") #eg. 02, 12, 08

# split date in the format and return multiple date formats colums
date1to3(data1, out.cols = c("\%B","\%d") )
date1to3(data1, out.cols = c("\%a","\%b","\%y") )
date1to3(data1, out.cols = c("\%A","\%B","\%Y","\%y") )
date1to3(data1, out.cols = c("\%j","\%Y","\%y","\%m") )
date1to3(data1, out.cols = c("\%U","\%Y","\%y","\%x") )
date1to3(data1, out.cols = c("\%m","\%Y","\%y","\%C") )

}
\references{
Adapted from Ecfun R package
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/insertInText.R
\name{insertInText}
\alias{insertInText}
\title{Shiny app function to insert string to current file in RStudio}
\usage{
insertInText(string)
}
\arguments{
\item{string}{what to insert}
}
\value{
Inserts into current position on opened file
}
\description{
Shorthand to insert content to opened file
}
\examples{
if(interactive()){
insertInText('hello rpkg.net')
insertInText('hello world')
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/trim_file.R
\name{trim.file}
\alias{trim.file}
\title{Remove Empty Lines from a File}
\usage{
trim.file(file_path, resave = TRUE)
}
\arguments{
\item{file_path}{A character string specifying the path to the text file.}

\item{resave}{A logical value indicating if the file content should be resaved or returned}
}
\value{
NULL This function modifies the file in place and does not return a value.
}
\description{
This function reads a text file, removes all empty lines, and saves the
modified content back to the same file.
}
\examples{
\donttest{
if(interactive()){
# Remove empty lines from a file
trim.file("path/to/your/file.txt")
}
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/math.R
\name{math.qt}
\alias{math.qt}
\alias{math.mm}
\title{Miscellaneous math computations: Corresponding m-m and quantile for confident intervals}
\usage{
math.qt(ci = 0.9)

math.mm(Vmax, S, Km, V, round = NULL)
}
\arguments{
\item{ci}{confident interval eg. 0.9 for 90 percent confident intervals}

\item{Vmax}{The maximum velocity of the enzymatic reaction.}

\item{S}{The substrate concentration.}

\item{Km}{The substrate concentration at which the reaction rate is half of Vmax.}

\item{V}{The current velocity of the enzymatic reaction}

\item{round}{round result to number of decimal places}
}
\value{
vector of two numeric values for the quantile based on the confident interval chosen

result of calculation of Michaelis-Menten equation
}
\description{
Compute the corresponding quantile given confident interval bounds
}
\examples{

# Get the bounds for 90\% confident intervals
math.qt(0.9)

# Get the bounds for 95\% confident intervals
# use the bounds to obtain quartile
values = number(100)
values
ci = math.qt(0.95)
getquart = quantile(values, probs = ci)
getquart

math.mm(3,500,0.5)

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/decommissionedPkg.R
\name{rDecomPkg}
\alias{rDecomPkg}
\title{Check whether an R package has been decommissioned in CRAN}
\usage{
rDecomPkg(package)
}
\arguments{
\item{package}{package name to query}
}
\value{
the decommissioned status of a particular package based on the available packages using the utils package
}
\description{
Designed to assist users in checking the decommission status
of an R package on CRAN. In the context of R language,
CRAN stands for the Comprehensive R Archive Network.
}
\details{
CRAN is a network of servers around the world that store R packages
and their documentation, providing a centralized repository for
the R community. With the current function, users can quickly and
easily determine whether a specific R package has been decommissioned
on CRAN, ensuring they stay informed about the availability and
support status of the packages they rely on for their R
programming projects. This tool simplifies the process of package
management, helping users maintain up-to-date and reliable
dependencies in their R code.
}
\examples{
\dontrun{
# check if cattonum package is decommissioned
# the current package is expected to be decommissioned
rDecomPkg("cattonum")

# check if dplyr is decommissioned
# the current package is expected NOT to be decommissioned
rDecomPkg("dplyr")

# when a package never existed in CRAN
# the result of the function call should be NA
rDecomPkg("printy")
rDecomPkg("package0002312122312")
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/func_chain.R
\name{chain_func}
\alias{chain_func}
\title{Combine specific functions as store as one function}
\usage{
chain_func(..., otherargs = list())
}
\arguments{
\item{...}{functions to include. see example for how to use}

\item{otherargs}{other arguments for use in each function}
}
\value{
returns a function that combines multiple function
}
\description{
Use many functions in one call using predeclared calls
}
\examples{

# Example 1 with base functions
combf1 <- chain_func(unique, cumsum, print)
result <- combf1(sample(1:5,10,replace = TRUE))
#or
u = sample(1:5,10,replace = TRUE)
result <- combf1(u)

# Example 2 with base functions with arguments
combf2 <- chain_func(unique, print, otherargs = list(.= c(FALSE),.=c(2)))
result <- combf2(sample(1:3,10,replace = TRUE))

# Example 3 with custom functions
r = function(a,b,c){
  if(!missing(a))print(a)
  if(!missing(b))print(b)
  if(!missing(c))print(c)
  return(a)
}

r2 = function(a,b,c){
  if(!missing(a))message(a)
  if(!missing(b))message(b)
  if(!missing(c))message(c)
  return(a)
}

combf3 <- chain_func(r,r2, otherargs =list(.=c("apple","cat"),.=c("rice")))
res <- combf3(head(mtcars))
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/zsore.R
\name{zscore}
\alias{zscore}
\alias{zscoreGrowthCurve}
\title{Calculates Z-Scores of a distribution}
\usage{
zscore(.data, round, na.rm = TRUE)

zscoreGrowthCurve(Xi, Mi, Si, Li = !0)
}
\arguments{
\item{.data}{data object}

\item{round}{round output to how many decimal place description}

\item{na.rm}{remove NA values before calculating z-scores}

\item{Xi}{physical measurement (e.g. weight, length, head circumference, stature or calculated BMI value)}

\item{Mi}{values from the table (see reference) corresponding to the age in months of the child}

\item{Si}{values from the table (see reference) corresponding to the age in months of the child}

\item{Li}{values from the table (see reference) corresponding to the age in months of the child}
}
\value{
zscore calculated based on data object or parameters
}
\description{
Calculates Z-Scores based on data
}
\examples{
# Capture z-score from the following distribution x
x = c(6, 7, 7, 12, 13, 13, 15, 16, 19, 22)
z_scores = zscore(x, round = 2) # limit to 2 decimal place
z_scores = zscore(x) # no decimal place limit

df = data.frame(val = x, zscore = z_scores)
head(df)
#EXAMPLE for zscore based on CDC growth chart

# Calculate the zscore for a patient weighing 50kg
Li=-0.1600954
Mi=9.476500305
Si=0.11218624
Xi=50
zscoreGrowthCurve(Xi,Mi,Si,Li)

}
\references{
CDC growth chart Z score calculation: https://www.cdc.gov/growthcharts/cdc-data-files.htm
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/inc.R
\name{minus}
\alias{minus}
\title{Decrease vector by value}
\usage{
minus(., minus = 1L)
}
\arguments{
\item{.}{vector of number(s)}

\item{minus}{number to minus}
}
\value{
a vector decreased by a number
}
\description{
decrease the content of a vector and re-save as the vector
}
\details{
Similar to the inc and plus functions, the minus function is very useful when writing complex codes involving loops.
Apart from the for loop, minus can be useful to quickly decrease the value of a variable located outside the loop
by simply decreement the variable by 1 or other numbers. Check in the example section for a specific use.
Given the scope, one may also choose to use this function in any other instances, as it's simple purpose is
to decrease the value of a variable by a number and then re-save the new value to that variable.
}
\examples{
num1 <- sample(5:150,10)
num1

# decrease num1 by 1
num1 #before decrease
minus(num1)
num1 #after decrease

# decrease num1 by 5
num1 #before decrease
minus(num1, minus = 5)
num1 #after decrease



#when used in loops

#add and compare directly
rnum = 23
minus(rnum) == 220 #returns FALSE
rnum #the variable was also updated


# use in a for loop
ynum = 100

for( i in c("teacher","student","lawyer","pharmacist")){
message("This is the item number ")
message(ynum)
message(". For this item, I am a ")
message(i)

#decrement easily at each turn
minus(ynum,3)
}

#use in a repeat loop
xnum = 100
repeat{ #repeat until xnum is 85
message(xnum)
if(minus(xnum) == 85) break
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/mutate_filter.R
\name{mutate_filter}
\alias{mutate_filter}
\title{Mutate only a subset of dataset intact}
\usage{
mutate_filter(., sub.set, ...)
}
\arguments{
\item{.}{data object}

\item{sub.set}{subset of data to modify}

\item{...}{mutation syntax similar to dplyr::mutate}
}
\value{
data frame containing original data, but with a subset mutated
}
\description{
Extension of the dplyr::mutate function that allows the user to mutate only a specific filtered subset of a data, while leaving the other parts of the data intact
}
\examples{
#mutate a subsection filter of mtcars
dt = mtcars
names(dt)
head(dt)
mutate_filter(dt,mpg == 21.0 & cyl == 6, cyl=1000,hp=2000,vs=hp*2)


dt2 = beaver1
names(dt2)
head(dt2)
mutate_filter(dt2, day == 346 & time < 1200, activ = 12, temp = round(temp*10,1))
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/strings.R
\name{randString}
\alias{randString}
\title{Generate a random string}
\usage{
randString(n, length)
}
\arguments{
\item{n}{number of strings to create}

\item{length}{length of string to create}
}
\value{
one more random string of specific length
}
\description{
Create a random string of specified length
}
\examples{
# Task 1: create 1 random string string of length 5
randString(n = 1, length = 5)

# Task 2: create 5 random string string of length 10
randString(n = 5, length = 10)


# Task 3: create 4 random string string of length 16
randString(n = 4, length = 16)

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/decimals.R
\name{ndecimal}
\alias{ndecimal}
\title{Count the number of decimal places}
\usage{
ndecimal(num)
}
\arguments{
\item{num}{a numeric value}
}
\description{
Count the number of decimal places
}
\examples{
#example 1
ndecimal(9.000322)

#example 2
ndecimal(34)

#example 3
ndecimal(45.)

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/data_pop_filter.R
\name{data_pop_filter}
\alias{data_pop_filter}
\title{Remove elements from a data matching filter}
\usage{
data_pop_filter(., remove)
}
\arguments{
\item{.}{data object}

\item{remove}{expression for filter}
}
\value{
data filtered out based on the expression
}
\description{
Shorthand to remove elements from a data frame based on filter and save as the same name
}
\examples{
# this function removes rows matching the filter expression
data.01 <- mtcars
data.02 <- airquality

#task: remove all mpg > 20
data.01 #data.01 data before pop
data_pop_filter(data.01,mpg > 15) #computes and resaves to variable
#note: this is different from subset(data.01,data.01$mpg > 15)
data.01 #modified data after pop based on filter

#task: remove all multiple. remove all elements where Month   == 5 or Solar.R > 50
data.02 #data.02 data before pop
data_pop_filter(data.02,Month   == 5 | Solar.R > 50) #computes and resaves to variable
data.02 #modified data after pop based on filter

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/data_shuffle.R
\name{data_shuffle}
\alias{data_shuffle}
\title{Shuffle a data frame just like shuffle in PHP}
\usage{
data_shuffle(., which = c("rows", "cols"), seed = NULL)
}
\arguments{
\item{.}{data to shuffle as data frame}

\item{which}{what to shuffle, rows or columns}

\item{seed}{apply seed if indicated for reproducibility}
}
\value{
shuffled data frame of items store to the data frame name
}
\description{
Shorthand to shuffle a data frame and save
}
\examples{

#basic example
data.frame(ID=46:55,PK=c(rep("Treatment",5),rep("Placebo",5))) #before
data_shuffle(
  data.frame(ID=46:55,PK=c(rep("Treatment",5),rep("Placebo",5)))
) #after shuffle row
data_shuffle(
  data.frame(ID=46:55,PK=c(rep("Treatment",5),rep("Placebo",5))),
  which = "cols"
) #after shuffle column


# examples using object
df1<-data.frame(ID=46:55,PK=c(rep("Treatment",5),rep("Placebo",5)))

#illustrate basic functionality
data_shuffle(df1)
df1 #shuffle and resaved to variable

data.f2<-df1
data_shuffle(data.f2)
data.f2 #first output

data.f2<-df1
data_shuffle(data.f2)
data.f2 # different output from first output top

data.f2<-df1
data_shuffle(data.f2,seed = 344L)
data.f2 #second output

data.f2<-df1
data_shuffle(data.f2,seed = 344L)
data.f2 #the same output as second output top

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/is_desc_asc.R
\name{is.increasing}
\alias{is.increasing}
\alias{is.decreasing}
\title{Check is numbers in a vector are decreasing or increasing}
\usage{
is.increasing(., na.last = TRUE)

is.decreasing(., na.last = TRUE)
}
\arguments{
\item{.}{a numeric vector}

\item{na.last}{for controlling the treatment of NAs. If TRUE, missing values in the data are put last; if FALSE, they are put first; if NA, they are removed.}
}
\value{
boolean value to indicate if the values are increasing or decreasing
}
\description{
Test if numeric values of a vector are decreasing or increasing using the radix method
}
\examples{
# example code
doy1 <- rnorm(1e3)
doy2 <- sort(doy1, decreasing = FALSE)
doy3 <- sort(doy1, decreasing = TRUE)

is.increasing(doy1)
is.decreasing(doy1)

is.increasing(doy2)
is.decreasing(doy2)

is.increasing(doy3)
is.decreasing(doy3)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/color_compHist.R
\name{mix.color}
\alias{mix.color}
\title{Mix or Blend two or more colors}
\usage{
mix.color(color, type = 2, alpha = 1)
}
\arguments{
\item{color}{CHARACTER. color vector e.g see example}

\item{type}{NUMERIC. return type of the output}

\item{alpha}{NUMERIC. alpha or opacity of the resulting color}
}
\value{
hex for the combined color
}
\description{
Combine colors to generate a new color
}
\examples{
# color vector
colvec <- c("red", "blue", "violet", "green", "#ff0066")

# just one color
mix.color(colvec[1], type = 1, alpha = 1)

# add two colors
mix.color(colvec[1:2], type = 1, alpha = 1)

# add three colors
mix.color(colvec[1:3], type = 1, alpha = 1)


# return type = 2

# just one color
mix.color(colvec[1], type = 2, alpha = 1)

# add two colors
mix.color(colvec[1:2], type = 2, alpha = 1)

# add three colors
mix.color(colvec[1:3], type = 2, alpha = 1)


# opacity or alpha  0.5

# just one color
mix.color(colvec[1], type = 1, alpha = 0.5)

# add two colors
mix.color(colvec[1:2], type = 1, alpha = 0.5)

# add three colors
mix.color(colvec[1:3], type = 1, alpha = 0.5)

# add all colors
mix.color(colvec, type = 1, alpha = 0.5)

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/aided_chatgpt.R
\name{multihead_att}
\alias{multihead_att}
\title{Multi-Head Attention}
\usage{
multihead_att(query, key, value, num_heads)
}
\arguments{
\item{query}{A numeric matrix representing the query.}

\item{key}{A numeric matrix representing the key.}

\item{value}{A numeric matrix representing the value.}

\item{num_heads}{An integer specifying the number of attention heads.}
}
\value{
A numeric matrix representing the attention output.
}
\description{
This function implements a simplified version of multi-head attention for sequence data.
}
\examples{
query <- matrix(rnorm(12), nrow = 3)
key <- matrix(rnorm(12), nrow = 3)
value <- matrix(rnorm(12), nrow = 3)
multihead_att(query, key, value, num_heads = 2)

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/seq_range.R
\name{in.range}
\alias{in.range}
\title{If number falls within a range of values and get closest values}
\usage{
in.range(
  value,
  range.min,
  range.max,
  range.vec = NULL,
  closest = FALSE,
  rm.na = FALSE
)
}
\arguments{
\item{value}{NUMERIC. the vector of numbers to check}

\item{range.min}{NUMERIC. OPTIONAL. the minimum value of the range}

\item{range.max}{NUMERIC. OPTIONAL. the maximum value of the range}

\item{range.vec}{NUMERIC. OPTIONAL. a vector of numbers to use for the range}

\item{closest}{BOOLEAN. OPTIONAL. return closest value}

\item{rm.na}{BOOLEAN. OPTIONAL. remove NA values from input}
}
\value{
boolean to indicate if the value or set of values are within the range
}
\description{
With a defined range of values, the function systematically examines
each provided number to determine if it falls within the specified range. It may
also provide the values with the range that are closest to a desired number.
}
\details{
The described function serves the purpose of checking whether a given number
or set of numbers falls within a specified range. It operates by taking a range
of values as input and then systematically evaluates each provided number to
determine if it lies within the defined range. This function proves particularly
useful for scenarios where there is a need to assess numeric values
against predefined boundaries, ensuring they meet specific criteria or constraints.
In the same manner, this function allows the user to also retrieve values within
the range that are closest to each provided number.
}
\note{
The argument range.vec is utilized when users opt not to employ the range.min or
range.max arguments. If range.vec is specified, range.min and range.max are
disregarded. It's important to note that the use of range.vec is optional.
}
\examples{
# Task 1: Check if a number is within specified range
in.range(5, range.min = 3, range.max = 10) # TRUE
in.range(25, range.min = 12, range.max = 20) # FALSE

# Task 2: Check if a set of values are within a specified range
in.range(1:5, range.min = 2, range.max = 7) #
in.range(50:60, range.min = 16, range.max = 27) #


# Task 3: Check if a number is within the range of a set of numbers
in.range(5, range.vec = 1:10) # TRUE
in.range(345, range.vec = c(1001,1002,1003,1004,1005,
1006,1007,1008,1009,1010,1011,1012,1013,1014)) # FALSE

# Task 4: Check if a set of values are within the range of a set of numbers
in.range(1:5, range.vec = 4:19) #
in.range(50:60, range.vec = c(55,33,22,56,75,213,120)) #

# Task 5: remove NAs prior to processing
in.range(c(1,3,NA,3,4,NA,8), range.min = 4, range.max = 6, rm.na = FALSE) # do not remove NA
in.range(c(1,3,NA,3,4,NA,8), range.min = 4, range.max = 6, rm.na = TRUE) # remove NA
#in.range(c(NA), range.min = 4, range.max = 6, rm.na = TRUE) #This will return error

# Task 6: return the closest number to the value
in.range(5:23, range.vec = 7:19, closest = TRUE)
in.range(-5:10, range.vec = -2:19, closest = TRUE)
in.range(c(1:5,NA,6:9), range.vec = 4:19, closest = TRUE)
in.range(c(1:5,NA,6:9), range.vec = 4:19, closest = TRUE, rm.na = TRUE)

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/0.7_combo_func1.R
\name{read.table.print}
\alias{read.table.print}
\title{Read in a table and show first X rows and columns}
\usage{
read.table.print(
  file,
  header = FALSE,
  sep = "",
  quote = "\\"'",
  dec = ".",
  numerals = c("allow.loss", "warn.loss", "no.loss"),
  row.names,
  col.names,
  as.is = TRUE,
  na.strings = "NA",
  colClasses = NA,
  nrows = -1,
  skip = 0,
  check.names = TRUE,
  fill = NULL,
  strip.white = FALSE,
  blank.lines.skip = TRUE,
  comment.char = "#",
  allowEscapes = FALSE,
  flush = FALSE,
  stringsAsFactors = FALSE,
  fileEncoding = "",
  encoding = "unknown",
  skipNul = FALSE,
  dim = c(10L, 5L),
  ...
)
}
\arguments{
\item{file}{the name of the file which the data are to be read from.
    Each row of the table appears as one line of the file.  If it does
    not contain an \emph{absolute} path, the file name is
    \emph{relative} to the current working directory,
    \code{\link{getwd}()}. Tilde-expansion is performed where supported.
    This can be a compressed file (see \code{\link{file}}).

    Alternatively, \code{file} can be a readable text-mode
    \link{connection} (which will be opened for reading if
    necessary, and if so \code{\link{close}}d (and hence destroyed) at
    the end of the function call).  (If \code{\link{stdin}()} is used,
    the prompts for lines may be somewhat confusing.  Terminate input
    with a blank line or an \abbr{EOF} signal, \code{Ctrl-D} on Unix and
    \code{Ctrl-Z} on Windows.  Any pushback on \code{stdin()} will be
    cleared before return.)

    \code{file} can also be a complete URL.  (For the supported URL
    schemes, see the \sQuote{URLs} section of the help for
    \code{\link{url}}.)
  }

\item{header}{a logical value indicating whether the file contains the
    names of the variables as its first line.  If missing, the value is
    determined from the file format: \code{header} is set to \code{TRUE}
    if and only if the first row contains one fewer field than the
    number of columns.}

\item{sep}{the field separator character.  Values on each line of the
    file are separated by this character.  If \code{sep = ""} (the
    default for \code{read.table}) the separator is \sQuote{white space},
    that is one or more spaces, tabs, newlines or carriage returns.}

\item{quote}{the set of quoting characters. To disable quoting
    altogether, use \code{quote = ""}.  See \code{\link{scan}} for the
    behaviour on quotes embedded in quotes.  Quoting is only considered
    for columns read as character, which is all of them unless
    \code{colClasses} is specified.}

\item{dec}{the character used in the file for decimal points.}

\item{numerals}{string indicating how to convert numbers whose conversion
    to double precision would lose accuracy, see \code{\link[utils]{type.convert}}.
    Can be abbreviated.  (Applies also to complex-number inputs.)}

\item{row.names}{a vector of row names.  This can be a vector giving
    the actual row names, or a single number giving the column of the
    table which contains the row names, or character string giving the
    name of the table column containing the row names.

    If there is a header and the first row contains one fewer field than
    the number of columns, the first column in the input is used for the
    row names.  Otherwise if \code{row.names} is missing, the rows are
    numbered.

    Using \code{row.names = NULL} forces row numbering. Missing or
    \code{NULL} \code{row.names} generate row names that are considered
    to be \sQuote{automatic} (and not preserved by \code{\link{as.matrix}}).
  }

\item{col.names}{a vector of optional names for the variables.
    The default is to use \code{"V"} followed by the column number.}

\item{as.is}{controls conversion of character variables (insofar as
    they are not converted to logical, numeric or complex) to factors,
    if not otherwise specified by \code{colClasses}.
    Its value is either a vector of logicals (values are recycled if
    necessary), or a vector of numeric or character indices which
    specify which columns should not be converted to factors.

    Note: to suppress all conversions including those of numeric
    columns, set \code{colClasses = "character"}.

    Note that \code{as.is} is specified per column (not per
    variable) and so includes the column of row names (if any) and any
    columns to be skipped.
  }

\item{na.strings}{a character vector of strings which are to be
    interpreted as \code{\link{NA}} values.  Blank fields are also
    considered to be missing values in logical, integer, numeric and
    complex fields.  Note that the test happens \emph{after}
    white space is stripped from the input (if enabled), so \code{na.strings}
    values may need their own white space stripped in advance.}

\item{colClasses}{character.  A vector of classes to be assumed for
    the columns.  If unnamed, recycled as necessary.  If named, names
    are matched with unspecified values being taken to be \code{NA}.

    Possible values are \code{NA} (the default, when
    \code{\link[utils]{type.convert}} is used), \code{"NULL"} (when the column
    is skipped), one of the atomic vector classes (logical, integer,
    numeric, complex, character, raw), or \code{"factor"}, \code{"Date"}
    or \code{"POSIXct"}.  Otherwise there needs to be an \code{as}
    method (from package \pkg{methods}) for conversion from
    \code{"character"} to the specified formal class.

    Note that \code{colClasses} is specified per column (not per
    variable) and so includes the column of row names (if any).
  }

\item{nrows}{integer: the maximum number of rows to read in.  Negative
    and other invalid values are ignored.}

\item{skip}{integer: the number of lines of the data file to skip before
    beginning to read data.}

\item{check.names}{logical.  If \code{TRUE} then the names of the
    variables in the data frame are checked to ensure that they are
    syntactically valid variable names.  If necessary they are adjusted
    (by \code{\link{make.names}}) so that they are, and also to ensure
    that there are no duplicates.}

\item{fill}{logical. If \code{TRUE} then in case the rows have unequal
    length, blank fields are implicitly added.  See \sQuote{Details}.}

\item{strip.white}{logical. Used only when \code{sep} has
    been specified, and allows the stripping of leading and trailing
    white space from unquoted \code{character} fields (\code{numeric} fields
    are always stripped).  See \code{\link{scan}} for further details
    (including the exact meaning of \sQuote{white space}),
    remembering that the columns may include the row names.}

\item{blank.lines.skip}{logical: if \code{TRUE} blank lines in the
    input are ignored.}

\item{comment.char}{character: a character vector of length one
    containing a single character or an empty string.  Use \code{""} to
    turn off the interpretation of comments altogether.}

\item{allowEscapes}{logical.  Should C-style escapes such as
    \samp{\\n} be processed or read verbatim (the default)?   Note that if
    not within quotes these could be interpreted as a delimiter (but not
    as a comment character).  For more details see \code{\link{scan}}.}

\item{flush}{logical: if \code{TRUE}, \code{scan} will flush to the
    end of the line after reading the last of the fields requested.
    This allows putting comments after the last field.}

\item{stringsAsFactors}{logical: should character vectors be converted
    to factors?  Note that this is overridden by \code{as.is} and
    \code{colClasses}, both of which allow finer control.}

\item{fileEncoding}{character string: if non-empty declares the
    encoding used on a file when given as a character string (not on an
    existing connection) so the character data can
    be re-encoded.  See the \sQuote{Encoding} section of the help for
    \code{\link{file}}, the \sQuote{R Data Import/Export} manual and
    \sQuote{Note}.
  }

\item{encoding}{encoding to be assumed for input strings.  It is
    used to mark character strings as known to be in
    Latin-1 or UTF-8 (see \code{\link{Encoding}}): it is not used to
    re-encode the input, but allows \R to handle encoded strings in
    their native encoding (if one of those two).  See \sQuote{Value}
    and \sQuote{Note}.
  }

\item{skipNul}{logical: should \abbr{NUL}s be skipped?}

\item{dim}{dimension of table content to show}

\item{...}{Further arguments to be passed to \code{read.table}.}
}
\value{
read table content and a print out of the data head
}
\description{
The purpose of this function is combine the functionality of
\strong{read.table} and \strong{print}, which are often used together.\cr\cr
The purpose of this function is to read table from a file into a variable and
simultaneously display a preview of the data, showing either the first few rows or
columns based on the user's specification. It is important to emphasize that the
function expects the user to assign the result of the read operation to a variable
in order to achieve its intended purpose. eg. Use \strong{var1 = read.table.print(file1)} instead of
\strong{read.table.print(file1)}
}
\details{
Read a dataset of type table and show x rows and y columns
}
\examples{
\dontrun{
# Example: read a table file and print the first 10 lines
# declare file
new.file <- "test.csv"

# read file and preview default
dth3 <- read.table.print(file = new.file, sep=",",quote = "\"",dec = ".",
fill = TRUE, comment.char = "", header = TRUE)

# read file and preview 10 rows and all columns
dth1 <- read.table.print(file = new.file, sep=",",quote = "\"",dec = ".",
fill = TRUE, comment.char = "", header = TRUE, dim = 10)

# read file and preview 10 rows and 5 columns
dth2 <- read.table.print(file = new.file, sep=",",quote = "\"",dec = ".",
fill = TRUE, comment.char = "", header = TRUE, dim = c(10,5))
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/aided_chatgpt.R
\name{na.cumsum}
\alias{na.cumsum}
\title{Cumulative Sum with NA Removal}
\usage{
na.cumsum(x)
}
\arguments{
\item{x}{A numeric vector for which the cumulative sum is to be calculated.}
}
\value{
A numeric vector representing the cumulative sum with `NA` values removed.
        Returns an empty vector if the input is empty or contains only `NA` values.
}
\description{
This function calculates the cumulative sum of a numeric vector, removing any `NA` values before computation.
}
\examples{
# Example with numeric vector
numeric_vector <- c(1, 2, NA, 4, 5)
na.cumsum(numeric_vector)

# Example with all NAs
na.cumsum(c(NA, NA))

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/envobj.R
\name{summarize.envobj}
\alias{summarize.envobj}
\title{Get all the environment objects and their sizes}
\usage{
summarize.envobj(envir = parent.frame())
}
\arguments{
\item{envir}{the environment to retrieve objects from}
}
\value{
a dataframe of all the variables within the environment
}
\description{
Retrieve the size contribution of all the available objects in the environment
}
\examples{
# Get a data frame of all environment objects and their size
summarize.envobj()
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/bool.R
\name{yesNoBool}
\alias{yesNoBool}
\title{Convert Yes/No to Binary or Logical}
\usage{
yesNoBool(
  table,
  fldname,
  out = c("change", "append", "vector"),
  type = c("bin", "log")
)
}
\arguments{
\item{table}{data frame}

\item{fldname}{field name in the data frame}

\item{out}{output form, choices - change, append, vector}

\item{type}{output type, choices - bin, log}
}
\value{
converted Yes/No entries into 1/0 or TRUE/FALSE
}
\description{
Seamlessly convert a yes or no to either a binary or logical output
}
\details{
type - "bin" for binary, and "log" for logical
}
\examples{
\donttest{
# Declare data for example
usedata <- data.frame(ID = 1:32)
usedata #view the dataset

usedata$yess = rep(c("yes","n","no","YES","No","NO","yES","Y"),4) #create a new column
usedata #view the modified dataset

# Set all yess field as standardize boolean
# Task: convert the "yess" column content to 1/0 or TRUE/FALSE
# Notice that you have add the column name with or without quotes
yesNoBool(usedata,yess, type="bin") #set all as binary 1/0
yesNoBool(usedata,"yess", type="log") #set all as logical TRUE/FALSE


# Task: By default, the 'out' argument is set to "change"
# means that the original data field will be
# replaced with the results as above

# In this example, set the out variable to
# append data frame with a new column name containing the result

yesNoBool(usedata,yess,"append")
#or yesNoBool(usedata,"yess","append")

# In this example, return as vector
yesNoBool(usedata,yess,"vector")
#or yesNoBool(usedata,"yess","vector")

# Task: Return result as logical
yesNoBool(usedata,"yess",type = "log")
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/rcolorconst.R
\name{rcolorconst}
\alias{rcolorconst}
\title{R Color Constant}
\usage{
rcolorconst(title = "R Color Constants")
}
\arguments{
\item{title}{title of the output}
}
\value{
returns color constant
}
\description{
This function provides information that describes the color constants that exist in R
}
\details{
In addition to the color palette in R that can be represented as either color literals or hexadecimal values,
numeric values can also be used to add colorization to a plot. Numeric values ranging from 1 to 8 provide 8 basic colors that can be deployed. The rcolorconst
function returns both a Named Vector and a color palette plot that connects these numeric values with their corresponding color.
}
\examples{
# Without title
ex1 <- rcolorconst()

# With title
ex2 <- rcolorconst("My new color constant")

# More detailed example
set.seed(200)
x = data.frame(
  meas = rnorm(100),
  grp = sample(1:8, size = 100,
  replace = TRUE))
plot(x, pch = 16, col = x$grp)
colnums = rcolorconst()

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/nm_scale.R
\name{print.nonmem_scaling}
\alias{print.nonmem_scaling}
\title{Print method for s1_scaling objects}
\usage{
\method{print}{nonmem_scaling}(x, ...)
}
\arguments{
\item{x}{An object of class \code{s1_scaling}.}

\item{...}{Additional arguments (ignored).}
}
\description{
Print method for s1_scaling objects
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/duplicate.R
\name{ai.duplicate}
\alias{ai.duplicate}
\title{Prompt guided duplication and editing of files}
\usage{
ai.duplicate(file = NULL, new.name = NULL, open = TRUE)
}
\arguments{
\item{file}{file to duplicate}

\item{new.name}{OPTIONAL.name of new file}

\item{open}{open file after duplication}
}
\value{
duplicated files with edited texts
}
\description{
AI like duplication and editing of files
}
\examples{
\donttest{
if(interactive()){
file1s <- paste0(tempfile(),".R")
writeLines("message(
'Sample items: farm, shinyappstore, rpkg'
)", file1s)
ai.duplicate(file1s,'file2.R')
}
}


}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/data_rep.R
\name{data_rep}
\alias{data_rep}
\title{Duplicate a data rows or columns X times}
\usage{
data_rep(., n, which = c("rows", "cols"))
}
\arguments{
\item{.}{data frame variable}

\item{n}{multiples of duplicate}

\item{which}{where to append the duplicated data e.g. rows or cols}
}
\value{
the duplicated dataset store to a variable with the name of the first
}
\description{
Add a data to itself X times by rows or columns
}
\examples{
# initialize p1 and p2
init(p1,p2)
p1
p2

# declare p1 and p2 as data frame
p1 <- data.frame(PK=1:10,ID2=1:10)
p2 <- data.frame(PK=11:20,ID2=21:30)

p1
p2

#add p1  twice by row, and resave as p1
data_rep(p1,n=2,"rows")
p1 #p1 has been updated


#add p2  3 times by col, and resave as p2
data_rep(p2,n=3,"cols")
p2 #p2 has been updated
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/init.R
\name{init}
\alias{init}
\title{Initialize new variables and objects}
\usage{
init(..., value = NULL)
}
\arguments{
\item{...}{variable names to initialize}

\item{value}{value to initialize them to}
}
\value{
initialized objects set to the value specified
}
\description{
Shorthand to initialize one or more objects
}
\examples{
init(t,u,v)
message(t) # t = NULL
message(u) # u = NULL
message(v) # v = NULL
init(j,k,m,value = 7)
message(j) # j = 7
message(k) # k = 7
message(m) # m = 7

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/categorical_to_numeric.R
\name{cat_to_num}
\alias{cat_to_num}
\alias{cat_to_num2}
\title{Convert Categorical Values to Numeric Improvement to seq_along()}
\usage{
cat_to_num(x, decreasing = FALSE)

cat_to_num2(x, decreasing = FALSE)
}
\arguments{
\item{x}{A vector containing categorical values (character or factor)}

\item{decreasing}{logical. Should the sort be increasing or decreasing?}
}
\value{
A numeric vector where each category is mapped to a number based on its
sorted position. NA values in the input remain NA in the output.
}
\description{
Converts categorical values into numeric by sorting unique values
alphabetically/numerically and assigning sequential numbers starting from 1
}
\examples{
# Example 1: Basic character vector
fruits <- c("apple", "banana", "apple", "cherry", "banana")
cat_to_num(fruits, decreasing = FALSE)  # Returns: 1, 2, 1, 3, 2

# Example 2: Vector with NAs
grades <- c("A", NA, "C", "B", "A", NA)
cat_to_num(grades)  # Returns: 1, NA, 3, 2, 1, NA

# Example 3: Factor input
animals <- factor(c("dog", "cat", "bird", "cat", "dog"))
cat_to_num(animals)  # Returns: 3, 2, 1, 2, 3

# Example 4: Numeric categories
sizes <- c("XL", "S", "M", "XS", "L")
cat_to_num(sizes)  # Returns: 5, 2, 3, 1, 4

# Example 5: Single category
status <- c("active", "active", "active")
cat_to_num(status)  # Returns: 1, 1, 1

# Examples for cat_to_num2

# Assign numeric values to a set of categorical values
# Call the value assignment to assign
# a numeric value to a new categorical value

cat_to_num2(letters[1:7])("b")
cat_to_num2(letters[1:7], decreasing = FALSE)("d")
# the above assigns numeric values to letters a-g
# and then calls the number for "a"

cat_vals_5 <- c("apple","orange","guava")
num_equiv <- cat_to_num2(cat_vals_5)

applenum = num_equiv("apple")


}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/color_compHist.R
\name{mix.cols.btw}
\alias{mix.cols.btw}
\title{Mix or Blend colors between two or more colors}
\usage{
mix.cols.btw(colors, max = 20, alpha = 1, preview = FALSE)
}
\arguments{
\item{colors}{the vector of two colors}

\item{max}{maximum number of colors to blend between}

\item{alpha}{alpha for the new color blends}

\item{preview}{LOGICAL. preview all color generated}
}
\value{
color hex for all generated colors
}
\description{
Mix or blend multiple colors between two colors
}
\examples{
# simply mix/blend two colors
mix.cols.btw(c("red","brown"))

# simply mix/blend two colors, maximum number of colors at the end
mix.cols.btw(c("red","brown"), max = 8)

# simply mix/blend two colors with alpha=0.2 (opacity=0.2)
mix.cols.btw(c("yellow","green"),alpha = 0.2)

# also preview after mixing the two colors
mix.cols.btw(c("red","green"), preview = TRUE)
mix.cols.btw(c("blue","violet"),alpha = 0.2, preview = TRUE)

mix.cols.btw(c("red","purple","yellow","gray"), preview = TRUE)

mix.cols.btw(c("red","purple","yellow","gray"),alpha = 0.2, preview = TRUE)

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/uniquelen.R
\name{unique_len}
\alias{unique_len}
\title{Combine unique() and length()}
\usage{
unique_len(.)
}
\arguments{
\item{.}{object such as vector or names(dataframe)}
}
\value{
length of the unique items in a vector
}
\description{
Combine two frequently used function together to return the length of the unique items of a vector
}
\examples{
frenchnames1 = c("Léa","Obinna","Bastien","Léa","Obinna", "Hugo", "Amélie","Louis")
unique_len(frenchnames1)

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/OR.R
\name{error.out}
\alias{error.out}
\alias{\%eo\%}
\title{Error coalescing operator}
\usage{
error.out(test, alternative = "")

test \%eo\% alternative
}
\arguments{
\item{test}{an object to return}

\item{alternative}{alternative object to return}
}
\value{
value of test if error, else return value of alternative
}
\description{
Alternative return for error statements.Return alternative if the value of expression is erroneous
}
\examples{
# The following statement would produce
# error because 'stat1' does not exist

# stat1 + 1

# To prevent the statement from
# stopping the process, when can have alternative out
alt = 0
error.out(stats1 + 1, alt)

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/not.data.R
\name{not.data}
\alias{not.data}
\title{Not a data}
\usage{
not.data(x)
}
\arguments{
\item{x}{vector entry}
}
\value{
a boolean value to indicate if entry is a data table
}
\description{
Opposite of is.data.frame(). Check if entry is not a data object
}
\examples{
test.dt <- data.frame(ID=1:200,Type="RPKG.net")
test.notenv <- list(t=1)

is.data.frame(test.dt) # TRUE
not.data(test.dt) # FALSE

not.data(test.notenv) # TRUE
if(not.data(test.dt)) message("yes") # NULL

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/0.7_combo_func1.R
\name{strsplit.num}
\alias{strsplit.num}
\title{Split a string of numbers and return as numeric vector}
\usage{
strsplit.num(x, split, fixed = FALSE, perl = FALSE, useBytes = FALSE)
}
\arguments{
\item{x}{character vector, each element of which is to be split. Other inputs, including a factor, will give an error.}

\item{split}{character vector}

\item{fixed}{logical. If TRUE match split exactly, otherwise use regular expressions. Has priority over perl.}

\item{perl}{logical. Should Perl-compatible regexps be used?}

\item{useBytes}{logical. If TRUE the matching is done byte-by-byte rather than character-by-character, and inputs with marked encodings are not converted.}
}
\value{
numeric values based on split string
}
\description{
The purpose of this function is combine the functionality of
\strong{strsplit}, \strong{unlist} and \strong{as.numeric}, which are often used together.\cr\cr
}
\details{
Given a sting, split by a separator into numbers
}
\examples{
# Example 1
# string of numbers with separator " "
num.01 = "5 3 2 3 5 2 33 23 5 32 432 42 23 554"

# split a string of numbers and return as numeric
strsplit.num(num.01, split = " ")


# Example 2
# string of numbers with separator "|||"
num.02 = "0|||1|||4|||43|||6|||8|||00||| 1||| 0 1||| T |||F|||TRUE |||f"

# split a string of numbers and return as numeric
strsplit.num(num.02, split = "[|||]")

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/sort.R
\name{sort_length}
\alias{sort_length}
\alias{sort_file_type}
\title{Sort vector by length or file types of its content}
\usage{
sort_length(vec, asc = TRUE)

sort_file_type(files, asc = TRUE)
}
\arguments{
\item{vec}{a vector}

\item{asc}{A logical value indicating whether to sort in ascending (TRUE) or descending (FALSE) order. Default is TRUE.}

\item{files}{A character vector containing file names to be sorted}
}
\value{
vector of items sorted by length

A character vector of sorted file names
}
\description{
Sort the length or file types of the content of a vector

Takes a vector of file names and sorts them by their file extensions (file type)
}
\note{
This function removes all NAs prior to sorting the vector
}
\examples{
# sort by length of content
x = c("acs","tt","jdssr","h","grab")
sort_length(vec = x) # ascending order of length
sort_length(vec = x, asc = FALSE) # descending order of length


files <- c("doc1.pdf",
  "image.jpg", "house.csv", "notes.txt",
  "patab","doc2.pdf", "data.csv", "pic.png","cotab")
sort_file_type(files)
sort_file_type(files, asc = FALSE)

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/libraryAll.R
\name{libraryAll}
\alias{libraryAll}
\title{Load specific R libraries and clear environment}
\usage{
libraryAll(
  ...,
  lib.loc = NULL,
  quietly = FALSE,
  clear = TRUE,
  clearPkgs = FALSE
)
}
\arguments{
\item{...}{multiple library names}

\item{lib.loc}{OPTIONAL. library store location}

\item{quietly}{OPTIONAL. attach library quietly}

\item{clear}{OPTIONAL. clear environment after attach}

\item{clearPkgs}{Clear previous loaded packages, TRUE or FALSE}
}
\value{
loaded libraries and clear environment
}
\description{
Load specific packages, print a list of the loaded packages along with versions.
Only include libraries, don't install if library doesn't exist
}
\examples{
\donttest{
# load packages and print their versions to the console
libraryAll(base) #one package

libraryAll(
  base,
  tools,
  stats
) #multiple packages

libraryAll("grDevices") #with quotes

libraryAll(
  stats,
  utils,
  quietly = TRUE
) #load quietly

libraryAll(
  base,
clear = FALSE) #do not clear console after load

# clear previously loaded packages, then load r2resize and r2social
libraryAll(
  stats,
  utils,
  clearPkgs = TRUE
)
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/vector_shuffle.R
\name{vector_shuffle}
\alias{vector_shuffle}
\title{Shuffle a vector just like shuffle in PHP}
\usage{
vector_shuffle(., replace = FALSE, prob = NULL, seed = NULL)
}
\arguments{
\item{.}{vector to shuffle}

\item{replace}{replace selected value}

\item{prob}{probability of occurrence}

\item{seed}{apply seed if indicated for reproducibility}
}
\value{
shuffled vector of items store to the vector name
}
\description{
Shorthand to shuffle a vector and save
}
\examples{

#basic example
vector_shuffle(c(3,45,23,3,2,4,1))


#using objects
v1<-c(3,45,23,3,2,4,1)

#demonstrate vector_shuffle
vector_shuffle(v1)
v1 # show outputs

#demonstrate reproducibility in shuffle with seed
v0<-v1
vector_shuffle(v0)
v0 #first output

v0<-v1
vector_shuffle(v0)
v0 # different output from first output top

v0<-v1
vector_shuffle(v0,seed = 232L)
v0 #second output

v0<-v1
vector_shuffle(v0,seed = 232L)
v0 #the same output as second output top
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/list_push.R
\name{list_push}
\alias{list_push}
\title{Add elements to a list like array_push in PHP}
\usage{
list_push(., add)
}
\arguments{
\item{.}{first list}

\item{add}{list to add}
}
\value{
vector combining fist and second vector, but have name set to the first
}
\description{
Shorthand to add elements to a vector and save as the same name
}
\examples{
num1 <- list(sample(330:400,10))
num2 <-list("rpkg.net")
list_push(num1, add= num2)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/aided_chatgpt.R
\name{rows.rep}
\alias{rows.rep}
\title{Replicate Rows in a Data Frame}
\usage{
rows.rep(data, n)
}
\arguments{
\item{data}{A data frame whose rows are to be replicated.}

\item{n}{An integer specifying the number of times each row should be replicated.}
}
\value{
A data frame with each row replicated `n` times. If `n` is less than or equal to 0, an empty data frame is returned.
}
\description{
This function replicates each row in a data frame a specified number of times.
}
\examples{
# Example with a simple data frame
df <- data.frame(A = c(1, 2), B = c("x", "y"))
rows.rep(df, 3)

# Example with no replication (n = 0)
rows.rep(df, 0)

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/detect_outlier.R
\name{detect_outlier2}
\alias{detect_outlier2}
\title{Advanced Outlier Detection in Numeric Data with Optional Grouping}
\usage{
detect_outlier2(
  x,
  groups = NULL,
  method = c("iqr", "zscore", "modified_zscore", "all"),
  multiplier = 1.5,
  z_threshold = 3,
  modified_z_threshold = 3.5,
  plot = TRUE,
  plot_groups = TRUE
)
}
\arguments{
\item{x}{A numeric vector containing the data to analyze}

\item{groups}{Optional vector of group labels, must be same length as x}

\item{method}{Character string specifying the outlier detection method.
Options are "iqr", "zscore", "modified_zscore", or "all". Default is "iqr"}

\item{multiplier}{Numeric value specifying the IQR multiplier for
outlier detection. Default is 1.5}

\item{z_threshold}{Numeric value specifying the Z-score threshold.
Default is 3}

\item{modified_z_threshold}{Numeric value specifying the modified Z-score
threshold. Default is 3.5}

\item{plot}{Logical indicating whether to generate visualization plots.
Default is TRUE}

\item{plot_groups}{Logical indicating whether to create separate plots for each
group. Only used when groups are provided. Default is TRUE}
}
\value{
A list of class "outlier_analysis" containing:
  \itemize{
    \item overall: Overall analysis results
    \item by_group: Group-specific results (if groups provided)
    \item plots: List of generated plots
  }
}
\description{
Detects outliers in numeric data using multiple statistical methods and provides
comprehensive analysis including visualizations and summary statistics. Supports
grouped analysis when group vector is provided. The function supports three
detection methods: IQR, Z-score, and Modified Z-score.
}
\examples{
# Example 1: Basic grouped analysis
set.seed(123)
data <- c(rnorm(50), rnorm(50, 2), rnorm(50, 4))
groups <- rep(c("A", "B", "C"), each = 50)
resultA <- detect_outlier2(data) # no groups
resultB <- detect_outlier2(data, groups = groups) # with groups

# Example 2: Custom thresholds by group
test_scores <- c(65, 70, 75, 72, 68, 73, 78, 71, 69, 74,
                 90, 85, 92, 88, 95, 87, 91, 89, 86, 93)
class_groups <- rep(c("Morning", "Afternoon"), each = 10)
result <- detect_outlier2(test_scores,
                                   groups = class_groups,
                                   method = "all",
                                   z_threshold = 2)
result$overall
result$by_group
result$plots$overall$boxplot()
result$plots$overall$density()
result$plots$overall$comparison()
result$plots$by_group$Morning$boxplot()
result$plots$by_group$Morning$density()
result$plots$by_group$Afternoon$boxplot()
result$plots$by_group$Afternoon$density()

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/not.vector.R
\name{not.vector}
\alias{not.vector}
\title{Not a vector}
\usage{
not.vector(x)
}
\arguments{
\item{x}{vector entry}
}
\value{
a boolean value to indicate if entry is vector
}
\description{
Opposite of is.vector(). Check if entry is not vector
}
\examples{
vect1 = list(r=1,t=3:10)
vect2 = LETTERS
is.vector(vect1) # TRUE
not.vector(vect1) # FALSE
not.vector(vect2) # FALSE
if(not.vector(vect1)) message("yes") # NULL

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/color_compHist.R
\name{compHist}
\alias{compHist}
\title{Compare histograms of two distributions}
\usage{
compHist(
  x1,
  x2,
  title,
  col1 = "red",
  col2 = "yellow",
  xlab = "",
  ylab = "Frequency",
  separate = FALSE
)
}
\arguments{
\item{x1}{NUMERIC. the first distribution}

\item{x2}{NUMERIC. the second distribution}

\item{title}{CHARACTER. title of the histogram plot}

\item{col1}{CHARACTER. color fill for first distribution}

\item{col2}{CHARACTER. color fill for second distribution}

\item{xlab}{CHARACTER. label of the x-axis}

\item{ylab}{CHARACTER. label of the y-axis}

\item{separate}{LOGICAL. whether to separate the plots}
}
\value{
return histogram comparison using basic histogram plot
}
\description{
For comparing histograms of two data distributions.
Simply input the two distributions, and it generates a clear and
informative histogram that illustrates the differences between the data.
}
\details{
Users have the option to view individual histograms for each distribution
before initiating the comparison, allowing for a detailed examination of
each dataset's characteristics. This feature ensures a comprehensive
understanding of the data and enhances the user's ability to interpret
the results of the distribution comparison provided by this function.
}
\note{
- Hexadecimal values can also be passed \cr
  in for col1 and col2, see the example section
- For best visual results, \cr
  col1 should be a dark color and col2 should be passed as a light color. \cr
  For example, col1 = "black", col2 = "yellow"
}
\section{Some recommended color pairs}{

col1 = 'dodgerblue4' (and) col2 = 'darksalmon' \cr
col1 = 'brown' (and) col2 = 'beige' \cr
col1 = 'pink' (and) col2 = 'royalblue4' \cr
col1 = 'red' (and) col2 = 'yellow' \cr
col1 = 'limegreen' (and) col2 = 'blue' \cr
col1 = 'darkred' (and) col2 = 'aquamarine4' \cr
col1 = 'purple' (and) col2 = 'yellow' \cr
}

\examples{
# compare two normal distributions with means that differ a lot
# in this case, the overlap will not be observed
set.seed(123)
compHist(
  x1 = rnorm(1000, mean = 3),
  x2 = rnorm(1000, mean = 10),
  title = "Histogram of Distributions With Means 3 & 10",
  col1 = "yellow", col2 = "violet"
)


# compare two normal distributions with means that are close
# in this case, the overlap between the histograms will be observed
set.seed(123)
compHist(
  x1 = rnorm(1000, mean = 0),
  x2 = rnorm(1000, mean = 2),
  title = "Histogram of rnorm Distributions With Means 0 & 2",
  col1 = "lightslateblue", col2 = "salmon"
)

set.seed(123)
# separate the plots for preview
compHist(
  x1 = rnorm(1000, mean = 0),
  x2 = rnorm(1000, mean = 2),
  title = c("Plot Means 0", "Plot Means 2"),
  col1 = "#F96167", col2 = "#CCF381",
  separate = TRUE
)

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/add.header.R
\name{add.header}
\alias{add.header}
\title{Addin snippet function to add header comment to a current opened file}
\usage{
add.header()
}
\value{
Inserts header content for file
}
\description{
Shorthand to add header comment
}
\examples{
if(interactive())
add.header()
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/duplicate.R
\name{duplicate}
\alias{duplicate}
\title{Duplicate a file with global text replacement}
\usage{
duplicate(file, new.name, pattern = NULL, replacement = NULL, open = TRUE)
}
\arguments{
\item{file}{data frame to re-sample}

\item{new.name}{column to uniquely re-sample}

\item{pattern}{number of rows to return}

\item{replacement}{unique numeric value for reproducibility}

\item{open}{description}
}
\value{
data frame containing re-sampled rows from an original data frame
}
\description{
Shorthand to return a re-sample number of rows in a data frame by unique column
}
\examples{
\donttest{
if(interactive()){
# example to duplicate a file, and replace text1 within it
# NOTE that, by default, this function will also open the file within RStudio

#create sample file
file1s <- paste0(tempfile(),".R")
writeLines("message(
'Sample items: eggs, coke, fanta, book'
)", file1s)

file2s <- paste0(tempfile(),".R")
file3s <- paste0(tempfile(),".R")

duplicate(
  file = file1s,
  new.name = file2s,
  pattern = 'text1',
  replacement = 'replacement1'
 )

# duplicate the file, with multiple replacements
# replace 'book' with 'egg' and 'coke' with 'fanta'
duplicate(
  file1s, file2s,
  pattern = c('book','coke'),
  replacement = c('egg','fanta')
)



# duplicate the file with no replacement
duplicate(file1s,file3s) # this simply performs file.copy, and opens the new file



# duplicate the file but do not open for editing
duplicate(file1s,file3s, open = FALSE) # this does not open file after duplication
}
}
}
